<?php

namespace Pg\Modules\Dynamic_blocks\Models;

/**
 * Dynamic blocks module
 *
 * @package     PG_RealEstate
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

define('DYNBLOCKS_AREAS_TABLE', DB_PREFIX . 'dynblocks_areas');
define('DYNBLOCKS_AREA_BLOCK_TABLE', DB_PREFIX . 'dynblocks_area_block');
define('DYNBLOCKS_BLOCKS_TABLE', DB_PREFIX . 'dynblocks_blocks');
define('DYNBLOCKS_PRESETS_TABLE', DB_PREFIX . 'dynblocks_presets');
define('DYNBLOCKS_PRESETS_BLOCK_TABLE', DB_PREFIX . 'dynblocks_presets_block');

/**
 * Dynamic blocks main model
 *
 * @package     PG_RealEstate
 * @subpackage  Dynamic blocks
 *
 * @category    models
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Dynamic_blocks_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Properties dynamic block object in data source
     *
     * @var array
     */
    protected $block_attrs = array(
        'id',
        'gid',
        'module',
        'model',
        'method',
        'min_width',
        'params',
        'views',
        'date_add',
        'date_modified',
    );

    /**
     * Area block object properties in data source
     *
     * @var array
     */
    protected $fields_area_block = array(
        "id",
        "id_area",
        "id_block",
        "params",
        "view_str",
        "width",
        "sorter",
        "cache_time",
    );

    /**
     * Properties of area object in data source
     *
     * @param array
     */
    protected $_fields_area = array(
        "id",
        "name",
        "gid",
        "date_add",
    );

    /**
     * Preset object properties in data source
     *
     * @var array
     */
    protected $fields_preset = array(
        'id',
        'gid',
        'gid_area',
        'name',
        'logo',
        'comment',
        'editable',
        'date_add',
    );

    /**
     * Preset block object properties in data source
     *
     * @var array
     */
    protected $fields_preset_block = array(
        'id',
        'id_preset',
        'gid_area',
        'id_block',
        'params',
        'view_str',
        'width',
        'sorter',
        'cache_time',
    );

    /**
     * Cache directory of dynamic blocks
     *
     * @var string
     */
    protected $cache_dir = '';

    /**
     * Module name (GUID)
     *
     * @param string
     */
    public $module_gid = "dynamic_blocks";

    /**
     * Block types (GUID)
     *
     * @param string
     */
    public $block_types_gid = "block_types";

    /**
     * Preset upload type (GUID)
     *
     * @param string
     */
    public $preset_upload_type = "preset-logo";

    /**
     * Constructor
     *
     * @return Dynamic_blocks_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        $this->cache_dir = TEMPPATH . "dynamic_blocks/";
    }

    /**
     * Return area object by identifier
     *
     * @param integer $id area identifier
     *
     * @return array
     */
    public function get_area_by_id($id)
    {
        $data = array();
        $this->ci->db->select(implode(",", $this->_fields_area));
        $this->ci->db->from(DYNBLOCKS_AREAS_TABLE);
        $this->ci->db->where("id", $id);
        $result = $this->ci->db->get()->result_array();
        if (!empty($result)) {
            $area = $this->format_area(array($result[0]));
            $data = array_shift($area);
        }

        return $data;
    }

    /**
     * Return area object by guid
     *
     * @param string $gid area guid
     *
     * @return array
     */
    public function get_area_by_gid($gid)
    {
        $data = array();
        $this->ci->db->select(implode(",", $this->_fields_area));
        $this->ci->db->from(DYNBLOCKS_AREAS_TABLE);
        $this->ci->db->where("gid", $gid);
        $result = $this->ci->db->get()->result_array();
        if (!empty($result)) {
            $area = $this->format_area(array($result[0]));
            $data = array_shift($area);
        }

        return $data;
    }

    /**
     * Save area object to data source
     *
     * @param integer $id   area identifier
     * @param array   $data area data
     *
     * @return integer
     */
    public function save_area($id, $data)
    {
        if (empty($id)) {
            $data["date_add"] = date("Y-m-d H:i:s");
            $this->ci->db->insert(DYNBLOCKS_AREAS_TABLE, $data);
            $id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where('id', $id);
            $this->ci->db->update(DYNBLOCKS_AREAS_TABLE, $data);
        }

        return $id;
    }

    /**
     * Validate area object for saving to data source
     *
     * @param integer $id   area identifier
     * @param array   $data area data
     *
     * @return array
     */
    public function validate_area($id, $data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data["name"])) {
            $return["data"]["name"] = strip_tags($data["name"]);
            if (empty($return["data"]["name"])) {
                $return["errors"][] = l('error_name_mandatory_field', 'dynamic_blocks');
            }
        }

        if (isset($data["gid"])) {
            $return["data"]["gid"] = trim(strip_tags($data["gid"]));
            $return["data"]["gid"] = preg_replace('/[^a-z\-_0-9]+/i', '', $return["data"]["gid"]);
            $return["data"]["gid"] = preg_replace('/[\s\n\t\r]+/', '-', $return["data"]["gid"]);
            $return["data"]["gid"] = preg_replace('/\-{2,}/', '-', $return["data"]["gid"]);

            if (empty($return["data"]["gid"])) {
                $return["errors"][] = l('error_gid_mandatory_field', 'dynamic_blocks');
            } else {
                $this->ci->db->select('COUNT(*) AS cnt')->from(DYNBLOCKS_AREAS_TABLE)->where("gid", $return["data"]["gid"]);
                if (!empty($id)) {
                    $this->ci->db->where("id <>", $id);
                }
                $result = $this->ci->db->get()->result_array();
                if (!empty($result) && $result[0]["cnt"] > 0) {
                    $return["errors"][] = l('error_area_already_exists', 'dynamic_blocks');
                }
            }
        }

        return $return;
    }

    /**
     * Format area data
     *
     * @param array $data area data
     *
     * @return array
     */
    public function format_area($data)
    {
        return $data;
    }

    /**
     * Remove area data by identifier
     *
     * @param integer $id area identifier
     *
     * @return void
     */
    public function delete_area($id)
    {
        $this->ci->db->where("id", $id);
        $this->ci->db->delete(DYNBLOCKS_AREAS_TABLE);

        $this->ci->db->where("id_area", $id);
        $this->ci->db->delete(DYNBLOCKS_AREA_BLOCK_TABLE);

        return;
    }

    /**
     * Remove area data by guid
     *
     * @param string $gid area guid
     *
     * @return void
     */
    public function delete_area_by_gid($gid)
    {
        $area_data = $this->get_area_by_gid($gid);
        if (empty($area_data)) {
            return false;
        }

        $this->delete_area($area_data["id"]);
    }

    /**
     * Return areas' objects as array
     *
     * @param integer $page              page of results
     * @param integer $items_on_page     results per page
     * @param array   $order_by          sorting data
     * @param array   $params            filters data
     * @param array   $filter_object_ids filters identifiers
     *
     * @return array
     */
    public function get_areas_list($page = null, $items_on_page = null, $order_by = null, $params = array(), $filter_object_ids = null)
    {
        $this->ci->db->select(implode(',', $this->_fields_area))->from(DYNBLOCKS_AREAS_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->_fields_area)) {
                    $this->ci->db->order_by($field . " " . $dir);
                }
            }
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($items_on_page, $items_on_page * ($page - 1));
        }

        $data = array();
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $data = $results;
        }

        return $data;
    }

    /**
     * Return number of areas
     *
     * @param array $params            filters data
     * @param array $filter_object_ids filters identifiers
     *
     * @return integer
     */
    public function get_areas_count($params = array(), $filter_object_ids = null)
    {
        $this->ci->db->select('COUNT(*) AS cnt')->from(DYNBLOCKS_AREAS_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["cnt"]);
        }

        return 0;
    }

    /**
     * Return area block object by identifier
     *
     * @param integer $id area block identifier
     *
     * @return array
     */
    public function get_area_block_by_id($id)
    {
        $data = array();
        $this->ci->db->select(implode(",", $this->fields_area_block))->from(DYNBLOCKS_AREA_BLOCK_TABLE)->where("id", $id);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $results = $this->format_area_block(array($results[0]));
            $data = $results[0];
        }

        return $data;
    }

    /**
     * Return area blocks as array
     *
     * @param integer $id_area area identifier
     *
     * @return array
     */
    public function get_area_blocks($id_area)
    {
        $data = array();
        $this->ci->db->select(implode(",", $this->fields_area_block))->from(DYNBLOCKS_AREA_BLOCK_TABLE)->where("id_area", $id_area)->order_by("sorter ASC");
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $data = $this->format_area_block($results);
        }
        return $data;
    }

    /**
     * Return number of area blocks
     *
     * @param integer $id_area area idnetifier
     *
     * @return integer
     */
    public function get_area_blocks_count($id_area)
    {
        $this->ci->db->select('COUNT(*) AS cnt')->from(DYNBLOCKS_AREA_BLOCK_TABLE)->where("id_area", $id_area);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["cnt"]);
        }

        return 0;
    }

    /**
     * Save area block object to data source
     *
     * @param integer $id   area block identifier
     * @param array   $data area block data
     *
     * @return integer
     */
    public function save_area_block($id, $data)
    {
        if (empty($id)) {
            if (!isset($data["width"]) || !$data["width"]) {
                $data["width"] = 100;
            }
            $this->ci->db->insert(DYNBLOCKS_AREA_BLOCK_TABLE, $data);
            $id = $this->ci->db->insert_id();
            if (!isset($data['sorter'])) {
                $this->refresh_area_block_sorter($data["id_area"]);
            }
        } else {
            $this->ci->db->where('id', $id);
            $this->ci->db->update(DYNBLOCKS_AREA_BLOCK_TABLE, $data);
        }
        $this->cache_clear();
        return $id;
    }

    /**
     * Set sorting value of area block
     *
     * @param integer $id     area block identifier
     * @param integer $sorter sorting value
     * @param integer $width  width value
     *
     * @return void
     */
    public function save_area_block_sorter($id, $sorter, $width = 0)
    {
        $data["sorter"] = $sorter;
        if ($width) {
            $data["width"] = $width;
        }
        $this->ci->db->where('id', $id);
        $this->ci->db->update(DYNBLOCKS_AREA_BLOCK_TABLE, $data);

        return;
    }

    /**
     * Remove area block data
     *
     * @param integer $id area block identifier
     *
     * @return void
     */
    public function delete_area_block($id)
    {
        $area_block_data = $this->get_area_block_by_id($id);

        $this->ci->db->where('id', $id);
        $this->ci->db->delete(DYNBLOCKS_AREA_BLOCK_TABLE);

        $this->refresh_area_block_sorter($area_block_data["id_area"]);
    }

    /**
     * Validate area block object for saving to data source
     *
     * @param integer $block_id area block idnetifier
     * @param array   $data     area block data
     *
     * @return array
     */
    public function validate_area_block($block_id, $data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data['id'])) {
            $return['data']['id'] = intval($data['id']);
        }

        if (isset($data['id_area'])) {
            $return['data']['id_area'] = intval($data['id_area']);
        }

        if (isset($data['id_block'])) {
            $return['data']['id_block'] = intval($data['id_block']);
        }

        if (isset($data['params'])) {
            $return['data']['params'] = serialize($data['params']);
        }

        if (isset($data['view_str'])) {
            $return['data']['view_str'] = trim(strip_tags($data['view_str']));
        }

        if (isset($data['width'])) {
            $return['data']['width'] = intval($data['width']);
        }

        if (isset($data['sorter'])) {
            $return['data']['sorter'] = intval($data['sorter']);
        }

        if (isset($data['cache_time'])) {
            $return['data']['cache_time'] = intval($data['cache_time']);
        }

        return $return;
    }

    /**
     * Format area block data
     *
     * @param array $data area block data
     *
     * @return array
     */
    public function format_area_block($data)
    {
        foreach ($data as $i => $block) {
            $data[$i]["params"] = $block["params"] ? unserialize($block["params"]) : array();
            $data[$i]["width"] = $block["width"] ? $block["width"] : 100;
        }

        return $data;
    }

    /**
     * Resort area blocks
     *
     * @param integer $id_area area identifier
     *
     * @return void
     */
    public function refresh_area_block_sorter($id_area)
    {
        $blocks = $this->get_area_blocks($id_area);
        if (empty($blocks)) {
            return false;
        }

        $sorter = 0;
        $prev_block = null;
        $row_width = 0;
        foreach ($blocks as $block) {
            $row_width += $block['width'];
            if ($row_width > 100) {
                $prev_block["width"] += 100 - ($row_width - $block["width"]);
                $this->save_area_block_sorter($prev_block["id"], $sorter, $prev_block['width']);
                $row_width = $block["width"];
            } elseif ($row_width == 100) {
                $row_width = 0;
            }
            $prev_block = $block;
            $this->save_area_block_sorter($block["id"], ++$sorter, $block['width']);
        }

        if ($row_width) {
            $prev_block["width"] += 100 - $row_width;
            $this->save_area_block_sorter($prev_block["id"], $sorter, $prev_block['width']);
        }
    }

    /**
     * Return block object by identifier
     *
     * @param integer $id block identifier
     *
     * @return array
     */
    public function get_block_by_id($id)
    {
        $data = array();
        $this->ci->db->select(implode(', ', $this->block_attrs))->from(DYNBLOCKS_BLOCKS_TABLE);
        if (is_array($id)) {
            $this->ci->db->where_in("id", $id);
        } else {
            $this->ci->db->where("id", $id);
        }
        $result = $this->ci->db->get()->result_array();
        if (!empty($result)) {
            $data = $result[0];
        }

        return $data;
    }

    /**
     * Return block object by guid
     *
     * @param string $gid block guid
     *
     * @return array
     */
    public function get_block_by_gid($gid)
    {
        $data = array();
        $result = $this->ci->db->select(implode(', ', $this->block_attrs))->from(DYNBLOCKS_BLOCKS_TABLE)->where("gid", $gid)->get()->result_array();
        if (!empty($result)) {
            $data = $result[0];
        }

        return $data;
    }

    /**
     * Save block object to data source
     *
     * @param integer $id    block identifier
     * @param array   $data  block data
     * @param array   $langs languages data
     *
     * @return integer
     */
    public function save_block($id, $data, $langs = null)
    {
        if (is_null($id)) {
            $data["date_add"] = $data["date_modified"] = date("Y-m-d H:i:s");
            $this->ci->db->insert(DYNBLOCKS_BLOCKS_TABLE, $data);
            $id = $this->ci->db->insert_id();
        } else {
            $data["date_add"] = date("Y-m-d H:i:s");
            $this->ci->db->where('id', $id);
            $this->ci->db->update(DYNBLOCKS_BLOCKS_TABLE, $data);
        }

        if (!empty($langs)) {
            $languages = $this->ci->pg_language->languages;
            if (!empty($languages)) {
                foreach ($languages as $language) {
                    $lang_ids[] = $language["id"];
                }
                foreach ($langs as $gid => $strings_data) {
                    foreach ($strings_data as $string => $lang_data) {
                        $this->ci->pg_language->pages->set_string_langs($gid, $string, $lang_data, $lang_ids);
                    }
                }
            }
        }

        return $id;
    }

    /**
     * Validate block object for saving to data source
     *
     * @param integer $id    block identifier
     * @param array   $data  block data
     * @param array   $langs languages data
     *
     * @return array
     */
    public function validate_block($id, $data, $langs = null)
    {
        $return = array("errors" => array(), "data" => array(), 'temp' => array(), 'langs' => array());

        if (isset($data["gid"])) {
            $return["data"]["gid"] = strtolower(preg_replace('/[\s\n\r_]{1,}/', '_', trim(preg_replace('/[^a-z_0-9]/i', '_', strip_tags($data["gid"])))));
            if (empty($return["data"]["gid"])) {
                $return["errors"][] = l('error_gid_empty', 'dynamic_blocks');
            }
        }

        if (isset($data["module"])) {
            $return["data"]["module"] = strtolower(preg_replace("/[\s\n\r]{1,}/", '_', $data["module"]));
        }
        if (isset($data["model"])) {
            $return["data"]["model"] = ucfirst(preg_replace("/[\s\n\r]{1,}/", '_', $data["model"]));
        }
        if (isset($data["method"])) {
            $return["data"]["method"] = preg_replace("/[\s\n\r]{1,}/", '_', $data["method"]);
        }

        if (empty($id) && (empty($return["data"]["module"]) || empty($return["data"]["model"]) || empty($return["data"]["method"]))) {
            $return["errors"][] = l('error_function_empty', 'dynamic_blocks');
        }

        if (!(empty($return["data"]["module"]) || empty($return["data"]["model"]) || empty($return["data"]["method"]))) {
            $result = $this->is_method_callable($return["data"]["module"], $return["data"]["model"], $return["data"]["method"]);
            if (!$result) {
                $return["errors"][] = l('error_function_invalid', 'dynamic_blocks');
            }
        }

        if (isset($data["min_width"]) && !empty($data["min_width"])) {
            $return["data"]["min_width"] = $data['min_width'] ?: '30';
        }
        if (!empty($data["params"])) {
            foreach ($data["params"] as $key => $param) {
                $gid = strtolower(preg_replace('/[\s\n\r_]{1,}/', '_', trim(preg_replace('/[^a-z_0-9]/i', '_', strip_tags($param["gid"])))));
                if (!empty($gid)) {
                    $return["data"]["params"][$gid]["type"] = $param["type"] ?: 'text';
                    $return["data"]["params"][$gid]["default"] = trim($param["default"]);
                    $return["temp"]["params"][$key] = $gid;
                }
            }
            $return["data"]["params"] = serialize($return["data"]["params"]);
        }

        if (isset($data["views"]) && !empty($data["views"])) {
            foreach ($data["views"] as $key => $view) {
                $gid = strtolower(preg_replace('/[\s\n\r_]{1,}/', '_', trim(preg_replace('/[^a-z_0-9]/i', '_', strip_tags($view["gid"])))));
                if (!empty($gid)) {
                    $return["data"]["views"][] = $gid;
                    $return["temp"]["views"][$key] = $gid;
                }
            }
            $return["data"]["views"] = serialize($return["data"]["views"]);
        }

        if (!empty($langs)) {
            $languages = $this->ci->pg_language->languages;
            $current_lang_id = $this->ci->pg_language->current_lang_id;
            $lang_gid = $return["data"]["module"] . "_dynamic_blocks";

            if (isset($langs["name"])) {
                $lang_i = "block_" . $return["data"]["gid"];
                foreach ($languages as $language) {
                    $str = (!empty($langs["name"][$language["id"]])) ? $langs["name"][$language["id"]] : $langs["name"][$current_lang_id];
                    $str = trim(strip_tags($str));

                    $return["langs"][$lang_gid][$lang_i][$language["id"]] = $str;
                }
            }

            if (isset($langs["params"])) {
                foreach ($langs["params"] as $key => $param) {
                    if (empty($return["temp"]["params"][$key])) {
                        continue;
                    }
                    $lang_i = 'param_' . $return["data"]["gid"] . '_' . $return["temp"]["params"][$key];
                    foreach ($languages as $language) {
                        $str = (!empty($langs["params"][$key][$language["id"]])) ? $langs["params"][$key][$language["id"]] : $langs["params"][$key][$current_lang_id];
                        $str = trim(strip_tags($str));

                        $return["langs"][$lang_gid][$lang_i][$language["id"]] = $str;
                    }
                }
            }

            if (isset($langs["views"])) {
                foreach ($langs["views"] as $key => $param) {
                    if (empty($return["temp"]["views"][$key])) {
                        continue;
                    }
                    $lang_i = 'view_' . $return["data"]["gid"] . '_' . $return["temp"]["views"][$key];
                    foreach ($languages as $language) {
                        $str = (!empty($langs["views"][$key][$language["id"]])) ? $langs["views"][$key][$language["id"]] : $langs["views"][$key][$current_lang_id];
                        $str = trim(strip_tags($str));

                        $return["langs"][$lang_gid][$lang_i][$language["id"]] = $str;
                    }
                }
            }
        }

        return $return;
    }

    /**
     * Format block data
     *
     * @param array   $data      block data
     * @param boolean $get_langs with languages data
     *
     * @return array
     */
    public function format_block($data, $get_langs = false)
    {
        $data["params"] = $data["params"] ? (array) unserialize($data["params"]) : array();
        $data["views"] = $data["views"] ? (array) unserialize($data["views"]) : array();

        $data["lang_gid"] = $data["module"] . "_dynamic_blocks";
        $data["name_i"] = "block_" . $data["gid"];
        $data["name"] = ($get_langs) ? (l($data["name_i"], $data["lang_gid"])) : "";

        if (!empty($data["params"])) {
            foreach ($data["params"] as $param_gid => $param_data) {
                $param_i = 'param_' . $data["gid"] . '_' . $param_gid;
                $data["params_data"][] = array(
                    "gid" => $param_gid,
                    "type" => isset($param_data["type"]) ? $param_data["type"] : '',
                    "default" => isset($param_data["default"]) ? $param_data["default"] : '',
                    "i" => $param_i,
                    "name" => ($get_langs) ? (l($param_i, $data["lang_gid"])) : "",
                );
            }
        }

        if (!empty($data["views"])) {
            $get_views_arr = function($view_gid, $data) use ($get_langs) {
                $param_i = 'view_' . $data["gid"] . '_' . $view_gid;
                return array(
                    "gid" => $view_gid,
                    "i" => $param_i,
                    "name" => ($get_langs) ? (l($param_i, $data["lang_gid"])) : "",
                );
            };
            foreach ($data["views"] as $view_gid) {
                if (is_array($view_gid)) {
                    foreach ($view_gid as $v_g) {
                        $data["views_data"][] = $get_views_arr($v_g, $data);
                    }
                } else {
                    $data["views_data"][] = $get_views_arr($view_gid, $data);
                }
            }
        }

        return $data;
    }

    /**
     * Remove block object by identifier
     *
     * @param integer $id block identifier
     *
     * @return void
     */
    public function delete_block($id)
    {
        $raw_data = $this->get_block_by_id($id);
        if (empty($raw_data)) {
            return false;
        }

        $block_data = $this->format_block($raw_data);

        $this->ci->db->where("id", $id);
        $this->ci->db->delete(DYNBLOCKS_BLOCKS_TABLE);

        $this->ci->db->where("id_block", $id);
        $this->ci->db->delete(DYNBLOCKS_AREA_BLOCK_TABLE);

        $this->ci->pg_language->pages->delete_string($block_data['lang_gid'], $block_data['name_i']);
        if (!empty($block_data["params_data"])) {
            foreach ($block_data["params_data"] as $param) {
                $this->ci->pg_language->pages->delete_string($block_data['lang_gid'], $param['i']);
            }
        }
        if (!empty($block_data["views_data"])) {
            foreach ($block_data["views_data"] as $view) {
                $this->ci->pg_language->pages->delete_string($block_data['lang_gid'], $view['i']);
            }
        }

        return;
    }

    /**
     * Remove block object by guid
     *
     * @param string $gid block guid
     *
     * @return void
     */
    public function delete_block_by_gid($gid)
    {
        $block_data = $this->get_block_by_gid($gid);
        return $this->delete_block($block_data["id"]);
    }

    /**
     * Return blocks objects as array
     *
     * @param integer $page              page of results
     * @param integer $items_on_page     results per page
     * @param array   $order_by          sorting data
     * @param array   $params            filters data
     * @param array   $filter_object_ids filters identifiers
     *
     * @return array
     */
    public function get_blocks_list($page = null, $items_on_page = null, $order_by = null, $params = array(), $filter_object_ids = null)
    {
        $this->ci->db->select(implode(', ', $this->block_attrs))->from(DYNBLOCKS_BLOCKS_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->block_attrs)) {
                    $this->ci->db->order_by($field . " " . $dir);
                }
            }
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($items_on_page, $items_on_page * ($page - 1));
        }

        $data = array();
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $result) {
                $data[] = $this->format_block($result);
            }
        }
        return $data;

    }

    /**
     * Return blocks objects as asscoiated array (<id> => <data>)
     *
     * @param integer $page              page of results
     * @param integer $items_on_page     results per page
     * @param array   $order_by          sorting data
     * @param array   $params            filters data
     * @param array   $filter_object_ids filters identifiers
     *
     * @return array
     */
    public function get_blocks_list_by_id($page = null, $items_on_page = null, $order_by = null, $params = array(), $filter_object_ids = null)
    {
        $return_blocks = array();
        $blocks = $this->get_blocks_list($page, $items_on_page, $order_by, $params, $filter_object_ids);
        if (!empty($blocks)) {
            foreach ($blocks as $block) {
                $return_blocks[$block["id"]] = $block;
            }
        }

        return $return_blocks;
    }

    /**
     * Return number of dynamic blocks
     *
     * @param array $params            filters data
     * @param array $filter_object_ids filters identifiers
     *
     * @return integer
     */
    public function get_blocks_count($params = array(), $filter_object_ids = null)
    {
        $this->ci->db->select("COUNT(*) AS cnt")->from(DYNBLOCKS_BLOCKS_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["cnt"]);
        }

        return 0;
    }

    /**
     * Check method is callable
     *
     * @param string $module module name
     * @param string $model  model name
     * @param string $method method name
     *
     * @return boolean
     */
    protected function is_method_callable($module, $model, $method)
    {
        $result = false;

        $model_url = $module . '/models/' . $model;
        $model_path = MODULEPATH . strtolower($model_url) . EXT;

        if (file_exists($model_path)) {
            $this->ci->load->model($model_url);
            $object = array($this->ci->$model, $method);
            $result = is_callable($object);
        }

        return $result;
    }

    /**
     * Return file name of content cache
     *
     * @param integer $id_area_block area block identifier
     * @param integer $lang_id       language idnetifier
     *
     * @return string
     */
    protected function cache_get_file_name($id_area_block, $lang_id = null)
    {
        if (!$lang_id) {
            $lang_id = $this->ci->pg_language->current_lang_id;
        }

        return $this->cache_dir . "block_content_" . $id_area_block . "_" . $lang_id . ".txt";
    }

    /**
     * Return content from cache
     *
     * @param string  $file_path  file path
     * @param integer $cache_time cache time
     *
     * @return string/false
     */
    protected function cache_get_content($file_path, $cache_time = 0)
    {
        if (!$cache_time || !file_exists($file_path)) {
            return false;
        }

        clearstatcache();
        $stat = stat($file_path);
        $mod_time = $stat["mtime"];

        if (time() < $mod_time + $cache_time) {
            return file_get_contents($file_path);
        }

        return false;
    }

    /**
     * Write content to cache
     *
     * @param string $file_path file path
     * @param string $content   block content
     *
     * @return void
     */
    protected function cache_set_content($file_path, $content)
    {
        $h = fopen($file_path, 'w');
        if ($h) {
            fwrite($h, $content);
            fclose($h);
        }

        return;
    }

    /**
     * Clear overall cache
     *
     * @return void
     */
    public function cache_clear()
    {
        $languages = $this->ci->pg_language->languages;
        $blocks = $this->get_blocks_list();
        foreach ($blocks as $block) {
            foreach ($languages as $lang) {
                $filename = $this->cache_get_file_name($block['id'], $lang['id']);
                if (file_exists($filename)) {
                    unlink($filename);
                }
            }
        }
    }

    /**
     * Clear block cache
     * @param int $block_id
     */
    public function cacheBlockClear($block_id)
    {
        foreach ($this->ci->pg_language->languages as $lang) {
            $filename = $this->cache_get_file_name($block_id, $lang['id']);
            if (file_exists($filename)) {
                unlink($filename);
            }
        }
    }

    /**
     * Clear area cache
     *
     * @param integer $area_id area identifier
     *
     * @return void
     */
    public function cache_area_clear($area_id)
    {
        $languages = $this->ci->pg_language->languages;
        $blocks = $this->get_area_blocks($area_id);
        foreach ($blocks as $block) {
            foreach ($languages as $lang) {
                $filename = $this->cache_get_file_name($block['id'], $lang['id']);
                if (file_exists($filename)) {
                    unlink($filename);
                }
            }
        }
    }

    /**
     * Clear all cache
     *
     * @return void
     */
    public function cache_all_clear()
    {
        $cache = scandir($this->cache_dir);
        foreach ($cache as $file) {
            if ($file == '.' || $file == '..' || $file == 'index.html') {
                continue;
            }
            unlink($this->cache_dir . $file);
        }
    }

    /**
     * Return content of dynamic blocks area by identifier
     *
     * @param integer $id_area area identifier
     *
     * @return string
     */
    public function html_area_blocks($id_area)
    {
        $template_preview_mode = $this->ci->input->get('template_preview_mode');

        //$area = $this->get_area_by_id($id_area);

        $area_blocks = $this->get_area_blocks($id_area);
        if (empty($area_blocks)) {
            return "";
        }

        foreach ($area_blocks as $block) {
            $block_ids[] = $block["id_block"];
        }

        $blocks = $this->get_blocks_list_by_id(null, null, null, array(), $block_ids);

        $layout = array();
        $row = array();
        $row_width = 0;

        foreach ($area_blocks as $block) {
            $row_width += $block["width"];
            $row[] = $block;
            if ($row_width < 100) {
                continue;
            }

            if ($row_width > 100) {
                array_pop($row);
                $count = count($row);
                $row[$count - 1]["width"] += 100 - ($row_width - $block["width"]);
                $layout[] = $row;
                $row = array($block);
                $row_width = $block["width"];
            } else {
                $layout[] = $row;
                $row = array();
                $row_width = 0;
            }
        }

        if ($row_width) {
            $count = count($row);
            $row[$count - 1]["width"] += 100 - $row_width;
            $layout[] = $row;
        }

        $dynamic_blocks = array();

        foreach ($layout as $row => $layout_blocks) {
            $dynamic_blocks_row = array();

            foreach ($layout_blocks as $area_block) {
                $temp_content = false;
                if ($area_block["cache_time"] && !$template_preview_mode) {
                    $file_name = $this->cache_get_file_name($area_block["id"]);
                    $temp_content = $this->cache_get_content($file_name, $area_block["cache_time"]);
                }

                if (!$temp_content) {
                    $module = $blocks[$area_block["id_block"]]['module'];
                    $model = $blocks[$area_block["id_block"]]['model'];
                    $method = $blocks[$area_block["id_block"]]['method'];
                    $model_url = $module . "/models/" . $model;
                    $this->ci->load->model($model_url);
                    $temp_content = $this->ci->$model->$method($area_block["params"], $area_block["view_str"], $area_block['width']);

                    if ($area_block["cache_time"] && !$template_preview_mode) {
                        $this->cache_set_content($file_name, $temp_content);
                    }
                }

                if ($area_block["width"] == 100 && strlen(strip_tags(trim($temp_content))) == 0) {
                    continue;
                }

                $width = ($area_block["width"] != 40 ? $area_block["width"] : 30);

                $dynamic_blocks_row[] = array('content' => $temp_content, 'width' => $width);
            }

            if (empty($dynamic_blocks_row)) {
                continue;
            }

            $dynamic_blocks[] = $dynamic_blocks_row;
        }

        $this->ci->view->assign('dynamic_blocks', $dynamic_blocks);

        return $this->ci->view->fetch('html_area_blocks', 'user', 'dynamic_blocks');
    }

    /**
     * Return content of dynamic blocks area by guid
     *
     * @param string $gid_area area guid
     *
     * @return string
     */
    public function html_area_blocks_by_gid($gid_area)
    {
        $area = $this->get_area_by_gid($gid_area);
        return $this->html_area_blocks($area["id"]);
    }

    /**
     * Import languages data to dynamic blocks
     *
     * @param array $data       languages data
     * @param array $langs_file languages data
     * @param array $langs_ids  languages identifiers
     *
     * @return void
     */
    public function update_langs(array $data, array $langs_file, array $langs_ids = array())
    {
        if(empty($data)) {
            return false;
        }
        $module = $data[0]['module'] . '_dynamic_blocks';
        foreach ($data as $block) {
            $this->ci->pg_language->pages->set_string_langs(
                    $module, 'block_' . $block['gid'], $langs_file['block_' . $block['gid']], (array) $langs_ids
            );
            if (!empty($block['views'])) {
                $views = unserialize($block['views']);
                if (is_array($views)) {
                    foreach ($views as $value) {
                        if (isset($langs_file['view_' . $block['gid'] . '_' . $value])) {
                            $this->ci->pg_language->pages->set_string_langs(
                                    $module, 'view_' . $block['gid'] . '_' . $value, $langs_file['view_' . $block['gid'] . '_' . $value], (array) $langs_ids
                            );
                        }
                    }
                }
            }

            if (!empty($block['params'])) {
                $params = unserialize($block['params']);
                if (is_array($params)) {
                    foreach ($params as $key => $value) {
                        $this->ci->pg_language->pages->set_string_langs(
                                $module, 'param_' . $block['gid'] . '_' . $key, $langs_file['param_' . $block['gid'] . '_' . $key], (array) $langs_ids
                        );
                        if($value['type'] == 'select'){
                            $this->ci->pg_language->pages->set_string_langs(
                                $module, 'param_' . $block['gid'] . '_' . $key . '_default', $langs_file['param_' . $block['gid'] . '_' . $key . '_default'], (array) $langs_ids
                            );
                        }

                    }
                }
            }
        }
    }

    /**
     * Export languages from dynamic blocks
     *
     * @param array $data      languages data
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function export_langs($data, $langs_ids = null)
    {
        $gids = array();
        $module = $data[0]['module'] . '_dynamic_blocks';
        foreach ($data as $block) {
            $gids[] = 'block_' . $block['gid'];
            if (!empty($block['views'])) {
                $views = unserialize($block['views']);
                if (is_array($views)) {
                    foreach ($views as $value) {
                        $gids[] = 'view_' . $block['gid'] . '_' . $value;
                    }
                }
            }
            if (!empty($block['params'])) {
                $params = unserialize($block['params']);
                if (is_array($params)) {
                    foreach ($params as $key => $value) {
                        $gids[] = 'param_' . $block['gid'] . '_' . $key;
                    }
                }
            }
        }

        return $this->ci->pg_language->export_langs($module, $gids, $langs_ids);
    }

    /**
     * Import view languages data to dynamic blocks
     *
     * @param string $module    module guid
     * @param string $block_gid block guid
     * @param string $value    view value
     * @param array $langs_file languages data
     * @param array $langs_ids  languages identifiers
     *
     * @return void
     */
    public function update_view_langs($module, $block_gid, $value, $langs_file, $langs_ids)
    {
        $module = $module . '_dynamic_blocks';
        $this->ci->pg_language->pages->set_string_langs(
            $module, 'view_' . $block_gid . '_' . $value, $langs_file['view_' . $block_gid . '_' . $value], (array) $langs_ids
        );
    }

    /**
     * Export view languages from dynamic blocks
     *
     * @param string $module    module guid
     * @param string $block_gid block guid
     * @param string $value     value
     * @param array $langs_ids  languages identifiers
     *
     * @return array
     */
    public function export_view_langs($module, $block_gid, $value, $langs_ids = null)
    {
        $module = $module . '_dynamic_blocks';
        return $this->ci->pg_language->export_langs($module, array('view_' . $block_gid . '_' . $value), $langs_ids);
    }

    /**
     * Remove view languages from dynamic blocks
     *
     * @param string $module    module guid
     * @param string $block_gid block guid
     * @param string $value     value
     *
     * @return array
     */
    public function delete_view_langs($module, $block_gid, $value)
    {
        $module = $module . '_dynamic_blocks';
        $this->ci->pg_language->pages->delete_string($module, 'view_' . $block_gid . '_' . $value);
    }

    /**
     * Return content of html dynamic block
     *
     * Dynamic block callback method
     *
     * @param array   $params block parametes
     * @param string  $view   block view
     * @param integer $width  block size
     *
     * @return string
     */
    public function _dynamic_block_get_html_code($params, $view = 'default', $width = 100)
    {
        $lang_id = $this->ci->pg_language->current_lang_id;
        if (!isset($params['html'])) {
            $params['html'] = '';
        }
        if (!isset($params['title_' . $lang_id])) {
            $params['title_' . $lang_id] = '';
        }
        $this->ci->view->assign('block_title', $params['title_' . $lang_id]);
        $this->ci->view->assign('block_html', $params['html']);
        $this->ci->view->assign('block_view', $view);
        $this->ci->view->assign('block_width', $width);

        return $this->ci->view->fetch('dynamic_block_html', 'user', 'dynamic_blocks');
    }

    /**
     * Return dynamic blocks presets as array
     *
     * @param integer $page              page of results
     * @param integer $items_on_page     results per page
     * @param array   $order_by          sorting data
     * @param array   $params            sql criteria
     * @param array   $filter_object_ids filter idnetifiers
     *
     * @return array
     */
    protected function _get_presets_list($page = null, $items_on_page = null, $order_by = null, $params = array(), $filter_object_ids = null)
    {
        $this->ci->db->select(implode(',', $this->fields_preset))->from(DYNBLOCKS_PRESETS_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->fields_preset)) {
                    $this->ci->db->order_by($field . " " . $dir);
                }
            }
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($items_on_page, $items_on_page * ($page - 1));
        }

        $data = array();
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $data = $this->format_preset($results);
        }

        return $data;
    }

    /**
     * Return number of dynamic blocks presets
     *
     * @param integer $page              page of results
     * @param integer $items_on_page     results per page
     * @param array   $order_by          sorting data
     * @param array   $params            sql criteria
     * @param array   $filter_object_ids filter idnetifiers
     *
     * @return integer
     */
    private function _get_presets_count($page = null, $items_on_page = null, $order_by = null, $params = array(), $filter_object_ids = null)
    {
        $this->ci->db->select('COUNT(*) as cnt')->from(DYNBLOCKS_PRESETS_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["cnt"]);
        }

        return 0;
    }

    /**
     * Return area presets as array
     *
     * @param string  $area_gid      area guid
     * @param integer $page          page of results
     * @param integer $items_on_page results per page
     * @param array   $order_by      sorting data
     *
     * @return array
     */
    public function get_area_presets_list($area_gid, $page = null, $items_on_page = null, $order_by = null)
    {
        $params = array('where' => array('gid_area' => $area_gid));

        return $this->_get_presets_list($page, $items_on_page, $order_by, $params);
    }

    /**
     * Return number of area presets
     *
     * @param string $area_gid area guid
     *
     * @return integer
     */
    public function get_area_presets_count($area_gid)
    {
        $params = array('where' => array('gid_area' => $area_gid));

        return $this->_get_presets_count($params);
    }

    /**
     * Return dynamic blocks presets as array
     *
     * @param integer $page              page of results
     * @param integer $items_on_page     results per page
     * @param array   $order_by          sorting data
     * @param array   $params            sql criteria
     * @param array   $filter_object_ids filter idnetifiers
     *
     * @return array
     */
    protected function _get_presets_blocks_list($page = null, $items_on_page = null, $order_by = null, $params = array(), $filter_object_ids = null)
    {
        $this->ci->db->select(implode(',', $this->fields_preset_block))->from(DYNBLOCKS_PRESETS_BLOCK_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->fields_preset_block)) {
                    $this->ci->db->order_by($field . " " . $dir);
                }
            }
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($items_on_page, $items_on_page * ($page - 1));
        }

        $data = array();
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $data = $this->format_preset_block($results);
        }

        return $data;
    }

    /**
     * Set area preset
     *
     * @param integer $area_id   area idnetifier
     * @param integer $preset_id preset identifier
     *
     * @return void
     */
    public function set_area_preset($area_id, $preset_id)
    {
        $this->cache_area_clear($area_id);

        $this->ci->db->where('id_area', $area_id);
        $this->ci->db->delete(DYNBLOCKS_AREA_BLOCK_TABLE);

        $params = array('where' => array('id_preset' => $preset_id));
        $blocks = $this->_get_presets_blocks_list(null, null, null, $params);
        foreach ($blocks as $block) {
            unset($block['id']);
            $validate_data = $this->validate_area_block(null, $block);
            $this->save_area_block(null, $validate_data['data']);
        }
        $this->refresh_area_block_sorter($area_id);
    }

    /**
     * Return preset object by identifier
     *
     * @param integer $id preset identifier
     *
     * @return array
     */
    public function get_preset_by_id($id)
    {
        $data = array();
        $this->ci->db->select(implode(",", $this->fields_preset));
        $this->ci->db->from(DYNBLOCKS_PRESETS_TABLE);
        $this->ci->db->where("id", $id);
        $result = $this->ci->db->get()->result_array();
        if (!empty($result)) {
            $preset = $this->format_preset(array($result[0]));
            $data = array_shift($preset);
        }

        return $data;
    }

    /**
     * Return preset object by guid
     *
     * @param string $gid area guid
     *
     * @return array
     */
    public function get_preset_by_gid($gid)
    {
        $data = array();
        $this->ci->db->select(implode(",", $this->fields_preset));
        $this->ci->db->from(DYNBLOCKS_PRESETS_TABLE);
        $this->ci->db->where("gid", $gid);
        $result = $this->ci->db->get()->result_array();
        if (!empty($result)) {
            $preset = $this->format_preset(array($result[0]));
            $data = array_shift($preset);
        }

        return $data;
    }

    /**
     * Save preset object to data source
     *
     * @param integer $preset_id preset identifier
     * @param array   $data      preset data
     *
     * @return integer
     */
    public function save_preset($preset_id, $data)
    {
        if (empty($preset_id)) {
            $data['date_add'] = date('Y-m-d H:i:s');
            $this->ci->db->insert(DYNBLOCKS_PRESETS_TABLE, $data);
            $preset_id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where('id', $preset_id);
            $this->ci->db->update(DYNBLOCKS_PRESETS_TABLE, $data);
        }

        return $preset_id;
    }

    /**
     * Remove preset block object
     *
     * @param integer $preset_id preset identifier
     *
     * @return void
     */
    public function remove_preset($preset_id)
    {
        $preset_data = $this->get_preset_by_id($preset_id);

        $this->ci->db->where('id', $preset_id);
        $this->ci->db->delete(DYNBLOCKS_PRESETS_TABLE);

        $this->ci->db->where('id_preset', $preset_id);
        $this->ci->db->delete(DYNBLOCKS_PRESETS_BLOCK_TABLE);

        if (!$preset_data['logo']) {
            return;
        }

        $this->ci->load->model("Uploads_model");
        $this->ci->Uploads_model->delete_upload($this->upload_config_id, $preset_data["prefix"], $preset_data["logo"]);
    }

    /**
     * Upload preset logo
     *
     * @param integer $preset_id   preset identifier
     * @param string  $upload_name upload name
     *
     * @return array
     */
    public function save_preset_logo($preset_id, $upload_name)
    {
        $return = array('errors' => array(), 'data' => array());

        if (empty($upload_name) || empty($_FILES[$upload_name]) || !is_array($_FILES[$upload_name]) ||
                !is_uploaded_file($_FILES[$upload_name]["tmp_name"])) {
            return $return;
        }

        $preset_data = $this->get_preset_by_id($preset_id);

        $this->ci->load->model("Uploads_model");
        $return = $this->ci->Uploads_model->upload($this->preset_upload_type, $preset_data["prefix"], $upload_name);

        if (!empty($return["errors"])) {
            return $return;
        }

        $this->save_preset($preset_id, array("logo" => $return["file"]));

        return $return;
    }

    /**
     * Remove preset logo
     *
     * @param integer $preset_id preset identifier
     *
     * @return void
     */
    public function delete_preset_logo($preset_id)
    {
        $preset_data = $this->get_preset_by_id($preset_id);
        if (!$preset_data['logo']) {
            return;
        }
        $this->ci->load->model("Uploads_model");
        $this->ci->Uploads_model->delete_upload($this->upload_config_id, $preset_data["prefix"], $preset_data["logo"]);
    }

    /**
     * Format preset object
     *
     * @param array $data preset data
     *
     * @return array
     */
    public function format_preset($data)
    {
        if (empty($data)) {
            return $data;
        }

        $is_upload_installed = $this->ci->pg_module->is_module_active('uploads');
        if ($is_upload_installed) {
            $this->ci->load->model('Uploads_model');
        }

        foreach ($data as $key => $preset) {
            $preset['prefix'] = $preset['id'];

            if ($is_upload_installed) {
                $preset["media"]["logo"] = $this->ci->Uploads_model->format_upload($this->preset_upload_type, $preset["prefix"], $preset["logo"]);
            }

            $data[$key] = $preset;
        }

        return $data;
    }

    /**
     * Validate preset object for saving to data source
     *
     * @param integer $preset_id preset idnetifier
     * @param array   $data      preset data
     *
     * @return array
     */
    public function validate_preset($preset_id, $data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data['id'])) {
            $return['data']['id'] = intval($data['id']);
        }

        if (isset($data['gid'])) {
            $return['data']['gid'] = trim(strip_tags($data['gid']));
        }

        if (isset($data['gid_area'])) {
            $return['data']['gid_area'] = trim(strip_tags($data['gid_area']));
        }

        if (isset($data['name'])) {
            $return['data']['name'] = trim(strip_tags($data['name']));
        }

        if (isset($data['logo'])) {
            $return['data']['logo'] = trim(strip_tags($data['logo']));
        }

        if (isset($data['comment'])) {
            $return['data']['comment'] = trim(strip_tags($data['comment']));
        }

        if (isset($data['editable'])) {
            $return['data']['editable'] = trim(strip_tags($data['editable']));
        }

        return $return;
    }

    /**
     * Save preset block object to data source
     *
     * @param integer $id   preset block identifier
     * @param array   $data preset block data
     *
     * @return integer
     */
    public function save_preset_block($id, $data)
    {
        if (empty($id)) {
            if (!isset($data["width"]) || !$data["width"]) {
                $data["width"] = 100;
            }
            $this->ci->db->insert(DYNBLOCKS_PRESETS_BLOCK_TABLE, $data);
            $id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where('id', $id);
            $this->ci->db->update(DYNBLOCKS_PRESETS_BLOCK_TABLE, $data);
        }

        return $id;
    }

    /**
     * Format area block data
     *
     * @param array $data area block data
     *
     * @return array
     */
    public function format_preset_block($data)
    {
        if (empty($data)) {
            return;
        }

        $area_gids = array();

        foreach ($data as $i => $block) {
            $data[$i]["params"] = $block["params"] ? unserialize($block["params"]) : array();
            $data[$i]["width"] = $block["width"] ? $block["width"] : 100;
            if (!in_array($block['gid_area'], $area_gids)) {
                $area_gids[] = $block['gid_area'];
            }
        }

        $areas_by_gid = array();
        $areas = $this->get_areas_list(null, null, null, array('where_in' => array('gid' => $area_gids)));
        foreach ($areas as $i => $area) {
            $areas_by_gid[$area['gid']] = $area['id'];
        }

        foreach ($data as $i => $block) {
            $data[$i]['id_area'] = $areas_by_gid[$block['gid_area']];
        }

        return $data;
    }

    /**
     * Validate preset block object for saving to data source
     *
     * @param integer $block_id preset block idnetifier
     * @param array   $data     preset block data
     *
     * @return array
     */
    public function validate_preset_block($block_id, $data)
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data['id'])) {
            $return['data']['id'] = intval($data['id']);
        }

        if (isset($data['id_preset'])) {
            $return['data']['id_preset'] = intval($data['id_preset']);
        }

        if (isset($data['gid_area'])) {
            $return['data']['gid_area'] = trim(strip_tags($data['gid_area']));
        }

        if (isset($data['id_block'])) {
            $return['data']['id_block'] = intval($data['id_block']);
        }

        if (isset($data['params'])) {
            $return['data']['params'] = serialize($data['params']);
        }

        if (isset($data['view_str'])) {
            $return['data']['view_str'] = trim(strip_tags($data['view_str']));
        }

        if (isset($data['width'])) {
            $return['data']['width'] = intval($data['width']);
        }

        if (isset($data['sorter'])) {
            $return['data']['sorter'] = intval($data['sorter']);
        }

        if (isset($data['cache_time'])) {
            $return['data']['cache_time'] = intval($data['cache_time']);
        }

        return $return;
    }

    /**
     * Install wish list fields depended on language
     *
     * @param integer $lang_id language identifier
     *
     * @return void
     */
    public function lang_dedicate_module_callback_add($lang_id = false)
    {
        if (!$lang_id) {
            return;
        }

        $default_lang_id = $this->ci->pg_language->get_default_lang_id();

        $this->ci->db->select(implode(",", $this->fields_area_block))
                ->from(DYNBLOCKS_AREA_BLOCK_TABLE);

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $blocks_data = $this->format_area_block($results);
            foreach ($blocks_data as $block_data) {
                foreach ($block_data['params'] as $param_name => $param_value) {
                    if (!preg_match('/_' . $default_lang_id . '$/', $param_name)) {
                        continue;
                    }
                    $block_data['params'][preg_replace('/_' . $default_lang_id . '$/', '_' . $lang_id, $param_name)] = $param_value;
                    $save_data = array('params' => $block_data['params']);
                    $validate_data = $this->ci->Dynamic_blocks_model->validate_area_block($block_data['id'], $save_data);
                    if (empty($validate_data["errors"])) {
                        $this->ci->Dynamic_blocks_model->save_area_block($block_data['id'], $validate_data["data"]);
                    }
                }
            }
        }

        $blocks_data = $this->_get_presets_blocks_list();
        $blocks_data = $this->format_preset_block($blocks_data);
        foreach ($blocks_data as $block_data) {
            foreach ($block_data['params'] as $param_name => $param_value) {
                if (!preg_match('/_' . $default_lang_id . '$/', $param_name)) {
                    continue;
                }
                $block_data['params'][preg_replace('/_' . $default_lang_id . '$/', '_' . $lang_id, $param_name)] = $param_value;
                $save_data = array('params' => $block_data['params']);
                $validate_data = $this->ci->Dynamic_blocks_model->validate_preset_block($block_data['id'], $save_data);
                if (empty($validate_data["errors"])) {
                    $this->ci->Dynamic_blocks_model->save_preset_block($block_data['id'], $validate_data["data"]);
                }
            }
        }
    }

    /**
     * Uninstall wish list fields depended on language
     *
     * @param integer $lang_id language identifier
     *
     * @return void
     */
    public function lang_dedicate_module_callback_delete($lang_id = false)
    {
        if (!$lang_id) {
            return;
        }

        $this->ci->db->select(implode(",", $this->fields_area_block))
                ->from(DYNBLOCKS_AREA_BLOCK_TABLE);

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $blocks_data = $this->format_area_block($results);
            foreach ($blocks_data as $block_data) {
                foreach ($block_data['params'] as $param_name => $param_value) {
                    if (!preg_match('/_' . $lang_id . '$/', $param_name)) {
                        continue;
                    }
                    unset($block_data['params'][$param_name]);
                    $save_data = array('params' => $block_data['params']);
                    $validate_data = $this->ci->Dynamic_blocks_model->validate_area_block($block_data['id'], $save_data);
                    if (empty($validate_data["errors"])) {
                        $this->ci->Dynamic_blocks_model->save_area_block($block_data['id'], $validate_data["data"]);
                    }
                }
            }
        }

        $blocks_data = $this->_get_presets_blocks_list();
        $blocks_data = $this->format_preset_block($blocks_data);
        foreach ($blocks_data as $block_data) {
            foreach ($block_data['params'] as $param_name => $param_value) {
                if (!preg_match('/_' . $default_lang_id . '$/', $param_name)) {
                    continue;
                }
                unset($block_data['params'][$param_name]);
                $save_data = array('params' => $block_data['params']);
                $validate_data = $this->ci->Dynamic_blocks_model->validate_preset_block($block_data['id'], $save_data);
                if (empty($validate_data["errors"])) {
                    $this->ci->Dynamic_blocks_model->save_preset_block($block_data['id'], $validate_data["data"]);
                }
            }
        }
    }

    /**
     * Install dynamic blocks
     * All blocks should belong to the same module
     *
     * @param array $module_blocks
     */
    public function installBatch(array $module_blocks)
    {
        $area_ids = array();
        foreach ((array) $module_blocks as $block_data) {
            $validate_data = $this->validate_block(null, $block_data);
            if (!empty($validate_data['errors'])) {
                continue;
            }
            $id_block = $this->save_block(null, $validate_data['data']);
            if (!empty($block_data['area'])) {
                foreach ($block_data['area'] as $block_area) {
                    if (!isset($area_ids[$block_area['gid']])) {
                        $area = $this->get_area_by_gid($block_area['gid']);
                        $area_ids[$block_area['gid']] = $area['id'];
                    }
                    $block_area['id_area'] = $area_ids[$block_area['gid']];
                    $block_area['id_block'] = $id_block;
                    $validate_data = $this->validate_area_block($block_area);

                    if (empty($validate_data['errors'])) {
                        $this->save_area_block(null, $validate_data['data']);
                    }
                }
            }
            if (!empty($block_data['presets'])) {
                foreach ($block_data['presets'] as $preset_data) {
                    $preset = $this->get_preset_by_gid($preset_data['gid_preset']);
                    if (!empty($preset)) {
                        $preset_data['id_preset'] = $preset['id'];
                        $preset_data['id_block'] = $id_block;
                        unset($preset_data['gid_preset']);
                        $this->save_preset_block(null, $preset_data);
                    }
                }
            }
        }
    }

    /**
     * Check that dynamic blocks are installed
     *
     * @param array $blocks
     *
     * @return array
     */
    private function checkBlocksInstalled(array $blocks)
    {
        $data = array();
        foreach ($blocks as $block) {
            $block = $this->get_block_by_gid($block['gid']);
            if ($block) {
                $data[] = $block;
            }
        }

        return $data;
    }

    /**
     * Update langs
     *
     * @param array      $module_blocks
     * @param array|null $langs_ids
     *
     * @return boolean
     */
    public function updateLangsByModuleBlocks(array $module_blocks, $langs_ids = null)
    {
        $installed_blocks = $this->checkBlocksInstalled($module_blocks);
        if (empty($installed_blocks)) {
            return false;
        }
        $module = $installed_blocks[array_shift(array_keys($installed_blocks))]['module'];
        $this->ci->load->model('Install_model');
        $langs_file = $this->ci->Install_model->language_file_read($module, 'dynamic_blocks', $langs_ids);
        if (!$langs_file) {
            log_message('info', 'Empty dynamic_blocks langs data (' . $module . ')');

            return false;
        }

        $this->updateLangs($installed_blocks, $langs_file, $langs_ids);

        return true;
    }
}
