<?php

namespace Pg\Modules\Listings\Models;

/**
 * Listings module
 *
 * @package 	PG_RealEstate
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
define('LISTINGS_STATS_TABLE', DB_PREFIX . 'listings_stats');

/**
 * Listings Statistics Model
 * 
 * @package 	PG_RealEstate
 * @subpackage 	Listings
 * @category	models
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Listings_stat_model extends \Model
{

    /**
     * Link to CodeIgniter object
     * 
     * @var object
     */
    protected $ci;

    /**
     * Properties of statistic object in data source
     * 
     * @var array
     */
    protected $_fields = array(
        'id',
        'id_user',
        'gid',
        'sale_cnt',
        'buy_cnt',
        'rent_cnt',
        'lease_cnt',
        'sold_cnt',
        'date_modified',
    );

    /**
     * Constructor
     *
     * return Listings_stat_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = & get_instance();
    }

    /**
     * Return statistic object by ID or GUID
     * 
     * @param string $stat_id statistic identifier/guid
     * @return array
     */
    public function get_stat_by_id($stat_id)
    {
        $field = 'id';
        if (!intval($stat_id)) {
            $field = 'gid';
            $stat_id = preg_replace('/[^a-z0-9_]/', '', strtolower($stat_id));
        }
        if (!$stat_id) {
            return false;
        }

        $this->ci->db->select(implode(', ', $this->_fields));
        $this->ci->db->from(LISTINGS_STATS_TABLE);
        $this->ci->db->where($field, $stat_id);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $data = $this->format_stat(array($results[0]));
            return $data[0];
        }
        return array();
    }

    /**
     * Return statistics objects by GUIDs
     * 
     * @param array $stat_gids GUIDs of statistics objects
     * @return array
     */
    public function get_stat_by_gids($stat_gids)
    {
        if (empty($stat_gids)) {
            return array();
        }
        $this->ci->db->select(implode(', ', $this->_fields));
        $this->ci->db->from(LISTINGS_STATS_TABLE);
        $this->ci->db->where_in('gid', $stat_gids);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $return = array();
            foreach ($results as $result) {
                $return[$result['gid']] = $result;
            }
            return $this->format_stat($return);
        }
        return array();
    }

    /**
     * Save statistic object to data source
     * 
     * @param integer $stat_id statistic identifier
     * @param array $data statistic data
     * @return integer
     */
    public function save_stat($stat_id, $data)
    {
        if (!$stat_id) {
            if (!isset($data['gid']) || !$data['gid']) {
                return false;
            }

            $data['gid'] = preg_replace('/[^a-z0-9_]/', '', strtolower($data['gid']));

            $type = $this->get_stat_by_id($data['gid']);
            if ($type) {
                return $type['id'];
            }

            $this->ci->db->insert(LISTINGS_STATS_TABLE, $data);
            $stat_id = $this->ci->db->insert_id();
        } else {
            $fields = array_flip($this->_fields);
            foreach ($data as $key => $value) {
                if (!isset($fields[$key])) {
                    unset($data[$key]);
                }
            }

            if (empty($data)) {
                return false;
            }

            $this->ci->db->where('id', $stat_id);
            $this->ci->db->update(LISTINGS_STATS_TABLE, $data);
        }
        return $stat_id;
    }

    /**
     * Remove statistic object from data source by ID or GUID
     * 
     * @param mixed $stat_id statistics identifier integer ID / string GUID
     * @return void
     */
    public function delete_stat($stat_id)
    {
        $stat = $this->get_type_by_id($stat_id);
        $this->ci->db->where('id', $stat['id']);
        $this->ci->db->delete(LISTINGS_STATS_TABLE);
    }

    /**
     * Validate statistic object for saving to data source
     * 
     * @param integer $stat_id statistic identifier
     * @param array $data statistic data
     * @return array
     */
    public function validate_stat($stat_id, $data)
    {
        $return = array('errors' => array(), 'data' => array());

        if (isset($data['id'])) {
            $return['data']['id'] = intval($data['id']);
            if (empty($return['data']['id'])) {
                unset($return['data']['id']);
            }
        }

        if (isset($data['id_user'])) {
            $return['data']['id_user'] = intval($data['id_user']);
        }

        if (isset($data['gid'])) {
            $return['data']['gid'] = trim(strip_tags($data['gid']));
            if (empty($return['data']['gid'])) {
                $return['errors'][] = l('error_empty_stat_gid', 'listings');
            }
        } elseif (!$stat_id) {
            $return['errors'][] = l('error_empty_stat_gid', 'listings');
        }

        if (isset($data['sale_cnt'])) {
            $return['data']['sale_cnt'] = intval($data['sale_cnt']);
        }

        if (isset($data['buy_cnt'])) {
            $return['data']['buy_cnt'] = intval($data['buy_cnt']);
        }

        if (isset($data['rent_cnt'])) {
            $return['data']['rent_cnt'] = intval($data['rent_cnt']);
        }

        if (isset($data['lease_cnt'])) {
            $return['data']['lease_cnt'] = intval($data['lease_cnt']);
        }

        if (isset($data['sold_cnt'])) {
            $return['data']['sold_cnt'] = intval($data['sold_cnt']);
        }

        if (isset($data['date_modified'])) {
            $value = strtotime($data['date_modified']);
            if ($value > 0) {
                $return['data']['date_modified'] = date('Y-m-d', $value);
            } else {
                $return['data']['date_modified'] = '0000-00-00 00:00:00';
            }
        }

        return $return;
    }

    /**
     * Format statistic data
     * 
     * @param array $data statistic data
     * @return array
     */
    public function format_stat($data)
    {
        return $data;
    }

    /**
     * Update statistics on saving listing
     * 
     * @param array $listing_old old listing data
     * @param array $listing_new new listing data
     * @return void
     */
    public function onsave_listing($listing_old, $listing_new)
    {
        $this->update_module_statistics($listing_old, $listing_new);
    }

    /**
     * Update statistics on removing listing
     * 
     * @param array $listing listing data
     * @return void
     */
    public function ondelete_listing($listing)
    {
        $this->update_module_statistics($listing);
    }

    /**
     * Update listing statistics
     * 
     * @param array $listing_old old listing data
     * @param array $listing_new new listing data
     * @return void
     */
    public function update_module_statistics($listing_old, $listing_new = null)
    {
        if (empty($listing_old['status']) && empty($listing_new['status'])) {
            return;
        }

        if (empty($listing_new)) {
            $listing_new['status'] = 0;
        } elseif (!isset($listing_new['status'])) {
            $listing_new['status'] = $listing_old['status'];
        }

        if (!empty($listing_old['sold'])) {
            $listing_old['status'] = 0;
        }

        if (isset($listing_new['sold']) && $listing_new['sold'] || !isset($listing_new['sold']) && $listing_old['sold']) {
            $listing_new['status'] = 0;
        }

        $this->ci->load->model('Listings_model');

        // listings by categories & locations
        if (isset($listing_new['status']) && !empty($listing_old['status']) && !$listing_new['status']) {
            $type_old = $this->ci->Listings_model->getOperationTypeById($listing_old['id_type']);

            $gids = [
                'property_type_' . $listing_old['id_category'],
                'property_type_' . $listing_old['id_category'] . '_' . $listing_old['property_type'],
                'country_' . $listing_old['id_country'],
                'region_' . $listing_old['id_region'],
                'city_' . $listing_old['id_city'],
            ];
            $type_name = $type_old . '_cnt';
            foreach($gids as $stat_gid) {
                $stat = $this->get_stat_by_id($stat_gid);
                if ($stat) {
                    $save_data = array($type_name => max($stat[$type_name] - 1, 0));
                    $validate_data = $this->validate_stat($stat['id'], $save_data);
                    $this->save_stat($stat['id'], $validate_data['data']);
                }
            }

            $is_users_installed = $this->ci->pg_module->is_module_active("users");
            if ($is_users_installed) {
                $this->ci->load->model('Users_model');
                $user = $this->ci->Users_model->get_user_by_id($listing_old['id_user']);
                if ($user) {
                    $this->ci->Users_model->update_listings_stat($user['id'], $type_old, false);
                }
            }
        } elseif (isset($listing_new['status']) && empty($listing_old['status']) && $listing_new['status']) {
            $type_id = isset($listing_new['id_type']) ? $listing_new['id_type'] : $listing_old['id_type'];
            $type_new = $this->ci->Listings_model->getOperationTypeById($type_id);

            if (isset($listing_new['id_category']) || isset($listing_old['id_category'])) {
                $type_name = $type_new . '_cnt';
                $stat_gid = 'property_type_' . (isset($listing_new['id_category']) ? $listing_new['id_category'] : $listing_old['id_category']);
                $stat = $this->get_stat_by_id($stat_gid);
                if ($stat) {
                    $save_data = array($type_name => $stat[$type_name] + 1);
                    $validate_data = $this->validate_stat($stat['id'], $save_data);
                    $this->save_stat($stat['id'], $validate_data['data']);
                } else {
                    $save_data = array('gid' => $stat_gid, $type_name => 1);
                    $validate_data = $this->validate_stat(null, $save_data);
                    $this->save_stat(null, $validate_data['data']);
                }
            }

            if ((isset($listing_new['id_category']) || isset($listing_old['id_category'])) &&
                (isset($listing_new['property_type']) || isset($listing_old['property_type']))) {
                $type_name = $type_new . '_cnt';
                $stat_gid = 'property_type_' . (isset($listing_new['id_category']) ? $listing_new['id_category'] : $listing_old['id_category']) . '_' . (isset($listing_new['property_type']) ? $listing_new['property_type'] : $listing_old['property_type']);
                $stat = $this->get_stat_by_id($stat_gid);
                if ($stat) {
                    $save_data = array($type_name => $stat[$type_name] + 1);
                    $validate_data = $this->validate_stat($stat['id'], $save_data);
                    $this->save_stat($stat['id'], $validate_data['data']);
                } else {
                    $save_data = array('gid' => $stat_gid, $type_name => 1);
                    $validate_data = $this->validate_stat(null, $save_data);
                    $this->save_stat(null, $validate_data['data']);
                }
            }

            if (isset($listing_new['id_country']) || isset($listing_old['id_country'])) {
                $type_name = $type_new . '_cnt';
                $stat_gid = 'country_' . (isset($listing_new['id_country']) ? $listing_new['id_country'] : $listing_old['id_country']);
                $stat = $this->get_stat_by_id($stat_gid);
                if ($stat) {
                    $save_data = array($type_name => $stat[$type_name] + 1);
                    $validate_data = $this->validate_stat($stat['id'], $save_data);
                    $this->save_stat($stat['id'], $validate_data['data']);
                } else {
                    $save_data = array('gid' => $stat_gid, $type_name => 1);
                    $validate_data = $this->validate_stat(null, $save_data);
                    $this->save_stat(null, $validate_data['data']);
                }
            }

            if (isset($listing_new['id_region']) || isset($listing_old['id_region'])) {
                $type_name = $type_new . '_cnt';
                $stat_gid = 'region_' . (isset($listing_new['id_region']) ? $listing_new['id_region'] : $listing_old['id_region']);
                $stat = $this->get_stat_by_id($stat_gid);
                if ($stat) {
                    $save_data = array($type_name => $stat[$type_name] + 1);
                    $validate_data = $this->validate_stat($stat['id'], $save_data);
                    $this->save_stat($stat['id'], $validate_data['data']);
                } else {
                    $save_data = array('gid' => $stat_gid, $type_name => 1);
                    $validate_data = $this->validate_stat(null, $save_data);
                    $this->save_stat(null, $validate_data['data']);
                }
            }

            if (isset($listing_new['id_city']) && isset($listing_old['id_city'])) {
                $type_name = $type_new . '_cnt';
                $stat_gid = 'city_' . (isset($listing_new['id_city']) ? $listing_new['id_city'] : $listing_old['id_city']);
                $stat = $this->get_stat_by_id($stat_gid);
                if ($stat) {
                    $save_data = array($type_name => $stat[$type_name] + 1);
                    $validate_data = $this->validate_stat($stat['id'], $save_data);
                    $this->save_stat($stat['id'], $validate_data['data']);
                } else {
                    $save_data = array('gid' => $stat_gid, $type_name => 1);
                    $validate_data = $this->validate_stat(null, $save_data);
                    $this->save_stat(null, $validate_data['data']);
                }
            }

            if (isset($listing_new['id_user']) || isset($listing_old['id_user'])) {
                $is_users_installed = $this->ci->pg_module->is_module_active("users");
                if ($is_users_installed) {
                    $this->ci->load->model('Users_model');
                    $user = $this->ci->Users_model->get_user_by_id((isset($listing_new['id_user']) ? $listing_new['id_user'] : $listing_old['id_user']));

                    if ($user) {
                        $this->ci->Users_model->update_listings_stat($user['id'], $type_new);
                    }
                }
            }
        } elseif ((!isset($listing_new['status']) && $listing_old['status']) || (isset($listing_new['status']) && $listing_new['status'])) {
            $type_id = isset($listing_new['id_type']) ? $listing_new['id_type'] : $listing_old['id_type'];
            $type_old = $this->ci->Listings_model->getOperationTypeById($listing_old['id_type']);
            $type_new = $this->ci->Listings_model->getOperationTypeById($type_id);

            if (($listing_old['id_type'] != $type_id) || (isset($listing_new['id_category']) && $listing_old['id_category'] != $listing_new['id_category'])) {
                $stat_gid = 'property_type_' . $listing_old['id_category'];
                $stat = $this->get_stat_by_id($stat_gid);
                if ($stat) {
                    $type_name = $type_old . '_cnt';
                    $save_data = array($type_name => $stat[$type_name] ? $stat[$type_name] - 1 : 0);
                    $validate_data = $this->validate_stat($stat['id'], $save_data);
                    $this->save_stat($stat['id'], $validate_data['data']);
                }

                $type_name = $type_new . '_cnt';
                $stat_gid = 'property_type_' . (isset($listing_new['id_category']) ? $listing_new['id_category'] : $listing_old['id_category']);
                $stat = $this->get_stat_by_id($stat_gid);
                if ($stat) {
                    $save_data = array($type_name => $stat[$type_name] + 1);
                    $validate_data = $this->validate_stat($stat['id'], $save_data);
                    $this->save_stat($stat['id'], $validate_data['data']);
                } else {
                    $save_data = array('gid' => $stat_gid, $type_name => 1);
                    $validate_data = $this->validate_stat(null, $save_data);
                    $this->save_stat(null, $validate_data['data']);
                }
            }

            if (($listing_old['id_type'] != $type_id) || (isset($listing_new['property_type']) && $listing_old['property_type'] != $listing_new['property_type'])) {
                $stat_gid = 'property_type_' . $listing_old['id_category'] . '_' . $listing_old['property_type'];
                $stat = $this->get_stat_by_id($stat_gid);
                if ($stat) {
                    $type_name = $type_old . '_cnt';
                    $save_data = array($type_name => $stat[$type_name] ? $stat[$type_name] - 1 : 0);
                    $validate_data = $this->validate_stat($stat['id'], $save_data);
                    $this->save_stat($stat['id'], $validate_data['data']);
                }

                $type_name = $type_new . '_cnt';
                $stat_gid = 'property_type_' . (isset($listing_new['id_category']) ? $listing_new['id_category'] : $listing_old['id_category']) . '_' . (isset($listing_new['property_type']) ? $listing_new['property_type'] : $listing_old['property_type']);
                $stat = $this->get_stat_by_id($stat_gid);
                if ($stat) {
                    $save_data = array($type_name => $stat[$type_name] + 1);
                    $validate_data = $this->validate_stat($stat['id'], $save_data);
                    $this->save_stat($stat['id'], $validate_data['data']);
                } else {
                    $save_data = array('gid' => $stat_gid, $type_name => 1);
                    $validate_data = $this->validate_stat(null, $save_data);
                    $this->save_stat(null, $validate_data['data']);
                }
            }

            if (($listing_old['id_type'] != $type_id) || (isset($listing_new['id_country']) && $listing_old['id_country'] != $listing_new['id_country'])) {
                $stat_gid = 'country_' . $listing_old['id_country'];
                $stat = $this->get_stat_by_id($stat_gid);
                if ($stat) {
                    $type_name = $type_old . '_cnt';
                    $save_data = array($type_name => $stat[$type_name] ? $stat[$type_name] - 1 : 0);
                    $validate_data = $this->validate_stat($stat['id'], $save_data);
                    $this->save_stat($stat['id'], $validate_data['data']);
                }

                $type_name = $type_new . '_cnt';
                $stat_gid = 'country_' . (isset($listing_new['id_country']) ? $listing_new['id_country'] : $listing_old['id_country']);
                $stat = $this->get_stat_by_id($stat_gid);
                if ($stat) {
                    $save_data = array($type_name => $stat[$type_name] + 1);
                    $validate_data = $this->validate_stat($stat['id'], $save_data);
                    $this->save_stat($stat['id'], $validate_data['data']);
                } else {
                    $save_data = array('gid' => $stat_gid, $type_name => 1);
                    $validate_data = $this->validate_stat(null, $save_data);
                    $this->save_stat(null, $validate_data['data']);
                }
            }

            if (($listing_old['id_type'] != $type_id) || (isset($listing_new['id_region']) && $listing_old['id_region'] != $listing_new['id_region'])) {
                $stat_gid = 'region_' . $listing_old['id_region'];
                $stat = $this->get_stat_by_id($stat_gid);
                if ($stat) {
                    $type_name = $type_old . '_cnt';
                    $save_data = array($type_name => max($stat[$type_name] - 1, 0));
                    $validate_data = $this->validate_stat($stat['id'], $save_data);
                    $this->save_stat($stat['id'], $validate_data['data']);
                }

                $type_name = $type_new . '_cnt';
                $stat_gid = 'region_' . (isset($listing_new['id_region']) ? $listing_new['id_region'] : $listing_old['id_region']);
                $stat = $this->get_stat_by_id($stat_gid);
                if ($stat) {
                    $save_data = array($type_name => $stat[$type_name] + 1);
                    $validate_data = $this->validate_stat($stat['id'], $save_data);
                    $this->save_stat($stat['id'], $validate_data['data']);
                } else {
                    $save_data = array('gid' => $stat_gid, $type_name => 1);
                    $validate_data = $this->validate_stat(null, $save_data);
                    $this->save_stat(null, $validate_data['data']);
                }
            }

            if (($listing_old['id_type'] != $type_id) || (isset($listing_new['id_city']) && $listing_old['id_city'] != $listing_new['id_city'])) {
                $stat_gid = 'city_' . $listing_old['id_city'];
                $stat = $this->get_stat_by_id($stat_gid);
                if ($stat) {
                    $type_name = $type_old . '_cnt';
                    $save_data = array($type_name => max($stat[$type_name] - 1, 0));
                    $validate_data = $this->validate_stat($stat['id'], $save_data);
                    $this->save_stat($stat['id'], $validate_data['data']);
                }

                $type_name = $type_new . '_cnt';
                $stat_gid = 'city_' . (isset($listing_new['id_city']) ? $listing_new['id_city'] : $listing_old['id_city']);
                $stat = $this->get_stat_by_id($stat_gid);
                if ($stat) {
                    $save_data = array($type_name => $stat[$type_name] + 1);
                    $validate_data = $this->validate_stat($stat['id'], $save_data);
                    $this->save_stat($stat['id'], $validate_data['data']);
                } else {
                    $save_data = array('gid' => $stat_gid, $type_name => 1);
                    $validate_data = $this->validate_stat(null, $save_data);
                    $this->save_stat(null, $validate_data['data']);
                }
            }


            if (($listing_old['id_type'] != $type_id) || (isset($listing_new['id_user']) && $listing_old['id_user'] != $listing_new['id_user'])) {
                $is_users_installed = $this->ci->pg_module->is_module_active("users");
                if ($is_users_installed) {
                    $this->ci->load->model('Users_model');
                    $user = $this->ci->Users_model->get_user_by_id($listing_old['id_user']);

                    if ($user) {
                        $this->ci->Users_model->update_listings_stat($user['id'], $type_old, false);
                    }

                    if (isset($listing_new['id_user']))
                        $user = $this->ci->Users_model->get_user_by_id($listing_new['id_user']);
                    if ($user) {
                        $this->ci->Users_model->update_listings_stat($user['id'], $type_new);
                    }
                }
            }
        }
    }

}
