<?php

namespace Pg\Modules\Listings\Models;

/**
 * Listings module
 *
 * @package 	PG_RealEstate
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

define('LISTINGS_VISITOR_TABLE', DB_PREFIX . 'listings_profile_visitors');
define('LISTINGS_VISITS_TABLE', DB_PREFIX . 'listings_profile_visits');

/**
 * Listings visitor model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Listings
 * @category	models
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Listings_visitor_model extends \Model
{

    /**
     * Link to CodeIgniter object
     * 
     * @var object
     */
    protected $CI;

    /**
     * Link to database object
     * 
     * @var object
     */
    protected $DB;

    /**
     * Properties of visit object in data source
     * 
     * @var array
     */
    protected $fields = array(
        'id',
        'id_listing',
        'id_user',
        'id_visitor',
        'last_visit_date',
        'visits_count',
    );

    /**
     * Constructor
     *
     * @return Listings_visitor_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = & get_instance();
        $this->DB = &$this->CI->db;
    }

    /**
     * Return visitor object from data source by identifier
     * 
     * @param integer $visitor_id visitor record identifier
     * @return array/false
     */
    public function get_visitor_by_id($visitor_id)
    {
        $result = $this->DB->select(implode(', ', $this->fields))
                ->from(LISTINGS_VISITOR_TABLE)
                ->where('id', $visitor_id)
                ->get()->result_array();
        if (empty($result))
            return false;
        return $result[0];
    }

    /**
     * Return visit object from data source by visitor
     * 
     * @param integer $listing_id listing identifier
     * @param integer $user_id user identifier
     * @param integer $visitor_id record identifier
     * @return array/false
     */
    public function get_visit_by_visitor($listing_id, $user_id, $visitor_id)
    {
        $result = $this->DB->select(implode(', ', $this->fields))
                ->from(LISTINGS_VISITOR_TABLE)
                ->where('id_listing', $listing_id)
                ->where('id_user', $user_id)
                ->where('id_visitor', $visitor_id)
                ->get()->result_array();
        if (empty($result))
            return false;
        return $result[0];
    }

    /**
     * Return visitor object from data source by visitor
     * 
     * @param integer $user_id user record identifier
     * @param integer $visitor_id visitor record identifier
     * @return array/false
     */
    public function get_visitor_by_visitor($user_id, $visitor_id)
    {
        $result = $this->DB->select(implode(', ', $this->fields))
                ->from(LISTINGS_VISITOR_TABLE)
                ->where('id_listing', 0)
                ->where('id_user', $user_id)
                ->where('id_visitor', $visitor_id)
                ->get()->result_array();
        if (empty($result))
            return false;
        return $result[0];
    }

    /**
     * Save visitor object to data source 
     * 
     * @param integer $visitor_id visitor identifier
     * @param array $data visitor data
     * @return integer
     */
    public function save_visitor($visitor_id, $data = array())
    {
        if (!isset($data['last_visit_date']))
            $data['last_visit_date'] = date('Y-m-d H:i:s');
        if (!$visitor_id) {
            if (!isset($data['visits_count']))
                $data['visits_count'] = 1;
            $this->DB->insert(LISTINGS_VISITOR_TABLE, $data);
            $visitor_id = $this->DB->insert_id();
        }else {
            if (!isset($data['visits_count']))
                $this->DB->set('visits_count', 'visits_count + 1', FALSE);
            $this->DB->where('id', $visitor_id);
            $this->DB->update(LISTINGS_VISITOR_TABLE, $data);
        }
        return $visitor_id;
    }

    /**
     * Return sql criteria by filters data
     * 
     * @param array $filters filters data
     * @return array
     */
    protected function _get_search_criteria($filters)
    {
        $params = array();
        $fields = array_flip($this->fields);
        foreach ($filters as $filter_name => $filter_data) {
            switch ($filter_name) {
                // By listing
                case 'only_listings':
                    if (!$filter_data)
                        continue;
                    $params = array_merge_recursive($params, array('where' => array(LISTINGS_VISITOR_TABLE . '.id_listing >' => 0)));
                    break;
                case 'all_listings':
                    if (!$filter_data)
                        continue;
                    $params = array_merge_recursive($params, array('where' => array(LISTINGS_VISITOR_TABLE . '.id_listing' => 0)));
                    break;
                // By listing
                case 'user':
                    if (!$filter_data)
                        continue;
                    $params = array_merge_recursive($params, array('where' => array(LISTINGS_VISITOR_TABLE . '.id_user' => $filter_data)));
                    break;
                // By visitor
                case 'visitor':
                    if (!$filter_data)
                        continue;
                    $params = array_merge_recursive($params, array('where' => array(LISTINGS_VISITOR_TABLE . '.id_visitor' => $filter_data)));
                    break;
                // By week
                case 'week':
                    if (!$filter_data)
                        continue;
                    $params = array_merge_recursive($params, array('where_sql' => array('(' . LISTINGS_VISITOR_TABLE . '.last_visit_date >= DATE_SUB(CURDATE(), INTERVAL 1 WEEK))')));
                    break;
                // By month
                case 'month':
                    if (!$filter_data)
                        continue;
                    $params = array_merge_recursive($params, array('where_sql' => array('(' . LISTINGS_VISITOR_TABLE . '.last_visit_date >= DATE_SUB(CURDATE(), INTERVAL 1 MONTH))')));
                    break;
                default:
                    if (isset($fields[$filter_name])) {
                        if (is_array($filter_data)) {
                            if (empty($filter_data))
                                break;
                            $params = array_merge_recursive($params, array('where_in' => array(LISTINGS_VISITOR_TABLE . '.' . $filter_name => $filter_data)));
                        }else {
                            $params = array_merge_recursive($params, array('where' => array(LISTINGS_VISITOR_TABLE . '.' . $filter_name => trim($filter_data))));
                        }
                    }
                    break;
            }
        }

        return $params;
    }

    /**
     * Return visitors objects from data source as array
     * 
     * @param integer $page page of results
     * @param string $limits results limit
     * @param array $order_by sorting data
     * @param array $params sql criteria
     * @return array
     */
    protected function _get_visitors_list($page = null, $limits = null, $order_by = null, $params = array())
    {
        $this->DB->select(LISTINGS_VISITOR_TABLE . '.' . implode(', ' . LISTINGS_VISITOR_TABLE . '.', $this->fields));
        $this->DB->from(LISTINGS_VISITOR_TABLE);

        if (isset($params['join']) && is_array($params['join']) && count($params['join'])) {
            foreach ($params['join'] as $join) {
                $this->DB->join($join['table'], $join['condition'], $join['type']);
            }
        }

        if (isset($params['where']) && is_array($params['where']) && count($params['where'])) {
            foreach ($params['where'] as $field => $value) {
                $this->DB->where($field, $value);
            }
        }

        if (isset($params['where_in']) && is_array($params['where_in']) && count($params['where_in'])) {
            foreach ($params['where_in'] as $field => $value) {
                $this->DB->where_in($field, $value);
            }
        }

        if (isset($params['where_sql']) && is_array($params['where_sql']) && count($params['where_sql'])) {
            foreach ($params['where_sql'] as $value) {
                $this->DB->where($value);
            }
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->fields)) {
                    $this->DB->order_by($field . ' ' . $dir);
                }
            }
        } elseif ($order_by) {
            $this->DB->order_by($order_by);
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->DB->limit($limits, $limits * ($page - 1));
        }

        $results = $this->DB->get()->result_array();

        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $data[] = $r;
            }
            return $data;
        }
        return array();
    }

    /**
     * Return number of visitors in data source
     * 
     * @param array $params filters data
     * @return integer
     */
    protected function _get_visitors_count($params = null)
    {
        $this->DB->select('COUNT(*) AS cnt');
        $this->DB->from(LISTINGS_VISITOR_TABLE);

        if (isset($params['join']) && is_array($params['join']) && count($params['join'])) {
            foreach ($params['join'] as $join) {
                $this->DB->join($join['table'], $join['condition'], $join['type']);
            }
        }

        if (isset($params['where']) && is_array($params['where']) && count($params['where'])) {
            foreach ($params['where'] as $field => $value) {
                $this->DB->where($field, $value);
            }
        }

        if (isset($params['where_in']) && is_array($params['where_in']) && count($params['where_in'])) {
            foreach ($params['where_in'] as $field => $value) {
                $this->DB->where_in($field, $value);
            }
        }

        if (isset($params['where_sql']) && is_array($params['where_sql']) && count($params['where_sql'])) {
            foreach ($params['where_sql'] as $value) {
                $this->DB->where($value);
            }
        }

        $results = $this->DB->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]['cnt']);
        }
        return 0;
    }

    /**
     * Return filtered visitors from data source as array
     * 
     * @param array $filters filters data
     * @param integer $page page of results
     * @param integer $items_on_page items per page
     * @param string $order_by sorting data
     * @return array
     */
    public function get_visitors_list($filters = array(), $page = null, $items_on_page = null, $order_by = null)
    {
        $params = $this->_get_search_criteria($filters);
        return $this->_get_visitors_list($page, $items_on_page, $order_by, $params);
    }

    /**
     * Return number of filtered visitors in data source
     * 
     * @param array $filters filters data
     * @return integer
     */
    public function get_visitors_count($filters = array())
    {
        $params = $this->_get_search_criteria($filters);
        return $this->_get_visitors_count($params);
    }

    /**
     * Validate visitor data for saving to data source
     * 
     * @param integer $visitor_id visitor identifier
     * @param array $data visitor data
     * @return array
     */
    public function validate_visitor($visitor_id, $data)
    {
        $return = array('errors' => array(), 'data' => array());

        if (isset($data['id'])) {
            $return['data']['id'] = intval($data['id']);
            if (empty($return['data']['id']))
                unset($return['data']['id']);
        }

        if (isset($data['id_user'])) {
            $return['data']['id_user'] = intval($data['id_user']);
        }

        if (isset($data['id_visitor'])) {
            $return['data']['id_visitor'] = intval($data['id_visitor']);
        }

        if (isset($data['last_visit_date'])) {
            $value = strtotime($data['last_visit_date']);
            if ($value > 0)
                $return['data']['last_visit_date'] = date('Y-m-d', $value);
        }

        if (isset($data['visit_counts'])) {
            $return['data']['visit_counts'] = intval($data['visit_counts']);
        }

        return $return;
    }

    /**
     * Check listing view by user
     * 
     * @param integer $user_id user identifier
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function check_views_count($user_id, $listing_id)
    {
        $views_array = $this->CI->session->userdata('views_listings_array');
        if (!isset($views_array) || !is_array($views_array))
            $views_array = array();
        if (in_array($listing_id, $views_array))
            return;
        $views_array[] = $listing_id;
        $this->CI->session->set_userdata('views_listings_array', $views_array);
        $this->save_views_count($user_id, $listing_id);
    }

    /**
     * Save number views of listing profile by user
     * 
     * @param integer $user_id listing identifier
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function save_views_count($user_id, $listing_id)
    {
        if ($this->CI->session->userdata('auth_type') == 'user') {
            $visitor_id = intval($this->CI->session->userdata('user_id'));
            if ($user_id != $visitor_id) {
                $this->CI->load->model('listings/models/Listings_visitor_model');

                $visit = $this->CI->Listings_visitor_model->get_visit_by_visitor($listing_id, $user_id, $visitor_id);
                $save_data = array('id_listing' => $listing_id, 'id_user' => $user_id, 'id_visitor' => $visitor_id);
                $this->CI->Listings_visitor_model->save_visitor($visit['id'], $save_data);

                $visitor = $this->CI->Listings_visitor_model->get_visitor_by_visitor($user_id, $visitor_id);
                $save_data = array('id_listing' => 0, 'id_user' => $user_id, 'id_visitor' => $visitor_id);
                $this->CI->Listings_visitor_model->save_visitor($visit['id'], $save_data);
            }
        } else {
            $visitor_id = 0;
        }

        //$this->DB->set('views', 'views + 1', FALSE);
        //$this->DB->where('id_listing', $listing_id);
        //$this->DB->update(LISTINGS_STATS_DAY_TABLE);

        $save_data = array(
            'id_listing' => $listing_id,
            'id_visitor' => $visitor_id,
            'date_add' => date('Y-m-d H:i:s'),
        );
        $this->DB->insert(LISTINGS_VISITS_TABLE, $save_data);
    }

    /**
     * Return number views of listings' profiles by last day
     * 
     * @param array $listings_ids listings identifiers
     * @return array
     */
    public function get_views_count($listings_ids)
    {
        $this->DB->select('id_listing, COUNT(*) AS count')
            ->from(LISTINGS_VISITS_TABLE)
            ->where_in('id_listing', $listings_ids)
            ->group_by(array('id_listing'));
        $results = $this->DB->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $data = array();
            foreach ($results as $result) {
                $data[$result['id_listing']] = $result['count'];
            }
            return $data;
        }
        return array();
    }

    /**
     * Update number views of listings' profiles by cron
     * 
     * @return void
     */
    public function update_views_count_cron()
    {
        $date = date('Y-m-d', time()) . ' 00:00:00';

        $ids = array();
        $listings = array();
        $this->DB->select('id, id_listing, id_visitor')->from(LISTINGS_VISITS_TABLE)->where('date_add <', $date);
        $results = $this->DB->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $result) {
                $ids[] = $result['id'];
                if (!isset($listings[$result['id_listing']]))
                    $listings[$result['id_listing']] = 0;
                $listings[$result['id_listing']] ++;
            }

            $this->DB->where('date_add <', $date)->delete(LISTINGS_VISITS_TABLE);

            $this->CI->load->model('Listings_model');

            $this->CI->Listings_model->set_format_settings('use_format', false);
            $listings_data = $this->CI->Listings_model->get_listings_list(array('id' => array_keys($listings)));
            $this->CI->Listings_model->set_format_settings('use_format', true);

            foreach ($listings_data as $listing_data) {
                $save_data = array('views' => $listing_data['views'] + $listings[$listing_data['id']]);
                $this->CI->Listings_model->save_listing($listing_data['id'], $save_data);
            }
        }
    }

}
