<?php

namespace Pg\Modules\Widgets\Models;

/**
 * Widgets module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

define('WIDGETS_TABLE', DB_PREFIX . 'widgets');

/**
 * Widgets model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Widgets
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Widgets_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @return object
     */
    protected $ci;

    /**
     * Widget properties in data source
     *
     * @var array
     */
    private $_fields = array(
        'id',
        'gid',
        'module',
        'url',
        'size',
        'colors',
        'settings',
        'date_created',
        'date_modified',
    );

    /**
     * Settings for formatting widget object
     *
     * @var array
     */
    private $format_settings = array(
        'use_format' => true,
        'get_content' => false,
    );

    /**
     * Constructor
     *
     * @return Widgets_model
     */
    public function __construct()
    {
        parent::__construct();

        $this->ci = &get_instance();

        foreach ($this->ci->pg_language->languages as $id => $value) {
            $this->_fields[] = 'title_' . $value['id'];
            $this->_fields[] = 'footer_' . $value['id'];
        }
    }

    /**
     * Return widget object by identifier
     *
     * @param integer $widget_id widget identifier
     *
     * @return array
     */
    public function get_widget_by_id($widget_id)
    {
        $this->ci->db->select(implode(', ', $this->_fields));
        $this->ci->db->from(WIDGETS_TABLE);
        $this->ci->db->where('id', $widget_id);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return $this->format_widget($results[0]);
        }

        return array();
    }

    /**
     * Return widget by GUID
     *
     * @param string $widget_gid widget guid
     * @param string $url        request url
     *
     * @return array
     */
    public function get_widget_by_gid($widget_gid, $url = null)
    {
        $this->ci->db->select(implode(', ', $this->_fields));
        $this->ci->db->from(WIDGETS_TABLE);
        $this->ci->db->where('gid', $widget_gid);
        if (!empty($url)) {
            $this->ci->db->where('(url=' . $this->ci->db->escape($url) . ' OR url=' . $this->ci->db->escape('') . ')');
        }
        $this->ci->db->order_by('url DESC');
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return $this->format_widget($results[0]);
        }

        return array();
    }

    /**
     * Return sql criteria for widget filters
     *
     * @param array $filters filters data
     *
     * @return array
     */
    private function _get_search_criteria($filters)
    {
        $params = array();

        $fields = array_flip($this->_fields);
        foreach ($filters as $filter_name => $filter_data) {
            if (!$filter_data) {
                continue;
            }
        }

        return $params;
    }

    /**
     * Return widgets objects from data source as array
     *
     * @param integer $page      page of results
     * @param string  $limits    results limit
     * @param array   $order_by  sorting order
     * @param array   $params    sql criteria
     * @param boolean $formatted format results
     *
     * @return array
     */
    private function _get_widgets_list($page = null, $limits = null, $order_by = null, $params = array(), $formatted = true)
    {
        $this->ci->db->select(implode(', ', $this->_fields));
        $this->ci->db->from(WIDGETS_TABLE);

        if (isset($params['where']) && is_array($params['where']) && count($params['where'])) {
            foreach ($params['where'] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params['where_in']) && is_array($params['where_in']) && count($params['where_in'])) {
            foreach ($params['where_in'] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params['where_not_in']) && is_array($params['where_not_in']) && count($params['where_not_in'])) {
            foreach ($params['where_not_in'] as $field => $value) {
                $this->ci->db->where_not_in($field, $value);
            }
        }

        if (isset($params['where_sql']) && is_array($params['where_sql']) && count($params['where_sql'])) {
            foreach ($params['where_sql'] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->_fields)) {
                    $this->ci->db->order_by($field . ' ' . $dir);
                }
            }
        } elseif ($order_by) {
            $this->ci->db->order_by($order_by);
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($limits, $limits * ($page - 1));
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            if ($formatted) {
                $results = $this->format_widgets($results);
            }

            return $results;
        }

        return array();
    }

    /**
     * Return number of widgets in data source
     *
     * @param array $params filters criteria
     *
     * @return integer
     */
    private function _get_widgets_count($params = null)
    {
        $this->ci->db->select('COUNT(*) AS cnt');
        $this->ci->db->from(WIDGETS_TABLE);

        if (isset($params['where']) && is_array($params['where']) && count($params['where'])) {
            foreach ($params['where'] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params['where_in']) && is_array($params['where_in']) && count($params['where_in'])) {
            foreach ($params['where_in'] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params['where_sql']) && is_array($params['where_sql']) && count($params['where_sql'])) {
            foreach ($params['where_sql'] as $value) {
                $this->ci->db->where($value);
            }
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]['cnt']);
        }

        return 0;
    }

    /**
     * Return filtered widgets objects from data source as array
     *
     * @param array   $filters       filter data
     * @param integer $page          page of results
     * @param integer $items_on_page items per page
     * @param string  $order_by      sorting data
     * @param boolean $formatted     format results
     *
     * @return array
     */
    public function get_widgets_list($filters = array(), $page = null, $items_on_page = null, $order_by = null, $formatted = true)
    {
        $params = $this->_get_search_criteria($filters);

        return $this->_get_widgets_list($page, $items_on_page, $order_by, $params, $formatted);
    }

    /**
     * Return number of filtered widgets in data source
     *
     * @param array $filters filters data
     *
     * @return integer
     */
    public function get_widgets_count($filters = array())
    {
        $params = $this->_get_search_criteria($filters);

        return $this->_get_widgets_count($params);
    }

    /**
     * Save widget object to data source
     *
     * @param integer $widget_id widget identifier
     * @param array   $data      widget data
     *
     * @return integer
     */
    public function save_widget($widget_id, $data)
    {
        if (!$widget_id) {
            $data['date_created'] = $data['date_modified'] = date('Y-m-d H:i:s');
            $this->ci->db->insert(WIDGETS_TABLE, $data);
            $widget_id = $this->ci->db->insert_id();
        } else {
            $data['date_modified'] = date('Y-m-d H:i:s');
            $this->ci->db->where('id', $widget_id);
            $this->ci->db->update(WIDGETS_TABLE, $data);
        }

        return $widget_id;
    }

    /**
     * Remove widget object from data source
     *
     * @param string $widget_gid widget guid
     *
     * @return void
     */
    public function delete_widget($widget_gid)
    {
        $this->ci->db->where('gid', $widget_gid);
        $this->ci->db->delete(WIDGETS_TABLE, $data);
    }

    /**
     * Validate widget data for saving to data source
     *
     * @param integer $widget_id widget identifier
     * @param array   $data      widget data
     *
     * @return array
     */
    public function validate_widget($widget_id, $data)
    {
        $return = array('errors' => array(), 'data' => array());

        if (isset($data['id'])) {
            $return['data']['id'] = intval($data['id']);
            if (empty($return['data']['id'])) {
                unset($return['data']['id']);
            }
        }

        if (isset($data['gid'])) {
            $return['data']['gid'] = trim(strip_tags($data['gid']));
            if (empty($return['data']['gid'])) {
                unset($return['data']['gid']);
            }
        }

        if (isset($data['module'])) {
            $return['data']['module'] = trim(strip_tags($data['module']));
        }

        if (isset($data['model'])) {
            $return['data']['model'] = trim(strip_tags($data['model']));
        }

        if (isset($data['title'])) {
            $return['data']['title'] = trim(strip_tags($data['title']));
        }

        if (isset($data['footer'])) {
            $return['data']['footer'] = trim(strip_tags($data['footer']));
        }

        if (isset($data['size'])) {
            $return['data']['size'] = intval($data['size']);
        }

        if (isset($data['colors'])) {
            $return['data']['colors'] = serialize($data['colors']);
        }

        if (isset($data['settings'])) {
            $return['data']['settings'] = serialize($data['settings']);
        }

        if (isset($data['date_created'])) {
            $value = strtotime($data['date_created']);
            if ($value > 0) {
                $return['data']['date_created'] = date('Y-m-d', $value);
            } else {
                $return['data']['date_created'] = '0000-00-00 00:00:00';
            }
        }

        if (isset($data['date_modified'])) {
            $value = strtotime($data['date_modified']);
            if ($value > 0) {
                $return['data']['date_modified'] = date('Y-m-d', $value);
            } else {
                $return['data']['date_modified'] = '0000-00-00 00:00:00';
            }
        }

        $lang_title = $this->validateLangValue('title', $data);
        
        $default_lang_id = $this->ci->pg_language->current_lang_id;
        if (isset($data['title_' . $default_lang_id])) {
            $return['data']['title_' . $default_lang_id] = trim(strip_tags($data['title_' . $default_lang_id]));
            foreach ($this->pg_language->languages as $lid => $lang_data) {
                if ($lid == $default_lang_id) {
                    continue;
                }
                if (empty($data['title_' . $lid])) {
                    $return['data']['title_' . $lid] = $return['data']['title_' . $default_lang_id];
                } else {
                    $return['data']['title_' . $lid] = trim(strip_tags($data['title_' . $lid]));
                }
            }
        }

        $default_lang_id = $this->ci->pg_language->current_lang_id;
        if (isset($data['footer_' . $default_lang_id])) {
            $return['data']['footer_' . $default_lang_id] = trim(strip_tags($data['footer_' . $default_lang_id]));
            foreach ($this->pg_language->languages as $lid => $lang_data) {
                if ($lid == $default_lang_id) {
                    continue;
                }
                if (empty($data['footer_' . $lid])) {
                    $return['data']['footer_' . $lid] = $return['data']['footer_' . $default_lang_id];
                } else {
                    $return['data']['footer_' . $lid] = trim(strip_tags($data['footer_' . $lid]));
                }
            }
        }
        
        return $return;
    }

    private function validateLangValue($field, $data)
    {
        $default_lang_id = $this->ci->pg_language->current_lang_id;
        $default_lang_key = $field . '_' . $default_lang_id;
        if (empty($data[$default_lang_key])) {
            return [];
        }
        $filtered_data = [];
        $filtered_data[$default_lang_key] = trim(strip_tags($data[$default_lang_key]));
        foreach (array_keys($this->pg_language->languages) as $lang_id) {
            if ($lang_id == $default_lang_id) {
                continue;
            }
            $lang_key = $field . '_' . $lang_id;
            if (empty($data[$lang_key])) {
                $filtered_data[$lang_key] = $filtered_data[$default_lang_key];
            } else {
                $filtered_data[$lang_key] = trim(strip_tags($data[$lang_key]));
            }
        }
        return $filtered_data;
    }

    /**
     * Change settings for formatting widget object
     *
     * @param string $name  parameter name
     * @param mixed  $value parameter value
     *
     * @return void
     */
    public function set_format_settings($name, $value = false)
    {
        if (!is_array($name)) {
            $name = array($name => $value);
        }
        if (empty($name)) {
            return;
        }
        foreach ($name as $key => $item) {
            $this->format_settings[$key] = $item;
        }
    }

    /**
     * Format widget data
     *
     * @param array $data widget data
     *
     * @return array
     */
    public function format_widget($data)
    {
        return current($this->format_widgets(array($data)));
    }

    /**
     * Format widgets data
     *
     * @param array $data widgets data
     *
     * @return array
     */
    public function format_widgets($data)
    {
        if (!$this->format_settings['use_format']) {
            return $data;
        }

        foreach ($data as $key => $widget) {
            $widget['title'] = $widget['title_' . $this->pg_language->current_lang_id];
            $widget['footer'] = $widget['footer_' . $this->pg_language->current_lang_id];
            $widget['colors'] = $widget['colors'] ? unserialize($widget['colors']) : array();
            $widget['settings'] = $widget['settings'] ? unserialize($widget['settings']) : array();

            if ($this->format_settings['get_content']) {
                if ($this->ci->pg_module->is_module_active($widget['module'])) {
                    $model_name = ucfirst($widget['gid']);
                    $this->ci->load->model($widget['module'] . '/widgets/' . $model_name, $model_name);
                    $widget['content'] = $this->ci->{$model_name}->generate($widget['settings'], $widget['colors']);
                }
            }

            $data[$key] = $widget;
        }

        return $data;
    }

    /**
     * Install widget properties depended on languages
     *
     * @param integer $lang_id language identifier
     *
     * @return void
     */
    public function lang_dedicate_module_callback_add($lang_id = false)
    {
        if (!$lang_id) {
            return;
        }

        $this->ci->load->dbforge();

        $field = array('title_' . $lang_id => array('type' => 'TEXT', 'null' => true));
        $this->ci->dbforge->add_column(WIDGETS_TABLE, $field);

        $field = array('footer_' . $lang_id => array('type' => 'TEXT', 'null' => true));
        $this->ci->dbforge->add_column(WIDGETS_TABLE, $field);

        $default_lang_id = $this->ci->pg_language->get_default_lang_id();
        if ($lang_id != $default_lang_id) {
            $this->ci->db->set('title_' . $lang_id, 'title_' . $default_lang_id, false);
            $this->ci->db->set('footer_' . $lang_id, 'footer_' . $default_lang_id, false);
            $this->ci->db->update(WIDGETS_TABLE);
        }
    }

    /**
     * Uninstall widget properties depended on languages
     *
     * @param integer $lang_id language identifier
     *
     * @return void
     */
    public function lang_dedicate_module_callback_delete($lang_id = false)
    {
        if (!$lang_id) {
            return;
        }

        $this->ci->load->dbforge();

        $fields_exists = $this->ci->db->list_fields(WIDGETS_TABLE);

        $fields = array('title_' . $lang_id, 'footer_' . $lang_id);
        foreach ($fields as $field_name) {
            if (!in_array($field_name, $fields_exists)) {
                continue;
            }
            $this->ci->dbforge->drop_column(WIDGETS_TABLE, $field_name);
        }
    }

}
