<?php

namespace Pg\Modules\Listings\Controllers;

/**
 * Listings module
 *
 * @package     PG_RealEstate
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;
use Pg\Modules\Listings\Models\Listings_model;

/**
 * Listings user side controller
 *
 * @package     PG_RealEstate
 * @subpackage  Listings
 * @category    controller
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Listings extends \Controller
{

    /**
     * Constructor
     *
     * @return Listings
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Listings_model');
        $this->load->model('Menu_model');
    }

    private function filterOperationType($operation_type, array $operation_types)
    {
        if (in_array($operation_type, $operation_types)) {
            return $operation_type;
        }
        foreach ($operation_types as $type) {
            $operation_type_l = l("operation_search_$type", 'listings');
            if ($operation_type_l === $operation_type) {
                return $type;
            }
        }
        return $this->Listings_model->get_operation_type_gid_default(true);
    }

    /**
     * Listings management
     *
     * @param string $operation_type_param operation type
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    protected function _list($operation_type_param = null, $order = 'default', $order_direction = 'DESC', $page = 1, array $filters = null)
    {
        $operation_types = $this->Listings_model->get_operation_types(true);
        $this->view->assign('operation_types', $operation_types);

        $operation_type = $this->filterOperationType($operation_type_param, $operation_types);
        $this->view->assign('current_operation_type', $operation_type);

        // TODO: IMPORTANT! listBlock and listBlockList are equal except for the return format.
        $view_mode = isset($_SESSION['listings_view_mode']) ? $_SESSION['listings_view_mode'] : 'list';
        if ($view_mode === 'list') {
            $this->view->assign('block', $this->listBlock($operation_type, $order, $order_direction, $page, false, false, $filters));
        } else {
            $this->view->assign('list_result', $this->listBlockList($operation_type, $order, $order_direction, $page, $filters));
        }

        if ($this->session->userdata('auth_type') != 'user') {
            $this->view->assign('is_guest', true);
        } else {
            $user_type = $this->session->userdata('user_type');
            $parent = $user_type . '-main-find-item';
            $this->Menu_model->breadcrumbs_set_parent($parent);
            $this->Menu_model->set_menu_active_item($user_type . '_main_menu', $parent);
        }
        $this->view->assign('use_rss', $this->pg_module->get_module_config('listings', 'use_rss'));
        $this->view->assign('use_poll_in_search', $this->pg_module->get_module_config('listings', 'use_poll_in_search'));
        $this->load->helper('seo');

        $search_action_data = $menu_action_data = $this->Listings_model->get_search_link_data();
        $method = !empty($search_action_data['keyword']) ? 'search' : 'index';
        $search_link = rewrite_link('listings', $method, $search_action_data);
        $this->pg_seo->set_seo_data($search_action_data);

        $menu_action_data['operation_type'] = '[operation_type]';
        $this->view->assign('menu_action_link', rewrite_link('listings', $method, $menu_action_data));

        $operation_types_links = array();
        foreach ($operation_types as $type) {
            $menu_action_data['operation_type'] = $type;
            $menu_action_data['operation_type_str'] = l("operation_search_$type", 'listings');
            $operation_types_links[$type] = array(
                'gid' => $type,
                'str' => $menu_action_data['operation_type_str'],
                'link' => $this->pg_seo->create_url('listings', $method, $menu_action_data),
            );
        }
        $this->view->assign('operation_types_links', $operation_types_links);
        $this->Menu_model->breadcrumbs_set_active(l('link_search_in_listings', 'listings'), $search_link);
        $this->Menu_model->breadcrumbs_set_active(l('link_search_results', 'listings'));

        $view_mode = isset($_SESSION['listings_view_mode']) ? $_SESSION['listings_view_mode'] : 'list';
        $this->view->render('listings_' . $view_mode, 'user', 'listings');
    }

    private function initSearchSettings($operation_type, array $raw_settings = array())
    {
        $current_settings = $raw_settings;
        if (!isset($current_settings['data'])) {
            $current_settings['data'] = array();
        }
        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'default';
        }
        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        if (!isset($current_settings['page'])) {
            $current_settings['page'] = 1;
        }

        if (!isset($current_settings['data']['type']) || $current_settings['data']['type'] != $operation_type) {
            $current_settings['data']['type'] = $operation_type;
            $current_settings['page'] = 1;

            //booking
            if (!in_array($operation_type, array('rent', 'lease'))) {
                if (isset($current_settings['data']['booking_date_start'])) {
                    unset($current_settings['data']['booking_date_start']);
                }
                if (isset($current_settings['data']['booking_date_end'])) {
                    unset($current_settings['data']['booking_date_end']);
                }
                if (isset($current_settings['data']['booking_guests'])) {
                    unset($current_settings['data']['booking_guests']);
                }
            }
            if (isset($current_settings['data']['field_editor_data'])) {
                unset($current_settings['data']['field_editor_data']);
            }
        }

        return $current_settings;
    }

    /**
     * Block of listings management
     *
     * @param string $operation_type operation type
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return string
     */
    private function listBlock($operation_type, $order = 'default', $order_direction = 'DESC', $page = 1, $order_user_filter = false, $get_short_list = false, array $search_settings = null)
    {
        if (!is_array($search_settings)) {
            $search_settings = array();
        }
        $current_settings = $this->initSearchSettings($operation_type, $search_settings);

        $filters = $current_settings['data'];
        $filters['active'] = '1';

        $order = trim(strip_tags($order));
        if (!empty($order)) {
            $current_settings['order'] = $order;
        }
        $order = $current_settings['order'];
        $this->view->assign('order', $order);

        $order_direction = strtoupper(trim(strip_tags($order_direction)));
        if (!empty($order_direction)) {
            $current_settings['order_direction'] = $order_direction;
        }
        $order_direction = $current_settings['order_direction'];
        $this->view->assign('order_direction', $order_direction);

        $page = intval($page);
        if (!empty($page)) {
            $current_settings['page'] = $page;
        }
        $page = $current_settings['page'];
        $this->view->assign('page', $page);

        if ($order_direction != 'ASC') {
            $order_direction = 'DESC';
        }

        $view_mode = isset($_SESSION['listings_view_mode']) ? $_SESSION['listings_view_mode'] : 'list';
        $items_on_page = $this->pg_module->get_module_config('start', 'index_items_per_page');
        $items_count = $this->Listings_model->get_listings_count($filters);
        $this->load->helper('sort_order');
        $current_settings['page'] = get_exists_page_number($page, $items_count, $items_on_page);

        $_SESSION['listings_pagination'] = array(
            'order' => $order,
            'order_direction' => $order_direction,
            'page' => $page,
            'data' => $filters,
        );
        $_SESSION['listings_search'] = $current_settings;

        if (!empty($filters['id_country'])) {
            if (!empty($filters['id_region'])) {
                if (!empty($filters['id_city'])) {
                    $this->view->assign('location_filter', 'city');
                } else {
                    $this->view->assign('location_filter', 'region');
                }
            } else {
                $this->view->assign('location_filter', 'country');
            }
        }

        $this->load->helper('seo');

        $search_action_data = $menu_action_data = $this->Listings_model->get_search_link_data();

        unset($search_action_data['order']);
        unset($search_action_data['order_direction']);
        unset($search_action_data['page']);

        $method = !empty($search_action_data['keyword']) ? 'search' : 'index';
        $url = rewrite_link('listings', $method, $search_action_data);

        // sorting
        $sort_data = array(
            'url' => $url,
            'order' => $current_settings['order'],
            'direction' => $current_settings['order_direction'],
            'links' => array('default' => l('field_default_sorter', 'listings')),
        );
        if ($this->pg_module->is_module_active('reviews')) {
            $this->load->helper('reviews');
            $review_link = get_reviews_sorter();
            if (is_array($review_link)) {
                $sort_data['links'] = array_merge($sort_data['links'], $review_link);
            }
        }
        $sort_data['links']['views'] = l('field_views', 'listings');
        $sort_data['links']['price'] = l('field_price', 'listings');
        $sort_data['links']['open'] = l('field_date_open', 'listings');
        $sort_data['links']['modified'] = l('field_date_modified', 'listings');
        $this->view->assign('sort_data', $sort_data);

        $use_map_in_search = true;
        if ($view_mode == 'list') {
            $use_map_in_search = $this->pg_module->get_module_config('listings', 'use_map_in_search');
        }
        $this->view->assign('use_map_in_search', $use_map_in_search);

        $order_user_filter = true;

        $use_leader = false;
        $markers = array();
        if ($items_count > 0) {
            $order_array = array();
            switch ($order) {
                case 'default':
                    if (!empty($filters['id_city'])) {
                        $order_array['is_lift_up_city'] = $order_direction;
                    }
                    if (!empty($filters['id_region'])) {
                        $order_array['is_lift_up_region'] = $order_direction;
                    }
                    if (!empty($filters['id_country'])) {
                        $order_array['is_lift_up_country'] = $order_direction;
                    }
                    $order_array['is_lift_up'] = $order_direction;

                    if (!empty($filters['keyword'])) {
                        $boolean_mode = isset($filters['keyword_mode']);
                        $keyword = trim($filters['keyword']);
                        $this->load->model('Field_editor_model');
                        $property_type_gid = $this->Listings_model->get_field_editor_type(
                            array('type' => $filters['type'], 'id_category' => $filters['id_category'])
                        );
                        $this->Field_editor_model->initialize($property_type_gid);
                        $temp_criteria = $this->Field_editor_model->return_fulltext_criteria($keyword, $boolean_mode);
                        $field_editor_table = constant('LISTINGS_' . strtoupper($property_type_gid) . '_TABLE');
                        $fe_table_name = substr($field_editor_table, strlen(DB_PREFIX));
                        if (!empty($temp_criteria[$fe_table_name]['field'])) {
                            $order_array[str_replace(
                                    array('search_field', ' AS score'), array($field_editor_table . '.search_field', ''), $temp_criteria[$fe_table_name]['field']
                                )] = 'DESC';
                        }
                    }
                    $order_user_filter = false;
                    $order_array['id'] = $order_direction;
                    $use_leader = true;
                    break;
                case 'price':
                    if (isset($filters['price_range'])) {
                        $order_array['price_negotiated'] = $order_direction === 'ASC' ? 'DESC' : 'ASC';
                    }
                    $order_array['price_sorting'] = $order_direction;
                    break;
                case 'open':
                    $order_array['date_open'] = $order_direction;
                    break;
                case 'modified':
                    $order_array['date_modified'] = $order_direction;
                    $order_array['id'] = $order_direction;
                    break;
                default:
                    $order_array[$order] = $order_direction;
                    break;
            }

            $this->Listings_model->set_format_settings(['get_saved_listings' => true, 'get_description' => true,]);
            $listings = $this->Listings_model->get_listings_list(
                $filters, $current_settings['page'], $items_on_page, $order_array, true, $order_user_filter, $order_direction
            );
            $this->Listings_model->set_format_settings(['get_saved_listings' => false, 'get_description' => false,]);
            $this->view->assign('listings', $listings);
            if ($this->pg_module->is_module_active('users_services')) {
                $this->load->model('Users_services_model');
                $show_logo = !$this->Users_services_model->is_active_show_logo_services();
                $this->view->assign('show_logo', $show_logo);
            } else {
                $this->view->assign('show_logo', true);
            }

            if ($use_map_in_search) {
                $this->view->assign('view_mode', $view_mode);
                foreach ($listings as $listing) {
                    if ((float) $listing['lat'] == 0 && (float) $listing['lon'] == 0) {
                        continue;
                    }
                    $this->view->assign('listing', $listing);
                    $info = $this->view->fetch('listing_map_block', 'user', 'listings');
                    $markers[] = array(
                        'gid' => $listing['id'],
                        'country' => $listing['country'],
                        'region' => $listing['region'],
                        'city' => $listing['city'],
                        'address' => $listing['address'],
                        'postal_code' => $listing['zip'],
                        'lat' => (float) $listing['lat'],
                        'lon' => (float) $listing['lon'],
                        'info' => $info,
                    );
                }
            }
        }
        $this->view->assign('markers', $markers);
        if ($use_map_in_search) {
            $current_language = $this->pg_language->get_lang_by_id($this->pg_language->current_lang_id);
            $view_settings = array('lang' => $current_language['code']);
            $this->view->assign('map_settings', $view_settings);
        }

        $current_search_data = $this->Listings_model->get_search_data();
        $this->view->assign('current_search_data', $current_search_data);

        $search_filters_block = $this->view->fetch('search_filters_block', 'user', 'listings');
        $this->view->assign('search_filters_block', addslashes($search_filters_block));

        $this->load->helper('navigation');
        $page_data = get_user_pages_data(
            $url . '/' . $order . '/' . $order_direction . '/', $items_count, $items_on_page, $current_settings['page'], 'briefPage'
        );
        $page_data['use_leader'] = $use_leader;
        $this->view->assign('page_data', $page_data);
        $this->view->assign('tstamp', time());
        $this->view->assign('show_user_info', 1);
        $this->view->assign('animate_available', 1);
        if ($get_short_list) {
            return json_encode(
                array(
                    'html' => $this->view->fetch('listings_block_short_list'),
                    'markers' => $markers,
                )
            );
        } else {
            $this->view->assign('is_list_block', 1);
            return $this->view->fetch('listings_block_' . $view_mode);
        }
    }

    public function ajax_create_video_content()
    {
        $post_data = $this->input->post('data', true);
        
        $format_settings['use_format'] = true;
        $format_settings['get_video'] = true;
        $this->Listings_model->set_format_settings($format_settings);
        $data = $this->Listings_model->get_listing_by_id($post_data['id']);
        
        $this->view->assign('data', $data);
        
        $return['video_content'] = $this->view->fetch("listing_form_video", null, "listings");
        $this->view->assign($return);
        $this->view->render();
    }

    public function ajax_create_file_content($is_edit_form = false)
    {
        if (!$is_edit_form) {
            $post_data = $this->input->post('data', true);
            $data['id'] = $post_data['listing_id'];
            $data['listing_file_name'] = $post_data['listing_file_name'];
            $data['date_add'] = $post_data['listing_file_date'];
            $data['orig_name'] = $post_data['listing_file'];
            $this->view->assign('data', $data);
            $return['file_content'] = $this->view->fetch("listing_form_file", null, "listings");
        } else {
            $this->view->assign('edit_data', $this->input->post('data', true));
            $this->view->assign('maxcount', 60);
            $return['file_content'] = $this->view->fetch("file_description_block", null, "upload_gallery");
        }
        $this->view->assign($return);
        $this->view->render();
    }

    public function ajax_save_video($listing_id = null)
    {
        $listing_id = (int)$listing_id;
        
        if (empty($listing_id)) {
            show_404();
        }
        
        $return = array();
        
        $listing_video_embed = $this->input->post('listing_video_embed');
        
        $this->load->model('listings/models/Listings_uploads_model');
        $validate_data = $this->Listings_uploads_model->save_video($listing_id, 'listing_video', $listing_video_embed, true);
        if (!empty($validate_data['errors'])) {
            $return['error'] = $validate_data['errors'];
        } elseif (!empty($validate_data['success'])) {
            $return['success'] = l('success_video_uploaded', 'listings');
            $format_settings['use_format'] = true;
            $format_settings['get_video'] = true;
            $this->Listings_model->set_format_settings($format_settings);
            $data = $this->Listings_model->get_listing_by_id($listing_id);
            $return['video'] = $data;
            $return['video_block'] = $this->view->fetch("listing_form_video", null, "listings");
        }
        
        $this->view->assign($return);
        $this->view->render();
    }

    public function ajax_delete_video($listing_id = null)
    {
        $return = array();
        if ($listing_id) {
            $this->load->model('listings/models/Listings_uploads_model');
            $this->Listings_uploads_model->delete_video($listing_id);
            $return['success'] = l('success_video_deleted', 'listings');
        }
        $this->view->assign($return);
        $this->view->render();
    }

    public function ajax_save_file($listing_id = null)
    {
        $return = array();
        if ($listing_id) {
            $post_data = $this->input->post('data', true);
            $this->load->model('listings/models/Listings_uploads_model');
            $validate_data = $this->Listings_uploads_model->save_file($listing_id, 'listing_file', $post_data, true);
            if (!empty($validate_data['errors'])) {
                $return['error'] = $validate_data['errors'];
            } elseif (isset($validate_data['data']['file'])) {
                $return['success'] = l('success_file_uploaded', 'listings');
                $validate_data['file']['listing_id'] = $listing_id;
                $return['data'] = $validate_data['file'];
            } elseif ($validate_data['file']) {
                $validate_data['file']['listing_id'] = $listing_id;
                $return['data'] = $validate_data['file'];
            }
        }
        $this->view->assign($return);
        $this->view->render();
    }

    public function ajax_delete_file($listing_id = null)
    {
        $return = array();
        if ($listing_id) {
            $format_settings['use_format'] = true;
            $format_settings['get_file'] = true;
            $this->Listings_model->set_format_settings($format_settings);
            $data = $this->Listings_model->get_listing_by_id($listing_id);
            $this->load->model('File_uploads_model');
            $this->File_uploads_model->delete_upload($this->Listings_uploads_model->file_config_id, $data['prefix'], $data['listing_file']);
            $listing_data = array('listing_file' => '', 'listing_file_name' => '', 'listing_file_comment' => '');
            $this->Listings_model->save_listing($listing_id, $listing_data, true);
            $return['success'] = l('success_file_deleted', 'listings');
        }
        $this->view->assign($return);
        $this->view->render();
    }

    /**
     * TODO: Remove this method. It's almost identical to listBlock().
     *
     * returns listings_block_list template all the time
     *
     * @param string $operation_type operation type
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return string
     */
    private function listBlockList($operation_type, $order = 'default', $order_direction = 'DESC', $page = 1, array $search_settings = null)
    {
        if (!is_array($search_settings)) {
            $search_settings = array();
        }
        $current_settings = $this->initSearchSettings($operation_type, $search_settings);

        $filters = $current_settings['data'];
        $filters['active'] = '1';

        $order = trim(strip_tags($order));
        if (!empty($order)) {
            $current_settings['order'] = $order;
        }
        $order = $current_settings['order'];
        $this->view->assign('order', $order);

        $order_direction = strtoupper(trim(strip_tags($order_direction)));
        if (!empty($order_direction)) {
            $current_settings['order_direction'] = $order_direction;
        }
        $order_direction = $current_settings['order_direction'];
        $this->view->assign('order_direction', $order_direction);

        if (!empty($page)) {
            $current_settings['page'] = $page;
        } else {
            $page = $current_settings['page'];
        }
        $this->view->assign('page', $page);

        if ($order_direction != 'ASC') {
            $order_direction = 'DESC';
        }

        $view_mode = isset($_SESSION['listings_view_mode']) ? $_SESSION['listings_view_mode'] : 'list';
        $items_on_page = $this->pg_module->get_module_config('start', 'index_items_per_page');
        $items_count = $this->Listings_model->get_listings_count($filters);
        $this->load->helper('sort_order');
        $current_settings['page'] = get_exists_page_number($page, $items_count, $items_on_page);

        $_SESSION['listings_pagination'] = array(
            'order' => $order,
            'order_direction' => $order_direction,
            'page' => $page,
            'data' => $filters,
        );
        $_SESSION['listings_search'] = $current_settings;

        if (!empty($filters['id_country'])) {
            if (!empty($filters['id_region'])) {
                if (!empty($filters['id_city'])) {
                    $location_filter = 'city';
                } else {
                    $location_filter = 'region';
                }
            } else {
                $location_filter = 'country';
            }
        } else {
            $location_filter = null;
        }
        $this->view->assign('location_filter', $location_filter);

        $this->load->helper('seo');
        $search_action_data = $menu_action_data = $this->Listings_model->get_search_link_data();

        unset($search_action_data['order']);
        unset($search_action_data['order_direction']);
        unset($search_action_data['page']);

        $method = !empty($search_action_data['keyword']) ? 'search' : 'index';
        $url = rewrite_link('listings', $method, $search_action_data);

        // sorting
        $sort_data = array(
            'url' => $url,
            'order' => $current_settings['order'],
            'direction' => $current_settings['order_direction'],
            'links' => array('default' => l('field_default_sorter', 'listings')),
        );
        if ($this->pg_module->is_module_active('reviews')) {
            $this->load->helper('reviews');
            $review_link = get_reviews_sorter();
            if (is_array($review_link)) {
                $sort_data['links'] = array_merge($sort_data['links'], $review_link);
            }
        }
        $sort_data['links']['views'] = l('field_views', 'listings');
        $sort_data['links']['price'] = l('field_price', 'listings');
        $sort_data['links']['open'] = l('field_date_open', 'listings');
        $sort_data['links']['modified'] = l('field_date_modified', 'listings');
        $this->view->assign('sort_data', $sort_data);

        $use_leader = false;
        $use_map_in_search = $view_mode == 'map';
        if ($items_count > 0) {
            $order_array = array();
            switch ($order) {
                case 'default':
                    if (!empty($filters['id_city'])) {
                        $order_array['is_lift_up_city'] = $order_direction;
                    }
                    if (!empty($filters['id_region'])) {
                        $order_array['is_lift_up_region'] = $order_direction;
                    }
                    if (!empty($filters['id_country'])) {
                        $order_array['is_lift_up_country'] = $order_direction;
                    }
                    $order_array['is_lift_up'] = $order_direction;

                    if (!empty($filters['keyword'])) {
                        $boolean_mode = isset($filters['keyword_mode']);
                        $keyword = trim($filters['keyword']);
                        $this->load->model('Field_editor_model');
                        $property_type_gid = $this->Listings_model->get_field_editor_type(
                            array('type' => $filters['type'], 'id_category' => $filters['id_category'])
                        );
                        $this->Field_editor_model->initialize($property_type_gid);
                        $temp_criteria = $this->Field_editor_model->return_fulltext_criteria($keyword, $boolean_mode);
                        $field_editor_table = constant('LISTINGS_' . strtoupper($property_type_gid) . '_TABLE');
                        $fe_table_name = substr($field_editor_table, strlen(DB_PREFIX));
                        if (!empty($temp_criteria[$fe_table_name]['field'])) {
                            $order_array[str_replace(
                                    array('search_field', ' AS score'), array($field_editor_table . '.search_field', ''), $temp_criteria[$fe_table_name]['field']
                                )] = 'DESC';
                        }
                    }
                    $order_array['id'] = $order_direction;
                    $use_leader = true;
                    break;
                case 'price':
                    if (isset($filters['price_range'])) {
                        $order_array['price_negotiated'] = $order_direction === 'ASC' ? 'DESC' : 'ASC';
                    }
                    $order_array['price_sorting'] = $order_direction;
                    break;
                case 'open':
                    $order_array['date_open'] = $order_direction;
                    break;
                case 'modified':
                    $order_array['date_modified'] = $order_direction;
                    $order_array['id'] = $order_direction;
                    break;
                default:
                    $order_array[$order] = $order_direction;
                    break;
            }

            $this->Listings_model->set_format_settings(['get_saved_listings' => true, 'get_description' => true,]);
            $listings = $this->Listings_model->get_listings_list(
                $filters, $current_settings['page'], $items_on_page, $order_array
            );
            $this->Listings_model->set_format_settings(['get_saved_listings' => false, 'get_description' => false,]);

            $this->view->assign('listings', $listings);
            if ($this->pg_module->is_module_active('users_services')) {
                $this->load->model('Users_services_model');
                $show_logo = !$this->Users_services_model->is_active_show_logo_services();
                $this->view->assign('show_logo', $show_logo);
            } else {
                $this->view->assign('show_logo', true);
            }

            if ($use_map_in_search) {
                $markers = array();
                $this->view->assign('view_mode', $view_mode);
                foreach ($listings as $listing) {
                    if ((float) $listing['lat'] == 0 && (float) $listing['lon'] == 0) {
                        continue;
                    }
                    $this->view->assign('listing', $listing);
                    $info = $this->view->fetch('listing_map_block', 'user', 'listings');
                    $markers[] = array(
                        'gid' => $listing['id'],
                        'country' => $listing['country'],
                        'region' => $listing['region'],
                        'city' => $listing['city'],
                        'address' => $listing['address'],
                        'postal_code' => $listing['zip'],
                        'lat' => (float) $listing['lat'],
                        'lon' => (float) $listing['lon'],
                        'info' => $info,
                    );
                }
                $this->view->assign('markers', $markers);
            }
        }

        if ($use_map_in_search) {
            $current_language = $this->pg_language->get_lang_by_id($this->pg_language->current_lang_id);
            $view_settings = array('lang' => $current_language['code']);
            $this->view->assign('map_settings', $view_settings);
        }

        $this->view->assign('current_search_data', $this->Listings_model->get_search_data());
        $this->view->assign('search_filters_block', addslashes($this->view->fetch('search_filters_block', 'user', 'listings')));

        $this->load->helper('navigation');
        $page_data = get_user_pages_data($url . '/' . $order . '/' . $order_direction . '/', $items_count, $items_on_page, $current_settings['page'], 'briefPage');
        $page_data['use_leader'] = $use_leader;

        $this->view->assign('page_data', $page_data);
        $this->view->assign('tstamp', time());
        $this->view->assign('show_user_info', 1);
        $this->view->assign('animate_available', 1);
        $this->view->assign('hide_bottom_pages', 'map' === $view_mode);
        return $this->view->fetch('listings_block_short_list');
    }

    /**
     * Listings catalogue
     *
     * @param string $operation_type operation type
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function index($operation_type = null, $order = 'default', $order_direction = 'DESC', $page = 1)
    {
        $search_settings = isset($_SESSION['listings_search']) ? $_SESSION['listings_search'] : array();
        /*if ('sale' === $operation_type) {
            unset($search_settings['data']);
        }*/

        $this->_list($operation_type, $order, $order_direction, $page, $search_settings);
    }

    /**
     * Listings catalogue by ajax
     *
     * @param string $operation_type operation type
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function ajax_index($operation_type, $order = null, $order_direction = null, $page = null)
    {
        $view_mode = isset($_SESSION['listings_view_mode']) ? $_SESSION['listings_view_mode'] : 'list';
        $get_short_list = $this->input->post('get_short_list') == '1';
        if ($view_mode === 'list') {
            $use_map_in_search = $this->pg_module->get_module_config('listings', 'use_map_in_search');
        } else {
            $use_map_in_search = true;
        }
        $this->view->assign('update_map', $use_map_in_search);
        $this->view->assign('update_operation_type', $operation_type);
        $this->view->assign('update_search_block', 1);
        $this->view->assign('update_rss_link', 1);

        $search_settings = isset($_SESSION['listings_search']) ? $_SESSION['listings_search'] : array();

        $this->view->output($this->listBlock($operation_type, $order, $order_direction, $page, true, $get_short_list, $search_settings));

        $this->view->render();
    }

    public function ajax_update_map()
    {
        $params["map_id"] = $this->input->post('map_id', true);
        $params["markers"] = $this->input->post('markers', true);
        $this->load->helper('geomap');
        update_default_map($params);
    }

    /**
     * Listings catalogue by category
     *
     * @param string $category_id category identifier
     * @param integer $property_type property type
     * @param string $operation_type operation type
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function category($category_id, $property_type = null, $operation_type = null, $order = 'default', $order_direction = 'DESC', $page = 1)
    {
        $categories_ids = $this->Listings_model->get_category_ids();
        if (!in_array($category_id, $categories_ids)) {
            show_404();
        }

        if ($property_type) {
            $this->load->model('Properties_model');
            $property_items = $this->Properties_model->get_properties($category_id, $this->pg_language->current_lang_id);
            if (!array_key_exists($property_type, $property_items)) {
                show_404();
            }
        }

        $_SESSION['listings_search']['data'] = array('id_category' => $category_id);
        if ($property_type) {
            $_SESSION['listings_search']['data']['property_type'] = $property_type;
        }
        $this->_list($operation_type, $order, $order_direction, $page, $_SESSION['listings_search']);
    }

    /**
     * Listings catalogue by open houses
     *
     * @param string $operation_type operation type
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function open_house($operation_type = null, $order = 'default', $order_direction = 'DESC', $page = 1)
    {
        $_SESSION['listings_search']['data'] = array('by_open_house' => '1');
        $this->_list($operation_type, $order, $order_direction, $page, $_SESSION['listings_search']);
    }

    /**
     * Listings catalogue by owners (private persons)
     *
     * @param string $operation_type operation type
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function privates($operation_type = null, $order = 'default', $order_direction = 'DESC', $page = 1)
    {
        $_SESSION['listings_search']['data'] = array('by_private' => '1');
        $this->_list($operation_type, $order, $order_direction, $page, $_SESSION['listings_search']);
    }

    /**
     * Listings catalogue by agents (agency + agent)
     *
     * @param string $operation_type operation type
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function agents($operation_type = null, $order = 'default', $order_direction = 'DESC', $page = 1)
    {
        $_SESSION['listings_search']['data'] = array('by_agents' => '1');
        $this->_list($operation_type, $order, $order_direction, $page, $_SESSION['listings_search']);
    }

    /**
     * Listings catalogue by location
     *
     * @param string $country country code
     * @param integer $region region identifier
     * @param integer $city city identifier
     * @param string $operation_type operation type
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function location($country, $region = null, $city = null, $operation_type = null, $order = 'default', $order_direction = 'DESC', $page = 1)
    {
        $this->load->model('Countries_model');
        if (!empty($city)) {
            $city_obj = $this->Countries_model->get_city($city);
            if (!empty($city_obj)) {
                $_SESSION['listings_search']['data'] = array(
                    'id_country' => $city_obj['country_code'],
                    'id_region' => $city_obj['id_region'],
                    'id_city' => $city,
                );
            }
        } elseif (!empty($region)) {
            $region_obj = $this->Countries_model->get_region($region);
            if (!empty($region_obj)) {
                $_SESSION['listings_search']['data'] = array(
                    'id_country' => $region_obj['country_code'],
                    'id_region' => $region,
                );
            }
        } elseif (!empty($country)) {
            $_SESSION['listings_search']['data'] = array(
                'id_country' => $country,
            );
        } else {
            $_SESSION['listings_search']['data'] = array();
        }

        $this->_list($operation_type, $order, $order_direction, $page, $_SESSION['listings_search']);
    }

    /**
     * Listings catalogue by discounts
     *
     * @param string $operation_type operation type
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function discount($operation_type = null, $order = 'default', $order_direction = 'DESC', $page = 1)
    {
        $_SESSION['listings_search']['data'] = array('by_discount' => '1');
        $this->_list($operation_type, $order, $order_direction, $page, $_SESSION['listings_search']);
    }

    /**
     * Show listing profile
     *
     * @param integer $listing_id listing identifier
     * @param string $section_gid section GUID
     * @param boolean $pdf pdf-version
     * @return void
     */
    public function view($listing_id, $section_gid = 'overview', $pdf = false)
    {
        if (!$listing_id) {
            show_404();
            return;
        }

        if (!isset($this->Listings_model->display_sections['top'][$section_gid]) &&
            !isset($this->Listings_model->display_sections['bottom'][$section_gid])) {
            $section_gid = 'overview';
        }

        $this->Listings_model->set_format_settings(array(
            'get_user' => false,
            'get_saved_listings' => true,
        ));

        if ($pdf) {
            $ajax_sections = array('print' => 1);
            $this->Listings_model->set_format_settings(array(
                'get_description' => true,
                'get_qr_code' => true,
                'get_photos' => true,
                'get_booking' => true,
            ));
        } else {
            $this->Listings_model->set_format_settings('get_views', true);

            $ajax_sections = array(
                'gallery' => 0,
                'virtual_tour' => 0,
                'file' => 0,
                'video' => 0,
                'overview' => 0,
                'calendar' => 0,
            );

            $map_only_load_scripts = 0;
            $panorama_only_load_scripts = 1;

            switch ($section_gid) {
                case 'gallery':
                    break;
                case 'virtual_tour':
                    $this->Listings_model->set_format_settings('get_virtual_tours', true);
                    $ajax_sections['virtual_tour'] = 1;
                    break;
                case 'file':
                    $this->Listings_model->set_format_settings('get_file', true);
                    $ajax_sections['file'] = 1;
                    break;
                case 'video':
                    $this->Listings_model->set_format_settings('get_video', true);
                    $ajax_sections['video'] = 1;
                    break;
                case 'map':
                    $map_only_load_scripts = 0;
                    break;
                case 'panorama':
                    $panorama_only_load_scripts = 0;
                    break;
                case 'overview':
                    $this->Listings_model->set_format_settings(array('get_description' => true, 'get_qr_code' => true));
                    $ajax_sections['overview'] = 1;
                    break;
            }

            if (!array_key_exists($section_gid, $this->Listings_model->display_sections['top'])) {
                $this->Listings_model->set_format_settings('get_photos', true);
                $ajax_sections['gallery'] = 1;
            }

            if (!array_key_exists($section_gid, $this->Listings_model->display_sections['bottom'])) {
                $this->Listings_model->set_format_settings(array('get_description' => true, 'get_qr_code' => true));
                $ajax_sections['overview'] = 1;
            }

            $this->Listings_model->set_format_settings('get_photos', true);
            $this->Listings_model->set_format_settings('get_virtual_tours', true);
            $this->Listings_model->set_format_settings('get_video', true);
            $ajax_sections['gallery'] = 1;

            $this->view->assign('map_only_load_scripts', $map_only_load_scripts);
            $this->view->assign('panorama_only_load_scripts', $panorama_only_load_scripts);
        }

        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        if (!$listing) {
            show_404();
            return;
        }
        $this->Listings_model->set_format_settings(array(
            'get_user' => true,
            'get_description' => false,
            'get_qr_code' => false,
            'get_photos' => false,
            'get_virtual_tours' => false,
            'get_saved_listings' => false,
            'get_file' => false,
            'get_video' => false,
            'get_booking' => false,
            'get_views' => false,
        ));

        $is_users_installed = $this->pg_module->is_module_active('users');
        if ($is_users_installed) {
            $this->load->model('Users_model');
            $this->Users_model->set_format_settings(array(
                'get_contact' => true,
                'get_company' => true,
                'get_field_editor' => true,
                'get_microdata' => true,
            ));
            $listing['user'] = $this->Users_model->get_user_by_id($listing['id_user']);
            $this->Users_model->set_format_settings(array(
                'get_contact' => false,
                'get_company' => false,
                'get_field_editor' => false,
                'get_microdata' => false,
            ));
        } else {
            $this->load->model('Start_model');
            $listing['user'] = $this->Start_model->getUserDefaultOnSite();
        }

        $this->view->assign('listing', $listing);

        $is_owner = 0;
        if ($this->session->userdata('auth_type') === 'user') {
            $user_id = $this->session->userdata('user_id');
            $this->view->assign('user_id', $user_id);
            if ($user_id == $listing['id_user']) {
                $is_owner = 1;
            }
        }
        $this->view->assign('is_listing_owner', $is_owner);

        $display_sections = $this->Listings_model->get_display_sections($listing);
        foreach ($display_sections as $sections) {
            foreach ($sections as $display_section_gid => $used) {
                if (isset($ajax_sections[$display_section_gid])) {
                    $used = $ajax_sections[$display_section_gid];
                }
                if (!$used) {
                    continue;
                }
                ob_start();
                $this->_get_section_html($listing, $display_section_gid);
                $listing_content[$display_section_gid] = ob_get_contents();
                ob_end_clean();
            }
        }

        $this->view->assign('display_sections', $display_sections);
        $this->view->assign('listing_content', $listing_content);
        $this->view->assign('section_gid', $section_gid);

        $slider_auto = $this->pg_module->get_module_config('listings', 'slider_auto');
        $slider_rotation = $this->pg_module->get_module_config('listings', 'slider_rotation');

        $page_data = array(
            'slider_auto' => $slider_auto,
            'rotation' => $slider_rotation,
        );
        $this->view->assign('page_data', $page_data);

        $is_guest = $this->session->userdata('auth_type') !== 'user';
        $this->view->assign('is_guest', $is_guest);

        // search seo
        $this->load->helper('seo');
        $search_action_data = $this->Listings_model->get_search_link_data();
        $search_link = rewrite_link('listings', $search_action_data['keyword'] ? 'search' : 'index', $search_action_data);

        $this->Menu_model->breadcrumbs_set_active(l('link_search_in_listings', 'listings'), $search_link);
        $this->Menu_model->breadcrumbs_set_active($listing['output_name']);

        $this->load->model('listings/models/Listings_visitor_model');
        $this->Listings_visitor_model->check_views_count($listing['id_user'], $listing_id);

        if ($pdf) {
            $this->view->assign('pdf', 1);
            $this->pg_theme->print_type = 'pdf';
            $tpl_output = $this->view->fetch('view_listing');
            $this->Listings_model->get_pdf_file($tpl_output, $listing);
        } else {
            $seo_data = array(
                'id' => $listing_id,
                'section' => 'overview',
                'operation_type' => $listing['operation_type'],
                'property' => $listing['property'],
                'country' => $listing['country'],
                'region' => $listing['region'],
                'city' => $listing['city'],
                'address' => $listing['address'],
            );

            $lang_canonical = true;

            if ($this->pg_module->is_module_active('seo')) {
                $lang_canonical = $this->pg_module->get_module_config('seo', 'lang_canonical');
            }

            if ($lang_canonical) {
                $lang_id = $this->pg_language->get_default_lang_id();
                if ($lang_id != $this->pg_language->current_lang_id) {
                    $this->load->model('Properties_model');
                    $property_items = $this->Properties_model->get_all_properties('property_types', $lang_id);
                    $seo_data['property'] = $property_items[$listing['id_category']][$listing['property_type']];

                    $this->load->model('Countries_model');
                    $country = $this->Countries_model->get_country($listing['id_country'], $lang_id);
                    $seo_data['country'] = !empty($country) ? $country['name'] : '';
                    $region = $this->Countries_model->get_region($listing['id_region'], $lang_id);
                    $seo_data['region'] = !empty($region) ? $region['name'] : '';
                    $city = $this->Countries_model->get_city($listing['id_city'], $lang_id);
                    $seo_data['city'] = !empty($city) ? $city['name'] : '';
                }
            }

            if ($listing['id_seo_settings']) {
                $this->load->model('Seo_model');
                $seo_settings = $this->Seo_model->parse_seo_tags($listing['id_seo_settings']);
                $seo_settings['canonical'] = rewrite_link('listings', 'view', $seo_data, false, null, $lang_canonical);
                $seo_settings['og_image'] = $listing['media']['photo']['thumbs']['big'];
                $this->pg_seo->set_seo_tags($seo_settings);
            } else {
                $this->load->helper('listings');
                $seo_settings = $listing;
                $seo_settings['canonical'] = rewrite_link('listings', 'view', $seo_data, false, null, $lang_canonical);
                $seo_settings['image'] = $listing['media']['photo']['thumbs']['big'];
                $seo_settings['section'] = $section_gid;
                $seo_settings['price'] = trim(strip_tags(listing_price_block(array('data' => $listing, 'template' => 'small'))));
                $seo_settings['sold'] = $listing['sold_output'];
                $this->pg_seo->set_seo_data($seo_settings);
            }

            $text_listing_id = l('field_id', 'listings');
            $text_listing_message = str_replace(
                '[listing]', $listing['output_name'] . ' (' . $text_listing_id . ' ' . $listing['id'] . ')', l('text_listing_message', 'listings')
            );
            $this->view->assign('user_contact_message', l('text_user_message', 'users'));
            $this->view->assign('user_contact_message', $text_listing_message);
            $this->view->render('view_listing', 'user', 'listings');
        }
    }

    private function prepareGallerySection($listing)
    {
        $total_count = $listing['photo_count'] + $listing['virtual_tour_count'];
        if (!empty($listing['listing_video'])) {
            $total_count++;
        }
        $result['total_count'] = $total_count;
        $result['content_types'] = ['photo', 'virtual_tour', 'video'];
        $this->view->assign('gallery', $result);
    }

    /**
     * Section content of listing profile
     *
     * @param array $data listing data
     * @param string $section_gid section GUID
     * @return void
     */
    protected function _get_section_html($data, $section_gid = 'overview')
    {
        $this->view->assign('section_gid', $section_gid);
        $this->view->assign('data', $data);
        switch ($section_gid) {
            case 'overview':
                $this->load->model('Field_editor_model');
                $property_type_gid = $data['field_editor_type'];
                $this->Field_editor_model->initialize($property_type_gid);
                $property_type_sections = $this->Field_editor_model->get_section_list();
                $this->view->assign('sections_data', array_values($property_type_sections));
                $this->view->assign('sections_data_count', count($property_type_sections));
                break;
            case 'gallery':
                $this->prepareGallerySection($data);
            case 'virtual_tour':
                $page_data = array(
                    'rand' => rand(100000, 999999),
                    'visible' => 7,
                );
                $this->view->assign('page_data', $page_data);
                break;
            case 'map':
            case 'panorama':
                $markers = array(
                    array(
                        'gid' => $data['id'],
                        'country' => $data['country'],
                        'region' => $data['region'],
                        'city' => $data['city'],
                        'address' => $data['address'],
                        'postal_code' => $data['zip'],
                        'lat' => (float) $data['lat'],
                        'lon' => (float) $data['lon'],
                        'info' => ($data['location'] ? $data['location'] : ''),
                        'dragging' => false,
                    ),
                );
                $this->view->assign('markers', $markers);

                $current_language = $this->pg_language->get_lang_by_id($this->pg_language->current_lang_id);
                $view_settings = array(
                    'width' => '630',
                    'height' => '250',
                    'class' => ($section_gid === 'panorama' ? 'hide' : ''),
                    'lang' => $current_language['code'],
                    'panorama_disable' => $section_gid !== 'panorama',
                    'lat' => (float) $data['lat'],
                    'lon' => (float) $data['lon'],
                );
                $this->view->assign('map_settings', $view_settings);
                break;
        }

        $this->view->render('view_listing_content');
    }

    /**
     * Section Sontent of listing profile by ajax
     *
     * @param integer $listing_id listing identifier
     * @param string $section_gid section GUID
     * @return void
     */
    public function ajax_get_section($listing_id, $section_gid)
    {
        $this->load->model('Field_editor_model');

        switch ($section_gid) {
            case 'overview':
                $this->Listings_model->set_format_settings(array('get_description' => true, 'get_qr_code' => true));
                break;
            case 'gallery':
                $this->Listings_model->set_format_settings('get_photos', true);
                break;
            case 'virtual_tour':
                $this->Listings_model->set_format_settings('get_virtual_tours', true);
                break;
            case 'map':
                $this->view->assign('map_only_load_content', 1);
                break;
            case 'panorama':
                $this->view->assign('panorama_only_load_content', 1);
                break;
            case 'file':
                $this->Listings_model->set_format_settings('get_file', true);
                break;
            case 'video':
                $this->Listings_model->set_format_settings('get_video', true);
                break;
            case 'calendar':
                $this->Listings_model->set_format_settings('get_booking', true);
                break;
            case 'print':
                $this->Listings_model->set_format_settings('get_photos', true);
                break;
        }
        $this->view->assign('ajax', 1);
        $this->_get_section_html(
            $this->Listings_model->get_listing_by_id($listing_id), $section_gid
        );
    }

    /**
     * Pdf version of listing
     *
     * @param integer $listing_id listing identifier
     * @return string
     */
    public function pdf($listing_id)
    {
        return $this->view($listing_id, 'overview', true);
    }

    /**
     * Find similar listings
     *
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function similar_listing($listing_id)
    {
        if ($listing_id) {
            $this->Listings_model->set_format_settings('use_format', false);
            $listing = $this->Listings_model->get_listing_by_id($listing_id);
            $this->Listings_model->set_format_settings('use_format', true);
            $operation_type = $this->Listings_model->getOperationTypeById($listing['id_type']);
            if ($operation_type === Listings_model::OPERATION_SALE && $listing['sold']) {
                $operation_type = Listings_model::OPERATION_SOLD;
            }
            $_SESSION['listings_search']['data'] = array(
                'type' => $operation_type,
                'id_country' => $listing['id_country'],
                'id_region' => $listing['id_region'],
                'id_city' => $listing['id_city'],
                'id_category' => $listing['id_category'],
                'property_type' => $listing['property_type'],
                'exclude_id' => $listing_id,
            );
        }
        $this->search();
    }

    protected function getServicesData()
    {
        if ($this->pg_module->is_module_active('users_services')) {
            $this->load->model('Users_services_model');
            $use_services = $this->Users_services_model->is_active_post_services();
        } else {
            $use_services = false;
        }

        $user_id = $this->session->userdata('user_id');
        $return = array('public_listing_data' => array(), 'free_publications' => array(),
                        'temp_data_services' => array(),  'services_data' => array(), 'is_membership' => false);

        if ($use_services) {
            $free_publications = $this->Users_services_model->getUserPostServices($user_id);
            $post_count = null;
            if (!empty($free_publications )) {
                $return['free_publications'] = $free_publications;
                $first_for_use = null;
                $name_service = null;
                foreach ($free_publications as $value) {
                    if (!$first_for_use) {
                        $first_for_use = $value['id'];
                        $name_service = $value['name'];
                    }
                    $post_count += (int) $value['available_data_array']['post_count'];
                    
                    if (!empty($value['id_users_membership'])) {
                        $return['is_membership'] = true; 
                    }
                }
                $return['public_listing_data'] = array('count' => $post_count, 'first_id' => $first_for_use, 'service_name' => $name_service);
            }
        }

        $this->load->model('listings/models/Listings_service_model');
        $services = array('listings_featured_service', 'listings_lift_up_service', 'listings_lift_up_country_service',
                                'listings_lift_up_region_service', 'listings_lift_up_city_service', 'listings_highlight_service',
                                'listings_slide_show_service');

        $temp_data_services = array();
        foreach ($services as $value) {
            $temp = str_replace("listings_", "", $value);
            $key = str_replace("_service", "", $temp);
            $temp_data_services[$key] = $this->Listings_service_model->getFreeServices($user_id, $value);
        }

        $services_data = array();
        if (!empty($temp_data_services)) {
            $return['temp_data_services'] = $temp_data_services;
            foreach ($temp_data_services as $service_key => $value) {
                if (!empty($value)) {
                    $count = null;
                    $first_for_use = null;
                    $name_service = null;
                    foreach ($value as $el) {
                        if (!$first_for_use) {
                            $first_for_use = $el['id'];
                            $name_service = $el['name'];
                        }
                        $count += (int) $el['available_data_array']['count'];
                    }
                    $services_data[$service_key] = array("count" => $count, "first_id" => $first_for_use, 'service_name' => $name_service);
                }
            }
        }
        if (!empty($services_data)) {
            $return['services_data'] = $services_data;
        }

        return $return;
    }

    /**
     * Block of my listings
     *
     * @param string $operation_type opearation type
     * @param array $data filters data
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return string
     */
    protected function myListingsBlock($operation_type, $data = array(), $order = 'date_modified', $order_direction = 'DESC', $page = 1)
    {
        $current_settings = isset($_SESSION['my_listings']) ? $_SESSION['my_listings'] : array();
        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'date_modified';
        }
        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        if (!isset($current_settings['page'])) {
            $current_settings['page'] = 1;
        }
        if (!isset($current_settings['filters'])) {
            $current_settings['filters'] = array();
        }

        if (!empty($data)) {
            $current_settings['filters'] = $data;
        }

        if (!isset($current_settings['filters']['type']) || $current_settings['filters']['type'] != $operation_type) {
            $current_settings['filters']['type'] = $operation_type;
            $page = 1;
        }

        if (empty($current_settings['filters']['user'])) {
            $current_settings['filters']['user'] = $this->session->userdata('user_id');
        }

        if (!empty($order)) {
            $current_settings['order'] = $order;
        } else {
            $order = $current_settings['order'];
        }
        $this->view->assign('order', $current_settings['order']);

        if (!empty($order_direction)) {
            $current_settings['order_direction'] = $order_direction;
        } else {
            $order_direction = $current_settings['order_direction'];
        }
        $this->view->assign('order_direction', $order_direction);

        if (!empty($page)) {
            $current_settings['page'] = $page;
        } else {
            $page = $current_settings['page'];
        }
        $operation_types = $this->Listings_model->get_operation_types(true);
        $this->view->assign('operation_types', $operation_types);

        $this->view->assign('current_operation_type', $operation_type);

        $items_on_page = $this->pg_module->get_module_config('start', 'index_items_per_page');

        $sort_data = array(
            'url' => site_url() . 'listings/my/' . $operation_type,
            'order' => $order,
            'direction' => $order_direction,
            'links' => array(
                'views' => l('field_views', 'listings'),
                'price' => l('field_price', 'listings'),
                'open' => l('field_date_open', 'listings'),
                'date_modified' => l('field_date_modified', 'listings'),
            )
        );
        if ($this->pg_module->is_module_active('reviews')) {
            $this->load->helper('reviews');
            $review_link = get_reviews_sorter();
            if (is_array($review_link)) {
                $first = array_splice($sort_data['links'], 0, 1);
                $sort_data['links'] = array_merge($first, $review_link, $sort_data['links']);
            }
        }
        $this->view->assign('sort_data', $sort_data);

        $listings_count = $this->Listings_model->get_listings_count($current_settings['filters']);
        if ($listings_count > 0) {
            $order_array = array();
            if ($order == 'price') {
                $order_array['price_sorting'] = $order_direction;
            } elseif ($order == 'open') {
                $order_array['date_open'] = $order_direction;
            } elseif ($order == 'date_modified') {
                $order_array['date_modified'] = $order_direction;
            } else {
                $order_array[$order] = $order_direction;
            }
            if ($order == 'price' || isset($current_settings['filters']['price_range'])) {
                $price_order_array = array('price_negotiated' => $order_direction == 'ASC' ? 'DESC' : 'ASC');
                $order_array = array_merge($price_order_array, $order_array);
            }
            $this->Listings_model->applyFormatSettings(array('get_moderation' => true, 'get_views' => true));
            $listings = $this->Listings_model->get_listings_list($current_settings['filters'], $page, $items_on_page, $order_array, true, true);
            $this->Listings_model->revertFormatSettings();
        } else {
            $listings = array();
        }

        $this->view->assign('listings', $listings);
        $this->load->helper('sort_order');
        $page = get_exists_page_number($page, $listings_count, $items_on_page);
        $current_settings['page'] = $page;
        $_SESSION['my_listings'] = $current_settings;
        if ($this->pg_module->is_module_active('users')) {
            $this->load->helper('users');
            $this->view->assign('is_login_enabled', is_login_enabled());
        }

        // services data
        $services_full_data = $this->getServicesData();
        if (!empty($services_full_data)) {
            if (!empty($services_full_data['services_data'])) {
                $this->view->assign('services_data', $services_full_data['services_data']);
            }
            if (!empty($services_full_data['public_listing_data'])) {
                $this->view->assign('public_listing_data', $services_full_data['public_listing_data']);
            }
        }
        
        $this->view->assign('listings_services', $this->getListingServices());

        $this->load->helper('navigation');
        $url = site_url() . 'listings/my/' . $operation_type . '/' . $order . '/' . $order_direction;
        $page_data = get_user_pages_data($url, $listings_count, $items_on_page, $page, 'briefPage');
        foreach ($current_settings['filters'] as $filter_name => $filter_data) {
            $page_data[$filter_name] = $filter_data;
        }

        if (Listings_model::USER_TYPE_COMPANY === $this->session->userdata('user_type')) {
            $this->load->model('Users_model');
            $filter_user_ids = $this->Users_model->getAgentsListIds($this->session->userdata('user_id'));
            $filter_user_ids[] = $this->session->userdata('user_id');
            $page_data['users_filter'] = [
                'id' => $filter_user_ids,
            ];
        }
        $this->view->assign('animate_available', 1);
        $this->view->assign('page_data', $page_data);
        $this->view->assign('tstamp', time());
        return $this->view->fetch('my_block_list', 'user', 'listings');
    }

    /**
     * My listings
     *
     * @param array $operation_type operation type
     * @param string $order sorting order
     * @param string $order_direction sorting direction
     * @param integer $page page of results
     * @return void
     */
    public function my($operation_type = 'all', $order = 'date_modified', $order_direction = 'DESC', $page = 1)
    {
        $data = $this->input->post('data', true);

        $operation_types = $this->Listings_model->get_operation_types(true);
        $this->view->assign('operation_types', $operation_types);

        // operation type
        if (isset($data['type']) && in_array($data['type'], $operation_types)) {
            $operation_type = $data['type'];
        } elseif ($operation_type && in_array($operation_type, $operation_types)) {
            $data['type'] = $operation_type;
        } else {
            $operation_type = 'all';
        }
        $this->view->assign('current_operation_type', $operation_type);

        foreach (['id', 'id_country', 'id_region', 'id_city', 'id_district', 'category', 'keyword', 'user_id'] as $field) {
            if (isset($data[$field]) && empty($data[$field])) {
                unset($data[$field]);
            }
        }

        if (isset($data['category'])) {
            $chunks = explode('_', $data['category']);
            if (count($chunks) > 1) {
                $data['property_type'] = $chunks[1];
            }
            $data['id_category'] = $chunks[0];
            unset($data['category']);
        }

        $user_id = $this->session->userdata('user_id');
        $user_type = $this->session->userdata('user_type');

        $this->view->assign('hide_filter', false);
        if (Listings_model::USER_TYPE_COMPANY !== $user_type) {
            $this->view->assign('hide_fields', ['user' => true]);
        }

        $this->view->assign('operation_type', $operation_type);

        $this->view->assign('block', $this->myListingsBlock($operation_type, $data, $order, $order_direction, $page));

        if ($this->session->userdata('auth_type') != 'user') {
            $this->view->assign('is_guest', 1);
        }

        $listings_count = array();
        foreach ($operation_types as $gid) {
            $listings_count[$gid] = $this->Listings_model->get_listings_count(
                array('type' => $gid, 'user' => $user_id)
            );
        }
        
        $listings_count_sum = array_sum($listings_count);
        
        $listing_count['all'] = $listings_count_sum;
        
        $this->view->assign('listings_count', $listings_count);
        $this->view->assign(
            'listings_count_sum', $listings_count_sum
        );
        $this->view->assign('noshow_add_button', $this->Listings_model->is_operation_types_disabled());

        $this->Menu_model->breadcrumbs_set_parent($user_type . '_my_listings_item');

        $current_settings = isset($_SESSION['my_listings']) ? $_SESSION['my_listings'] : array();
        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'date_created';
        }
        $this->view->assign('order', $current_settings['order']);

        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        $this->view->assign('order_direction', $current_settings['order_direction']);

        if (!isset($current_settings['page'])) {
            $current_settings['page'] = 1;
        }
        $this->view->assign('page', $current_settings['page']);
        $this->view->render('my_list', 'user', 'listings');
    }

    /**
     * My listings by ajax
     *
     * @param string $operation_type operation type
     * @param string $order sorting order
     * @param string $order_direction sorting direction
     * @param integer $page page of results
     * @reeturn void
     */
    public function ajax_my($operation_type='all', $order = 'date_modified', $order_direction = 'DESC', $page = 1)
    {
        $data = $this->input->post('data', true);
        if (!empty($data)) {
            foreach ($data as $key=>$value) {
                if ($value == "") {
                    unset($data[$key]);
                }
            }
        }
        $this->view->output($this->myListingsBlock($operation_type, $data, $order, $order_direction, $page));
        $this->view->render();
    }

    /**
     * Edit listing data
     *
     * @param integer $listing_id listing identifier
     * @param string $section_gid section guid
     * @param integer $step processing step
     * @return void
     */
    public function getFieldsDescription($field_editor_type = '', $data = array())
    {
        $this->load->model('Field_editor_model');
        $this->Field_editor_model->initialize($field_editor_type);
        $property_type_sections = $this->Field_editor_model->get_section_list();
        if (!empty($property_type_sections)) {
            $sections = array();
            foreach ($property_type_sections as $section) {
                $sections[] = $section['gid'];
            }
            $this->view->assign('sections_data', array_values($property_type_sections));
            if (!empty($sections)) {
                $params['where']['editor_type_gid'] = $field_editor_type;
                $params['where_in']['section_gid'] = $sections;
                $fields_data = $this->Field_editor_model->get_form_fields_list($data, $params);
                return $fields_data;
            }
        }
        return 0;
    }

    public function edit($listing_id = 0, $section_gid = 'overview')
    {
        if ($this->Listings_model->is_operation_types_disabled() && $listing_id == 0) {
            show_404();
        }

        $user_id = $this->session->userdata('user_id');
        $user_type = $this->session->userdata('user_type');

        $this->Menu_model->breadcrumbs_set_parent($user_type . '_my_listings_item');

        $format_settings = array('get_moderation' => true);

        switch ($section_gid) {
            case 'overview':
                $format_settings['get_location'] = true;
                break;
            case 'description':
                $format_settings['get_description'] = true;
                $format_settings['get_photos_all'] = true;
                $format_settings['get_virtual_tours_all'] = true;
                $format_settings['get_file'] = true;
                $format_settings['get_video'] = true;
                break;
            case 'pricing':
                $format_settings['get_booking_special_prices'] = true;
                break;
            case 'booked_periods':
                $format_settings['get_booking_periods'] = true;
                break;
            case 'seo':
                $format_settings['get_location'] = true;
                break;
        }

        $this->Listings_model->set_format_settings($format_settings);

        if ($listing_id) {
            $data = $this->Listings_model->get_listing_by_id($listing_id);
            if (empty($data) || $user_id != $data['id_user']) {
                show_404();
                return;
            }

            $moderation_alerts = array();
            $is_moderation_installed = $this->pg_module->is_module_active('moderation');
            if ($is_moderation_installed) {
                $this->load->model('listings/models/Listings_moderation_model');

                $admin_alert = $this->Listings_moderation_model->get_alert($listing_id);
                if (!empty($admin_alert)) {
                    $moderation_alerts[$admin_alert['status']] = $admin_alert['content'];
                }

                if ($this->Listings_moderation_model->is_listing_moderated($listing_id)) {
                    $fields_for_select = $this->Listings_moderation_model->get_fields_for_moderate();
                    $this->Listings_moderation_model->set_dop_fields($fields_for_select);
                    $moderated_data = $this->Listings_moderation_model->get_listing_by_id($listing_id);
                    $data = array_merge($data, $moderated_data);
                    $moderation_alerts['info'] = l('info_your_listing_on_moderation', 'listings');
                }
            }

            $this->view->assign('moderation_alerts', $moderation_alerts);

            $this->Menu_model->breadcrumbs_set_active(l('header_listing_edit', 'listings'), site_url() . 'listings/edit/' . $listing_id);
            $this->Menu_model->breadcrumbs_set_active(l('filter_section_' . $section_gid, 'listings'));
        } else {
            $data = array();
            $this->Menu_model->breadcrumbs_set_active(l('link_add_listing', 'listings'), site_url() . 'listings/edit');
        }

        $is_back_to_my = false;

        if (isset($data['square']) && $data['square'] && ($data['price'] || $data['price_auction'])) {
            $this->view->assign('edit', '');
            $is_back_to_my = 'true';
        } else {
            $this->view->assign('edit', '1');
        }

        $back_url = site_url() . 'listings/my';

        if ($this->input->post('save_btn') == 1) {
            $post_data = $this->input->post('data', true);
            switch ($section_gid) {
                case 'overview':
                    $post_data['id_country'] = $this->input->post('id_country', true);
                    $post_data['id_region'] = $this->input->post('id_region', true);
                    $post_data['id_city'] = $this->input->post('id_city', true);
                    $post_data['id_district'] = $this->input->post('id_district', true);
                    $post_data['id_user'] = $user_id;
                    //map
                    $this->load->model('geomap/models/Geomap_model');
                    $this->load->model('geomap/models/Geomap_settings_model');
                    $post_settings = $this->input->post('map', true);
                    $post_settings['lat'] = $post_data['lat'];
                    $post_settings['lon'] = $post_data['lon'];
                    $map_gid = $this->Geomap_model->get_default_driver_gid();
                    $validate_map = $this->Geomap_settings_model->validate_settings($post_settings);
                    if (!empty($validate_map['errors'])) {
                        break;
                    }
                    $this->Geomap_settings_model->save_settings($map_gid, $user_id, $listing_id, 'listing_view', $validate_map['data']);
                    $listing_data['lat'] = $post_data['lat'];
                    $listing_data['lon'] = $post_data['lon'];
                    //map
                    if ($listing_id) {
                        if (isset($post_data['category'])) {
                            $pos = strpos($post_data['category'], '_');
                            $post_data['property_type'] = substr($post_data['category'], $pos + 1);
                            unset($post_data['category']);
                        }
                    } else {
                        $operation_type = $this->input->post('type', true);

                        $post_data['id_type'] = $this->Listings_model->get_operation_type_by_gid($operation_type);

                        if (isset($post_data['category'])) {
                            $pos = strpos($post_data['category'], '_');
                            $post_data['id_category'] = substr($post_data['category'], 0, $pos);
                            $post_data['property_type'] = substr($post_data['category'], $pos + 1);
                            unset($post_data['category']);
                        }
                    }

                    $validate_data = $this->Listings_model->validate_listing($listing_id, $post_data);

                    if (!empty($validate_data['errors'])) {
                        $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
                    } else {
                        if (!empty($validate_data['data'])) {
                            if ($listing_id) {
                                $this->Listings_model->save_listing($listing_id, $validate_data['data'], true);
                                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_update_listing', 'listings'));
                            } else {
                                $listing_id = $this->Listings_model->save_listing(null, $validate_data['data'], true);
                                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_add_listing', 'listings'));
                            }
                        }
                        $url = null;
                        if ($this->pg_module->is_module_active('field_editor')) {
                            $url = site_url() . 'listings/edit/' . $listing_id . '/description';
                        }
                        if ($is_back_to_my) {
                            $url = $back_url;
                        }
                        $this->view->setRedirect($url);
                    }
                    break;
                case 'description':
                    if (!$listing_id) {
                        break;
                    }
                    $this->load->model('Field_editor_model');
                    $field_editor_section_gid = $this->input->post('field_editor_section_gid', true);
                    $property_type_gid = $this->Listings_model->get_field_editor_type($data);
                    $this->Field_editor_model->initialize($property_type_gid);
                    $fields_data = $this->Field_editor_model->get_fields_for_select($field_editor_section_gid);
                    foreach ($fields_data as $field) {
                        $post_data[$field] = $this->input->post($field, true);
                    }
                    $validate_data = $this->Listings_model->validate_listing($listing_id, $post_data);
                    if (!empty($validate_data['errors'])) {
                        $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
                    } else {
                        $this->load->model('Field_editor_model');
                        $this->Field_editor_model->initialize($data['field_editor_type']);
                        $property_type_sections = $this->Field_editor_model->get_section_list();
                        $property_type_sections_count = max(count($property_type_sections), 1);
                        if (!empty($validate_data['data'])) {
                            if ($listing_id) {
                                $this->Listings_model->save_listing($listing_id, $validate_data['data'], true);
                                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_update_listing', 'listings'));
                            }
                        }
                        $url = site_url() . 'listings/edit/' . $listing_id . '/pricing';

                        if ($is_back_to_my) {
                            $url = $back_url;
                        }

                        $this->view->setRedirect($url);
                    }
                    break;
                case 'pricing':
                    if (!$listing_id) {
                        break;
                    }
                    $post_data['date_open'] = $this->input->post('date_open', true);
                    if ($post_data['date_open']) {
                        $post_data['date_open'] = $post_data['date_open'];
                    }
                    $post_data['date_available'] = $this->input->post('date_available', true);
                    if ($post_data['date_available']) {
                        $post_data['date_available'] = $post_data['date_available'];
                    }

                    $validate_data = $this->Listings_model->validate_listing($listing_id, $post_data);

                    if (!empty($validate_data['errors'])) {
                        $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
                    } else {
                        if (!empty($validate_data['data'])) {
                            if ($listing_id) {
                                $this->Listings_model->save_listing($listing_id, $validate_data['data'], true);
                                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_update_listing', 'listings'));
                            }
                        }
                        if ($data['operation_type'] == 'rent' && $this->pg_module->is_module_active('listings_booking')) {
                            $url = site_url() . 'listings/edit/' . $listing_id . '/booked_periods';
                        } elseif ($this->pg_module->is_module_active('services')) {
                            $url = site_url() . 'listings/edit/' . $listing_id . '/publish';
                        } else {
                            $url = $back_url;
                        }

                        if ($is_back_to_my) {
                            $url = $back_url;
                        }

                        $this->view->setRedirect($url);
                    }
                    break;
                case 'booked_periods':
                    $url = site_url() . 'listings/activity/' . $listing_id;
                    $this->view->setRedirect($url);
                    break;
                case 'publish':
                    $validate_data = $this->Listings_model->validateForActivation($data);
                    if (!empty($validate_data['errors'])) {
                        $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
                        $this->view->setRedirect(site_url() . 'listings/edit/' . $listing_id . '/publish');
                        return;
                    }
                
                    $services = $this->input->post('services', true);

                    $date_expired = strtotime($data['date_expire']);
                    if ($date_expired > time()) {
                        $this->Listings_model->activate_listing($data['id'], 1);
                        $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_activate_listing', 'listings'));
                        $services = array_diff($services, ['listings_post_service']);
                    } elseif (!$this->isPostServices()) {
                        $period = $this->pg_module->get_module_config('listings', 'default_activation_period');
                        if (!$data['initial_moderation']) {
                            $this->Listings_model->save_listing($data['id'], array('initial_activity' => $period), false);
                            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_activate_listing'));
                        } else {
                            $date_expired = strtotime($data['date_expire']);
                            if ($date_expired > time()) {
                                $uts = $date_expired;
                            } else {
                                $uts = time();
                            }
                            $date_expired = date('Y-m-d H:i:s', $uts + $period * 24 * 60 * 60);
                            $this->Listings_model->activate_listing($listing_id, 1, $date_expired);
                            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_activate_listing', 'listings'));
                        }
                    }
                    
                    if (in_array('listings_post_service', $services)) {
                        $publications = $this->Users_services_model->getUserPostServices($user_id);
                        foreach ($publications as $publication) {
                            if (!empty($publication['id_users_membership'])) {
                                $services = array_diff($services, ['listings_post_service']);
                                $uts = strtotime($publication['date_modified']);
                                $date_expired = date('Y-m-d H:i:s', $uts + $publication['available_data_array']['post_period'] * 24 * 60 * 60);
                                $this->Listings_model->activate_listing($listing_id, 1, $date_expired);
                                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_activate_listing', 'listings'));
                                break;
                            }
                        }
                    }
                  
                    if (!empty($services)) {
                        $activate_public = $this->input->post('activate_public', true);
                        $price = $this->input->post('price', true);
                        $this->session->set_userdata(array('service_redirect' => 'listings/edit/' . $listing_id . '/publish'));
                        $retHTML = "<html>\n<body onLoad=\"document.send_form.submit();\">\n";
                        $retHTML .= "<form method=\"post\" name=\"send_form\" action=\"" . site_url() . 'services/form' . "\">\n";
                        foreach ($services as $service) {
                            $retHTML .= "<input type=\"hidden\" name=\"services[]\" value=\"" . addslashes($service) . "\">\n";
                        }
                        if ($activate_public) {
                            $retHTML .= "<input type=\"hidden\" name=\"activate_public\" value=\"" . addslashes($activate_public) . "\">\n";
                        }
                        $retHTML .= "<input type=\"hidden\" name=\"price\" value=\"" . addslashes($price) . "\">\n";
                        $retHTML .= "<input type=\"hidden\" name=\"id_listing\" value=\"" . addslashes($listing_id) . "\">\n";
                        $retHTML .= "</form>\n</body>\n</html>";
                        $this->view->output($retHTML);
                        $this->view->render();
                        return;
                    } else {
                        $this->view->setRedirect(site_url() . 'listings/edit/' . $listing_id . '/publish');
                        return;
                    }

                    break;
                case 'seo':
                    $this->load->model('Seo_model');
                    $seo_fields = $this->Seo_model->get_seo_fields();

                    foreach ($seo_fields as $key => $section_data) {
                        if ($this->input->post('btn_save_' . $section_data['gid'])) {
                            $post_data = array();
                            $post_data[$section_data['gid']] = $this->input->post($section_data['gid'], true);
                            $validate_data = $this->Seo_model->validate_seo_tags($listing_id, $post_data);
                            if (empty($validate_data['errors'])) {
                                $listing_data['id_seo_settings'] = $this->Seo_model->save_seo_tags($data['id_seo_settings'], $validate_data['data']);
                                if (!$data['id_seo_settings']) {
                                    $data['id_seo_settings'] = $listing_data['id_seo_settings'];
                                    $this->Listings_model->save_listing($listing_id, $listing_data, false);
                                }
                                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_settings_updated', 'seo'));
                            }
                            break;
                        }
                    }
                    break;
            }

            if (!empty($post_data)) {
                $this->Listings_model->set_format_settings('use_format', false);
                $data = $this->Listings_model->get_listing_by_id($listing_id);
                $this->Listings_model->set_format_settings('use_format', true);

                $post_data = $this->Listings_model->format_listings(array(array_merge($data, $post_data)));
                $data = $post_data[0];
            }
        }

        if ($this->input->post('btn_deactivate') == 1) {
            $this->Listings_model->activate_listing($listing_id, 0);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_deactivate_listing', 'listings'));
            $this->view->setRedirect(site_url() . 'listings/edit/' . $listing_id . '/publish');
            return;
        }

        if ($this->input->post('btn_make_sold') == 1) {
            $this->Listings_model->make_sold($listing_id, 1);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_make_sold', 'listings'));
            $this->view->setRedirect(site_url() . 'listings/edit/' . $listing_id . '/publish');
            return;
        }

        if ($this->input->post('btn_delete_sold')) {
            $this->Listings_model->make_sold($listing_id, 0);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_remove_sold', 'listings'));
            $this->view->setRedirect(site_url() . 'listings/edit/' . $listing_id . '/publish');
            return;
        }

        switch ($section_gid) {
            case 'overview':
                $operation_types = $this->Listings_model->get_operation_types(false);
                $this->view->assign('operation_types', $operation_types);
                $this->view->assign('operation_types_count', count($operation_types));
                //map
                if (isset($data['id'])) {
                    $markers = array(
                        array(
                            'gid' => $data['id'],
                            'country' => $data['country'],
                            'region' => $data['region'],
                            'city' => $data['city'],
                            'address' => $data['address'],
                            'postal_code' => $data['zip'],
                            'lat' => (float) $data['lat'],
                            'lon' => (float) $data['lon'],
                            'dragging' => true,
                            'info' => ($data['location'] ? $data['location'] : ''),
                        ),
                    );
                    $this->view->assign('markers', $markers);
                    $this->view->assign('markers_data', json_encode($markers));
                }

                $current_language = $this->pg_language->get_lang_by_id($this->pg_language->current_lang_id);
                if (!isset($data['lat'])) {
                    $lat = 0;
                } else {
                    $lat = $data['lat'];
                }
                if (!isset($data['lon'])) {
                    $lon = 0;
                } else {
                    $lon = $data['lon'];
                }
                $view_settings = array(
                    'width' => '630',
                    'height' => '400',
                    'disable_smart_zoom' => 1,
                    'zoom_listener' => 'get_listing_zoom_data',
                    'type_listener' => 'get_listing_type_data',
                    'drag_listener' => 'get_listing_drag_data',
                    'panorama_listener' => 'get_listing_panorama',
                    'lang' => $current_language['code'],
                    'lat' => (float) $lat,
                    'lon' => (float) $lon,
                );
                $this->view->assign('map_settings', $view_settings);
                $this->view->assign('map_settings_data', json_encode($view_settings));

                if ($listing_id) {
                    $this->load->model('geomap/models/Geomap_model');
                    $this->load->model('geomap/models/Geomap_settings_model');
                    $map_gid = $this->Geomap_model->get_default_driver_gid();
                    $map_settings = $this->Geomap_settings_model->get_settings(
                        $map_gid, $user_id, $listing_id, 'listing_view'
                    );
                    $this->view->assign('listing_map_settings', $map_settings);
                    $this->view->assign('listing_map_settings_data', json_encode($map_settings));
                }
                //square
                $this->view->assign('square_units', $this->Listings_model->getSquareUnits());
                break;
            case 'description':
                $fields_data = $this->getFieldsDescription($data['field_editor_type'], $data);
                if ($fields_data) {
                    $this->view->assign('fields_data', $fields_data);
                }
                $this->load->model('listings/models/Listings_uploads_model');
                // photos
                $this->view->assign('gallery_type', $this->Listings_uploads_model->gallery_type);
                // virtual tours
                $this->view->assign('vtour_type', $this->Listings_uploads_model->virtual_tour_type);
                $this->load->model('upload_gallery/models/Upload_gallery_model');
                // files
                $file_type_config = $this->Listings_uploads_model->get_file_type();
                $file_type_config['max_size_str'] = $this->Listings_uploads_model->getSize($file_type_config['max_size'], 'listings');
                $file_type_config['config'] = $this->Upload_gallery_model->get_type_by_gid($this->Listings_uploads_model->file_type);
                $this->view->assign('file_settings', $file_type_config);
                // videos
                $video_type_config = $this->Listings_uploads_model->get_video_type();
                $video_type_config['max_size_str'] = $this->Listings_uploads_model->getSize($video_type_config['max_size'], 'listings');
                $video_type_config['config'] = $this->Upload_gallery_model->get_type_by_gid($this->Listings_uploads_model->video_type);
                $this->view->assign('video_settings', $video_type_config);
                break;
            case 'pricing':
                $operation_types = $this->Listings_model->get_operation_types(false);
                $this->view->assign('operation_types', $operation_types);
                $this->view->assign('operation_types_count', count($operation_types));
                if ($this->pg_module->is_module_active('currencies')) {
                    $this->load->model('Currencies_model');
                    $params["where"]['status'] = 1;
                    $currencies = $this->Currencies_model->get_currency_list($params);
                    $currencies_data['data'] = $currencies;
                    $currencies_data['count'] = count($currencies_data['data']);
                    $this->view->assign('currencies', $currencies_data);
                    $this->view->assign('currency_default', $this->Currencies_model->get_currency_default());
                }

                $dayhours = $this->pg_language->ds->get_reference('start', 'dayhour-names');
                $this->view->assign('dayhours', $dayhours);
                $data['calendar_special_prices_lang'] = $this->getSpecialPricesLang();
                $this->view->assign('current_lang_id', $this->pg_language->current_lang_id);
                $this->view->assign('langs', $this->pg_language->languages);
                break;
            case 'booked_periods':
                break;
            case 'publish':
                $this->view->assign('action', site_url() . 'listings/edit/' . $listing_id . '/publish');
                $services_full_data = $this->getServicesData();
                $this->view->assign('services_data', $services_full_data);
                $this->view->assign('services', $this->services($data['id']));
                $this->view->assign('is_resume_available', $data['status'] == 0 && time() < strtotime($data['date_expire']));
                break;
            case 'seo':
                $this->load->model('Seo_model');
                $seo_fields = $this->Seo_model->get_seo_fields();
                $this->view->assign('seo_fields', $seo_fields);

                $languages = $this->pg_language->languages;
                $this->view->assign('languages', $languages);

                $current_lang_id = $this->pg_language->current_lang_id;
                $this->view->assign('current_lang_id', $current_lang_id);

                $seo_settings = $this->Seo_model->get_seo_tags($data['id_seo_settings']);
                $this->view->assign('seo_settings', $seo_settings);
                break;
        }
        $this->view->assign('back_url', $back_url);
        $this->view->assign('section_gid', $section_gid);
        $this->view->assign('data', $data);
        $this->Menu_model->set_menu_active_item($user_type . '_account_menu', 'my_listings_item');
        $this->view->render('listing_form');
    }

    public function getSpecialPricesLang()
    {
        return array(
            'calendar_prices_details' => l('calendar_prices_details', 'listings_booking'),
            'exclusive_prices' => l('exclusive_prices', 'listings_booking'),
            'standart_price' => l('standart_price', 'listings_booking'),
        );
    }

    private function isPostServices() {
        if ($this->pg_module->is_module_active('users_services')) {
            $this->load->model('Users_services_model');
            return $this->Users_services_model->is_active_post_services();
        }
        return false;
    }

    /**
     * Available listing services
     *
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function services($listing_id)
    {
        $user_id = $this->session->userdata('user_id');
        $user_type = $this->session->userdata('user_type');

        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        if ($user_id != $listing['id_user']) {
            show_404();
            return;
        }
        if ($this->pg_module->is_module_active('services')) {
            $this->load->model('Services_model');
            // check pay services exists
            $templates_gids = array(
                'listings_featured_template',
                'listings_lift_up_template',
                'listings_lift_up_country_template',
                'listings_lift_up_region_template',
                'listings_lift_up_city_template',
                'listings_highlight_template',
                'listings_slide_show_template',
                'listings_post_template'
            );
            $services = $this->Services_model->get_user_type_active_services($user_type, $templates_gids);

            $page_data['listings_services'] = $services;
            $page_data['use_services'] = count($services) > 0;
            $this->view->assign('image_path', 'uploads/services-logo/');
            $services_params = array();
            foreach ($page_data['listings_services'] as $service) {
                foreach ($service['data_admin'] as $param_gid => $param_value) {
                    $services_params[$param_gid] = $service['template']['data_admin_array'][$param_gid]['name'];
                }
            }
            if (count($services_params) === 1) {
                $this->view->assign('services_one_param', 1);
                $this->view->assign('services_param_gid', key($services_params));
                $this->view->assign('services_param_name', current($services_params));
            }

            $params = array(
                'where' => array(
                    'id_user' => $user_id,
                    'status' => 1,
                ),
                'where_in' => array(
                    'template_gid' => $templates_gids,
                ),
            );

            $this->load->model('listings/models/Listings_service_model');
            $user_services = $this->Listings_service_model->get_services_list(null, $params);

            $this->view->assign('user_services', $user_services);

            $user_services_params = array();
            foreach ($user_services as $user_service) {
                foreach ($user_service['available_data_array'] as $param_gid => $param_value) {
                    if (isset($user_service['template_data_array']['data_admin_array'][$param_gid]['name'])) {
                        $user_services_params[$param_gid] = $user_service['template_data_array']['data_admin_array'][$param_gid]['name'];
                    } else {
                        if (isset($user_service['template_data_array']['data_membership_array'][$param_gid])) {
                            $user_services_params[$param_gid] = $user_service['template_data_array']['data_membership_array'][$param_gid]['name'];
                        }
                    }
                }
            }

            if (count($user_services_params) === 1) {
                $this->view->assign('user_services_one_param', 1);
                $this->view->assign('user_services_param_gid', key($user_services_params));
                $this->view->assign('user_services_param_name', current($user_services_params));
            }
        } else {
            $page_data['use_services'] = false;
        }

        $this->view->assign('is_resume_available', $listing['status'] == 0 && time() < strtotime($listing['date_expire']));
        $this->view->assign('is_post_services', $this->isPostServices());
        $this->view->assign('default_activation_period',
        $this->pg_module->get_module_config('listings', 'default_activation_period'));

        $this->view->assign('filter_service_templates', $templates_gids);
        $this->view->assign('section_gid', 'services');
        $this->view->assign('data', $listing);
        $this->view->assign('page_data', $page_data);
        $this->load->model('Currencies_model');
        $this->view->assign('base_currency', $this->Currencies_model->get_currency_default(true));
        return $this->view->fetch('listing_services_form', 'user', 'listings');
    }
    
    private function getListingServices() 
    {
        if ($this->pg_module->is_module_active('services')) {
            $this->load->model('Services_model');
            
            $user_type = $this->session->userdata('user_type');
            
            // check pay services exists
            $templates_gids = array(
                    'listings_featured_template',
                    'listings_lift_up_template',
                    'listings_lift_up_country_template',
                    'listings_lift_up_region_template',
                    'listings_lift_up_city_template',
                    'listings_highlight_template',
                    'listings_slide_show_template',
                    'listings_post_template'
                );
            
            $services = $this->Services_model->get_user_type_active_services($user_type, $templates_gids);
            
            $services_by_gid = [];
            
            foreach ($services as $service) {
                $services_by_gid[$service['gid']] = $service;
            }
                
            return $services_by_gid;
        } else {
            return [];
        }
    }

    /**
     * Apply selected service to listing object
     *
     * @param integer $listing_id listing identifier
     * @param string $service_gid service GUID
     * @return void
     */
    public function apply_service($listing_id, $service_gid)
    {
        $user_id = $this->session->userdata('user_id');

        $data = $this->Listings_model->get_listing_by_id($listing_id);
        if (!$data || $user_id != $data['id_user']) {
            show_404();
            return;
        }
        $this->session->set_userdata(array('service_redirect' => 'listings/my'));

        $this->load->helper('payments');
        $params = array('id_listing' => $listing_id);
        post_location_request(site_url() . 'services/form/' . $service_gid, $params);
    }

    private function activateListing($listing, $user_service_id = null)
    {
        $validate_data = $this->Listings_model->validateForActivation($listing);
        if (!empty($validate_data['errors'])) {
            $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
            return;
        }
        
        if ($this->pg_module->is_module_active('users_services')) {
            $this->load->model('Users_services_model');
            $use_services = $this->Users_services_model->is_active_post_services();
        } else {
            $use_services = false;
        }
        $date_expired = strtotime($listing['date_expire']);
        if ($date_expired > time()) {
            $this->Listings_model->activate_listing($listing['id'], 1);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_activate_listing', 'listings'));
            return;
        }
        if ($use_services) {
            $this->view->setRedirect(site_url() . 'listings/edit/' . $listing['id'] . '/publish');
        } else {
            $period = $this->pg_module->get_module_config('listings', 'default_activation_period');
            if (!$listing['initial_moderation']) {
                $this->Listings_model->save_listing($listing['id'], array('initial_activity' => $period), false);
                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_activate_listing'));
            } else {
                $date_expired = strtotime($listing['date_expire']);
                if ($date_expired > time()) {
                    $uts = $date_expired;
                } else {
                    $uts = time();
                }
                $date_expired = date('Y-m-d H:i:s', $uts + $period * 24 * 60 * 60);
                $this->Listings_model->activate_listing($listing_id, 1, $date_expired);
                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_activate_listing', 'listings'));
            }
        }
    }

    public function setStatus($listing_id, $status)
    {
        $statuses = ['active', 'inactive', 'sold', 'unsold'];

        if (!in_array($status, $statuses)) {
            show_404();
            return;
        }

        $this->Listings_model->set_format_settings(array('get_user' => false, 'get_location' => false));
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings(array('get_user' => true, 'get_location' => true));

        if ($this->session->userdata('user_id') != $listing['id_user']) {
            show_403();
            return;
        }

        if ('inactive' === $status) {
            $this->Listings_model->activate_listing($listing_id, 0);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_deactivate_listing', 'listings'));
        } elseif ('active' === $status) {
            $result = $this->activateListing($listing);
        } elseif ('sold' === $status) {
            $this->Listings_model->make_sold($listing_id, 1);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_make_sold', 'listings'));
        } elseif ('unsold' === $status) {
            $this->Listings_model->make_sold($listing_id, 0);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_remove_sold', 'listings'));
        }
        $this->view->setRedirect($_SERVER['HTTP_REFERER']);
    }

    /**
     * Listing activity management
     *
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function activity($listing_id)
    {
        $user_id = $this->session->userdata('user_id');
        $user_type = $this->session->userdata('user_type');

        $this->Listings_model->set_format_settings(array('get_user' => false, 'get_location' => false));
        $data = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings(array('get_user' => true, 'get_location' => true));
        if ($user_id != $data['id_user']) {
            show_404();
            return;
        }

        $this->Menu_model->breadcrumbs_set_parent($user_type . '_my_listings_item');
        $this->Menu_model->breadcrumbs_set_active(l('link_edit_listing', 'listings'), site_url() . 'listings/edit/' . $listing_id);
        $this->Menu_model->breadcrumbs_set_active(l('filter_section_activity', 'listings'));

        if ($this->pg_module->is_module_active('users_services')) {
            $this->load->model('Users_services_model');
            $page_data['use_services'] = $this->Users_services_model->is_active_post_services();
        } else {
            $page_data['use_services'] = false;
        }

        if ($data['status'] == 0 && strtotime($data['date_expire']) > time()) {
            $page_data['can_reactivate'] = true;
        }

        // actions
        if ($this->input->post('btn_activate')) {
            $validate_data = $this->Listings_model->validateForActivation($data);
            if (!empty($validate_data['errors'])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
                $this->view->setRedirect(site_url() . 'listings/activity/' . $listing_id);
                return;
            }
            
            if ($page_data['use_services']) {
                $user_service_id = $this->input->post('id_user_service', true);
                $user_service_id = intval($user_service_id);

                if (!empty($user_service_id)) {
                    $user_service = $this->Users_services_model->get_service($user_service_id);
                } else {
                    $this->system_messages->addMessage(View::MSG_ERROR, l('error_service_activating', 'listings'));
                    $this->view->setRedirect(site_url() . 'listings/activity/' . $listing_id);
                }

                if (empty($user_service)) {
                    $this->system_messages->addMessage(View::MSG_ERROR, l('error_service_activating', 'listings'));
                    $this->view->setRedirect(site_url() . 'listings/activity/' . $listing_id);
                }

                $user_data = array('listing' => $listing_id);

                $validate_data = $this->Users_services_model->validate_post_activate($user_service, $user_data);
                if (!empty($validate_data['errors'])) {
                    $this->system_messages->addMessage(View::MSG_ERROR, l('error_service_activating', 'listings'));
                    $this->view->setRedirect(site_url() . 'listings/activity/' . $listing_id);
                } else {
                    $this->Users_services_model->service_post_activate($user_service, $user_data);
                }
            } else {
                $period = $this->pg_module->get_module_config('listings', 'default_activation_period');

                if (!$data['initial_moderation']) {
                    $this->Listings_model->save_listing($listing_id, array('initial_activity' => $period), false);
                } else {
                    $date_expired = strtotime($data['date_expire']);

                    if ($date_expired > time()) {
                        $uts = $date_expired;
                    } else {
                        $uts = time();
                    }

                    $date_expired = date('Y-m-d H:i:s', $uts + $period * 24 * 60 * 60);

                    $this->Listings_model->activate_listing($listing_id, 1, $date_expire);
                }
            }

            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_service_activating', 'listings'));
            $this->view->setRedirect(site_url() . 'listings/activity/' . $listing_id);
        }

        if ($this->input->post('btn_deactivate') && $data['status'] == 1) {
            $this->Listings_model->activate_listing($listing_id, 0);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_service_deactivating', 'listings'));
            $this->view->setRedirect(site_url() . 'listings/activity/' . $listing_id);
        }

        if ($this->input->post('btn_reactivate') && $page_data['can_reactivate']) {
            $this->Listings_model->activate_listing($listing_id, 1);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_service_activating', 'listings'));
            $this->view->setRedirect(site_url() . 'listings/activity/' . $listing_id);
        }

        if ($this->input->post('btn_make_sold') && $data['status'] == 1) {
            $this->Listings_model->make_sold($listing_id, 1);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_make_sold', 'listings'));
            $this->view->setRedirect(site_url() . 'listings/activity/' . $listing_id);
        }

        if ($this->input->post('btn_delete_sold') && $data['status'] == 1) {
            $this->Listings_model->make_sold($listing_id, 0);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_remove_sold', 'listings'));
            $this->view->setRedirect(site_url() . 'listings/activity/' . $listing_id);
        }

        if ($page_data['use_services']) {
            // get user services
            $page_data['user_services'] = $this->Users_services_model->getUserPostServices($user_id);
        } else {
            $page_data['default_activation_period'] = $this->pg_module->get_module_config('listings', 'default_activation_period');
            $page_data['end_activation_date'] = date('Y-m-d H:i:s', time() + $page_data['default_activation_period'] * 24 * 60 * 60);
        }

        $this->view->assign('section_gid', 'activity');
        $this->view->assign('data', $data);
        $this->view->assign('page_data', $page_data);

        $this->Menu_model->set_menu_active_item($user_type . '_account_menu', 'my_listings_item');
        $this->view->render('listing_activity_form');
    }

    /**
     * Remove listing
     *
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function delete($listing_id)
    {
        $user_id = $this->session->userdata('user_id');

        $this->Listings_model->set_format_settings('use_format', false);
        $data = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);

        if (empty($data) || $user_id != $data['id_user']) {
            show_404();
            return;
        }

        if (!empty($listing_id)) {
            $this->Listings_model->delete_listing($listing_id);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_listings_delete', 'listings'));
        }
        $this->view->setRedirect(site_url() . 'listings/my/');
    }

    /**
     * Close moderation alert
     *
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function alert($listing_id)
    {
        if ($this->session->userdata('auth_type') != 'user') {
            show_404();
            return;
        }

        $user_id = $this->session->userdata('user_id');
        $this->Listings_model->set_format_settings('use_format', false);
        $data = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);

        $this->view->render('alert_form', 'user', 'listings');
    }

    /**
     * Close moderation alert by ajax
     *
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function ajax_alert($listing_id)
    {
        $user_id = $this->session->userdata('user_id');

        $this->Listings_model->set_format_settings('use_format', false);
        $data = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);

        $this->view->assign('data', $data);

        $content = $this->view->fetch('alert_form', 'user', 'listings');

        $this->view->assign('content', $content);
        $this->view->render();
    }

    /**
     * Show main search form by ajax
     *
     * @param integer $rand random number
     * @return void
     */
    public function ajax_get_main_search_form($rand)
    {
        $result = array('short' => '', 'full' => '');

        $this->view->assign('form_settings', array('rand' => $rand));

        $operation_type = $this->input->post('type', true);
        $category = $this->input->post('category', true);
        if ($category) {
            $category_chunks = explode('_', $category);
            $category_id = $category_chunks[0];
        } else {
            $category_id = 0;
        }

        $width = intval($this->input->post('width', true));
        if (!$width) {
            $width = 100;
        }

        $data = array('operation_type' => $operation_type, 'id_category' => $category_id);
        $property_type_gid = $this->Listings_model->get_field_editor_type($data);

        $this->view->assign('block_width', $width);

        $this->load->model('listings/models/Listings_form_model');
        $main_search_form_data = $this->Listings_form_model->get_search_extend_form($property_type_gid, 'main_search_form_gid', false);
        if (empty($main_search_form_data)) {
            $this->view->render();
            return;
        }
        $form_data = array();
        foreach ($main_search_form_data as $field_data) {
            $form_data[$field_data['field']['gid']] = array('range' => array('min' => '', 'max' => ''));
        }
        $this->load->helper('listings');
        $main_search_form = listings_main_search_form_block($property_type_gid, $form_data);
        $result['short'] = '';
        $result['full'] = $main_search_form;

        $this->view->assign($result);
        $this->view->render();
    }

    /**
     * Show quick search form by ajax
     *
     * @return void
     */
    public function ajax_get_search_quick_form()
    {
        $operation_type = $this->input->post('type', true);

        $category_id = $this->input->post('id_category', true);
        $category_id = explode('_', $category_id);

        $data = array('operation_type' => $operation_type, 'id_category' => $category_id[0]);

        $property_type_gid = $this->Listings_model->get_field_editor_type($data);

        $this->load->model('listings/models/Listings_form_model');
        $quick_search_form_data = $this->Listings_form_model->get_search_extend_form(
            $property_type_gid, 'quick_search_form_gid', false
        );


        if (!empty($quick_search_form_data)) {
            $this->view->assign('form_data', $quick_search_form_data);

            $form_data = array();
            foreach ($quick_search_form_data as $field_data) {
                $form_data[$field_data['field']['gid']] = array('range' => array('min' => '', 'max' => ''));
            }
            $this->view->assign('data', $form_data);
            $this->view->assign('form_rand', rand(100000, 999999));
            $this->view->output($this->view->fetch('helper_search_form_block', 'user', 'listings'));            
        }
        
        $this->view->render();
    }

    /**
     * Show advanced search form by ajax
     *
     * @return void
     */
    public function ajax_get_search_advanced_form()
    {
        $operation_type = $this->input->post('type', true);

        $category_id = $this->input->post('id_category');
        $category_id = explode('_', $category_id);

        $data = array('operation_type' => $operation_type, 'id_category' => $category_id[0]);

        $property_type_gid = $this->Listings_model->get_field_editor_type($data);

        $this->load->model('listings/models/Listings_form_model');
        $advanced_search_form_data = $this->Listings_form_model->get_search_extend_form($property_type_gid, 'advanced_search_form_gid', false);

        if (count($advanced_search_form_data) > 0 && is_array($advanced_search_form_data)) {
            $this->view->assign('form_data', $advanced_search_form_data);

            $form_data = array();
            foreach ($advanced_search_form_data as $field_data) {

                $form_data[$field_data['field']['gid']] = array('range' => array('min' => '', 'max' => ''));
            }
            $this->view->assign('data', $form_data);
            $this->view->assign('form_rand', rand(100000, 999999));
            $this->view->output($this->view->fetch('helper_search_form_block', 'user', 'listings'));
        }
        $this->view->render();
    }

    /**
     * Show export extended form by ajax
     *
     * @return void
     */
    public function ajax_get_export_extend_form()
    {
        $operation_type = $this->input->post('type', true);
        $category_id = $this->input->post('id_category');
        $category_id = explode('_', $category_id);
        $data = array('operation_type' => $operation_type, 'id_category' => $category_id[0]);
        $property_type_gid = $this->Listings_model->get_field_editor_type($data);

        $this->load->model('listings/models/Listings_export_model');

        $this->view->output($this->Listings_export_model->get_export_extend_form($property_type_gid, 'user_export_form_gid', false));
        $this->view->render();
    }

    /**
     * Show found listings search
     *
     * @param string $keyword keyword string
     * @param integer $operation_type listing operation type
     * @param string $order order value
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function search($keyword, $operation_type = null, $order = 'default', $order_direction = 'DESC', $page = 1)
    {
        $_SESSION['listings_search']['data']['keyword'] = trim(strip_tags($keyword));
        $this->_list($operation_type, $order, $order_direction, $page, $_SESSION['listings_search']);
    }

    /**
     * Show found listings by ajax
     *
     * @return void
     */
    public function ajax_search()
    {
        $current_settings = isset($_SESSION['listings_search']) ? $_SESSION['listings_search'] : array();
        $current_settings['order'] = 'default';
        $current_settings['order_direction'] = 'DESC';
        $current_settings['page'] = 1;

        $new = $this->input->post('new');
        if ($new) {
            $current_settings['data'] = array();
        }

        $data = $this->input->post('filters', true);

        // operation type
        $operation_types = $this->Listings_model->get_operation_types(true);
        if (isset($data['type'])) {
            if (in_array($data['type'], $operation_types)) {
                $current_settings['data']['type'] = $data['type'];
            }
            unset($data['type']);
        }

        // category
        if (isset($data['category'])) {
            $category_chunks = explode('_', $data['category']);
            if ($category_chunks[0]) {
                if (isset($current_settings['data']['field_editor_data'])) {
                    $this->load->model('field_editor/models/Field_editor_forms_model');

                    if (isset($current_settings['data']['id_category']) && $current_settings['data']['id_category'] != $category_chunks[0]) {
                        unset($current_settings['data']['field_editor_data']);
                    } else {
                        $property_type_gid = $this->Listings_model->get_field_editor_type(array('type' => $current_settings['data']['type'], 'id_category' => $current_settings['data']['id_category'])); //field_editor_types[$current_settings['data']['id_category']];

                        $form_gids = array();
                        $this->load->model('listings/models/Listings_form_model');
                        foreach (array_keys($current_settings['data']['field_editor_data']) as $form_name) {
                            $form_gids[$form_name] = $this->Listings_form_model->{$form_name . '_' . $property_type_gid . '_gid'};
                        }

                        $params = array();
                        $params['where']['editor_type_gid'] = $property_type_gid;
                        $params['where_in']['gid'] = $form_gids;

                        $forms = $this->Field_editor_forms_model->get_forms_list($params);
                        $forms = array_intersect_key(array_flip($form_gids), $forms);

                        foreach ($forms as $form) {
                            unset($current_settings['data']['field_editor_data'][$form]);
                        }
                        if (empty($current_settings['data']['field_editor_data'])) {
                            unset($current_settings['data']['field_editor_data']);
                        }
                    }
                }
                $current_settings['data']['id_category'] = $category_chunks[0];
            } else {
                if (isset($current_settings['data']['id_category'])) {
                    unset($current_settings['data']['id_category']);
                }
                if (isset($current_settings['data']['property_type'])) {
                    unset($current_settings['data']['property_type']);
                }

                if (isset($current_settings['data']['field_editor_data'])) {
                    $this->load->model('field_editor/models/Field_editor_forms_model');

                    if (isset($current_settings['data']['id_category'])) {
                        $category_id = $current_settings['data']['id_category'];
                    } else {
                        $category_id = 0;
                    }

                    $field_data = array('operation_type' => $current_settings['data']['type'], 'id_category' => $category_id);
                    $property_type_gid = $this->Listings_model->get_field_editor_type($field_data);

                    $form_gids = array();
                    foreach (array_keys($current_settings['data']['field_editor_data']) as $form_name) {
                        $form_gids[$form_name] = $this->Listings_model->{$form_name . '_' . $property_type_gid . '_gid'};
                    }

                    $params = array();
                    $params['where']['editor_type_gid'] = $property_type_gid;
                    $params['where_in']['gid'] = $form_gids;

                    $forms = $this->Field_editor_forms_model->get_forms_list($params);
                    $forms = array_intersect_key(array_flip($form_gids), $forms);

                    foreach ($forms as $form) {
                        unset($current_settings['data']['field_editor_data'][$form]);
                    }
                    if (empty($current_settings['data']['field_editor_data'])) {
                        unset($current_settings['data']['field_editor_data']);
                    }
                }
            }
            if (count($category_chunks) > 1 && $category_chunks[1]) {
                $current_settings['data']['property_type'] = $category_chunks[1];
            } else {
                if (isset($current_settings['data']['property_type'])) {
                    unset($current_settings['data']['property_type']);
                }
            }
            unset($data['category']);
        }

        if (isset($current_settings['data']['type'])) {
            $operation_type = $current_settings['data']['type'];
        } else {
            $operation_type = $this->Listings_model->get_operation_type_gid_default();
        }

        $this->load->model('listings/models/Listings_module_model');

        //price
        $price_ranges = $this->Listings_module_model->get_price_range();
        $price_range = isset($price_ranges[$operation_type]) ? $price_ranges[$operation_type] : array('max_price' => 0);

        if (isset($data['price_max'])) {
            if (!empty($data['price_max']) && $data['price_max'] < $price_range['max_price']) {
                $current_settings['data']['price_max'] = $data['price_max'];
            } else {
                if (isset($current_settings['data']['price_max'])) {
                    unset($current_settings['data']['price_max']);
                }
            }
            unset($data['price_max']);
        }

        if (isset($data['price'])) {
            if (!empty($data['price'])) {
                $chunks = explode('-', $data['price']);
                if ($chunks[0] > 0) {
                    $current_settings['data']['price_min'] = $chunks[0];
                } else {
                    if (isset($current_settings['data']['price_min'])) {
                        unset($current_settings['data']['price_min']);
                    }
                }
                if (count($chunks) > 1 && $chunks[1] > 0 && $chunks[1] < $price_range['max_price']) {
                    $current_settings['data']['price_max'] = $chunks[1];
                } else {
                    if (isset($current_settings['data']['price_max'])) {
                        unset($current_settings['data']['price_max']);
                    }
                }
            } else {
                if (isset($current_settings['data']['price_min'])) {
                    unset($current_settings['data']['price_min']);
                }
                if (isset($current_settings['data']['price_max'])) {
                    unset($current_settings['data']['price_max']);
                }
            }
            unset($data['price']);
        }

        //square
        $square_ranges = $this->Listings_module_model->get_square_range();
        $square_range = isset($square_ranges[$operation_type]) ? $square_ranges[$operation_type] : array('max_square' => 0);

        if (isset($data['square_max'])) {
            if (!empty($data['square_max']) && $data['square_max'] < $square_range['max_square']) {
                $current_settings['data']['square_max'] = $data['square_max'];
            } else {
                if (isset($current_settings['data']['square_max'])) {
                    unset($current_settings['data']['square_max']);
                }
            }
            unset($data['square_max']);
        }

        if (isset($data['square'])) {
            if (!empty($data['square'])) {
                $chunks = explode('-', $data['square']);
                if ($chunks[0] > 0) {
                    $current_settings['data']['square_min'] = $chunks[0];
                } else {
                    if (isset($current_settings['data']['square_min'])) {
                        unset($current_settings['data']['square_min']);
                    }
                }
                if (count($chunks) > 1 && $chunks[1] > 0 && $chunks[1] < $square_range['max_square']) {
                    $current_settings['data']['square_max'] = $chunks[1];
                } else {
                    if (isset($current_settings['data']['square_max'])) {
                        unset($current_settings['data']['square_max']);
                    }
                }
            } else {
                if (isset($current_settings['data']['square_min'])) {
                    unset($current_settings['data']['square_min']);
                }
                if (isset($current_settings['data']['square_max'])) {
                    unset($current_settings['data']['square_max']);
                }
            }
            unset($data['square']);
        }

        //booking
        if (in_array($operation_type, array(Listings_model::OPERATION_RENT, Listings_model::OPERATION_LEASE))) {
            if (isset($data['booking_date_start'])) {
                $value = strtotime($data['booking_date_start']);
                if ($value > 0) {
                    $current_settings['data']['booking_date_start'] = date('Y-m-d', $value);
                }
                unset($data['booking_date_start']);
            }

            if (isset($data['booking_date_end'])) {
                $value = strtotime($data['booking_date_end']);
                if ($value > 0) {
                    $current_settings['data']['booking_date_end'] = date('Y-m-d', $value);
                }
                unset($data['booking_date_end']);
            }
        } else {
            if (isset($data['booking_date_start'])) {
                unset($data['booking_date_start']);
            }
            if (isset($data['booking_date_end'])) {
                unset($data['booking_date_end']);
            }
            if (isset($data['booking_guests'])) {
                unset($data['booking_guests']);
            }
            if (isset($current_settings['data']['booking_date_start'])) {
                unset($current_settings['data']['booking_date_start']);
            }
            if (isset($current_settings['data']['booking_date_end'])) {
                unset($current_settings['data']['booking_date_end']);
            }
            if (isset($current_settings['data']['booking_guests'])) {
                unset($current_settings['data']['booking_guests']);
            }
        }

        //location
        $region_name = $this->input->post('region_name', true);
        if (isset($region_name) && empty($data['id_country']) && empty($data['id_region']) && empty($data['id_city']) && empty($data['id_district'])) {
            if (isset($data['keyword'])) {
                $data['keyword'] .= ' ' . $region_name;
            } else {
                $data['keyword'] = $region_name;
            }
        }

        //keyword
        if (isset($data['keyword'])) {
            $data['keyword'] = trim(strip_tags($data['keyword']));
            if (!empty($data['keyword'])) {
                $current_settings['data']['keyword'] = $data['keyword'];
            } else {
                unset($current_settings['data']['keyword']);
            }
            unset($data['keywords']);
        }

        // field editor
        $form = $this->input->post('form', true);

        if ($form) {
            $data['field_editor_data'][$form] = $this->input->post('data', true);
            if (!is_array($data['field_editor_data'][$form])) {
                $data['field_editor_data'][$form] = array();
            }
            $field_editor_range = $this->input->post('field_editor_range', true);
            if (!is_array($field_editor_range)) {
                $field_editor_range = array();
            }
            $short_value = $this->input->post('short_value', true);

            foreach ($data['field_editor_data'][$form] as $field_gid => $field_value) {
                if (isset($field_value['range']['min'])) {
                    if (isset($field_editor_range[$field_gid]['min'])) {
                        $min_value = $field_editor_range[$field_gid]['min'];
                    } else {
                        $min_value = 0;
                    }
                    if ($field_value['range']['min'] <= $min_value) {
                        unset($data['field_editor_data'][$form][$field_gid]['range']['min']);
                    }
                }
                if (isset($field_value['range']['max'])) {
                    if (isset($field_editor_range[$field_gid]['max'])) {
                        $max_value = $field_editor_range[$field_gid]['max'];
                        if ($field_value['range']['max'] >= $max_value) {
                            unset($data['field_editor_data'][$form][$field_gid]['range']['max']);
                        }
                    } elseif (empty($field_value['range']['max'])) {
                        unset($data['field_editor_data'][$form][$field_gid]['range']['max']);
                    }
                }
                if (isset($data['field_editor_data'][$form][$field_gid]['range']) &&
                    empty($data['field_editor_data'][$form][$field_gid]['range'])) {
                    unset($data['field_editor_data'][$field_gid]['range']);
                }
            }
            if (isset($current_settings['data']['field_editor_data'])) {
                foreach ($current_settings['data']['field_editor_data'] as $form_name => $form_data) {
                    if ($form_name == $form) {
                        continue;
                    }
                    $data['field_editor_data'][$form_name] = array_diff_key($form_data, $data['field_editor_data'][$form]);
                    if (empty($data['field_editor_data'][$form_name])) {
                        unset($data['field_editor_data'][$form_name]);
                    }
                }
            }
            foreach ($data['field_editor_data'][$form] as $field_gid => $field_data) {
                if (empty($field_data)) {
                    unset($data['field_editor_data'][$form][$field_gid]);
                } elseif (is_array($field_data)) {
                    $empty = true;
                    foreach ($field_data as $value) {
                        if (!empty($value)) {
                            $empty = false;
                            break;
                        }
                    }
                    if ($empty) {
                        unset($data['field_editor_data'][$form][$field_gid]);
                    }
                }
            }
            if (empty($data['field_editor_data'][$form])) {
                unset($data['field_editor_data'][$form]);
            }
        }
        foreach ($data as $key => $value) {
            if (!empty($value)) {
                $current_settings['data'][$key] = $value;
            } elseif (isset($current_settings['data'][$key])) {
                unset($current_settings['data'][$key]);
            }
        }
        $_SESSION['listings_search'] = $current_settings;

        $this->view->output($this->getNewLink());
        $this->view->render();
    }

    /**
     * Number of found listings
     *
     * @return void
     */
    public function ajax_search_counts()
    {
        $result = array('count' => 0, 'error' => '', 'string' => '');
        $filters = isset($_SESSION['listings_search']['data']) ? $_SESSION['listings_search']['data'] : array();
        $filters['active'] = '1';
        if ($this->session->userdata('auth_type') == 'user') {
            $filters['not_user'] = $this->session->userdata('user_id');
        }
        $result['count'] = $this->Listings_model->get_listings_count($filters);
        $this->view->assign($result);
        $this->view->render();
    }

    /**
     * Rewrite link for catalogue or search page
     *
     * @return void
     */
    public function ajax_get_search_link()
    {
        $this->load->helper('seo');

        //load properties
        $this->load->model('Properties_model');
        $property_types = $this->Properties_model->get_categories('property_types', $this->pg_language->current_lang_id);
        $this->view->assign('property_types', $property_types);
        $property_items = $this->Properties_model->get_all_properties('property_types', $this->pg_language->current_lang_id);
        $this->view->assign('property_items', $property_items);

        $data = array(
            'id_category' => 0,
            'category' => 'all',
            'property_type' => 0,
            'property' => '',
            'order' => 'default',
            'order_direction' => 'DESC',
            'page' => 0,
        );

        if (isset($_SESSION['listings_search']) && !empty($_SESSION['listings_search'])) {
            $data['order'] = $_SESSION['listings_search']['order'];
            $data['order_direction'] = $_SESSION['listings_search']['order_direction'];
            if (isset($_SESSION['listings_search']['data']['type'])) {
                $data['type'] = $_SESSION['listings_search']['data']['type'];
            }
            if (isset($_SESSION['listings_search']['data']['id_category'])) {
                $data['id_category'] = $_SESSION['listings_search']['data']['id_category'];
                $data['category'] = $property_types[$this->Listings_model->field_editor_types[$data['id_category']]];
                if (isset($_SESSION['listings_search']['data']['property_type'])) {
                    $data['property_type'] = $_SESSION['listings_search']['data']['property_type'];
                    $data['property'] = $property_items[$data['id_category']][$data['property_type']];
                }
            }
            if (isset($_SESSION['listings_search']['data']['keyword'])) {
                $data['keyword'] = $_SESSION['listings_search']['data']['keyword'];
            }
        }

        $post_data = (array) $this->input->post('filters', true);

        $operation_types = $this->Listings_model->get_operation_types(true);
        if (isset($post_data['operation_type']) && in_array($post_data['operation_type'], $operation_types)) {
            $data['operation_type'] = $post_data['operation_type'];
        } else {
            $data['operation_type'] = current($operation_types);
        }

        if (isset($post_data['id_category'])) {
            $data['id_category'] = $post_data['id_category'];
            $data['category'] = $property_types[$post_data['id_category']];
            if (isset($post_data['property_type']) && $post_data['property_type']) {
                $data['property_type'] = $post_data['property_type'];
                $data['property'] = $property_items[$post_data['id_category']][$post_data['property_type']];
            }
        }

        if (isset($post_data['category']) && !empty($post_data['category'])) {
            $category_chunks = explode('_', $post_data['category']);
            $data['id_category'] = $category_chunks[0];
            if (count($category_chunks) > 1) {
                $data['property_type'] = $category_chunks[1];
            }
        }

        if (isset($post_data['keyword']) && !empty($post_data['keyword'])) {
            $data['keyword'] = trim(strip_tags($post_data['keyword']));
        } else {
            $region_name = $this->input->post('region_name', true);
            $region_name = trim($region_name);
            if (!empty($region_name)) {
                $data['keyword'] = $region_name;
            }
        }

        if (!empty($data['keyword'])) {
            $url = rewrite_link('listings', 'search', $data);
        } else {
            $url = rewrite_link('listings', 'index', $data);
        }
        $this->view->output($url);
        $this->view->render();
    }

    /**
     * Remove search criteria
     *
     * @param string $gid filter guid
     * @param $field_editor_gid filter guid of field editor
     * @return void
     */
    public function delete_search_criteria($gid, $field_editor_gid = '')
    {
        $delete = array($gid);
        switch ($gid) {
            case 'id_category':
            case 'property_type':
                $delete[] = 'id_category';
                $delete[] = 'property_type';
                break;
            case 'price':
                $delete[] = 'price_min';
                $delete[] = 'price_max';
                break;
            case 'square':
                $delete[] = 'square_min';
                $delete[] = 'square_max';
                break;
            case 'field_editor':
                if (!isset($_SESSION['listings_search']['data']['field_editor_data'])) {
                    break;
                }
                foreach ($_SESSION['listings_search']['data']['field_editor_data'] as $form_name => $form_data) {
                    if (isset($form_data[$field_editor_gid])) {
                        unset($form_data[$field_editor_gid]);
                        if (empty($form_data)) {
                            unset($_SESSION['listings_search']['data']['field_editor_data'][$form_name]);
                            if (empty($_SESSION['listings_search']['data']['field_editor_data'])) {
                                unset($_SESSION['listings_search']['data']['field_editor_data']);
                            }
                        } else {
                            $_SESSION['listings_search']['data']['field_editor_data'][$form_name] = $form_data;
                        }
                        break;
                    }
                }
                break;
            default:
                $delete[] = $gid;
                break;
        }
        foreach ($delete as $field_gid) {
            if (isset($_SESSION['listings_search']['data'][$field_gid])) {
                unset($_SESSION['listings_search']['data'][$field_gid]);
            }
        }
        $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_delete_search_criteria', 'listings'));

        //search seo
        $this->load->helper('seo');

        $search_action_data = $this->Listings_model->get_search_link_data();

        unset($search_action_data['page']);

        $search_link = rewrite_link('listings', !empty($search_action_data['keyword']) ? 'search' : 'index', $search_action_data);
        $this->setRedirect($search_link);
    }

    /**
     * Remove search criteria by ajax
     *
     * @param string $gid filter guid
     * @param string $field_editor_gid filter guid of field editor
     * @return void
     */
    public function ajax_delete_search_criteria($gid, $field_editor_gid = '')
    {
        $return = array('error' => '', 'success' => '', 'data' => '');

        $delete = array();

        switch ($gid) {
            case 'id_category':
            case 'property_type':
                $delete[] = 'id_category';
                $delete[] = 'property_type';
                break;
            case 'price':
                $delete[] = 'price_min';
                $delete[] = 'price_max';
                break;
            case 'square':
                $delete[] = 'square_min';
                $delete[] = 'square_max';
                break;
            case 'location':
                $delete[] = 'id_country';
                $delete[] = 'id_region';
                $delete[] = 'id_city';
                $delete[] = 'id_district';
            case 'field_editor':
                if (!isset($_SESSION['listings_search']['data']['field_editor_data']))
                    break;
                foreach ($_SESSION['listings_search']['data']['field_editor_data'] as $form_name => $form_data) {
                    if (isset($form_data[$field_editor_gid])) {
                        unset($form_data[$field_editor_gid]);
                        if (empty($form_data)) {
                            unset($_SESSION['listings_search']['data']['field_editor_data'][$form_name]);
                            if (empty($_SESSION['listings_search']['data']['field_editor_data'])) {
                                unset($_SESSION['listings_search']['data']['field_editor_data']);
                            }
                        } else {
                            $_SESSION['listings_search']['data']['field_editor_data'][$form_name] = $form_data;
                        }
                        break;
                    }
                }
                break;
            default:
                $delete[] = $gid;
        }
        foreach ($delete as $field_gid) {
            if (isset($_SESSION['listings_search']['data'][$field_gid])) {
                unset($_SESSION['listings_search']['data'][$field_gid]);
            }
        }

        if (!isset($_SESSION['listings_search']['data']['type'])) {
            $operation_types = $this->Listings_model->get_operation_types(true);
            $_SESSION['listings_search']['data']['type'] = current($operation_types);
        }

        $current_search_data = $this->Listings_model->get_search_data();
        $this->view->assign('current_search_data', $current_search_data);

        $search_filters_block = $this->view->fetch('search_filters_block', 'user', 'listings');
        $this->view->assign('search_filters_block', $search_filters_block);

        $return['data'] = $this->view->fetch('search_filters_block', 'user', 'listings');
        $return['type'] = $_SESSION['listings_search']['data']['type'];
        $return['success'] = l('success_delete_search_criteria', 'listings');
        $this->view->assign($return);
        $this->view->render();
    }

    /**
     * Show main search form by ajax
     *
     * @param string $operation_type operation type
     * @param integer $type form type
     * @return void
     */
    public function ajax_search_form($operation_type = null, $type = 'short')
    {
        if ($operation_type == 'sold') {
            $operation_type = 'sale';
        }
        
        $operation_types = $this->Listings_model->get_operation_types(true);
        if (!$operation_type || !in_array($operation_type, $operation_types)) {
            $operation_type = $this->Listings_model->get_operation_type_gid_default();
        }

        $_SESSION['listings_search']['data']['type'] = $operation_type;
        $this->load->helper('listings');

        $rand = $this->input->post('rand', true);
        if (!$rand) {
            $rand = rand(100000, 999999);
        }

        $show_data = (bool) $this->input->post('show_data', true);

        $width = intval($this->input->post('width', true));
        if (!$width) {
            $width = 100;
        }

        $page_data = array(
            'object' => $operation_type,
            'rand' => $rand,
        );
        $this->view->assign('form_settings', $page_data);

        $form_block = listings_main_search_form($type, $show_data, $width, $operation_type);
        $this->view->output($form_block);
        $this->view->render();
    }

    /**
     * Save search criteria
     *
     * @return void
     */
    public function save_search()
    {
        if (strpos($_SERVER['HTTP_REFERER'], site_url()) !== false) {
            $url = $_SERVER['HTTP_REFERER'];
        } else {
            $this->load->helper('seo');
            $search_action_data = $this->get_search_link_data();
            $url = rewrite_link('listings', $search_action_data['keyword'] ? 'search' : 'index', $search_action_data);
        }

        $user_id = $this->session->userdata('user_id');
        if (!$user_id) {
            $this->system_messages->addMessage(View::MSG_ERROR, l('error_empty_user_search_save', 'listings'));
            $this->view->setRedirect($url);
        }

        $current_settings = isset($_SESSION['listings_search']) ? $_SESSION['listings_search'] : array();
        $data = (!empty($current_settings['data'])) ? $current_settings['data'] : array();

        if (empty($data)) {
            $this->system_messages->addMessage(View::MSG_ERROR, l('error_empty_search_save', 'listings'));
            $this->view->setRedirect($url);
        }

        $save_data = array(
            'id_user' => $user_id,
            'search_data' => $data
        );

        $this->load->model('listings/models/Listings_search_model');
        $validate_data = $this->Listings_search_model->validateSearch(null, $save_data, true);

        if (!empty($validate_data['errors'])) {
            $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
        } else {
            $this->Listings_search_model->saveSearch(null, $validate_data['data']);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_save_search', 'listings'));
        }

        $this->view->setRedirect($url);
    }

    /**
     * Save search criteria by ajax
     *
     * @return void
     */
    public function ajax_save_search()
    {
        $return = array('success' => false, 'error' => '', 'data' => '');

        $user_id = $this->session->userdata('user_id');
        if (!$user_id) {
            $return['error'] = l('error_empty_user_search_save', 'listings');
            $this->view->assign($return);
            $this->view->render();
            return;
        }

        $current_settings = isset($_SESSION['listings_search']) ? $_SESSION['listings_search'] : array();
        $data = (!empty($current_settings['data'])) ? $current_settings['data'] : array();

        if (empty($data)) {
            $return['error'] = l('error_empty_search_save', 'listings');
            $this->view->assign($return);
            $this->view->render();
            return;
        }

        $save_data = array(
            'id_user' => $user_id,
            'search_data' => $data
        );

        $this->load->model('listings/models/Listings_search_model');
        $validate_data = $this->Listings_search_model->validateSearch(null, $save_data, true);

        if (!empty($validate_data['errors'])) {
            $return['error'] = implode('<br>', $validate_data['errors']);
        } else {
            $this->Listings_search_model->saveSearch(null, $validate_data['data']);

            $filters = array('user' => $user_id);
            $save_search_items_on_page = $this->pg_module->get_module_config('listings', 'save_search_items_per_page');
            $save_search_data = $this->Listings_search_model->getSearchesList($filters, 1, $save_search_items_on_page, array('id' => 'DESC'));
            $this->view->assign('save_search_data', $save_search_data);
            $return['data'] = $this->view->fetch('saved_searches_block', 'user', 'listings');

            $return['success'] = l('success_save_search', 'listings');
        }
        $this->view->assign($return);
        $this->view->render();
        return;
    }

    /**
     * My saved searches
     *
     * @param integer $page page of results
     * @return void
     */
    public function preferences($page = 1)
    {
        $this->load->model('listings/models/Listings_search_model');

        $user_id = $this->session->userdata('user_id');
        $user_type = $this->session->userdata('user_type');

        $items_on_page = $this->pg_module->get_module_config('start', 'index_items_per_page');

        $params['user'] = $user_id;

        $searches_count = $this->Listings_search_model->getSearchesCount($params);
        if ($searches_count) {
            $searches = $this->Listings_search_model->getSearchesList($params, $page, $items_on_page, array('id' => 'DESC'));
            $this->view->assign('searches', $searches);
        }

        $this->load->helper('navigation');
        $url = site_url() . 'listings/preferences/';
        $page_data = get_user_pages_data($url, $searches_count, $items_on_page, $page, 'briefPage');
        $this->view->assign('page_data', $page_data);

        $this->Menu_model->set_menu_active_item($user_type . '_account_menu', 'my_searches_item');
        $this->Menu_model->breadcrumbs_set_parent($user_type . '_my_searches_item');

        $this->view->render('listings_searches_list');
    }

    /**
     * Load saved search
     *
     * @param integer $search_id saved search identifier
     * @return void
     */
    public function load_saved_search($search_id)
    {
        $user_id = $this->session->userdata('user_id');

        $this->load->model('listings/models/Listings_search_model');

        $search = $this->Listings_search_model->getSearchById($search_id);
        if ($search['id_user'] != $user_id) {
            show_404();
            return;
        }

        $current_settings = isset($_SESSION['listings_search']) ? $_SESSION['listings_search'] : array();
        $current_settings['data'] = $search['search_data'];

        $_SESSION['listings_search'] = $current_settings;

        $this->load->helper('seo');

        $search_action_data = $this->Listings_model->get_search_link_data();

        unset($search_action_data['page']);

        $search_link = rewrite_link('listings', $search_action_data['keyword'] ? 'search' : 'index', $search_action_data);

        $this->view->setRedirect($search_link);
    }

    /**
     * Remove saved search
     *
     * @param integer $search_id saved search identifier
     * @return void
     */
    public function delete_saved_search($search_id)
    {
        $this->load->model('listings/models/Listings_search_model');
        $user_id = $this->session->userdata('user_id');
        $search = $this->Listings_search_model->getSearchById($search_id);
        if ($search['id_user'] != $user_id) {
            show_404();
            return;
        }
        $this->Listings_search_model->deleteSearch($search_id);
        $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_delete_search', 'listings'));
        $this->view->setRedirect(site_url() . 'listings/preferences');
    }

    private function getNewLink()
    {
        $this->load->model('Listings_model');
        $search_action_data = $this->Listings_model->get_search_link_data();

        unset($search_action_data['order']);
        unset($search_action_data['order_direction']);
        unset($search_action_data['page']);

        $this->load->helper('seo');
        return rewrite_link('listings', $search_action_data['keyword'] ? 'search' : 'index', $search_action_data);
    }

    public function ajax_reset_search_data()
    {
        unset($_SESSION['listings_search']);
        $this->view->output($this->getNewLink());
        $this->view->render();
    }

    /**
     * Remove saved search by ajax
     *
     * @param integer $search_id saved search identifier
     * @return void
     */
    public function ajax_delete_saved_search($search_id)
    {
        $return = array('error' => '', 'success' => '', 'data' => '');
        if ($this->session->userdata('auth_type') != 'user') {
            $return['error'] = l('error_empty_user', 'listings');
        } else {
            $this->load->model('listings/models/Listings_search_model');
            $user_id = $this->session->userdata('user_id');
            $search = $this->Listings_search_model->getSearchById($search_id);
            if ($search['id_user'] != $user_id) {
                $return['error'] = l('error_not_access', 'listings');
                $this->view->assign($return);
                $this->view->render();
                return;
            }
            $this->Listings_search_model->deleteSearch($search_id);
            $filters = array('user' => $user_id);
            $save_search_items_on_page = $this->pg_module->get_module_config('listings', 'save_search_items_per_page');
            $save_search_data = $this->Listings_search_model->getSearchesList($filters, 1, $save_search_items_on_page, array('id' => 'DESC'));
            $this->view->assign('save_search_data', $save_search_data);
            $return['data'] = $this->view->fetch('saved_searches_block', 'user', 'listings');
            $return['success'] = l('success_delete_search', 'listings');
        }
        $this->view->assign($return);
        $this->view->render();
    }

    public function ajax_get_save_search_block()
    {
        $this->load->helper('listings');
        $this->view->assign(listings_save_search_block());
        $this->view->render();
    }

    /**
     * Block of my saved listings
     *
     * @param string $operation_type operation type
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    protected function _saved_listings_list_block($operation_type, $order = 'default', $order_direction = 'DESC', $page = 1)
    {
        $current_settings = isset($_SESSION['saved_list']) ? $_SESSION['saved_list'] : array();

        if (!isset($current_settings['type'])) {
            $current_settings['type'] = $this->Listings_model->get_operation_type_gid_default();
        }
        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'modified';
        }
        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        if (!isset($current_settings['page'])) {
            $current_settings['page'] = 1;
        }

        if ($current_settings['type'] != $operation_type) {
            $current_settings['type'] = $operation_type;
            $page = 1;
        }

        if (!empty($order)) {
            $current_settings['order'] = $order;
        }
        $order = $current_settings['order'];
        $this->view->assign('order', $current_settings['order']);

        if (!empty($order_direction)) {
            $current_settings['order_direction'] = $order_direction;
        }
        $order_direction = $current_settings['order_direction'];
        $this->view->assign('order_direction', $order_direction);

        $page = intval($page);
        if (!empty($page)) {
            $current_settings['page'] = $page;
        }
        $page = $current_settings['page'];

        $_SESSION['saved_list'] = $current_settings;

        $items_on_page = $this->pg_module->get_module_config('start', 'index_items_per_page');

        if ($this->session->userdata('auth_type') == 'user') {
            $user_id = $this->session->userdata('user_id');
            $saved_listings = $this->session->userdata('saved_listings');
            if (!isset($saved_listings) || empty($saved_listings)) {
                $this->load->model('listings/models/Saved_listings_model');
                $list = $this->Saved_listings_model->get_saved_list_by_user((int) $user_id);
                $saved_listings = array();
                foreach ($list as $item) {
                    $saved_listings[] = $item['id_listing'];
                }
                $this->session->set_userdata('saved_listings', $saved_listings);
            }
        } elseif (isset($_SESSION['saved_listings'])) {
            $saved_listings = $_SESSION['saved_listings'];
        } else {
            $saved_listings = array();
        }

        if (!empty($saved_listings)) {
            $items_count = array();
            $filters['ids'] = $saved_listings;
            $filters['active'] = 1;
            $filters['type'] = $operation_type;
            $items_count = $this->Listings_model->get_listings_count($filters);
        } else {
            $items_count = 0;
        }

        $this->load->helper('sort_order');
        $page = get_exists_page_number($page, $items_count, $items_on_page);

        $sort_data = array(
            'url' => site_url() . 'listings/saved/' . $operation_type,
            'order' => $order,
            'direction' => $order_direction,
            'links' => array(
                'views' => l('field_views', 'listings'),
                'price' => l('field_price', 'listings'),
                'open' => l('field_date_open', 'listings'),
                'modified' => l('field_date_modified', 'listings'),
            )
        );
        if ($this->pg_module->is_module_active('reviews')) {
            $this->load->helper('reviews');
            $review_link = get_reviews_sorter();
            if (is_array($review_link)) {
                $first = array_splice($sort_data['links'], 0, 1);
                $sort_data['links'] = array_merge($first, $review_link, $sort_data['links']);
            }
        }
        $this->view->assign('sort_data', $sort_data);

        $use_leader = false;
        if ($items_count > 0) {
            $order_array = array();
            if ($order == 'price') {
                $order_array['price_sorting'] = $order_direction;
            } elseif ($order == 'open') {
                $order_array['date_open'] = $order_direction;
            } elseif ($order == 'modified') {
                $order_array['date_modified'] = $order_direction;
            } else {
                $order_array[$order] = $order_direction;
            }
            $listings = $this->Listings_model->get_listings_list($filters, $page, $items_on_page, $order_array);
            $this->view->assign('listings', $listings);
        } else {
            $this->view->assign('listings', array());
        }


        $this->load->helper('navigation');
        $url = site_url() . 'listings/saved/' . $operation_type . '/' . $order . '/' . $order_direction . '/';
        $page_data = get_user_pages_data($url, $items_count, $items_on_page, $page, 'briefPage');

        $page_data['use_leader'] = $use_leader;

        $this->view->assign('page_data', $page_data);

        $use_save_search = false;
        $this->view->assign('use_save_search', $use_save_search);

        return $this->view->fetch('saved_listings_block', 'user', 'listings');
    }

    /**
     * My saved listings
     *
     * @param strung $operation_type operation type
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function saved($operation_type = null, $order = null, $order_direction = null, $page = 1)
    {
        $operation_types = $this->Listings_model->get_operation_types(true);
        $this->view->assign('operation_types', $operation_types);

        if (!in_array($operation_type, $operation_types)) {
            $operation_type = $this->Listings_model->get_operation_type_gid_default();
        }
        $this->view->assign('current_operation_type', $operation_type);

        $block = $this->_saved_listings_list_block($operation_type, $order, $order_direction, $page);
        $this->view->assign('block', $block);

        if ($this->session->userdata('auth_type') == 'user') {
            $user_id = $this->session->userdata('user_id');
            $user_type = $this->session->userdata('user_type');
            $saved_listings = $this->session->userdata('saved_listings');
            if (!isset($saved_listings) || empty($saved_listings)) {
                $this->load->model('listings/models/Saved_listings_model');
                $list = $this->Saved_listings_model->get_saved_list_by_user((int) $user_id);
                $saved_listings = array();
                foreach ($list as $item) {
                    $saved_listings[] = $item['id_listing'];
                }
                $this->session->set_userdata('saved_listings', $saved_listings);
            }
            $this->Menu_model->breadcrumbs_set_parent($user_type . '_my_saved_listings');
        } else {
            if (isset($_SESSION['saved_listings'])) {
                $saved_listings = $_SESSION['saved_listings'];
            } else {
                $saved_listings = array();
            }
            $this->Menu_model->breadcrumbs_set_active(l('guest_my_saved_listings', 'listings'));
        }

        if (!empty($saved_listings)) {
            $items_count = array();
            $filters['ids'] = $saved_listings;
            $filters['active'] = 1;
            foreach ($operation_types as $operation_type) {
                $filters['type'] = $operation_type;
                $items_count[$operation_type] = $this->Listings_model->get_listings_count($filters);
            }
            $this->view->assign('items_count', $items_count);
            $this->view->assign('saved_count', array_sum($items_count));
        }

        $current_settings = isset($_SESSION['saved_list']) ? $_SESSION['saved_list'] : array();
        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'date_created';
        }
        $this->view->assign('order', $current_settings['order']);

        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        $this->view->assign('order_direction', $current_settings['order_direction']);

        if (!isset($current_settings['page'])) {
            $current_settings['page'] = 1;
        }
        $this->view->assign('page', $current_settings['page']);

        $this->view->render('saved_listings');
    }

    /**
     * My saved listings by ajax
     *
     * @param string $operation_type operation type
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function ajax_saved($operation_type = null, $order = null, $order_direction = null, $page = null)
    {
        $operation_types = $this->Listings_model->get_operation_types(true);

        $current_settings = isset($_SESSION['saved_list']) ? $_SESSION['saved_list'] : array();

        if (!$current_settings['type']) {
            $current_settings['type'] = $this->Listings_model->get_operation_type_gid_default();
        }
        if (!$current_settings['order']) {
            $current_settings['order'] = 'date_modified';
        }
        if (!$current_settings['order_direction']) {
            $current_settings['order_direction'] = 'DESC';
        }
        if (!$current_settings['page']) {
            $current_settings['page'] = 1;
        }

        if (!in_array($operation_type, $operation_types)) {
            $operation_type = $current_settings['type'];
        }
        if (!$order) {
            $order = $current_settings['order'];
        }
        if (!$order_direction) {
            $order_direction = $current_settings['order_direction'];
        }
        if (!$page) {
            $page = $current_settings['page'];
        }

        $this->view->output($this->_saved_listings_list_block($operation_type, $order, $order_direction, $page));
        $this->view->render();
    }

    /**
     * Move listing into saved (internal)
     *
     * @param integer $listing_id listing identifier
     * @return array
     */
    protected function _save_listing($listing_id)
    {
        $return = array('error' => 1, 'success' => 0, 'message' => l('listings_error_saving', 'listings'));
        if (!$listing_id) {
            return $return;
        }

        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);
        if (!$listing) {
            return $return;
        }

        $this->load->model('listings/models/Saved_listings_model');
        if ($this->session->userdata('auth_type') == 'user') {
            $user_id = $this->session->userdata('user_id');
            if ($listing && $listing['id_user'] != $user_id) {
                $saved_listings = $this->session->userdata('saved_listings');
                if (!isset($saved_listings) || empty($saved_listings)) {
                    $list = $this->Saved_listings_model->get_saved_list_by_user((int) $user_id);
                    $saved_listings = array();
                    foreach ($list as $item) {
                        $saved_listings[] = $item['id_listing'];
                        $this->session->set_userdata('saved_listings', $saved_listings);
                    }
                }
                if (!isset($saved_listings) || !in_array($listing_id, $saved_listings)) {
                    $data['id_user'] = $user_id;
                    $data['id_listing'] = $listing_id;
                    $this->Saved_listings_model->save_saved(null, $data);

                    $saved_listings[] = $data['id_listing'];
                    $this->session->set_userdata('saved_listings', $saved_listings);
                }
                $return['error'] = 0;
                $return['success'] = 1;
                $return['id'] = $listing_id;
                $return['message'] = l('success_listings_saved', 'listings');
                $_SESSION['saved_listings'][] = $listing_id;
            } else {
                $return['message'] = l('listings_error_saving', 'listings');
            }
        } else {
            $return['error'] = 0;
            $return['success'] = 1;
            $return['id'] = $listing_id;
            $return['message'] = l('success_listings_saved', 'listings');
            $_SESSION['saved_listings'][] = $listing_id;
        }
        return $return;
    }

    /**
     * Move listing into saved
     *
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function save($listing_id)
    {
        $res = $this->_save_listing($listing_id);
        if ($res['error']) {
            $this->system_messages->addMessage(View::MSG_ERROR, $res['message']);
        } else {
            $this->system_messages->addMessage(View::MSG_SUCCESS, $res['message']);
        }

        $url = site_url() . 'listings/view/' . $listing_id;
        $this->view->setRedirect($url);
    }

    /**
     * Move listing into saved by ajax
     *
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function ajax_save($listing_id)
    {
        $this->view->assign($this->_save_listing($listing_id));
        $this->view->render();
    }

    /**
     * Remove listing from saved (internal)
     *
     * @param integer $listing_id listing identifier
     * @return array
     */
    protected function _delete_saved($listing_id)
    {
        $return = array('error' => 1, 'success' => 0, 'message' => '');
        if ($listing_id) {
            if ($this->session->userdata('auth_type') == 'user') {
                $saved_listings = $this->session->userdata('saved_listings');
                if (!empty($saved_listings)) {
                    $index = array_search($listing_id, $saved_listings);
                    if ($index !== false) {
                        unset($saved_listings[$index]);
                    }
                    $this->session->set_userdata('saved_listings', $saved_listings);
                }
                $user_id = $this->session->userdata('user_id');
                $this->load->model('listings/models/Saved_listings_model');
                $this->Saved_listings_model->delete_saved_by_listing($user_id, $listing_id);
            } elseif (isset($_SESSION['saved_listings'])) {
                $index = array_search($listing_id, $_SESSION['saved_listings']);
                if ($index !== false) {
                    unset($_SESSION['saved_listings'][$index]);
                }
            }
            $return['success'] = 1;
            $return['error'] = 0;
            $return['message'] = l('success_delete_listing', 'listings');
        } else {
            $return['message'] = l('error_with_delete', 'listings');
        }
        return $return;
    }

    /**
     * Remove listing from saved by ajax
     *
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function ajax_delete_saved($listing_id)
    {
        $this->view->assign($this->_delete_saved($listing_id));
        $this->view->render();
    }

    /**
     * Remove listing from saved
     *
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function delete_saved($listing_id)
    {
        $res = $this->_delete_saved($listing_id);
        if ($res['error']) {
            $this->system_messages->addMessage(View::MSG_ERROR, $res['message']);
        } else {
            $this->system_messages->addMessage(View::MSG_SUCCESS, $res['message']);
        }
        $url = site_url() . 'listings/saved';
        $this->view->setRedirect($url);
    }

    /**
     * Show form of share listing
     *
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function share($listing_id)
    {
        $data = $this->Listings_model->get_listing_by_id($listing_id);

        if ($this->input->post('btn_save')) {
            $post_data = array();
            $post_data['email'] = $this->input->post('email', true);
            $post_data['user'] = $this->input->post('user', true);
            $post_data['message'] = $this->input->post('message', true);

            $validate_data = $this->Listings_model->validate_share($listing_id, $post_data);

            if (!empty($validate_data['errors'])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
            } else {
                $email = $validate_data['data']['email'];
                unset($validate_data['data']['email']);

                $this->load->helper('seo');
                $this->load->library('Translit');

                $current_lang = $this->pg_language->get_lang_by_id($this->pg_language->current_lang_id);

                //get listing
                $listing = $this->Listings_model->get_listing_by_id($listing_id);

                $validate_data['data']['listing_id'] = $listing_id;
                $validate_data['data']['listing_name'] = $listing['output_name'];
                $validate_data['data']['listing_owner'] = $listing['user']['output_name'];
                $validate_data['data']['listing_url'] = rewrite_link('listings', 'view', $listing);

                //send mail
                $this->load->model('Notifications_model');
                $from = [
                    'email' => $this->session->userdata('email'),
                    'name' => $this->session->userdata('name'),
                ];
                $this->Notifications_model->send_notification(
                    $email, 'share_listing', $validate_data['data'], '', $this->pg_language->current_lang_id, $from
                );

                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_share_send', 'listings'));

                $url = site_url() . 'listings/share/' . $listing_id;
                $this->view->setRedirect($url);
            }
        }

        $this->view->assign('data', $data);

        $this->load->helper('seo');

        $search_action_data = $this->Listings_model->get_search_link_data();
        $search_link = rewrite_link('listings', $search_action_data['keyword'] ? 'search' : 'index', $search_action_data);
        $this->Menu_model->breadcrumbs_set_active(l('link_search_in_listings', 'listings'), $search_link);

        $view_link = rewrite_link('listings', 'view', $data);
        $this->Menu_model->breadcrumbs_set_active($data['output_name'], $view_link);

        $this->Menu_model->breadcrumbs_set_active(l('header_share', 'listings'), $view_link);

        $this->view->render('share', 'user', 'listings');
    }

    /**
     * Form of share listing by ajax
     *
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function ajax_share_form($listing_id)
    {
        $data = $this->Listings_model->get_listing_by_id($listing_id);
        $this->view->assign('data', $data);

        $is_facebook_enabled = false;
        $is_twitter_enabled = false;

        if ($this->pg_module->is_module_active('social_networking')) {
            $this->load->model('social_networking/models/Social_networking_pages_model');
            $page_data = $this->Social_networking_pages_model->get_page_data('listings', 'share');

            $this->load->model('social_networking/models/Social_networking_services_model');
            $services = $this->Social_networking_services_model->get_services_list(
                    array('id' => 'ASC'), array('where' => array('status' => 1))
            );

            foreach($services as $service) {
                if ($service['gid'] == 'facebook' && !empty($service['app_key'])) {
                    $is_facebook_enabled =
                        !empty($page_data['data']['share']['facebook']) &&
                        $page_data['data']['share']['facebook'] == 'on';
                }

                if ($service['gid'] == 'twitter' && !empty($service['app_key'])) {
                    $is_twitter_enabled =
                        !empty($page_data['data']['share']['twitter']) &&
                        $page_data['data']['share']['twitter'] == 'on';
                }
            }
        }

        $is_guest = $this->session->userdata('auth_type') !== 'user';
        $tabs = [
            'email' => [
                'is_enabled' => !$is_guest,
                'is_active' => !$is_guest,
            ],
            'facebook' => [
                'is_enabled' => $is_facebook_enabled,
                'is_active' => $is_guest && $is_facebook_enabled,
            ],
            'twitter' => [
                'is_enabled' => $is_twitter_enabled,
                'is_active' => $is_guest && !$is_facebook_enabled,
            ],
        ];

        if ($is_guest && !$is_facebook_enabled && !$is_twitter_enabled) {
            $error = &load_class('Exceptions');
            $error->show_403();
        }

        $this->view->assign('tabs', $tabs);
        $this->view->render('ajax_share', 'user', 'listings');
    }

    /**
     * Share listing by mail by ajax
     *
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function ajax_share_mail($listing_id)
    {
        $post_data = array();
        $post_data['email'] = $this->input->post('email', true);
        $post_data['user'] = $this->input->post('user', true);
        $post_data['message'] = $this->input->post('message', true);

        $validate_data = $this->Listings_model->validate_share($listing_id, $post_data);

        if (!empty($validate_data['errors'])) {
            $return['error'] = implode('<br>', $validate_data['errors']);
        } else {
            $email = $validate_data['data']['email'];
            unset($validate_data['data']['email']);

            $this->load->helper('seo');
            $this->load->library('Translit');

            //get listing
            $listing = $this->Listings_model->get_listing_by_id($listing_id);

            $validate_data['data']['listing_id'] = $listing_id;
            $validate_data['data']['listing_name'] = $listing['output_name'];
            $validate_data['data']['listing_owner'] = $listing['user']['output_name'];
            $validate_data['data']['listing_url'] = rewrite_link('listings', 'view', $listing);

            $from = [
                'email' => $this->session->userdata('email'),
                'name' => $this->session->userdata('name'),
            ];
            //send mail
            $this->load->model('Notifications_model');
            $this->Notifications_model->send_notification(
                $email, 'share_listing', $validate_data['data'], '', $this->pg_language->current_lang_id, $from
            );

            $return['success'] = l('success_share_send', 'listings');
        }

        $this->view->assign($return);
        $this->view->render();
        return;
    }

    /**
     * Block of user's listings
     *
     * @param integer $user user data
     * @param string $operation_type operation type
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return string
     */
    protected function _user_listings_block($user, $operation_type, $order = null, $order_direction = null, $page = null, $is_update=false)
    {
        $current_settings = isset($_SESSION['user_listings']) ? $_SESSION['user_listings'] : array();
        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'modified';
        }
        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        if (!isset($current_settings['page'])) {
            $current_settings['page'] = 1;
        }

        $view_mode = isset($_SESSION['listings_view_mode']) ? $_SESSION['listings_view_mode'] : 'list';

        $filters['type'] = $operation_type;
        $filters['user'] = $user['id'];
        $filters['active'] = 1;

        if (!$order) {
            $order = $current_settings['order'];
        }
        $this->view->assign('order', $order);
        $current_settings['order'] = $order;

        if (!$order_direction) {
            $order_direction = $current_settings['order_direction'];
        }
        $this->view->assign('order_direction', $order_direction);
        $current_settings['order_direction'] = $order_direction;

        $page = intval($page);
        if (!$page) {
            $page = $current_settings['page'];
        }

        $items_on_page = $this->pg_module->get_module_config('start', 'index_items_per_page');

        // save criteria for pagination
        $_SESSION['listings_pagination'] = array(
            'order' => $current_settings['order'],
            'order_direction' => $current_settings['order_direction'],
            'page' => $current_settings['page'],
            'data' => $filters,
        );

        $this->load->helper('seo');
        $action_data = array('id_user' => $user['id'], 'user' => $user['name'], 'operation_type' => $operation_type);
        $url = rewrite_link('listings', 'user', $action_data);

        // sorting
        $sort_data = array(
            'url' => $url,
            'order' => $order,
            'direction' => $order_direction,
            'links' => array(
                'views' => l('field_views', 'listings'),
                'price' => l('field_price', 'listings'),
                'open' => l('field_date_open', 'listings'),
                'modified' => l('field_date_modified', 'listings'),
            ),
        );
        if ($this->pg_module->is_module_active('reviews')) {
            $this->load->helper('reviews');
            $review_link = get_reviews_sorter();
            if (is_array($review_link)) {
                $first = array_splice($sort_data['links'], 0, 1);
                $sort_data['links'] = array_merge($first, $review_link, $sort_data['links']);
            }
        }
        $this->view->assign('sort_data', $sort_data);

        $use_map_in_search = $view_mode == 'map';
        $this->view->assign('use_map_in_search', $use_map_in_search);

        $listings_count = $this->Listings_model->get_listings_count($filters);
        if ($listings_count > 0) {
            $order_array = array();
            if ($order == 'price') {
                $order_array['price_sorting'] = $order_direction;
            } elseif ($order == 'open') {
                $order_array['date_open'] = $order_direction;
            } elseif ($order == 'modified') {
                $order_array['date_modified'] = $order_direction;
            } else {
                $order_array[$order] = $order_direction;
            }
            if ($order == 'price') {
                $order_array = array_merge(array('price_negotiated' => $order_direction == 'ASC' ? 'DESC' : 'ASC'), $order_array);
            }
            $this->Listings_model->set_format_settings('get_saved_listings', true);
            $listings = $this->Listings_model->get_listings_list($filters, $page, $items_on_page, $order_array);
            $this->Listings_model->set_format_settings('get_saved_listings', false);
            $this->view->assign('listings', $listings);

            if ($use_map_in_search) {
                $markers = array();
                foreach ($listings as $listing) {
                    if ((float) $listing['lat'] == 0 && (float) $listing['lon'] == 0) {
                        continue;
                    }
                    $this->view->assign('view_mode', $view_mode);
                    $this->view->assign('listing', $listing);
                    $info = $this->view->fetch('listing_map_block', 'user', 'listings');
                    $markers[] = array(
                        'gid' => $listing['id'],
                        'country' => $listing['country'],
                        'region' => $listing['region'],
                        'city' => $listing['city'],
                        'address' => $listing['address'],
                        'postal_code' => $listing['zip'],
                        'lat' => (float) $listing['lat'],
                        'lon' => (float) $listing['lon'],
                        'info' => $info,
                    );
                }
                $this->view->assign('markers', $markers);
            }
        } else {
            $this->view->assign('listings', array());
        }

        if ($use_map_in_search) {
            $current_language = $this->pg_language->get_lang_by_id($this->pg_language->current_lang_id);
            $view_settings = array(
                'lang' => $current_language['code'],
            );
            $this->view->assign('listings_map_settings', $view_settings);
        }

        $this->load->helper('sort_order');
        $page = get_exists_page_number($page, $listings_count, $items_on_page);
        $current_settings['page'] = $page;

        $_SESSION['user_listings'] = $current_settings;

        $this->load->helper('navigation');

        $page_data = get_user_pages_data($url . '/' . $order . '/' . $order_direction . '/', $listings_count, $items_on_page, $page, 'briefPage');
        $this->view->assign('page_data', $page_data);

        $is_owner = 0;
        if ($this->session->userdata('auth_type') == 'user') {
            if ($this->session->userdata('user_id') == $user['id']) {
                $is_owner = 1;
            }
        } else {
            $this->view->assign('is_guest', 1);
        }
        
        $this->view->assign('update_map', $is_update);
        
        $this->view->assign('is_listing_owner', $is_owner);
        $this->view->assign('tstamp', time());
        return $this->view->fetch('listings_block_' . $view_mode, 'user', 'listings');
    }

    /**
     * User's listings
     *
     * @param integer $user_id user identifier
     * @param string $operation_type operation type
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function user($user_id, $operation_type = null, $order = 'modified', $order_direction = 'DESC', $page = 1)
    {
        $this->load->helper('seo');
        $is_users_installed = $this->pg_module->is_module_active("users");
        if ($is_users_installed) {
            $this->load->model('Users_model');
            $this->Users_model->set_format_settings(array('get_contact' => true, 'get_company' => true, 'get_microdata' => true));
            $user = $this->Users_model->get_user_by_id($user_id, true);
            $this->Users_model->set_format_settings(array('get_contact' => false, 'get_company' => false, 'get_microdata' => false));
            $user_view_link = rewrite_link('users', 'view', $user);
        } else {
            $this->load->model('Start_model');
            $user = $this->Start_model->getUserDefaultOnSite();
            $user_view_link = rewrite_link('start', 'startUser');
        }

        if (!$user) {
            show_404();
        }
        $this->view->assign('user', $user);

        $operation_types = $this->Listings_model->get_operation_types(true);
        $this->view->assign('operation_types', $operation_types);

        if (!$operation_type) {
            $operation_type = $this->input->post('type');
        }
        if (!$operation_type || !in_array($operation_type, $operation_types)) {
            $operation_type = $this->Listings_model->get_operation_type_gid_default();
        }
        $this->view->assign('current_operation_type', $operation_type);
        $block_content = $this->_user_listings_block($user, $operation_type, $order, $order_direction, $page);
        $this->view->assign('block', $block_content);
        $this->pg_seo->set_seo_data(array('name' => $user['name'], 'user_type' => $user['user_type'], 'user_type_str' => $user['user_type_str'], 'operation_type' => $operation_type, 'operation_type_str' => l('operation_search_' . $operation_type, 'listings')));
        $menu_action_data = array('id_user' => $user['id'], 'user' => $user['name'], 'operation_type' => '[operation_type]');
        $menu_action_link = rewrite_link('listings', 'user', $menu_action_data);
        $this->view->assign('menu_action_link', $menu_action_link);
        $this->Menu_model->breadcrumbs_set_active($user['output_name'], $user_view_link);
        $this->Menu_model->breadcrumbs_set_active(l('text_user_listing', 'listings'));
        $current_settings = isset($_SESSION['user_listings']) ? $_SESSION['user_listings'] : array();
        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'date_created';
        }
        $this->view->assign('order', $current_settings['order']);
        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        $this->view->assign('order_direction', $current_settings['order_direction']);
        if (!isset($current_settings['page'])) {
            $current_settings['page'] = 1;
        }
        $this->view->assign('page', $current_settings['page']);
        $view_mode = isset($_SESSION['listings_view_mode']) ? $_SESSION['listings_view_mode'] : 'list';
        $this->view->render('user_listings_' . $view_mode, 'user', 'listings');
    }

    /**
     * User's listings by ajax
     *
     * @param integer $user_id user identifier
     * @param string $operation_type operation type
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function ajax_user($user_id, $operation_type, $order = 'modified', $order_direction = 'DESC', $page = 1)
    {
        $is_users_installed = $this->pg_module->is_module_active("users");
        if ($is_users_installed) {
            $this->load->model('Users_model');
            $user = $this->Users_model->get_user_by_id($user_id, true);
        } else {
            $this->load->model('Start_model');
            $user = $this->Start_model->getUserDefaultOnSite();
        }
        if (!$user) {
            return;
        }
        if (!$operation_type) {
            $operation_type = $this->input->post('type');
        }
        $this->view->output($this->_user_listings_block($user, $operation_type, $order, $order_direction, $page, true));
        $this->view->render();
    }

    /**
     * Manage wish lists
     *
     * @param string $order sorting order
     * @param string $order_direction sorting direction
     * @param integer $page page of results
     * @return void
     */
    public function wish_lists($order = 'date_created', $order_direction = 'DESC', $page = 1)
    {
        $this->load->model('listings/models/Wish_list_model');

        $items_on_page = $this->pg_module->get_module_config('start', 'index_items_per_page');

        $filters = array();
        $filters['active'] = 1;

        $wish_lists_count = $this->Wish_list_model->get_wish_lists_count($filters);
        if ($wish_lists_count > 0) {
            $wish_lists = $this->Wish_list_model->get_wish_lists_list($filters, $page, $items_on_page, array($order => $order_direction));
            $this->view->assign('wish_lists', $wish_lists);
        }

        $this->load->helper('navigation');

        $url = site_url() . 'listings/wish_lists/';
        $page_data = get_admin_pages_data($url, $wish_lists_count, $items_on_page, $page, 'briefPage');
        $this->view->assign('page_data', $page_data);

        $this->Menu_model->breadcrumbs_set_active(l('header_wish_list', 'listings'), site_url() . 'listings/wish_lists');
        $this->view->render('wish_lists');
    }

    /**
     * Content of wish list
     *
     * @param integer $wish_list_id wish list identifier
     * @param integer $page page of results
     * @return void
     */
    public function wish_list($wish_list_id, $page = 1)
    {
        $content = $this->_wish_list_content($wish_list_id, $page);
        $this->view->assign('content', $content);

        $wish_list = $this->Wish_list_model->get_wish_list_by_id($wish_list_id, true);

        $this->Menu_model->breadcrumbs_set_active(l('header_wish_list', 'listings'), site_url() . 'listings/wish_lists');
        $this->Menu_model->breadcrumbs_set_active($wish_list['output_name'], site_url() . 'listings/wish_list/' . $wish_list_id);
        $this->pg_seo->set_seo_data($wish_list);
        $this->view->render('wish_list_view');
    }

    /**
     * Render content of wish list by ajax
     *
     * @param integer $wish_list_id wish list identifier
     * @param integer $page page of results
     * @return void
     */
    public function ajax_wish_list($wish_list_id, $page = 1)
    {
        $this->view->output($this->_wish_list_content($wish_list_id, $page));
        $this->view->render();
    }

    /**
     * Block of wish list content
     *
     * @param integer $wish_list_id wish list identifier
     * @param integer $page page of results
     * @return void
     */
    protected function _wish_list_content($wish_list_id, $page = 1)
    {
        $this->load->model('listings/models/Wish_list_model');

        $this->Wish_list_model->set_format_content_settings('get_listings', true);
        $wish_list = $this->Wish_list_model->get_wish_list_by_id($wish_list_id, true);
        $this->Wish_list_model->set_format_content_settings('get_listings', false);
        if (empty($wish_list['status'])) {
            show_404();
        }
        $this->view->assign('wish_list', $wish_list);

        $page = max(intval($page), 1);

        $items_on_page = $this->pg_module->get_module_config('listings', 'rows_per_page') * 4;

        $listings_count = $this->Wish_list_model->get_wish_list_content_count($wish_list_id);
        if ($listings_count > 0) {
            $this->Wish_list_model->set_format_content_settings('get_listings', true);
            $listings = $this->Wish_list_model->get_wish_list_content_list($wish_list_id, $page, $items_on_page);
            $this->Wish_list_model->set_format_content_settings('get_listings', false);
            $this->view->assign('listings', $listings);
        } else {
            $this->view->assign('listings', array());
        }

        $this->load->helper('navigation');

        $url = site_url() . 'listings/wish_list/' . $wish_list_id . '/';
        $page_data = get_user_pages_data($url, $listings_count, $items_on_page, $page, 'briefPage');
        $this->view->assign('page_data', $page_data);

        return $this->view->fetch('wish_list_content', 'user', 'listings');
    }

    /**
     * Form to choose wish lists by ajax
     *
     * @param integer $max_select max wish list for selected
     * @param string $template template name
     * @return void
     */
    public function ajax_get_wish_lists_form($max_select = 1, $template = 'default')
    {
        $selected = $this->input->post('selected', true);
        $this->load->model('listings/models/Wish_list_model');

        if (!empty($selected)) {
            $filters['ids'] = $selected;
            $data['selected'] = $this->Wish_list_model->get_wish_lists_list($filters, null, null, array('name' => 'asc'));
        } else {
            $data['selected'] = array();
        }
        $data['max_select'] = $max_select ? $max_select : 0;

        $this->view->assign('select_data', $data);
        $this->view->render('ajax_wish_list_select_form_' . $template);
    }

    /**
     * Get wish lists data by ajax
     *
     * @param integer $page page of results
     * @return void
     */
    public function ajax_get_wish_lists($page = 1)
    {
        $selected = $this->input->post('selected', true);
        $search_string = trim(strip_tags($this->input->post('search', true)));
        $filters = $return = array();
        $items_on_page = 100;

        if (!empty($search_string)) {
            $filters['keyword'] = $search_string;
        }
        if (!empty($selected)) {
            $filters['not_ids'] = $selected;
        }

        $this->load->model('listings/models/Wish_list_model');

        $return['all'] = $this->Wish_list_model->get_wish_lists_count($filters);
        $return['pages'] = ceil($return['all'] / $items_on_page);
        $return['current_page'] = $page;
        $return['items'] = $this->Wish_list_model->get_wish_lists_list($filters, $page, $items_on_page, array('name' => 'asc'));

        $this->view->assign($return);
        $this->view->render();
    }

    /**
     * Get data of selected wish lists
     *
     * @return void
     */
    public function ajax_get_selected_wish_lists()
    {
        $selected = $this->input->post('selected', true);
        $this->load->model('listings/models/Wish_list_model');
        if (!empty($selected)) {
            $filters['ids'] = $selected;
            $return = $this->Wish_list_model->get_wish_lists_list($filters, null, null, array('name' => 'asc'));
        } else {
            $return = array();
        }

        $this->view->assign('listings', $return);
        $this->view->render();
    }

    /**
     * Form for choosing property type
     *
     * @param integer $max maximum selected property types
     * @return void
     */
    public function ajax_get_property_type_form($max = 1)
    {
        $property_types = array();

        $this->load->model('Properties_model');
        $initial_types = $this->Properties_model->get_categories('property_types', $this->pg_language->current_lang_id);
        $initial_items = $this->Properties_model->get_all_properties('property_types', $this->pg_language->current_lang_id);

        foreach ($initial_types as $type_gid => $type_data) {
            $property_types[$type_gid] = array(
                'gid' => $type_gid,
                'name' => $type_data['value'],
                'parent' => 0,
            );

            foreach ($initial_items[$type_gid] as $item_gid => $item_data) {
                $property_types[$type_gid . '_' . $item_gid] = array(
                    'gid' => $item_gid,
                    'name' => $item_data['value'],
                    'parent' => 0,
                );
            }
        }

        $data['max'] = $max;
        $data['property_types'] = $property_types;

        $this->view->assign('data', $data);
        $this->view->render('ajax_property_type_form');
    }

    /**
     * Get data of property types by ajax
     *
     * @param sring $var selected property type GUID
     * @return void
     */
    public function ajax_get_property_type_data($var)
    {
        $data = array();

        $property_types = array();

        $this->load->model('Properties_model');
        $initial_types = $this->Properties_model->get_categories('property_types', $this->pg_language->current_lang_id);
        $initial_items = $this->Properties_model->get_all_properties('property_types', $this->pg_language->current_lang_id);

        $var = intval($var);

        $i = 1;

        foreach ($initial_types as $type_gid => $type_data) {
            $id = $i++;
            $parent = $id;
            if (!$var) {
                $property_types[$type_gid] = array('id' => $id, 'gid' => $type_gid, 'name' => $type_data, 'parent' => 0);
            } elseif ($type_gid == $var) {
                $data['current_property_type'] = array('id' => $id, 'gid' => $type_gid, 'name' => $type_data, 'parent' => 0);
            }
            foreach ($initial_items[$type_gid] as $item_gid => $item_data) {
                $id = $i++;
                if (!$var || $type_gid == $var) {
                    $property_types[$id] = array(
                        'id' => $id,
                        'gid' => $item_gid,
                        'name' => $item_data,
                        'parent' => $parent,
                    );
                }
            }
        }
        $data['property_types'] = $property_types;

        $this->view->assign($data);
        $this->view->render();
        return;
    }

    /**
     * Save map settings of listing object by ajax
     *
     * @param integer $listing_id listing identifier
     * @param array $param map data
     * @return void
     */
    public function ajax_set_map($listing_id, $param)
    {
        $user_id = $this->session->userdata('user_id');

        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);

        if (!$listing) {
            return;
        }

        if ($listing['id_user'] == $user_id) {
            $user_id = 0;
        }

        $this->load->model('Geomap_model');
        $this->load->model('geomap/models/Geomap_settings_model');

        $save_data = array();

        $map_gid = $this->Geomap_model->get_default_driver_gid();
        $settings = $this->Geomap_settings_model->get_settings($map_gid, $user_id, $listing_id, 'listing_view');
        if (!$settings) {
            $settings = array('gid' => 'listing_view');
        }
        switch ($param) {
            case 'type': $settings['view_type'] = $this->input->post('type', true);
                break;
            case 'zoom': $settings['zoom'] = $this->input->post('zoom', true);
                break;
            case 'coordinates':
                $settings['lat'] = $save_data['lat'] = $this->input->post('lat', true);
                $settings['lon'] = $save_data['lon'] = $this->input->post('lon', true);
                break;
            case 'panorama':
                $save_data['is_map_panorama'] = $this->input->post('panorama', true);
                break;
        }

        $validate_data = $this->Geomap_settings_model->validate_settings($settings);
        if (empty($validate_data['errors']) && !empty($validate_data['data'])) {
            $this->Geomap_settings_model->save_settings($map_gid, 0, $listing_id, 'listing_view', $validate_data['data']);
        }

        $validate_data = $this->Listings_model->validate_listing($listing_id, $save_data);
        if (empty($validate_data['errors']) && !empty($validate_data['data'])) {
            $this->Listings_model->save_listing($listing_id, $validate_data['data']);
        }
    }

    /**
     * Load laguages for mortgage calculator by ajax
     *
     * @return void
     */
    public function ajax_mortgage_calc()
    {
        $result = array();
        $result['alert1'] = l('mortgage_calc_alert1', 'listings');
        $result['alert2'] = l('mortgage_calc_alert2', 'listings');
        $result['alert3'] = l('mortgage_calc_alert3', 'listings');
        $result['alert4'] = l('mortgage_calc_alert4', 'listings');
        $result['mes1'] = l('mortgage_calc_mes1', 'listings');
        $result['mes2'] = l('mortgage_calc_mes2', 'listings');
        $result['mes3'] = l('mortgage_calc_mes3', 'listings');
        $result['mes4'] = l('mortgage_calc_mes4', 'listings');
        $result['mes5'] = l('mortgage_calc_mes5', 'listings');
        $result['mes6'] = l('mortgage_calc_mes6', 'listings');
        $result['mes7'] = l('mortgage_calc_mes7', 'listings');
        $result['mes8'] = l('mortgage_calc_mes8', 'listings');
        $result['mes9'] = l('mortgage_calc_mes9', 'listings');
        $result['mes10'] = l('mortgage_calc_mes10', 'listings');
        $result['mes11'] = l('mortgage_calc_mes11', 'listings');
        $result['mes12'] = l('mortgage_calc_mes12', 'listings');
        $result['mes13'] = l('mortgage_calc_mes13', 'listings');
        $result['content1'] = l('mortgage_calc_content1', 'listings');
        $result['content2'] = l('mortgage_calc_content2', 'listings');

        $this->view->assign($result);
        $this->view->render();
    }

    /**
     * Close moderation alert by ajax
     *
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function ajax_close_alert($listing_id)
    {
        $user_id = $this->session->userdata('user_id');
        $return = array('success' => 0);

        $this->Listings_model->set_format_settings('use_format', false);
        $data = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);

        if ($user_id == $data['id_user']) {
            $is_moderation_installed = $this->pg_module->is_module_active('moderation');
            if ($is_moderation_installed) {
                $this->load->model('listings/models/Listings_moderation_model');
                $this->Listings_moderation_model->delete_alert($listing_id);
                $return['success'] = 1;
            }
        }
        $this->view->assign($return);
        $this->view->render();
        return;
    }

    /**
     * Generate rss link for found listings by guid
     *
     * @param string $rss_gid criteria identifier
     * @return void
     */
    public function rss($rss_gid = null)
    {
        $this->load->model('listings/models/Listings_rss_model');

        $rss_settings = $this->Listings_model->get_rss_settings();
        $this->load->library('rssfeed');
        $current_lang = $this->pg_language->languages[$this->pg_language->current_lang_id];

        $this->rssfeed->set_channel(
            site_url(), $rss_settings['rss_feed_channel_title'], $rss_settings['rss_feed_channel_description'], $current_lang['code']
        );

        if ($rss_settings['rss_feed_image_url']) {
            $this->rssfeed->set_image(
                $rss_settings['rss_feed_image_media']['thumbs']['rss'], $rss_settings['rss_feed_image_title'], site_url()
            );
        }

        if ($rss_gid) {
            $rss = $this->Listings_rss_model->get_rss_by_gid($rss_gid);
            if (!$rss) {
                show_404();
            }
            $filters = $rss['data'];
        }

        $filters['active'] = '1';

        $listings = $this->Listings_model->get_listings_list($filters, 1, $rss_settings['rss_listings_max_count'], array('date_add' => 'DESC'));
        if (!empty($listings)) {
            $this->load->helper('seo');
            $this->load->library('Translit');

            foreach ($listings as $item) {
                $url = rewrite_link('listings', 'view', $item);
                $this->view->assign('listing', $item);
                $content = $this->view->fetch('listing_rss_block', 'user', 'listings');
                $this->rssfeed->set_item($url, $item['output_name'], $content, $item['date_modified']);
            }
        }
        $this->rssfeed->send();
        return;
    }

    /**
     * Block of listings visitors
     *
     * @param string $field_name visitor/visit
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return string
     */
    protected function _visitors_block($field_name, $order = 'id', $order_direction = 'DESC', $page = 1)
    {
        $current_settings = isset($_SESSION['listings_visitors_list']) ? $_SESSION['listings_visitors_list'] : array();
        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'id';
        }
        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        if (!isset($current_settings['page'])) {
            $current_settings['page'] = 1;
        }
        if (!isset($current_settings['filters'])) {
            $current_settings['filters'] = array();
        }

        $visit_type = $field_name === 'id_visitor' ? 'visitors' : 'visits';

        $filters = $current_settings['filters'];
        $filters['only_listings'] = 1;

        if (!$order) {
            $order = $current_settings['order'];
        }
        $this->view->assign('order', $order);
        $current_settings['order'] = $order;

        if (!$order_direction) {
            $order_direction = $current_settings['order_direction'];
        }
        $this->view->assign('order_direction', $order_direction);
        $current_settings['order_direction'] = $order_direction;

        $page = intval($page);
        if (!$page) {
            $page = $current_settings['page'];
        }
        $items_on_page = $this->pg_module->get_module_config('start', 'index_items_per_page');

        $sort_data = array(
            'url' => site_url() . 'listings/' . $visit_type,
            'order' => $order,
            'direction' => $order_direction,
            'links' => array(
                'visits_count' => l('field_visits_count', 'listings'),
                'last_visit_date' => l('field_last_visit_date', 'listings'),
            )
        );
        $this->view->assign('sort_data', $sort_data);
        $this->load->model('listings/models/Listings_visitor_model');
        $listings_count = $this->Listings_visitor_model->get_visitors_count($filters);
        if ($listings_count > 0) {
            $order_array = array($order => $order_direction);
            $visitors = $this->Listings_visitor_model->get_visitors_list($filters, $page, $items_on_page, $order_array);

            $listings_for_search = array();
            foreach ($visitors as $value) {
                $listings_for_search[] = $value['id_listing'];
            }

            $listings_by_ids = array();
            $results = $this->Listings_model->get_listings_list(array('ids' => $listings_for_search));
            foreach ($results as $result) {
                $listings_by_ids[$result['id']] = $result;
            }
            $listings = array();
            foreach ($listings_for_search as $listing_id) {
                $listings[$listing_id] = isset($listings_by_ids[$listing_id]) ? $listings_by_ids[$listing_id] :
                    $this->Listings_model->format_default_listing($listing_id);
            }
            $this->view->assign('listings', $listings);
        } else {
            $this->view->assign('listings', array());
        }
        $this->view->assign('listings_count', $listings_count);
        $this->load->helper('sort_order');
        $page = get_exists_page_number($page, $listings_count, $items_on_page);
        $current_settings['page'] = $page;
        $this->load->helper('navigation');
        $url = site_url() . 'listings/' . $visit_type . '/' . $order . '/' . $order_direction . '/';
        $page_data = get_user_pages_data($url, $listings_count, $items_on_page, $page, 'briefPage');
        $this->view->assign('page_data', $page_data);

        $_SESSION['listings_visitors_list'] = $current_settings;

        return $this->view->fetch('visitors_block', 'user', 'listings');
    }

    /**
     * Show visitors of my listings
     *
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function visitors($order = 'id', $order_direction = 'DESC', $page = 1)
    {
        $current_settings = isset($_SESSION['listings_visitors_list']) ? $_SESSION['listings_visitors_list'] : array();

        $user_id = intval($this->session->userdata('user_id'));
        $filters = array('user' => $user_id);

        $current_settings['order'] = $order;
        $current_settings['order_direction'] = $order_direction;
        $current_settings['page'] = $page;
        $current_settings['filters'] = $filters;
        $_SESSION['listings_visitors_list'] = $current_settings;

        $listing_block = $this->_visitors_block('id_visitor', $order, $order_direction, $page);
        $this->view->assign('block', $listing_block);

        $current_settings = isset($_SESSION['listings_visitors_list']) ? $_SESSION['listings_visitors_list'] : array();
        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'date_created';
        }
        $this->view->assign('order', $current_settings['order']);

        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        $this->view->assign('order_direction', $current_settings['order_direction']);

        if (!isset($current_settings['page'])) {
            $current_settings['page'] = 1;
        }
        $this->view->assign('page', $current_settings['page']);
        $this->Menu_model->breadcrumbs_set_active(l('text_listings_visitors', 'listings'));
        $this->view->render('visitors', 'user', 'listings');
    }

    /**
     * Show visitors of my listings by ajax
     *
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function ajax_visitors($order = 'id', $order_direction = 'DESC', $page = 1)
    {
        $current_settings = isset($_SESSION['listings_visitors_list']) ? $_SESSION['listings_visitors_list'] : array();

        $user_id = intval($this->session->userdata('user_id'));
        $filters = array('user' => $user_id);

        $current_settings['order'] = $order;
        $current_settings['order_direction'] = $order_direction;
        $current_settings['page'] = $page;
        $current_settings['filters'] = $filters;
        $_SESSION['listings_visitors_list'] = $current_settings;

        $this->view->output($this->_visitors_block('id_visitor', $order, $order_direction, $page));
        $this->view->render();
    }

    /**
     * Show listings which i'm visit
     *
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function visits($order = 'id', $order_direction = 'DESC', $page = 1)
    {
        $current_settings = isset($_SESSION['listings_visitors_list']) ? $_SESSION['listings_visitors_list'] : array();

        $user_id = intval($this->session->userdata('user_id'));
        $filters = array('visitor' => $user_id);

        $current_settings['order'] = $order;
        $current_settings['order_direction'] = $order_direction;
        $current_settings['page'] = $page;
        $current_settings['filters'] = $filters;
        $_SESSION['listings_visitors_list'] = $current_settings;

        $this->view->assign(
            'block', $this->_visitors_block('id_user', $order, $order_direction, $page)
        );

        $current_settings = isset($_SESSION['listings_visitors_list']) ? $_SESSION['listings_visitors_list'] : array();
        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'date_created';
        }
        $this->view->assign('order', $current_settings['order']);

        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        $this->view->assign('order_direction', $current_settings['order_direction']);

        if (!isset($current_settings['page'])) {
            $current_settings['page'] = 1;
        }
        $this->view->assign('page', $current_settings['page']);
        $this->Menu_model->breadcrumbs_set_active(l('text_listings_visits', 'listings'));
        $this->view->render('visits', 'user', 'listings');
    }

    /**
     * Show listings wich i'm visit by ajax
     *
     * @param string $order soting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function ajax_visits($order = 'id', $order_direction = 'DESC', $page = 1)
    {
        $current_settings = isset($_SESSION['listings_visits_list']) ? $_SESSION['listings_visitors_list'] : array();

        $user_id = intval($this->session->userdata('user_id'));
        $filters = array('visitor' => $user_id);

        $current_settings['order'] = $order;
        $current_settings['order_direction'] = $order_direction;
        $current_settings['page'] = $page;
        $current_settings['filters'] = $filters;
        $_SESSION['listings_visitors_list'] = $current_settings;
        $this->view->output($this->_visitors_block('id_user', $order, $order_direction, $page));
        $this->view->render();
    }

    /**
     * Change view mode of listings
     *
     * @param string $view_mode view mode
     * @return void
     */
    public function set_view_mode($view_mode)
    {
        if (in_array($view_mode, array('list', 'map'))) {
            $_SESSION['listings_view_mode'] = $view_mode;
        }
        $this->view->setRedirect($_SERVER['HTTP_REFERER']);
    }

    /**
     * Generate rss link of found listings by ajax
     *
     * @return void
     */
    public function ajax_get_rss_link()
    {
        $use_rss = $this->pg_module->get_module_config('listings', 'use_rss');
        if (!$use_rss) {
            return '';
        }

        $this->load->model('listings/models/Listings_rss_model');

        $data = isset($_SESSION['listings_search']['data']) ? $_SESSION['listings_search']['data'] : array();

        if ($this->session->userdata('auth_type') == 'user') {
            $data['not_user'] = $this->session->userdata('user_id');
        }

        $rss_gid = $this->Listings_rss_model->generate_rss_gid(null, $data);
        $rss = $this->Listings_rss_model->get_rss_by_gid($rss_gid);
        if (!$rss) {
            $rss_data = array(
                'gid' => $rss_gid,
                'data' => $data,
            );
            $validate_data = $this->Listings_rss_model->validate_rss(null, $rss_data);
            if (empty($validate_data['errors'])) {
                $this->Listings_rss_model->save_rss(null, $validate_data['data']);
            }
        }

        $this->view->output(site_url() . 'listings/rss/' . $rss_gid);
        $this->view->render();
    }

    /**
     * Show listings by ajax
     *
     * @param integer $page page of results
     * @return void
     */
    public function ajax_get_listings($page = 1)
    {
        $selected = $this->input->post('selected', true);
        $operation_type = $this->input->post('operation_type', true);
        $id_user = $this->input->post('id_user', true);
        $params = $return = array();
        $items_on_page = 100;

        if (!empty($selected)) {
            $params['exclude_id'] = $selected;
        }

        if ($operation_type) {
            $params['type'] = $operation_type;
        }

        if ($id_user) {
            $params['id_user'] = $id_user;
        }

        $return['all'] = $this->Listings_model->get_listings_count($params);
        $return['pages'] = ceil($return['all'] / $items_on_page);
        $return['current_page'] = $page;
        $return['items'] = $this->Listings_model->get_listings_list($params, $page, $items_on_page, array('date_modified' => 'DESC'));
        $this->view->assign($return);
        $this->view->render();
        return;
    }

    /**
     * Show selected listings by ajax
     *
     * @return void
     */
    public function ajax_get_selected_listings()
    {
        $selected = $this->input->post('selected', true);

        if (!empty($selected)) {
            $params['ids'] = $selected;
            $return = $this->Listings_model->get_listings_list($params, null, null, array('date_modified' => 'DESC'));
        } else {
            $return = array();
        }

        if (!empty($return)) {
            $this->load->helper('seo');
            $this->load->helper('listings');
            foreach ($return as $key => $listing) {
                $listing['seo_url'] = rewrite_link('listings', 'view', $listing);
                $listing['price_str'] = listing_price_block(array('operation_type' => $listing['operation_type'], 'data' => $listing, 'template' => 'small'));
                $return[$key] = $listing;
            }
        }

        $this->view->assign('listings', $return);
        $this->view->render();
    }

    /**
     * Show form to choise listings by ajax
     *
     * @param integer $max_select max selected listings
     * @param string $template template name
     * @return void
     */
    public function ajax_get_listings_form($max_select = 1, $template = 'default')
    {
        $selected = $this->input->post('selected', true);
        if (!empty($selected)) {
            $params['ids'] = $selected;
            $data['selected'] = $this->Listings_model->get_listings_list($params, null, null, array('date_modified' => 'DESC'));
        } else {
            $data['selected'] = array();
        }
        $data['max_select'] = $max_select ? $max_select : 0;

        $this->view->assign('select_data', $data);
        $this->view->render('ajax_listing_select_form_' . $template);
    }

    /**
     * Show calendar block
     *
     * @param integer $listing_id listing identifier
     * @param integer $count number of calendars
     * @param integer $year year
     * @param integer $month month
     * @return void
     */
    public function ajax_get_calendar($listing_id, $count, $year, $month = 1)
    {
        $this->Listings_model->set_format_settings('get_booking', true);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('get_booking', false);

        $params = array(
            'listing' => $listing,
            'count' => $count,
            'year' => $year,
            'month' => $month,
            'template' => 'view',
        );

        $this->load->helper('listings_booking');
        $this->view->output(listings_calendar_block($params, true));
        $this->view->render();
    }

    /**
     * Catalogue of listings by country
     *
     * @param string $country country code
     * @param string $operation_type operation type
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function country($country, $operation_type = null, $order = 'default', $order_direction = 'DESC', $page = 1)
    {
        $_SESSION['listings_search']['data'] = array();
        if (!empty($country)) {
            $_SESSION['listings_search']['data']['id_country'] = $country;
        }
        $this->_list($operation_type, $order, $order_direction, $page, $_SESSION['listings_search']);
    }

    /**
     * Catalogue of listings by region
     *
     * @param string $country country code
     * @param integer $region region identifier
     * @param string $operation_type operation type
     * @param string $order sorting order
     * @param string $order_direction order direction
     * @param integer $page page of results
     * @return void
     */
    public function region($country, $region = null, $operation_type = null, $order = 'default', $order_direction = 'DESC', $page = 1)
    {
        $_SESSION['listings_search']['data'] = array();
        $this->load->model('Countries_model');
        if (!empty($region)) {
            $region_obj = $this->Countries_model->get_region($region);
            if (!empty($region_obj)) {
                $_SESSION['listings_search']['data']['id_country'] = $region_obj['country_code'];
                $_SESSION['listings_search']['data']['id_region'] = $region;
            }
        } elseif (!empty($country)) {
            $_SESSION['listings_search']['data']['id_country'] = $country;
        }
        $this->_list($operation_type, $order, $order_direction, $page, $_SESSION['listings_search']);
    }

    private function activate_service($listing_id, $user_service_id, $service_name)
    {
        $user_data = array('listing' => $listing_id);
        $this->load->model('listings/models/Listings_services_model');
        $user_service = $this->Listings_services_model->get_service($user_service_id);
        if (empty($user_service)) {
            show404();
        }
        $validation_method = "validate_{$service_name}_activate";
        $validate_data = $this->Listings_services_model->$validation_method($user_service, $user_data);
        if (!empty($validate_data['errors'])) {
            $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
        } else {
            $activation_method = "service_{$service_name}_activate";
            $this->Listings_services_model->$activation_method($user_service, $user_data);
            $message = l('success_service_activating', 'listings');
            $this->system_messages->addMessage(View::MSG_SUCCESS, $message);
        }
        $this->view->setRedirect($_SERVER['HTTP_REFERER']);
    }

    /**
     * Activate featured listing service
     *
     * @param integer $listing_id listing identifier
     * @param integer $id_user_service user service identifier
     * @return void
     */
    public function activate_featured($listing_id, $id_user_service)
    {
        $this->activate_service($listing_id, $id_user_service, 'featured');
    }

    /**
     * Activate lift up listing service
     *
     * @param integer $listing_id listing identifier
     * @param integer $id_user_service user service identifier
     * @return void
     */
    public function activate_lift_up($listing_id, $id_user_service)
    {
        $this->activate_service($listing_id, $id_user_service, 'lift_up');
    }

    /**
     * Activate lift up listing in country service
     *
     * @param integer $listing_id listing identifier
     * @param integer $id_user_service user service identifier
     * @return void
     */
    public function activate_lift_up_country($listing_id, $id_user_service)
    {
        $this->activate_service($listing_id, $id_user_service, 'lift_up_country');
    }

    /**
     * Activate lift up listing in region service
     *
     * @param integer $listing_id listing identifier
     * @param integer $id_user_service user service identifier
     * @return void
     */
    public function activate_lift_up_region($listing_id, $id_user_service)
    {
        $this->activate_service($listing_id, $id_user_service, 'lift_up_region');
    }

    /**
     * Activate lift up listing in city service
     *
     * @param integer $listing_id listing identifier
     * @param integer $id_user_service user service identifier
     * @return void
     */
    public function activate_lift_up_city($listing_id, $id_user_service)
    {
        $this->activate_service($listing_id, $id_user_service, 'lift_up_city');
    }

    /**
     * Activate highlight listing service
     *
     * @param integer $listing_id listing identifier
     * @param integer $id_user_service user service identifier
     * @return void
     */
    public function activate_highlight($listing_id, $id_user_service)
    {
        $this->activate_service($listing_id, $id_user_service, 'highlight');
    }

    /**
     * Activate slide show listing service
     *
     * @param integer $listing_id listing identifier
     * @param integer $id_user_service user service identifier
     * @return void
     */
    public function activate_slide_show($listing_id, $id_user_service)
    {
        $this->activate_service($listing_id, $id_user_service, 'slide_show');
    }

    /**
     * Activate user service
     *
     * @param integer $listing_id listing identifier
     * @param integer $user_service_id user service identifier
     * @return void
     */
    public function activate_user_service($listing_id, $user_service_id)
    {
        $user_data = array('listing' => $listing_id);
        $this->load->model('listings/models/Listings_service_model');
        $user_service = $this->Listings_service_model->get_service($user_service_id);
        if (empty($user_service)) {
            show404();
        }
        switch ($user_service['template_gid']) {
            case 'listings_featured_template': {
                    $service_gid = 'featured';
                    break;
                }
            case 'listings_lift_up_template': {
                    $service_gid = 'lift_up';
                    break;
                }
            case 'listings_lift_up_country_template': {
                    $service_gid = 'lift_up_country';
                    break;
                }
            case 'listings_lift_up_region_template': {
                    $service_gid = 'lift_up_region';
                    break;
                }
            case 'listings_lift_up_city_template': {
                    $service_gid = 'lift_up_city';
                    break;
                }
            case 'listings_highlight_template': {
                    $service_gid = 'highlight';
                    break;
                }
            case 'listings_slide_show_template': {
                    $service_gid = 'slide_show';
                    break;
                }
            default: {
                    show404();
                }
        }

        $validate_data = $this->Listings_service_model->{'validate_' . $service_gid . '_activate'}($user_service, $user_data);
        if (!empty($validate_data['errors'])) {
            $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
        } else {
            $this->Listings_service_model->{'service_' . $service_gid . '_activate'}($user_service, $user_data);
            $this->system_messages->addMessage(
                View::MSG_SUCCESS, l('success_service_activating', 'listings')
            );
        }
        $this->view->setRedirect($_SERVER['HTTP_REFERER']);
    }

}
