<?php

namespace Pg\Modules\Listings\Models;

use \Pg\Libraries\ModuleConfigTrait;
use Pg\Modules\Users\Models\Users_model;

/**
 * Listings module
 *
 * @package     PG_RealEstate
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
define('LISTINGS_TABLE', DB_PREFIX . 'listings');

/**
 * Listings Model
 *
 * @package     PG_RealEstate
 * @subpackage  Listings
 * @category    models
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Listings_model extends \Model
{

    use ModuleConfigTrait;

    const MODULE_GID = 'listings';
    const USER_TYPE_PRIVATE = 'private';
    const USER_TYPE_COMPANY = 'company';
    const USER_TYPE_AGENT = 'agent';
    const OPERATION_SALE = 'sale';
    const OPERATION_BUY = 'buy';
    const OPERATION_RENT = 'rent';
    const OPERATION_LEASE = 'lease';
    const OPERATION_SOLD = 'sold';
    const OPERATION_ALL = 'all';
    const STATUS_ACTIVE = 1;
    const STATUS_INACTIVE = 0;

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Properties of listings object in data source
     *
     * @var array
     */
    protected $fields = array(
        'id',
        'gid',
        'id_user',
        'id_type',
        'id_category',
        'user_type',
        'property_type',
        'status',
        'date_open',
        'date_open_begin',
        'date_open_end',
        'date_available',
        'sold',
        'date_created',
        'date_modified',
        'date_activity',
        'date_expire',
        'price',
        'price_old',
        'price_max',
        'price_week',
        'price_month',
        'price_reduced',
        'price_period',
        'price_type',
        'price_sorting',
        'gid_currency',
        'price_negotiated',
        'price_auction',
        'square',
        'square_max',
        'square_unit',
        'id_city',
        'id_region',
        'id_country',
        'id_district',
        'address',
        'zip',
        'lat',
        'lon',
        'headline',
        'listing_file',
        'listing_file_date',
        'listing_file_name',
        'listing_file_comment',
        'listing_video',
        'listing_video_image',
        'listing_video_data',
        'initial_moderation',
        'initial_activity',
        'logo_image',
        'slider_image',
        'id_wish_lists',
        'photo_count',
        'is_vtour',
        'use_calendar',
        'calendar_period_min',
        'calendar_period_max',
        'guest_count',
        'is_featured',
        'featured_date_end',
        'is_lift_up',
        'lift_up_date_end',
        'is_lift_up_country',
        'lift_up_country_date_end',
        'is_lift_up_region',
        'lift_up_region_date_end',
        'is_lift_up_city',
        'lift_up_city_date_end',
        'is_highlight',
        'highlight_date_end',
        'is_slide_show',
        'slide_show_date_end',
        'views',
        'set_to_subscribe',
        'is_map_panorama',
        'id_seo_settings',
    );

    /**
     * Review properties of listing
     *
     * @var array
     */
    protected $review_fields = array(
        'review_data',
        'review_count',
        'review_sorter',
        'review_value',
        'review_type'
    );

    /**
     * Settings for formatting listing object
     *
     * @var array
     */
    protected $format_settings = array(
        'use_format' => true,
        'get_user' => true,
        'get_location' => true,
        'get_description' => false,
        'get_photos' => false,
        'get_photos_all' => false,
        'get_virtual_tours' => false,
        'get_virtual_tours_all' => false,
        'get_video' => false,
        'get_file' => false,
        'get_moderation' => false,
        'get_saved_listings' => false,
        'get_qr_code' => false,
        'get_booking' => false,
        'get_booking_periods' => false,
        'get_booking_special_prices' => false,
        'get_views' => false,
        'get_price_str' => true,
    );
    protected $old_settings = array();

    /**
     * Moderation type of listing data
     *
     * @var string
     */
    protected $moderation_type = 'listings';

    /**
     * Moderation type of share data
     *
     * @var string
     */
    protected $moderation_share_type = 'share_listing';

    /**
     * Available operations types
     *
     * @var array
     */
    public $operations_arr = array(
        1 => self::OPERATION_SALE,
        2 => self::OPERATION_BUY,
        3 => self::OPERATION_RENT,
        4 => self::OPERATION_LEASE,
    );

    /**
     * Disabled operations types
     *
     * @var array
     */
    public $operations_arr_disabled = array();

    /**
     * Categories
     *
     * var array
     */
    public $category_arr = array(1 => 'residential', 2 => 'commercial', 3 => 'lot_and_land');

    /**
     * Configuration of field editor categories
     *
     * @var array
     */
    public $field_editor_types = array(
        self::OPERATION_SALE => array(1 => 'residential_sale', 2 => 'commercial_sale', 3 => 'lot_and_land_sale'),
        self::OPERATION_BUY => array(1 => 'residential_buy', 2 => 'commercial_buy', 3 => 'lot_and_land_buy'),
        self::OPERATION_RENT => array(1 => 'residential_rent', 2 => 'commercial_rent', 3 => 'lot_and_land_rent'),
        self::OPERATION_LEASE => array(1 => 'residential_lease', 2 => 'commercial_lease', 3 => 'lot_and_land_lease'),
    );

    /**
     * Schema to make listing GUID
     *
     * @var string
     */
    public $listing_gid_scheme = '[user_name]-[location]';

    /**
     * Data source of square units (GUID)
     *
     * @var string
     */
    public $square_units_gid = 'square_units';

    /**
     * Data source of searching radiuses (GUID)
     *
     * @var string
     */
    public $radius_data_gid = 'radius_data';

    /**
     * Display listing sections
     *
     * @var array
     */
    public $display_sections = array(
        'top' => array('gallery' => 0, 'map' => 1, 'panorama' => 1, 'virtual_tour' => 0, 'video' => 0),
        'bottom' => array('overview' => 1, 'calendar' => 0, 'reviews' => 1, 'file' => 1, 'provider' => 0, 'print' => 0),
    );

    /**
     * Listings rss logo upload (GUID)
     *
     * @var string
     */
    public $rss_config_id = 'rss-listings-logo';

    /**
     * Time of updating QR code in hours
     *
     * @var integer
     */
    protected $qr_update_time = 3;

    /**
     * Listings upload image (GUID)
     *
     *
     * @var string
     */
    public $upload_config_id = 'listing-search-photo';

    /**
     * Slider upload image (GUID)
     *
     * @var string
     */
    public $slider_config_id = 'listing-slider-photo';

    /**
     * Constructor
     *
     * return Listings_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();

        if (PACKAGE_NAME == 'agency' || PACKAGE_NAME == 'free') {
            $this->operations_arr = array(1 => self::OPERATION_SALE, 3 => self::OPERATION_RENT,);
        }

        $this->getModuleConfigOperations();

        // add reviews fields
        if ($this->ci->pg_module->is_module_active('reviews')) {
            $this->fields = array_merge($this->fields, $this->review_fields);
        }
    }

    protected $config_items = [
        'default_activation_period' => [
            'key' => 'default_activation_period',
            'type' => 'int',
            'name' => null,
            'value' => null,
        ],
        'for_rent_enabled' => [
            'key' => 'for_rent_enabled',
            'type' => 'checkbox',
            'name' => null,
            'value' => null,
        ],
        'for_sale_enabled' => [
            'key' => 'for_sale_enabled',
            'type' => 'checkbox',
            'name' => null,
            'value' => null,
        ],
        'for_buy_enabled' => [
            'key' => 'for_buy_enabled',
            'type' => 'checkbox',
            'name' => null,
            'value' => null,
        ],
        'for_lease_enabled' => [
            'key' => 'for_lease_enabled',
            'type' => 'checkbox',
            'name' => null,
            'value' => null,
        ],
        'is_show_contact' => [
            'key' => 'is_show_contact',
            'type' => 'checkbox',
            'name' => null,
            'value' => null,
        ],
        'similar_items' => [
            'key' => 'similar_items',
            'type' => 'int',
            'name' => null,
            'value' => null,
        ],
        'slide_show_images' => [
            'key' => 'slide_show_images',
            'type' => 'int',
            'name' => null,
            'value' => null,
        ],
        'slider_auto' => [
            'key' => 'slider_auto',
            'type' => 'checkbox',
            'name' => null,
            'value' => null,
        ],
        'slider_min_height' => [
            'key' => 'slider_min_height',
            'type' => 'int',
            'name' => null,
            'value' => null,
        ],
        'slider_min_width' => [
            'key' => 'slider_min_width',
            'type' => 'int',
            'name' => null,
            'value' => null,
        ],
        'slider_rotation' => [
            'key' => 'slider_rotation',
            'type' => 'int',
            'name' => null,
            'value' => null,
        ],
    ];

    private function getModuleConfigList()
    {
        if (PACKAGE_NAME === 'agency' || PACKAGE_NAME === 'free') {
            unset($this->config_items['for_buy_enabled'], $this->config_items['for_lease_enabled']);
        }
        return $this->config_items;
    }

    /**
     * Load module configuration
     *
     * @return void
     */
    protected function getModuleConfigOperations()
    {
        foreach ($this->operations_arr as $operation_type_gid) {
            $enabled = $this->ci->pg_module->get_module_config('listings', 'for_' . $operation_type_gid . '_enabled');
            if (!$enabled) {
                $this->operations_arr_disabled[] = $operation_type_gid;
            }
        }
    }

    /**
     * Return operation type name by identifier
     *
     * @param integer $operation_type_id type identifier
     * @return string
     */
    public function getOperationTypeById($operation_type_id)
    {
        if (!isset($this->operations_arr[$operation_type_id])) {
            return '';
        }
        $operation_type_gid = $this->operations_arr[$operation_type_id];
        return $operation_type_gid;
    }

    /**
     * Return operation type identifier by GUID
     *
     * @param string $operation_type_gid operation_type GUID
     * $return integer
     */
    public function get_operation_type_by_gid($operation_type_gid)
    {
        if (!in_array($operation_type_gid, $this->operations_arr)) {
            return 0;
        }
        return array_search($operation_type_gid, $this->operations_arr);
    }

    /**
     * Return operation type identifier by default
     *
     * @param boolean $is_search use all
     * @return string
     */
    public function get_operation_type_id_default($is_search = false)
    {
        $operation_types = $this->get_operation_types($is_search);
        if (!empty($operation_types)) {
            return array_shift(array_keys($operation_types));
        } else {
            return '';
        }
    }

    /**
     * Return operation type GUID by default
     *
     * @param boolean $is_search use all
     * @return string
     */
    public function get_operation_type_gid_default($is_search = false)
    {
        $operation_types = $this->get_operation_types($is_search);
        if (!empty($operation_types)) {
            return array_shift($operation_types);
        } else {
            return self::OPERATION_ALL;
        }
    }

    /**
     * Return enabled operation types as array
     *
     * @param boolean $is_search use all
     * @return array
     */
    public function get_operation_types($is_search = false)
    {
        $operation_types = array_diff($this->operations_arr, $this->operations_arr_disabled);
        if ($is_search) {
            if (in_array(self::OPERATION_SALE, $operation_types)) {
                $operation_types[] = self::OPERATION_SOLD;
            }
            if (empty($operation_types)) {
                $operation_types[] = self::OPERATION_ALL;
            }
        }
        return $operation_types;
    }

    /**
     * Check all types are disabled
     *
     * @return boolean
     */
    public function is_operation_types_disabled()
    {
        return count($this->get_operation_types()) == 0;
    }

    /**
     * Return category identifier by code
     *
     * @param string $category_code category code
     * @return string
     */
    public function get_category($category_code)
    {
        $operation_types = $this->operations_arr;

        $category_ids = array();
        foreach ($operation_types as $operation_type) {
            $category_ids = array_merge($category_ids, array_flip($this->field_editor_types[$operation_type]));
            foreach ($this->field_editor_types[$operation_type] as $key => $value) {
                $category_ids[preg_replace('/\_' . preg_quote($operation_type, '/') . '$/i', '', $value)] = $key;
            }
        }

        return $category_ids[$category_code];
    }

    /**
     * Return category code by default
     *
     * @return string
     */
    public function get_category_default()
    {
        $this->ci->load->model('Properties_model');
        $default_lang = $this->ci->pg_language->current_lang_id;
        $property_types = $this->ci->Properties_model->get_categories('property_types', $default_lang);
        return key($property_types);
    }

    /**
     * Return categories identifiers as array
     *
     * @return array
     */
    public function get_category_ids()
    {
        $operation_types = $this->get_operation_types();

        $category_types = array();
        foreach ($operation_types as $operation_type) {
            $category_types = array_merge($category_types, array_keys($this->field_editor_types[$operation_type]));
        }

        return array_unique($category_types);
    }

    /**
     * Return category code by identifier
     *
     * @param integer $category_id category identifier
     * @return string
     */
    public function get_category_code($category_id)
    {
        return isset($this->category_arr[$category_id]) ? $this->category_arr[$category_id] : current($this->category_arr);
    }

    /**
     * Return listing object from data source by identifier
     *
     * @param string $listing_id listing identifier
     * @return array
     */
    public function get_listing_by_id($listing_id)
    {
        $this->ci->db->select(implode(', ', $this->fields));
        $this->ci->db->from(LISTINGS_TABLE);
        $this->ci->db->where('id', $listing_id);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            $results = $this->format_listings(array($results[0]));
            return $results[0];
        }
        return array();
    }

    /**
     * Generate listing GUID
     *
     * @param array $data listing data
     * @return string
     */
    protected function _create_unique_gid($data)
    {
        $scheme = $this->listing_gid_scheme;

        $this->ci->load->library('Translit');
        $current_lang = $this->ci->pg_language->get_lang_by_id($this->ci->pg_language->current_lang_id);
        $current_lang_code = $current_lang['code'];

        ///[user_name] [user_id]
        $is_users_installed = $this->ci->pg_module->is_module_active("users");
        if ($is_users_installed) {
            if ($data['id_user']) {
                $this->ci->load->model('Users_model');
                $user = $this->ci->Users_model->get_user_by_id($data['id_user'], true);
                $user_name_replace = $this->ci->translit->convert($current_lang_code, $user['output_name']);
                $user_name_replace = str_replace(' ', '-', $user_name_replace);
                $user_name_replace = preg_replace('/[-]+/i', '-', $user_name_replace);
                $user_name_replace = preg_replace('/[^a-z0-9\-_]+/i', '', $user_name_replace);
                $user_id_replace = $data['id_user'];
            } else {
                $user_login_replace = $user_id_replace = '';
            }
        } else {
            $this->ci->load->model('Start_model');
            $user = $this->ci->Start_model->getUserDefaultOnSite();
            $user_name_replace = $this->ci->translit->convert($current_lang_code, $user['output_name']);
            $user_name_replace = str_replace(' ', '-', $user_name_replace);
            $user_name_replace = preg_replace('/[-]+/i', '-', $user_name_replace);
            $user_name_replace = preg_replace('/[^a-z0-9\-_]+/i', '', $user_name_replace);
            $user_id_replace = $data['id_user'];
        }
        $scheme = str_replace('[user_name]', $user_name_replace, $scheme);
        $scheme = str_replace('[user_id]', $user_id_replace, $scheme);

        ///[location]
        if (!empty($data['id_country']) && !empty($data['id_region']) && !empty($data['id_city'])) {
            $this->ci->load->helper('countries');
            if (!empty($data['id_district'])) {
                $listing_locations = districts_output_format(
                    array(
                        array(
                            'country' => $data['id_country'],
                            'region' => $data['id_region'],
                            'city' => $data['id_city'],
                            'district' => $data['id_district'],
                        ),
                    )
                );
            } else {
                $listing_locations = cities_output_format(
                    array(
                        array(
                            'country' => $data['id_country'],
                            'region' => $data['id_region'],
                            'city' => $data['id_city'],
                        ),
                    )
                );
            }
            if ($listing_locations[0]) {
                $location_replace = $this->ci->translit->convert($current_lang_code, $listing_locations[0]);
                $location_replace = str_replace(array(' ', ','), '-', $location_replace);
                $location_replace = preg_replace('/[-]+/i', '-', $location_replace);
                $location_replace = preg_replace('/[^a-z0-9\-_]+/i', '', $location_replace);
            } else {
                $location_replace = '';
            }
        } else {
            $location_replace = '';
        }
        $scheme = str_replace('[location]', $location_replace, $scheme);

        $gid_counts = $this->get_listings_count(array('gid' => $scheme));
        if ($gid_counts > 0) {
            if (strlen($scheme) > 96) {
                $scheme = substr($scheme, 0, 96);
            }
            $scheme = $scheme . '-' . rand(100, 999);
        }
        return $scheme;
    }

    /**
     * Save listing object to data source
     *
     * @param integer $listing_id listing identifier
     * @param array $data listing data
     * @param boolean $moderation use moderation
     * @return integer
     */
    public function save_listing($listing_id, $data, $moderation = false)
    {
        if (!$listing_id) {
            $listing_old = array();
        } else {
            $format_settings = $this->format_settings;
            $this->format_settings = array('use_format', false);
            $listing_old = $this->get_listing_by_id($listing_id);
            $this->format_settings = $format_settings;
        }
        $this->ci->load->model('listings/models/Listings_field_editor_model');

        $property_type_gid = $this->get_field_editor_type(array_merge($listing_old, $data));

        // get field editor fields
        $this->ci->load->model('Field_editor_model');
        $this->ci->Field_editor_model->initialize($property_type_gid);

        $fields_for_select = $this->ci->Listings_field_editor_model->get_fields_for_select($property_type_gid);

        $fields_data = array();
        foreach ($fields_for_select as $field) {
            if (!array_key_exists($field, $data)) {
                continue;
            }
            $fields_data[$field] = $data[$field];
            unset($data[$field]);
        }

        // check user type
        $is_users_installed = $this->ci->pg_module->is_module_active("users");
        if ($is_users_installed) {
            if (isset($data['id_user'])) {
                $this->ci->load->model('Users_model');
                $user = $this->ci->Users_model->get_user_by_id($data['id_user']);
                $data['user_type'] = $user['user_type'];
            }
        } else {
            $this->ci->load->model('Start_model');
            $user = $this->ci->Start_model->getUserDefaultOnSite();
            $data['user_type'] = $user['user_type'];
        }

        $is_moderation_installed = $this->ci->pg_module->is_module_active("moderation");

        if ($moderation && $is_moderation_installed) {
            $moderation = $this->ci->pg_module->get_module_config('listings', 'use_moderation');
        } else {
            $moderation = false;
        }


        if (!$listing_id) {
            $data['gid'] = $this->_create_unique_gid($data);
            $data['initial_moderation'] = $moderation ? 0 : 1;
            $data['date_created'] = $data['date_modified'] = date('Y-m-d H:i:s');

            if (in_array($this->operations_arr[$data['id_type']], array(self::OPERATION_RENT, self::OPERATION_LEASE)) && !isset($data['use_calendar'])) {
                $data['use_calendar'] = 1;
            }

            $this->ci->db->insert(LISTINGS_TABLE, $data);
            $listing_id = $this->ci->db->insert_id();

            $this->ci->load->model('listings/models/Listings_field_editor_model');
            $this->ci->Listings_field_editor_model->create_listing($listing_id, $fields_data, $property_type_gid);

            if ($moderation) {
                $this->ci->load->model('listings/models/Listings_moderation_model');
                $this->ci->Listings_moderation_model->save_listing($listing_id, array($data, $fields_data));
            }
        } else {
            if ($moderation) {
                $this->ci->load->model('listings/models/Listings_moderation_model');
                // check listing
                $already_moderated = $this->ci->Listings_moderation_model->is_listing_moderated($listing_id);

                // set dop fields
                $moderation_fields = $this->ci->Listings_moderation_model->get_fields_for_moderate();
                $moderation_field_editor = $this->ci->Listings_moderation_model->get_field_editor_for_moderate($property_type_gid);
                $moderation_fields = array_merge($moderation_fields, $moderation_field_editor);

                $this->ci->load->model('listings/models/Listings_field_editor_model');
                $saved_fields = $this->ci->Listings_field_editor_model->get_field_editor_data($property_type_gid, $fields_for_select, $listing_id);
                $saved_data = array_merge($listing_old, array_shift($saved_fields));

                $content_changed = $this->ci->Listings_moderation_model->is_data_changed($listing_id, $saved_data, array_merge($data, $fields_data));
                if ($already_moderated || $content_changed) {
                    // выбираем из данных поля для модерации
                    $moderation_data = array('id_listing' => $listing_id, 'id_user' => $saved_data['id_user']);
                    foreach ($moderation_fields as $field) {
                        if (isset($data[$field])) {
                            $moderation_data[$field] = $data[$field];
                            unset($data[$field]);
                        }
                        if (isset($fields_data[$field])) {
                            $moderation_data[$field] = $fields_data[$field];
                            unset($fields_data[$field]);
                        }
                    }
                    if (!empty($moderation_data)) {
                        $this->ci->Listings_moderation_model->save_listing($listing_id, $moderation_data);
                    }
                }
            }

            // remove from wish list
            if (array_key_exists('status', $data) && !$data['status'] && !empty($listing_old['id_wish_lists'])) {
                $this->ci->load->model('listings/models/Wish_list_model');

                foreach ($listing_old['id_wish_lists'] as $wish_list) {
                    $this->ci->Wish_list_model->delete_from_wish_list($wish_list, $listing_id, false);
                }

                $save_data = array('id_wish_lists' => array());
                $validate_data = $this->validate_listing($listing_id, $save_data);
                $data['id_wish_lists'] = $validate_data['data']['id_wish_lists'];
            }

            $data['date_modified'] = date('Y-m-d H:i:s');
            $this->ci->db->where('id', $listing_id);
            $this->ci->db->update(LISTINGS_TABLE, $data);

            if (!empty($fields_data)) {
                $this->ci->load->model('listings/models/Listings_field_editor_model');
                $this->ci->Listings_field_editor_model->save_listing($listing_id, $fields_data, $property_type_gid);
            }
        }

        $format_settings = $this->format_settings;
        $this->format_settings = array('use_format', false);
        $listing_new = $this->get_listing_by_id($listing_id);
        $this->format_settings = $format_settings;

        // update fulltext
        $this->ci->Field_editor_model->update_fulltext_field($listing_id);

        // update extern data
        $this->ci->load->model('listings/models/Listings_events_model');
        $this->ci->Listings_events_model->onsave_listing($listing_old, $listing_new);

        return $listing_id;
    }

    /**
     * Remove listing from data source by identifier
     *
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function delete_listing($listing_id)
    {
        $listing = $this->get_listing_by_id($listing_id);

        $property_type_gid = $this->get_field_editor_type($listing);

        // remove listing
        $this->ci->db->where('id', $listing_id);
        $this->ci->db->delete(LISTINGS_TABLE);

        // remove details
        $this->ci->load->model('listings/models/Listings_field_editor_model');
        $this->ci->Listings_field_editor_model->delete_listing($listing_id, $property_type_gid);

        $this->ci->load->model('Upload_gallery_model');
        $this->ci->load->model('listings/models/Listings_uploads_model');

        // remove uploads
        $this->ci->load->model('Uploads_model');
        if (!empty($listing['logo_image'])) {
            $this->ci->Uploads_model->delete_upload($this->upload_config_id, $listing_id . '/', $listing['logo_image']);
        }
        $this->ci->Uploads_model->delete_path($this->upload_config_id, $listing_id . '/');

        if (!empty($listing['slider_image'])) {
            $this->ci->Uploads_model->delete_upload($this->slider_config_id, $listing_id . '/', $listing['slider_image']);
        }
        $this->ci->Uploads_model->delete_path($this->slider_config_id, $listing_id . '/');

        // remove upload gallery
        $gallery_type = $this->ci->Upload_gallery_model->get_type_by_gid($this->ci->Listings_uploads_model->gallery_type);
        $photos_params = array('where' => array('type_id' => $gallery_type['id'], 'object_id' => $listing_id));
        $photos = $this->ci->Upload_gallery_model->get_files_by_param($photos_params);
        foreach ($photos as $photo) {
            $this->ci->Upload_gallery_model->delete_file($photo['id']);
        }
        $this->ci->Upload_gallery_model->delete_path($gallery_type, $listing_id);
        unset($photos);

        //remove virtual tour
        $virtual_tour_type = $this->ci->Upload_gallery_model->get_type_by_gid($this->ci->Listings_uploads_model->virtual_tour_type);
        $panorama_params = array('where' => array('type_id' => $virtual_tour_type['id'], 'object_id' => $listing_id));
        $panorama_arr = $this->ci->Upload_gallery_model->get_files_by_param($panorama_params);
        foreach ($panorama_arr as $panorama) {
            $this->ci->Upload_gallery_model->delete_file($panorama['id']);
        }
        $this->ci->Upload_gallery_model->delete_path($virtual_tour_type, $listing_id);
        unset($panorama_arr);

        // remove file
        if ($this->ci->pg_module->is_module_active('file_uploads') && !empty($listing['listing_file'])) {
            $this->ci->load->model('File_uploads_model');
            $this->ci->File_uploads_model->delete_upload($this->ci->Listings_uploads_model->file_config_id, $listing['prefix'], $listing['listing_file']);
        }

        if ($this->ci->pg_module->is_module_active('video_uploads') && !empty($listing['listing_video'])) {
            $this->ci->load->model('Video_uploads_model');
            $this->ci->Video_uploads_model->delete_upload($this->ci->Listings_uploads_model->video_config_id, $listing['prefix'], $listing['listing_video'], $listing['listing_video_data']['data']['upload_type']);
        }

        // remove from moderation
        $is_moderation_installed = $this->ci->pg_module->is_module_active('moderation');
        if ($is_moderation_installed) {
            $this->ci->load->model('listings/models/Listings_moderation_model');
            $this->ci->Listings_moderation_model->delete_listing($listing_id);
            $this->ci->Listings_moderation_model->delete_alert($listing_id);
        }

        // remove from wish lists
        if (!empty($listing['id_wish_lists'])) {
            $this->ci->load->model('listings/models/Wish_list_model');
            foreach ($listing['id_wish_lists'] as $wish_list) {
                $this->ci->Wish_list_model->delete_from_wish_list($wish_list, $listing_id);
            }
        }

        // remove seo settings
        if ($listing['id_seo_settings'] != 0) {
            $this->ci->load->model('Seo_model');
            $this->ci->Seo_model->delete_seo_tags($listing['id_seo_settings']);
        }

        // remove external data
        $this->ci->load->model('listings/models/Listings_events_model');
        $this->ci->Listings_events_model->ondelete_listing($listing);
    }

    /**
     * Activate/deactivate listing object
     *
     * Available status values: 1 - activate, 0 - deactivate
     *
     * @param integer $listing_id listing identifier
     * @param integer $status listing status
     * @param boolean $date_expired need saving expired date
     * @return void
     */
    public function activate_listing($listing_id, $status = self::STATUS_ACTIVE, $date_expired = false)
    {
        $data = array('status' => intval($status));
        if ($status) {
            $data['date_activity'] = date('Y-m-d H:i:s', time());
        }
        if ($date_expired !== false) {
            $data['date_expire'] = $date_expired;
        }
        $this->save_listing($listing_id, $data);
    }

    /**
     * Return sql criteria for listing searching as array
     *
     * @param array $filters filters data
     * @return array
     */
    protected function _get_search_criteria($filters)
    {
        $filters = array('data' => $filters, 'table' => LISTINGS_TABLE, 'type' => '');
        $use_optimization = $this->ci->pg_module->get_module_config('listings', 'use_optimization');
        if ($use_optimization) {
            $this->ci->load->model('optimize/models/Optimize_listings_model');
            $filters = $this->ci->Optimize_listings_model->get_filters($filters);
        }

        $params = array();

        $params['table'] = !empty($filters['table']) ? $filters['table'] : LISTINGS_TABLE;

        // by price range
        if (!empty($filters['data']['price_min'])) {
            $filters['data']['price_range'][0] = $filters['data']['price_min'];
            unset($filters['data']['price_min']);
        }

        if (!empty($filters['data']['price_max'])) {
            $filters['data']['price_range'][1] = $filters['data']['price_max'];
            unset($filters['data']['price_max']);
        }

        // by square range
        if (!empty($filters['data']['square_min'])) {
            $filters['data']['square_range'][0] = $filters['data']['square_min'];
            unset($filters['data']['square_min']);
        }

        if (!empty($filters['data']['square_max'])) {
            $filters['data']['square_range'][1] = $filters['data']['square_max'];
            unset($filters['data']['square_max']);
        }

        // by open house
        if (!empty($filters['data']['by_open_house'])) {
            $filters['data']['open_house'] = array(date('Y-m-d'), null);
            unset($filters['data']['by_open_house']);
        }

        // By private
        if (!empty($filters['data']['by_private'])) {
            $filters['data']['user_type'] = 'private';
            unset($filters['data']['by_private']);
        }

        // By agents
        if (!empty($filters['data']['by_agents'])) {
            if ($this->ci->pg_module->is_module_active('users')) {
                $this->ci->load->model('Users_model');
                $filters['data']['user_type'] = array_merge($this->ci->Users_model->getUserTypes(Users_model::GROUP_PROFESSIONALS));
            }
            unset($filters['data']['by_agents']);
        }

        // By keyword
        if (!empty($filters['data']['keyword'])) {
            $this->ci->load->model('listings/models/Listings_field_editor_model');
            $fe_criteria = $this->ci->Listings_field_editor_model->get_search_criteria($filters['data']);
            if (!empty($fe_criteria)) {
                $property_type_gids = $this->ci->Listings_field_editor_model->get_field_editor_types();
                $fe_criteria = array('where_sql' => array('(' . implode(' OR ', array_unique($fe_criteria)) . ')'));
                if (isset($filters['data']['field_editor'])) {
                    $filters['data']['field_editor'][0] = array_merge(
                        $filters['data']['field_editor'][0], $property_type_gids
                    );
                    $filters['data']['field_editor'][1] = array_merge(
                        $filters['data']['field_editor'][1], $fe_criteria
                    );
                } else {
                    $filters['data']['field_editor'] = array($property_type_gids, $fe_criteria);
                }
            }
        }

        // Field editor forms
        if (!empty($filters['data']['field_editor_data'])) {
            $field_editor_data = $filters['data']['field_editor_data'];
            unset($filters['data']['field_editor_data']);

            $fe_criteria = array();

            $property_type_gid = $this->get_field_editor_type($filters['data']);
            $this->ci->load->model('listings/models/Listings_form_model');
            $this->ci->load->model('field_editor/models/Field_editor_forms_model');
            foreach ($field_editor_data as $form_name => $form_data) {
                foreach ($form_data as $field_gid => $value) {
                    if (isset($value['range']) && is_array($value['range'])) {
                        if (isset($value['range']['min']))
                            $form_data[$field_gid . '_min'] = $value['range']['min'];
                        if (isset($value['range']['max']))
                            $form_data[$field_gid . '_max'] = $value['range']['max'];
                        unset($form_data[$field_gid]);
                    }
                }
                $form_gid = $this->ci->Listings_form_model->{$form_name . '_' . $property_type_gid . '_gid'};
                $temp = $this->ci->Field_editor_forms_model->get_search_criteria($form_gid, $form_data, $property_type_gid);
                if (!empty($temp)) {
                    $fe_criteria = array_merge($fe_criteria, $temp);
                }
            }
            if (!empty($fe_criteria)) {
                if (isset($filters['data']['field_editor'])) {
                    if (!in_array($property_type_gid, $filters['data']['field_editor'][0]))
                        $filters['data']['field_editor'][0][] = $property_type_gid;
                    $filters['data']['field_editor'][1] = array_merge($filters['data']['field_editor'][1], $fe_criteria);
                }else {
                    $filters['data']['field_editor'] = array(array($property_type_gid), $fe_criteria);
                }
            }
        }

        //booking
        if ($this->ci->pg_module->is_module_active('listings_booking')) {
            if (isset($filters['data']['booking_date_start'])) {
                $filters['data']['booking']['date_start'] = $filters['data']['booking_date_start'];
                unset($filters['data']['booking_date_start']);
            }

            if (isset($filters['data']['booking_date_end'])) {
                $filters['data']['booking']['date_end'] = $filters['data']['booking_date_end'];
                unset($filters['data']['booking_date_end']);
            }

            if (isset($filters['data']['booking_guests'])) {
                $filters['data']['booking']['guests'] = $filters['data']['booking_guests'];
                unset($filters['data']['booking_guests']);
            }
        }

        if (empty($filters['data'])) {
            return $params;
        }

        $fields = array_flip($this->fields);
        foreach ($filters['data'] as $filter_name => $filter_data) {
            if (!is_array($filter_data)) {
                $filter_data = trim($filter_data);
            }
            switch ($filter_name) {
                // By active
                case 'active':
                    if (!$filter_data) {
                        break;
                    }
                    $params = array_merge_recursive($params, array('where' => array($params['table'] . '.status' => self::STATUS_ACTIVE)));
                    break;
                // By not active
                case 'not_active':
                    if (!$filter_data) {
                        break;
                    }
                    $params = array_merge_recursive($params, array('where' => array($params['table'] . '.status' => self::STATUS_INACTIVE)));
                    break;
                // By not sold
                case 'not_sold':
                    if (!$filter_data) {
                        break;
                    }
                    $params = array_merge_recursive($params, array('where' => array($params['table'] . '.sold' => 0)));
                    break;
                // By country
                case 'country':
                    if (!$filter_data) {
                        break;
                    }
                    $params = array_merge_recursive($params, array('where' => array($params['table'] . '.id_country' => $filter_data)));
                    break;
                // By region
                case 'region':
                    if (intval($filter_data) == 0) {
                        break;
                    }
                    $params = array_merge_recursive($params, array('where' => array($params['table'] . '.id_region' => $filter_data)));
                    break;
                // By city
                case 'city':
                    if (intval($filter_data) == 0) {
                        break;
                    }
                    $params = array_merge_recursive($params, array('where' => array($params['table'] . '.id_city' => $filter_data)));
                    break;
                // By district
                case 'district':
                    if (intval($filter_data) == 0) {
                        break;
                    }
                    $params = array_merge_recursive($params, array('where' => array($params['table'] . '.id_district' => $filter_data)));
                    break;
                // By type
                case 'type':
                    if (empty($filter_data) || $filter_data == 'all') {
                        break;
                    }
                    if (!is_array($filter_data))
                        $filter_data = array($filter_data);
                    $type_ids = array();
                    foreach ($filter_data as $key => $value) {
                        if ($value == self::OPERATION_SOLD && in_array(self::OPERATION_SALE, $filter_data))
                            continue;
                        $type_ids[] = $this->get_operation_type_by_gid($value != self::OPERATION_SOLD ? $value : self::OPERATION_SALE);
                    }
                    if (empty($type_ids))
                        break;
                    $params = array_merge_recursive($params, array('where_in' => array($params['table'] . '.id_type' => $type_ids)));
                    if (in_array(self::OPERATION_SOLD, $filter_data)) {
                        if (!in_array(self::OPERATION_SALE, $filter_data)) {
                            $params = array_merge_recursive($params, array('where' => array($params['table'] . '.sold' => 1)));
                        }
                    } else {
                        if (in_array(self::OPERATION_SALE, $filter_data)) {
                            $params = array_merge_recursive($params, array('where' => array($params['table'] . '.sold' => 0)));
                        }
                    }
                    break;
                // By category
                case 'category':
                    if (!$filter_data)
                        break;
                    $params = array_merge_recursive($params, array('where' => array($params['table'] . '.id_category' => $filter_data)));
                    break;
                // By user
                case 'user':
                    if (!$filter_data) {
                        break;
                    }
                    if (is_array($filter_data)) {
                        $params = array_merge_recursive($params, array('where_in' => array($params['table'] . '.id_user' => $filter_data)));
                    } else {
                        $params = array_merge_recursive($params, array('where' => array($params['table'] . '.id_user' => $filter_data)));
                    }
                    break;
                // Exlude user
                case 'not_user':
                    if (!$filter_data)
                        break;
                    $params = array_merge_recursive($params, array('where' => array($params['table'] . '.id_user !=' => $filter_data)));
                    break;
                // By ids
                case 'ids':
                    if (empty($filter_data))
                        break;
                    if (is_array($filter_data)) {
                        $params = array_merge_recursive($params, array('where_in' => array($params['table'] . '.id' => $filter_data)));
                    } else {
                        $params = array_merge_recursive($params, array('where' => array($params['table'] . '.id' => $filter_data)));
                    }
                    break;
                // Exclude listing
                case 'exclude_id':
                    if (empty($filter_data))
                        break;
                    if (is_array($filter_data)) {
                        $params = array_merge_recursive($params, array('where_not_in' => array($params['table'] . '.id' => $filter_data)));
                    } else {
                        $params = array_merge_recursive($params, array('where' => array($params['table'] . '.id !=' => $filter_data)));
                    }
                    break;
                case 'price_range':
                    if (empty($filter_data))
                        break;
                    $where = array();
                    if ($this->ci->pg_module->is_module_active('currencies')) {
                        $this->ci->load->model('Currencies_model');
                        if (is_array($filter_data)) {
                            if (!empty($filter_data[1])) {
                                $filter_data[1] = $this->ci->Currencies_model->get_value_into_base_currency(
                                    $filter_data[1], $this->ci->Currencies_model->default_currency['gid']);
                                $where[] = $params['table'] . '.price_sorting<=' . $this->ci->db->escape($filter_data[1]);
                            }
                            if (!empty($filter_data[0])) {
                                $filter_data[0] = $this->ci->Currencies_model->get_value_into_base_currency(
                                    $filter_data[0], $this->ci->Currencies_model->default_currency['gid']);
                                switch ($filters['type']) {
                                    case self::OPERATION_SALE:
                                    case self::OPERATION_RENT:
                                    case self::OPERATION_SOLD:
                                        $where[] = $params['table'] . '.price_sorting>=' . $this->ci->db->escape($filter_data[0]);
                                        break;
                                    case self::OPERATION_BUY:
                                    case self::OPERATION_LEASE:
                                        $where[] = $params['table'] . '.price_max_sorting=0 OR price_max_sorting>=' . $this->ci->db->escape($filter_data[0]);
                                        break;
                                    default:
                                        $where[] = '(CASE WHEN ' . $params['table'] . '.id_type=' . $this->ci->db->escape($this->get_operation_type_by_gid(self::OPERATION_BUY)) . ' OR ' . $params['table'] . '.id_type=' . $this->ci->db->escape($this->get_operation_type_by_gid(self::OPERATION_LEASE)) . ' THEN (' . $params['table'] . '.price_max_sorting=0 OR price_max_sorting>=' . $this->ci->db->escape($filter_data[0]) . ') ELSE ' . $params['table'] . '.price_sorting>=' . $this->ci->db->escape($filter_data[0]) . ' END)';
                                        break;
                                }
                            }
                        } else {
                            $filter_data = $this->ci->Currencies_model->get_value_into_base_currency(
                                $filter_data, $this->ci->Currencies_model->default_currency['gid']);
                            $where[] = $params['table'] . '.price_sorting>=' . $this->ci->db->escape($filter_data);
                        }
                    } else {
                        if (is_array($filter_data)) {
                            if (!empty($filter_data[1])) {
                                $where[] = '(CASE WHEN ' . $params['table'] . '.price_reduced THEN ' . $params['table'] . '.price_reduced ELSE ' . $params['table'] . '.price END)<=' . $this->ci->db->escape($filter_data[1]);
                            }
                            if (!empty($filter_data[0])) {
                                switch ($filters['type']) {
                                    case self::OPERATION_SALE:
                                    case self::OPERATION_RENT:
                                    case self::OPERATION_SOLD:
                                        $where[] = '(CASE WHEN ' . $params['table'] . '.price_reduced THEN ' . $params['table'] . '.price_reduced ELSE ' . $params['table'] . '.price END)>=' . $this->ci->db->escape($filter_data[0]);
                                        break;
                                    case self::OPERATION_BUY:
                                    case self::OPERATION_LEASE:
                                        $where[] = $params['table'] . '.price_max=0 OR price_max>=' . $this->ci->db->escape($filter_data[0]);
                                        break;
                                    default:
                                        $where[] = '(CASE WHEN ' . $params['table'] . '.id_type=' . $this->ci->db->escape($this->get_operation_type_by_gid(self::OPERATION_BUY)) . ' OR ' . $params['table'] . '.id_type=' . $this->ci->db->escape($this->get_operation_type_by_gid(self::OPERATION_LEASE)) . ' THEN (' . $params['table'] . '.price_max=0 OR price_max>=' . $this->ci->db->escape($filter_data[0]) . ') ELSE (CASE WHEN ' . $params['table'] . '.price_reduced THEN ' . $params['table'] . '.price_reduced ELSE ' . $params['table'] . '.price END)>=' . $this->ci->db->escape($filter_data[0]) . ' END)';
                                        break;
                                }
                            }
                        } else {
                            $where[] = $params['table'] . '.price>=' . $this->ci->db->escape($filter_data);
                        }
                    }
                    if (!empty($where))
                        $params = array_merge_recursive($params, array('where_sql' => array('(' . $params['table'] . '.price_negotiated=' . $this->ci->db->escape(1) . ' OR ' . implode(' AND ', $where) . ')')));
                    break;
                case 'square_range':
                    if (empty($filter_data))
                        break;
                    $where = array();
                    if (is_array($filter_data)) {
                        if (!empty($filter_data[1])) {
                            $where[] = $params['table'] . '.square<=' . $this->ci->db->escape($filter_data[1]);
                        }
                        if (!empty($filter_data[0])) {
                            switch ($filters['type']) {
                                case self::OPERATION_SALE:
                                case self::OPERATION_RENT:
                                case self::OPERATION_SOLD:
                                    $where[] = $params['table'] . '.square>=' . $this->ci->db->escape($filter_data[0]);
                                    break;
                                case self::OPERATION_BUY:
                                case self::OPERATION_LEASE:
                                    $where[] = $params['table'] . '.square_max=0 OR square_max>=' . $this->ci->db->escape($filter_data[0]);
                                    break;
                                default:
                                    $where[] = '(CASE WHEN ' . $params['table'] . '.id_type=' . $this->ci->db->escape($this->get_operation_type_by_gid(self::OPERATION_BUY)) . ' OR ' . $params['table'] . '.id_type=' . $this->ci->db->escape($this->get_operation_type_by_gid(self::OPERATION_LEASE)) . ' THEN (' . $params['table'] . '.square_max=0 OR square_max>=' . $this->ci->db->escape($filter_data[0]) . ') ELSE ' . $params['table'] . '.square>=' . $this->ci->db->escape($filter_data[0]) . ' END)';
                                    break;
                            }
                        }
                    } else {
                        $where[] = $params['table'] . '.square>=' . $this->ci->db->escape($filter_data);
                    }
                    if (!empty($where))
                        $params = array_merge_recursive($params, array('where_sql' => array('(' . implode(' AND ', $where) . ')')));
                    break;
                case 'with_photo':
                    if (empty($filter_data))
                        break;
                    $params = array_merge_recursive($params, array('where' => array($params['table'] . '.photo_count >' => 0)));
                    break;
                case 'open_house':
                    if (empty($filter_data))
                        break;
                    if (is_array($filter_data)) {
                        $where = array();
                        if (!empty($filter_data[0])) {
                            $where[$params['table'] . '.date_open >='] = $filter_data[0];
                            if (!empty($filter_data[1])) {
                                $where[$params['table'] . '.date_open_begin >='] = $filter_data[1];
                            }
                            if (!empty($filter_data[2])) {
                                $where[$params['table'] . '.date_open_end <='] = $filter_data[2];
                            }
                        }
                    } else {
                        $where = array($params['table'] . '.date_open >=' => $filter_data);
                    }
                    $params = array_merge_recursive($params, array('where' => $where));
                    break;
                case 'user_type':
                    if (empty($filter_data))
                        break;
                    if (is_array($filter_data)) {
                        $params = array_merge_recursive($params, array('where_in' => array($params['table'] . '.user_type' => $filter_data)));
                    } else {
                        $params = array_merge_recursive($params, array('where' => array($params['table'] . '.user_type' => $filter_data)));
                    }
                    break;
                // By featured
                case 'featured':
                    if (empty($filter_data))
                        break;
                    $params = array_merge_recursive($params, array('where' => array($params['table'] . '.is_featured' => 1)));
                    break;
                // By slider image
                case 'slider_image':
                    if (empty($filter_data))
                        break;
                    $params = array_merge_recursive($params, array('where_sql' => array($params['table'] . '.slider_image !=' . $this->ci->db->escape(''))));
                    break;
                // By booking
                case 'booking':
                    if ($this->ci->pg_module->is_module_active('listings_booking')) {
                        if (!empty($filter_data['date_start']) && !empty($filter_data['date_end'])) {
                            $this->ci->load->model('listings_booking/models/Listings_booking_model');
                            $ids = $this->ci->Listings_booking_model->get_booking_listings($filter_data);
                            if (!empty($ids)) {
                                $params = array_merge_recursive($params, array('where_not_in' => array($params['table'] . '.id' => $ids)));
                            }
                        }
                        if (!empty($filter_data['guests'])) {
                            $params = array_merge_recursive($params, array('where_custom' => '(guest_count >= ' . intval($filter_data['guests']) . ' OR guest_count = 0)'));
                        }
                    }
                    break;
                // By field editor
                case 'field_editor':
                    if (empty($filter_data)) {
                        break;
                    }

                    $this->ci->load->model('listings/models/Listings_field_editor_model');
                    $filter_data[0] = $this->ci->Listings_field_editor_model->get_search_tables($filter_data[0]);

                    foreach ($filter_data[0] as $key => $field_editor_table) {
                        $filter_data[0][$key] = array('table' => $field_editor_table, 'condition' => $params['table'] . '.id=' . $field_editor_table . '.id_listing', 'type' => 'LEFT');
                    }
                    $params = array_merge_recursive($params, array('join' => $filter_data[0]), $filter_data[1]);
                    break;
                // By discount
                case 'by_discount':
                    if (empty($filter_data)) {
                        break;
                    }
                    $params = array_merge_recursive($params, array('where' => array($params['table'] . '.price_reduced >' => 0)));
                    break;
                case 'cron_date_expire':
                    $params = array_merge_recursive($params, array('where' => array($params['table'] . '.date_expire <' => date('Y-m-d H:i:s'), 'UNIX_TIMESTAMP(date_expire) >' => '86400')));
                    break;
                case 'cron_featured_date_end':
                    $params = array_merge_recursive($params, array('where' => array($params['table'] . '.featured_date_end <' => date('Y-m-d H:i:s'), 'UNIX_TIMESTAMP(featured_date_end) >' => '86400')));
                    break;
                case 'cron_lift_up_date_end':
                    $params = array_merge_recursive($params, array('where' => array($params['table'] . '.lift_up_date_end <' => date('Y-m-d H:i:s'), 'UNIX_TIMESTAMP(lift_up_date_end) >' => '86400')));
                    break;
                case 'cron_lift_up_country_date_end':
                    $params = array_merge_recursive($params, array('where' => array($params['table'] . '.lift_up_country_date_end <' => date('Y-m-d H:i:s'), 'UNIX_TIMESTAMP(lift_up_country_date_end) >' => '86400')));
                    break;
                case 'cron_lift_up_region_date_end':
                    $params = array_merge_recursive($params, array('where' => array($params['table'] . '.lift_up_region_date_end <' => date('Y-m-d H:i:s'), 'UNIX_TIMESTAMP(lift_up_region_date_end) >' => '86400')));
                    break;
                case 'cron_lift_up_city_date_end':
                    $params = array_merge_recursive($params, array('where' => array($params['table'] . '.lift_up_city_date_end <' => date('Y-m-d H:i:s'), 'UNIX_TIMESTAMP(lift_up_city_date_end) >' => '86400')));
                    break;
                case 'cron_highlight_date_end':
                    $params = array_merge_recursive($params, array('where' => array($params['table'] . '.highlight_date_end <' => date('Y-m-d H:i:s'), 'UNIX_TIMESTAMP(highlight_date_end) >' => '86400')));
                    break;
                case 'cron_slide_show_date_end':
                    $params = array_merge_recursive($params, array('where' => array($params['table'] . '.slide_show_date_end <' => date('Y-m-d H:i:s'), 'UNIX_TIMESTAMP(slide_show_date_end) >' => '86400')));
                    break;
                // By field
                default:
                    if (isset($fields[$filter_name])) {
                        if (is_array($filter_data)) {
                            $params = array_merge_recursive($params, array('where_in' => array($params['table'] . '.' . $filter_name => $filter_data)));
                        } else {
                            $params = array_merge_recursive($params, array('where' => array($params['table'] . '.' . $filter_name => $filter_data)));
                        }
                    }
                    break;
            }
        }

        return $params;
    }

    /**
     * Return listings object from data source as array
     *
     * @param integer $page page of results
     * @param string $limits results per page
     * @param array $order_by sorting data
     * @param array $params sql criteria
     * @param boolean $formatted format results
     * @return array
     */
    protected function _get_listings_list($page = null, $limits = null, $order_by = null, $params = array(), $formatted = true)
    {
        $table = isset($params['table']) ? $params['table'] : LISTINGS_TABLE;

        $this->ci->db->select($table . '.' . implode(', ' . $table . '.', $this->fields));
        $this->ci->db->from($table);

        if (isset($params['join']) && is_array($params['join']) && count($params['join'])) {
            $temp_tables = array();
            foreach ($params['join'] as $join) {
                if ($join['table'] == $table || in_array($join['table'], $temp_tables))
                    continue;
                $this->ci->db->join($join['table'], $join['condition'], $join['type']);
                $temp_tables[] = $join['table'];
            }
        }

        if (isset($params['where']) && is_array($params['where']) && count($params['where'])) {
            foreach ($params['where'] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params['or_where']) && is_array($params['or_where']) && count($params['or_where'])) {
            foreach ($params['or_where'] as $field => $value) {
                $this->ci->db->or_where($field, $value);
            }
        }

        if (isset($params['geolocation_where']) && is_array($params['geolocation_where']) && count($params['geolocation_where'])) {
            foreach ($params['geolocation_where'] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params['where_in']) && is_array($params['where_in']) && count($params['where_in'])) {
            foreach ($params['where_in'] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params['where_not_in']) && is_array($params['where_not_in']) && count($params['where_not_in'])) {
            foreach ($params['where_not_in'] as $field => $value) {
                $this->ci->db->where_not_in($field, $value);
            }
        }

        if (isset($params['where_sql']) && is_array($params['where_sql']) && count($params['where_sql'])) {
            foreach ($params['where_sql'] as $value) {
                $this->ci->db->where($value, null, false);
            }
        }
        if (isset($params['where_custom']) && !empty($params['where_custom'])) {
            $this->ci->db->where($params['where_custom']);
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->fields) || strpos($field, 'MATCH') === 0) {
                    $this->ci->db->order_by($field . " " . $dir);
                }
            }
        } elseif ($order_by) {
            $this->ci->db->order_by($order_by);
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($limits, $limits * ($page - 1));
        }

        if (isset($params['special_order_by'])) {

            if (!empty($params['get_listings_by_radius'])) {
                $this->ci->db->_reset_select();

                $distance = $params['get_listings_by_radius']['radius'];
                $latitude = $params['get_listings_by_radius']['lat'];
                $longitude = abs($params['get_listings_by_radius']['lon']);

                $query = $this->ci->db->query("SELECT *,
                                    ( 3959 * acos( cos( radians('{$latitude}') ) *
                                    cos( radians( " . LISTINGS_TABLE . ".lat ) ) *
                                    cos( radians(abs( " . LISTINGS_TABLE . ".lon) ) -
                                    radians('{$longitude}') ) +
                                    sin( radians('{$latitude}') ) *
                                    sin( radians( " . LISTINGS_TABLE . ".lat ) ) ) )
                                    AS distance FROM " . LISTINGS_TABLE . " WHERE ( 3959 * acos( cos( radians('{$latitude}') ) *
                                    cos( radians( " . LISTINGS_TABLE . ".lat ) ) *
                                    cos( radians( abs( " . LISTINGS_TABLE . ".lon) ) -
                                    radians('{$longitude}') ) +
                                    sin( radians('{$latitude}') ) *
                                    sin( radians( " . LISTINGS_TABLE . ".lat ) ) ) ) < '{$distance}' ORDER BY distance ASC LIMIT 0, 10");

                $results = $query->result_array();
            } else {

                $original_query = $this->ci->db->_compile_select();
                $query = (stripos($original_query, 'order by')) ? explode('ORDER BY', $original_query) : explode('LIMIT', $original_query);
                $query = $query[0];
                $this->ci->db->_reset_select();

                $query_limits = '';
                if (!is_null($page)) {
                    $page = intval($page) ? intval($page) : 1;
                    $query_limits = 'LIMIT ' . ($limits * ($page - 1)) . ', ' . $limits;
                }

                $query_order = 'ORDER BY ';
                if (isset($params['special_order_by']['field'])) {
                     $query_order = $query_order . "CASE WHEN " . $params['special_order_by']['field'] . " = '"
                        . $params['special_order_by']['value'] . "' THEN 1 ELSE 2 END, ";
                } elseif (is_array($params['special_order_by']) && count($params['special_order_by']) > 0) {
                    foreach ($params['special_order_by'] as $key => $value) {
                        $query_order = $query_order . $key
                             . " " . $value .",";
                    }
                }
                if (is_array($order_by) && count($order_by) > 0) {
                    foreach ($order_by as $field => $dir) {
                        if (in_array($field, $this->fields) || strpos($field, 'MATCH') === 0) {
                            $query_order = $query_order . $field . ' ' . $dir . ',';
                        }
                    }
                } elseif ($order_by) {
                    $query_order = $query_order . ", " . $order_by . " ";
                }
                if (empty($query_order)) {
                    $query_order = 'id ASC ';
                }

                $query_order = trim($query_order, ',');
                $query_order = $query_order . " ";

                $query = $this->ci->db->query($query . $query_order . $query_limits);

                $results = $query->result_array();
            }
        }elseif (!empty($params['get_listings_by_radius'])) {
            $this->ci->db->_reset_select();

            $distance = $params['get_listings_by_radius']['radius'];
            $latitude = $params['get_listings_by_radius']['lat'];
            $longitude = abs($params['get_listings_by_radius']['lon']);

            $query = $this->ci->db->query("SELECT *,
                                    ( 3959 * acos( cos( radians('{$latitude}') ) *
                                    cos( radians( " . LISTINGS_TABLE . ".lat ) ) *
                                    cos( radians(abs( " . LISTINGS_TABLE . ".lon) ) -
                                    radians('{$longitude}') ) +
                                    sin( radians('{$latitude}') ) *
                                    sin( radians( " . LISTINGS_TABLE . ".lat ) ) ) )
                                    AS distance FROM " . LISTINGS_TABLE . " WHERE ( 3959 * acos( cos( radians('{$latitude}') ) *
                                    cos( radians( " . LISTINGS_TABLE . ".lat ) ) *
                                    cos( radians( abs( " . LISTINGS_TABLE . ".lon) ) -
                                    radians('{$longitude}') ) +
                                    sin( radians('{$latitude}') ) *
                                    sin( radians( " . LISTINGS_TABLE . ".lat ) ) ) ) < '{$distance}' ORDER BY distance ASC LIMIT 0, 10");

            $results = $query->result_array();
        } else {
            $results = $this->ci->db->get()->result_array();
        }

        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $data[] = $r;
            }
            if ($formatted) {
                $data = $this->format_listings($data);
            }
            return $data;
        }
        return array();
    }

    /**
     * Return number of listings in data source
     *
     * @param array $params sql criteria
     * @return integer
     */
    protected function _get_listings_count($params = null)
    {
        $table = isset($params['table']) ? $params['table'] : LISTINGS_TABLE;

        $this->ci->db->select('COUNT(*) AS cnt');
        $this->ci->db->from($table);

        if (isset($params['join']) && is_array($params['join']) && count($params['join'])) {
            $temp_tables = array();
            foreach ($params['join'] as $join) {
                if ($join['table'] == $table || in_array($join['table'], $temp_tables))
                    continue;
                $this->ci->db->join($join['table'], $join['condition'], $join['type']);
                $temp_tables[] = $join['table'];
            }
        }

        if (isset($params['where']) && is_array($params['where']) && count($params['where'])) {
            foreach ($params['where'] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params['where_in']) && is_array($params['where_in']) && count($params['where_in'])) {
            foreach ($params['where_in'] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params['where_not_in']) && is_array($params['where_not_in']) && count($params['where_not_in'])) {
            foreach ($params['where_not_in'] as $field => $value) {
                $this->ci->db->where_not_in($field, $value);
            }
        }

        if (isset($params['where_sql']) && is_array($params['where_sql']) && count($params['where_sql'])) {
            foreach ($params['where_sql'] as $value) {
                $this->ci->db->where($value, null, false);
            }
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]['cnt']);
        }
        return 0;
    }

    /**
     * Return filtered listings objects from data source as array
     *
     * @param array $filters filters data
     * @param integer $page page of results
     * @param integer $items_on_page results per page
     * @param string $order_by sorting data
     * @param boolean $formatted format results
     * @return array
     */
    public function get_listings_list($filters = array(), $page = null, $items_on_page = null, $order_by = null, $formatted = true, $order_user_filter = false, $order_direction='DESC')
    {
        $this->ci->load->helper('cookie');

        $params = $this->_get_search_criteria($filters);

        /* order listings by geolocation */
        if ($this->ci->pg_module->get_module_config('country_identification', 'automatic_identification') && $order_user_filter === false) {
            $this->ci->load->model('countries/models/Countries_model');
            $user = array();

            if ($this->ci->session->userdata('auth_type') == 'user') {
                $this->ci->load->model('Users_model');
                $user = $this->ci->Users_model->get_user_by_id($this->ci->session->userdata('user_id'));
                if (empty($user['id_city'])) {
                    $user = array(
                        'id_city' => $this->ci->session->userdata('id_city'),
                        'id_region' => $this->ci->session->userdata('id_region'),
                        'id_country' => $this->ci->session->userdata('id_country'),
                    );
                }
            } else {
                $user = array(
                    'id_city' => $this->ci->session->userdata('id_city'),
                    'id_region' => $this->ci->session->userdata('id_region'),
                    'id_country' => $this->ci->session->userdata('id_country'),
                );
                if (empty($user['id_city']))
                    $user['id_city'] = get_cookie('id_city');
                if (empty($user['id_region']))
                    $user['id_region'] = get_cookie('id_region');
                if (empty($user['id_country']))
                    $user['id_country'] = get_cookie('id_country');
            }

            if (!empty($user)) {
                if (!empty($user['id_city'])) {
                    $params['geolocation_where'] = array('id_city' => $user['id_city']);
                    $if_listings_location_exists = $this->_get_listings_list($page, $items_on_page, $order_by, $params, $formatted);
                    unset($params['geolocation_where']);
                    if (!empty($if_listings_location_exists)) {
                        if (!empty($filters['get_local_listings'])) {
                            $params['where']['id_city'] = $user['id_city'];
                        }
                        if (!empty($filters['get_listings_by_radius'])) {
                            $params['get_listings_by_radius'] = $filters['get_listings_by_radius'];
                        }
                        $params['special_order_by'] = array("CASE WHEN is_lift_up = '1' THEN lift_up_date_end ELSE 2 END" => $order_direction,
                                "CASE WHEN id_city = '".$user['id_city']."' THEN 1 ELSE 2 END" => $order_direction == 'ASC' ? 'DESC' : 'ASC');
                        return $this->_get_listings_list($page, $items_on_page, $order_by, $params, $formatted);
                    }
                }
                if (!empty($user['id_region'])) {
                    $params['geolocation_where'] = array('id_region' => $user['id_region']);
                    $if_listings_location_exists = $this->_get_listings_list($page, $items_on_page, $order_by, $params, $formatted);
                    unset($params['geolocation_where']);
                    if (!empty($if_listings_location_exists)) {
                        if (!empty($filters['get_local_listings'])) {
                            $params['where']['id_region'] = $user['id_region'];
                        }
                        if (!empty($filters['get_listings_by_radius'])) {
                            $params['get_listings_by_radius'] = $filters['get_listings_by_radius'];
                        }
                        $params['special_order_by'] = array("CASE WHEN is_lift_up = '1' THEN lift_up_date_end ELSE 2 END" => $order_direction,
                                "CASE WHEN id_region = '".$user['id_region']."' THEN 1 ELSE 2 END" => $order_direction == 'ASC' ? 'DESC' : 'ASC');
                        return $this->_get_listings_list($page, $items_on_page, $order_by, $params, $formatted);
                    }
                }
                if (!empty($user['id_country'])) {
                    $country = $this->ci->Countries_model->get_country_by_id($user['id_country']);
                    $params['geolocation_where'] = array('id_country' => $country['code']);
                    $if_listings_location_exists = $this->_get_listings_list($page, $items_on_page, $order_by, $params, $formatted);
                    unset($params['geolocation_where']);
                    if (!empty($if_listings_location_exists)) {
                        if (!empty($filters['get_local_listings'])) {
                            $params['where']['id_country'] = $user['id_country'];
                        }
                        if (!empty($filters['get_listings_by_radius'])) {
                            $params['get_listings_by_radius'] = $filters['get_listings_by_radius'];
                        }
                         $params['special_order_by'] = array("CASE WHEN is_lift_up = '1' THEN lift_up_date_end ELSE 2 END" => $order_direction,
                                "CASE WHEN id_country = '".$country['code']."' THEN 1 ELSE 2 END" => $order_direction == 'ASC' ? 'DESC' : 'ASC');
                        return $this->_get_listings_list($page, $items_on_page, $order_by, $params, $formatted);
                    }
                }
            }
        }

        return $this->_get_listings_list($page, $items_on_page, $order_by, $params, $formatted);
    }

    /**
     * Return number of filtered listings objects in data source
     *
     * @param array $filters filters data
     * @return array
     */
    public function get_listings_count($filters = array())
    {
        $params = $this->_get_search_criteria($filters);
        return $this->_get_listings_count($params);
    }

    /**
     * Return filtered listings objects for pagination from data source
     *
     * @param array $filters filters data
     * @param integer $page page of results
     * @param integer $limits results per page
     * @param string $order_by sorting data
     * @param boolean $formatted format result
     * @return array
     */
    public function get_listings_pagination($filters = array(), $page = null, $limits = null, $order_by = null, $formatted = true)
    {
        $params = $this->_get_search_criteria($filters);

        $table = isset($params['table']) ? $params['table'] : LISTINGS_TABLE;

        $this->ci->db->select($table . '.id');
        $this->ci->db->from($table);

        if (isset($params['join']) && is_array($params['join']) && count($params['join'])) {
            foreach ($params['join'] as $join) {
                $this->ci->db->join($join['table'], $join['condition'], $join['type']);
            }
        }

        if (isset($params['where']) && is_array($params['where']) && count($params['where'])) {
            foreach ($params['where'] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params['where_in']) && is_array($params['where_in']) && count($params['where_in'])) {
            foreach ($params['where_in'] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params['where_sql']) && is_array($params['where_sql']) && count($params['where_sql'])) {
            foreach ($params['where_sql'] as $value) {
                $this->ci->db->where($value, null, false);
            }
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->fields)) {
                    $this->ci->db->order_by($field . ' ' . $dir);
                }
            }
        } elseif ($order_by) {
            $this->ci->db->order_by($order_by);
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($limits + ($page > 1 ? 4 : 2), $limits * ($page - 1) - ($page > 1 ? 2 : 0));
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $data[] = $r;
            }
            return $data;
        }
        return array();
    }

    /**
     * Return filtered listings objects from data source by key
     *
     * @param array $filters filters data
     * @param integer $page page of results
     * @param integer $items_on_page results per page
     * @param array $order_by sorting data
     * @param boolean $formatted format results
     * @return array
     */
    public function get_listings_list_by_key($filters = array(), $page = null, $items_on_page = null, $order_by = null, $formatted = true)
    {
        $params = $this->_get_search_criteria($filters);
        $listings_data = $this->_get_listings_list($page, $items_on_page, $order_by, $params, $formatted);
        $return_data = array();
        foreach ($listings_data as $data) {
            $return_data[$data['id']] = $data;
        }
        return $return_data;
    }

    /**
     * Return listings objects by identifiers set
     *
     * @param array $listing_ids listings identifiers set
     * @param boolean $formatted format results
     * @return array
     */
    public function get_listings_list_by_id($listing_ids, $formatted = true)
    {
        $params['where_in']['id'] = $listing_ids;
        return $this->_get_listings_list(null, null, null, $params, $formatted);
    }

    /**
     * Return search criteria for form from field editor
     *
     * @param string $form_gid GUID form from field editor
     * @param array $data listing data
     * @param string $property_type_gid property type (GUID)
     * @return array
     */
    public function get_listings_ids_by_form($form_gid, $data, $property_type_gid)
    {
        $this->ci->load->model('field_editor/models/Field_editor_forms_model');
        $params = $this->ci->Field_editor_forms_model->get_search_criteria($form_gid, $data, $property_type_gid);
        return $this->_get_listings_ids_from_category($property_type_gid, $params);
    }

    /**
     * Change settings for listings formatting
     *
     * @param string $name parameter name
     * @param mixed $value parameter value
     * @return void
     */
    public function set_format_settings($name, $value = false)
    {
        if (!is_array($name)) {
            $name = array($name => $value);
        }
        foreach ($name as $key => $item) {
            $this->format_settings[$key] = $item;
        }
    }

    /**
     * Validate listing data for saving to data source
     *
     * @param integer $listing_id listing identifier
     * @param array $data listing data
     * @return array
     */
    public function validate_listing($listing_id, $data)
    {
        $return = array('errors' => array(), 'data' => array());

        $is_moderation_installed = $this->ci->pg_module->is_module_active("moderation");

        if ($listing_id) {
            $this->set_format_settings('use_format', false);
            $listing = $this->get_listing_by_id($listing_id);
            $this->set_format_settings('use_format', true);
        } else {
            $listing = array('id_type' => 0);
        }

        if (isset($data['id'])) {
            $return['data']['id'] = intval($data['id']);
            if (empty($return['data']['id']))
                unset($return['data']['id']);
        }

        if (isset($data['gid'])) {
            $return['data']['gid'] = trim(strip_tags($data['gid']));
            if (empty($return['data']['gid']))
                unset($return['data']['gid']);
        }

        if ($this->ci->pg_module->is_module_active("users")) {
            if (isset($data['id_user']) && $data['id_user'] !== false) {
                $return['data']['id_user'] = intval($data['id_user']);
                if (empty($return['data']['id_user']))
                    $return['errors'][] = l('error_empty_user', 'listings');
            }elseif (!$listing_id) {
                $return['errors'][] = l('error_empty_user', 'listings');
            }
        }

        if (isset($data['id_type'])) {
            $return['data']['id_type'] = intval($data['id_type']);
            if (empty($return['data']['id_type'])) {
                $return['errors'][] = l('error_empty_operation_type', 'listings');
            } elseif (!array_key_exists($return['data']['id_type'], $this->get_operation_types())) {
                $return['errors'][] = l('error_invalid_operation_type', 'listings');
            } else {
                $listing['id_type'] = $return['data']['id_type'];
            }
        } elseif (!$listing_id) {
            $return['errors'][] = l('error_empty_operation_type', 'listings');
        }

        if (isset($data['id_category'])) {
            $return['data']['id_category'] = intval($data['id_category']);
            if (empty($return['data']['id_category'])) {
                $return['errors'][] = l('error_empty_category', 'listings');
            } elseif (!$this->get_field_editor_type(array_merge($listing, $return['data']))) {
                $return['errors'][] = l('error_invalid_category', 'listings');
            } else {
                $listing['id_category'] = $data['id_category'];
            }
        } elseif (!$listing_id) {
            $return['errors'][] = l('error_empty_category', 'listings');
        }

        if (isset($data['property_type'])) {
            $return['data']['property_type'] = intval($data['property_type']);
            if (empty($return['data']['property_type'])) {
                $return['errors'][] = l('error_empty_property_type', 'listings');
            } else {
                $listing['property_type'] = $return['data']['property_type'];

                $this->ci->load->helper('properties');
                $property_type = property_value($listing);
                if (!$property_type) {
                    $return['errors'][] = l('error_invalid_property_type', 'listings');
                }
            }
        } elseif (!$listing_id) {
            $return['errors'][] = l('error_empty_property_type', 'listings');
        }

        if (isset($data['status'])) {
            $return['data']['status'] = $listing['status'] = $data['status'] ? self::STATUS_ACTIVE : self::STATUS_INACTIVE;
        }

        if (isset($data['date_open'])) {
            $value = strtotime($data['date_open']);
            if ($value > 0) {
                $return['data']['date_open'] = date('Y-m-d', $value);
            } else {
                $return['data']['date_open'] = '0000-00-00 00:00:00';
            }
        }

        if (isset($data['date_open_begin'])) {
            $return['data']['date_open_begin'] = intval($data['date_open_begin']);
            if (empty($return['data']['date_open_begin'])) {
                unset($return['data']['date_open_begin']);
            } else {
                $value = ld_option('dayhour-names', 'start', $return['data']['date_open_begin']);
                if (empty($value))
                    unset($return['data']['date_open_begin']);
            }
        }

        if (isset($data['date_open_end'])) {
            $return['data']['date_open_end'] = intval($data['date_open_end']);
            if (empty($return['data']['date_open_end'])) {
                unset($return['data']['date_open_end']);
            } else {
                $value = ld_option('dayhour-names', 'start', $return['data']['date_open_end']);
                if (empty($value))
                    unset($return['data']['date_open_end']);
            }
        }

        if (isset($data['guest_count'])) {
            $return['data']['guest_count'] = intval($data['guest_count']);
        }

        if (isset($data['date_available'])) {
            $value = strtotime($data['date_available']);
            if ($value > 0) {
                $return['data']['date_available'] = date('Y-m-d', $value);
            } else {
                $return['data']['date_available'] = '0000-00-00 00:00:00';
            }
        }

        if (isset($data[self::OPERATION_SOLD])) {
            $return['data'][self::OPERATION_SOLD] = $data[self::OPERATION_SOLD] ? 1 : 0;
        }

        if (isset($data['date_created'])) {
            $value = strtotime($data['date_created']);
            if ($value > 0) {
                $return['data']['date_created'] = date('Y-m-d H:i:s', $value);
            } else {
                $return['data']['date_created'] = '0000-00-00 0000:00:00';
            }
        }

        if (isset($data['date_modified'])) {
            $value = strtotime($data['date_modified']);
            if ($value > 0) {
                $return['data']['date_modified'] = date('Y-m-d H:i:s', $value);
            } else {
                $return['data']['date_modified'] = '0000-00-00 0000:00:00';
            }
        }

        if (isset($data['date_activity'])) {
            $value = strtotime($data['date_activity']);
            if ($value > 0) {
                $return['data']['date_activity'] = date('Y-m-d H:i:s', $value);
            } else {
                $return['data']['date_activity'] = '0000-00-00 00:00:00';
            }
        }

        if (isset($data['date_expire'])) {
            $value = strtotime($data['date_expire']);
            if ($value > 0) {
                $return['data']['date_expire'] = date('Y-m-d H:i:s', $value);
            } else {
                $return['data']['date_expire'] = '0000-00-00 00:00:00';
            }
        }

        if (isset($data['price_negotiated'])) {
            $return['data']['price_negotiated'] = $listing['price_negotiated'] = $data['price_negotiated'] ? 1 : 0;
        }

        $is_payments_installed = $this->ci->pg_module->is_module_active('currencies');
        if ($is_payments_installed) {
            if (isset($data['gid_currency'])) {
                $return['data']['gid_currency'] = trim(strip_tags($data['gid_currency']));
                if (!isset($listing['price_negotiated']) || empty($listing['price_negotiated'])) {
                    if (empty($return['data']['gid_currency'])) {
                        $return['errors'][] = l('error_empty_currency', 'listings');
                    } else {
                        $is_currency = false;
                        $this->ci->load->model('Currencies_model');
                        $currencies = $this->ci->Currencies_model->get_currency_list();
                        foreach ($currencies as $currency) {
                            if ($currency['gid'] == $return['data']['gid_currency']) {
                                $is_currency = true;
                                break;
                            }
                        }
                        if (!$is_currency) {
                            $return['errors'][] = l('error_invalid_currency', 'listings');
                        } else {
                            $listing['gid_currency'] = $return['data']['gid_currency'];
                        }
                    }
                } else {
                    $listing['gid_currency'] = $return['data']['gid_currency'];
                }
            } elseif (!$listing_id) {
                if (isset($data['gid_currency'])) {
                    if (!isset($listing['price_negotiated']) || empty($listing['price_negotiated'])) {
                        $return['errors'][] = l('error_empty_currency', 'listings');
                    }
                }
            }
        }

        $need_min_price = in_array($this->getOperationTypeById($listing['id_type']), array(self::OPERATION_SALE, self::OPERATION_RENT));
        if (isset($data['price'])) {
            $return['data']['price'] = floatval(str_replace(' ', '', $data['price']));
            if (empty($return['data']['price']) && $need_min_price) {
                $return['errors'][] = l('error_empty_price', 'listings');
            } else {
                $listing['price'] = $return['data']['price'];
                if ($is_payments_installed && isset($listing['gid_currency'])) {
                    $this->ci->load->model('Currencies_model');
                    $return['data']['price_sorting'] = $this->ci->Currencies_model->
                        get_value_into_base_currency($listing['price'], $listing['gid_currency']);
                } else {
                    $return['data']['price_sorting'] = 0;
                }
            }
        } elseif (!$listing_id && $need_min_price) {
            if (isset($data['price'])) {
                if (!isset($listing['price_negotiated']) || empty($listing['price_negotiated'])) {
                    $return['errors'][] = l('error_empty_price', 'listings');
                }
            }
        }

        if (isset($data['price_max'])) {
            $use_price_max = in_array($this->getOperationTypeById($listing['id_type']), array(self::OPERATION_BUY, self::OPERATION_LEASE));
            if ($use_price_max) {
                $return['data']['price_max'] = floatval(str_replace(' ', '', $data['price_max']));
                if (!empty($return['data']['price_max'])) {
                    if (isset($listing['price']) && $listing['price'] > $return['data']['price_max']) {
                        $return['errors'][] = l('error_invalid_max_price', 'listings');
                    } else {
                        $listing['price_max'] = $return['data']['price_max'];
                        if ($is_payments_installed && isset($listing['gid_currency'])) {
                            $this->ci->load->model('Currencies_model');
                            $return['data']['price_max_sorting'] = $this->ci->Currencies_model->
                                get_value_into_base_currency($listing['price_max'], $listing['gid_currency']);
                        } else {
                            $return['data']['price_max_sorting'] = 0;
                        }
                    }
                }
            }
        }
        if (isset($listing['price']) && isset($listing['price_max'])) {
            if (empty($listing['price']) && empty($listing['price_max'])) {
                if (!isset($listing['price_negotiated']) || empty($listing['price_negotiated'])) {
                    $return['errors'][] = l('error_empty_price', 'listings');
                }
            }
        }

        if (isset($data['price_old'])) {
            $return['data']['price'] = floatval($data['price_old']);
        }

        if (in_array($this->getOperationTypeById($listing['id_type']), array(self::OPERATION_RENT))) {
            if (isset($data['price_week'])) {
                $return['data']['price_week'] = floatval(str_replace(' ', '', $data['price_week']));
            }

            if (isset($data['price_month'])) {
                $return['data']['price_month'] = floatval(str_replace(' ', '', $data['price_month']));
            }
        }

        if (in_array($this->getOperationTypeById($listing['id_type']), array(self::OPERATION_SALE, self::OPERATION_RENT))) {
            if (isset($data['price_reduced'])) {
                $return['data']['price_reduced'] = floatval(str_replace(' ', '', $data['price_reduced']));
                if (!empty($return['data']['price_reduced'])) {
                    if ($listing['price'] <= $return['data']['price_reduced']) {
                        $return['errors'][] = l('error_invalid_reduced_price', 'listings');
                    } else {
                        $listing['price_reduced'] = $return['data']['price_reduced'];
                        if ($is_payments_installed && isset($listing['gid_currency'])) {
                            $this->ci->load->model('Currencies_model');
                            $return['data']['price_sorting'] = $this->ci->Currencies_model->
                                get_value_into_base_currency($listing['price_reduced'], $listing['gid_currency']);
                        } else {
                            $return['data']['price_sorting'] = 0;
                        }
                    }
                }
            }
        }

        if (in_array($this->getOperationTypeById($listing['id_type']), array(self::OPERATION_RENT, self::OPERATION_LEASE))) {
            if (isset($data['price_period'])) {
                $return['data']['price_period'] = strval($data['price_period']);
                if (empty($return['data']['price_period'])) {
                    $return['errors'][] = l('error_empty_price_period', 'listings');
                } else {
                    $value = ld_option('price_period', 'listings', $return['data']['price_period']);
                    if (empty($value)) {
                        $return['errors'][] = l('error_invalid_price_period', 'listings');
                    }
                }
            }

            if (isset($data['price_type'])) {
                $return['data']['price_type'] = strval($data['price_type']);
                if (empty($return['data']['price_type'])) {
                    $return['errors'][] = l('error_empty_price_type', 'listings');
                } else {
                    $value = ld_option('price_type', 'listings', $return['data']['price_type']);
                    if (empty($value)) {
                        $return['errors'][] = l('error_invalid_price_type', 'listings');
                    }
                }
            }
        }

        if (isset($data['price_sorting'])) {
            $return['data']['price_sorting'] = floatval($data['price_sorting']);
        }

        if (isset($data['price_auction'])) {
            $return['data']['price_auction'] = $data['price_auction'] ? 1 : 0;
        }

        if (isset($data['square_unit'])) {
            $return['data']['square_unit'] = trim(strip_tags($data['square_unit']));
            if (empty($return['data']['square_unit'])) {
                $return['errors'][] = l('error_empty_square_unit', 'listings');
            } else {
                $square_unit = ld_option('square_unit', 'listings', $return['data']['square_unit']);
            }
        }

        if (isset($data['square'])) {
            $return['data']['square'] = floatval($data['square']);
            if (empty($return['data']['square'])) {
                $return['errors'][] = l('error_empty_square', 'listings');
            } else {
                $listing['square'] = $return['data']['square'];
            }
        }

        if (isset($data['square_max'])) {
            $return['data']['square_max'] = floatval($data['square_max']);
            if (!empty($return['data']['square_max'])) {
                if ($listing['square'] > $return['data']['square_max']) {
                    $return['errors'][] = l('error_invalid_max_square', 'listings');
                }
            }
        }

        if (isset($data['id_city'])) {
            $return['data']['id_city'] = intval($data['id_city']);
            if (empty($return['data']['id_city'])) {
                $return['errors'][] = l('error_empty_location', 'listings');
            } else {
                $this->ci->load->model('Countries_model');
                $city = $this->ci->Countries_model->get_city($return['data']['id_city']);
                if ($city) {
                    $return['data']['id_country'] = $listing['id_country'] = $city['country_code'];
                    $return['data']['id_region'] = $listing['id_region'] = $city['id_region'];
                    $return['data']['id_city'] = $listing['id_city'] = $city['id'];
                } else {
                    $return['errors'][] = l('error_invalid_location', 'listings');
                }
            }
        } elseif (!$listing_id) {
            $return['errors'][] = l('error_empty_location', 'listings');
        }

        if (isset($data['id_region']) && isset($listing['id_region'])) {
            $return['data']['id_region'] = intval($data['id_region']);
            if ($return['data']['id_region'] != $listing['id_region']) {
                $return['errors'][] = l('error_invalid_location', 'listings');
            }
        }

        if (isset($data['id_country']) && isset($listing['id_region']) && $listing['id_country']) {
            $return['data']['id_country'] = trim(strip_tags($data['id_country']));
            if ($return['data']['id_country'] != $listing['id_country']) {
                $return['errors'][] = l('error_invalid_location', 'listings');
            }
        }

        if (isset($data['id_district']) && isset($listing['id_city']) && $listing['id_region'] && $listing['id_country']) {
            $return['data']['id_district'] = intval($data['id_district']);
            if (!empty($return['data']['id_district'])) {
                $this->ci->load->model('Countries_model');
                $district = $this->ci->Countries_model->get_district($return['data']['id_district']);
                if ($district['country_code'] != $listing['id_country'] ||
                    $district['id_region'] != $listing['id_region'] ||
                    $district['id_city'] != $listing['id_city']) {
                    $return['errors'][] = l('error_invalid_location', 'listings');
                }
            }
        }

        if (isset($data['address'])) {
            $return['data']['address'] = trim(strip_tags($data['address']));
            if (!empty($return['data']['address'])) {
                if ($is_moderation_installed) {
                    $this->ci->load->model('moderation/models/Moderation_badwords_model');
                    $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type, $return['data']['address']);
                    if ($bw_count) {
                        $message = l('error_badwords_field', 'listings');
                        $message = str_replace('[field]', l('field_address', 'listings'), $message);
                        $return['errors'][] = $message;
                    }
                }
            }
        }

        if (isset($data['zip'])) {
            $return['data']['zip'] = trim(strip_tags($data['zip']));
            if (!empty($return['data']['zip'])) {
                if ($is_moderation_installed) {
                    $this->ci->load->model('moderation/models/Moderation_badwords_model');
                    $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type, $return['data']['zip']);
                    if ($bw_count) {
                        $message = l('error_badwords_field', 'listings');
                        $message = str_replace('[field]', l('field_zip', 'listings'), $message);
                        $return['errors'][] = $message;
                    }
                }
            }
        }

        if (isset($data['lat'])) {
            $return['data']['lat'] = floatval($data['lat']);
        }

        if (isset($data['lon'])) {
            $return['data']['lon'] = floatval($data['lon']);
        }

        if (isset($data['listing_file'])) {
            $return['data']['listing_file'] = $listing['listing_file'] = $data['listing_file'];
        }

        if (isset($data['listing_file_date'])) {
            $value = strtotime($data['listing_file_date']);
            if ($value > 0) {
                $return['data']['listing_file_date'] = date('Y-m-d', $value);
            } else {
                $return['data']['listing_file_date'] = '0000-00-00 00:00:00';
            }
        }

        if (isset($data['listing_file_name'])) {
            $return['data']['listing_file_name'] = trim(strip_tags($data['listing_file_name']));
            if (empty($return['data']['listing_file_name'])) {
                $return['errors'][] = l('error_empty_file_name', 'listings');
            } else {
                if ($is_moderation_installed) {
                    $this->ci->load->model('moderation/models/Moderation_badwords_model');
                    $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type, $return['data']['listing_file_name']);
                    if ($bw_count) {
                        $message = l('error_badwords_field', 'listings');
                        $message = str_replace('[field]', l('field_listing_file_name', 'listings'), $message);
                        $return['errors'][] = $message;
                    }
                }
            }
        }

        if (isset($data['listing_file_comment'])) {
            $return['data']['listing_file_comment'] = trim(strip_tags($data['listing_file_comment']));
            if (!empty($return['data']['listing_file_comment'])) {
                if ($is_moderation_installed) {
                    $this->ci->load->model('moderation/models/Moderation_badwords_model');
                    $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type, $return['data']['listing_file_comment']);

                    if ($bw_count) {
                        $message = l('error_badwords_field', 'listings');
                        $message = str_replace('[field]', l('field_listing_file_comment', 'listings'), $message);
                        $return['errors'][] = $message;
                    }
                }
            }
        }

        if (isset($data['listing_video'])) {
            $return['data']['listing_video'] = strval($data['listing_video']);
        }

        if (isset($data['listing_video_image'])) {
            $return['data']['listing_video_image'] = strval($data['listing_video_image']);
        }

        if (isset($data['listing_video_data'])) {
            $return['data']['listing_video_data'] = serialize($data['listing_video_data']);
        }

        if (isset($data['initial_moderation'])) {
            $return['data']['initial_moderation'] = intval($data['initial_moderation']);
        }

        if (isset($data['initial_activity'])) {
            $return['data']['initial_activity'] = intval($data['initial_activity']);
        }

        if (isset($data['logo_image'])) {
            $return['data']['logo_image'] = $data['logo_image'];
        }

        if (isset($data['slider_image'])) {
            $return['data']['slider_image'] = $data['slider_image'];
        }

        if (isset($data['id_wish_lists'])) {
            if (!empty($listing['status']) || empty($data['id_wish_lists'])) {
                $return['data']['id_wish_lists'] = serialize($data['id_wish_lists']);
            } else {
                $return['errors'][] = l('error_inactive_to_wish_list', 'listings');
            }
        }

        if (isset($data['photo_count'])) {
            $return['data']['photo_count'] = intval($data['photo_count']);
        }

        if (isset($data['is_vtour'])) {
            $return['data']['is_vtour'] = $data['is_vtour'] ? 1 : 0;
        }

        if (isset($data['use_calendar'])) {
            $return['data']['use_calendar'] = $data['use_calendar'] ? 1 : 0;
        }

        if (isset($data['calendar_period_min'])) {
            $listing['calendar_period_min'] = $return['data']['calendar_period_min'] = intval($data['calendar_period_min']);
        }

        if (isset($data['calendar_period_max'])) {
            $listing['calendar_period_max'] = $return['data']['calendar_period_max'] = intval($data['calendar_period_max']);
        }

        if (!empty($listing['calendar_period_max']) && !empty($listing['calendar_period_min']) &&
            $listing['calendar_period_min'] > $listing['calendar_period_max']) {
            $return['errors'][] = l('error_invalid_calendar_limits', 'listings');
        }

        if (isset($data['guest_count'])) {
            $return['data']['guest_count'] = intval($data['guest_count']);
        }

        if (isset($data['featured_date_end'])) {
            $value = strtotime($data['featured_date_end']);
            if ($value > 0) {
                $return['data']['featured_date_end'] = date('Y-m-d', $value);
                if ($value > time()) {
                    $return['data']['is_featured'] = 1;
                } else {
                    $return['data']['is_featured'] = 0;
                }
            } else {
                $return['data']['featured_date_end'] = '0000-00-00 00:00:00';
                $return['data']['is_featured'] = 0;
            }
        }

        if (isset($data['lift_up_date_end'])) {
            $value = strtotime($data['lift_up_date_end']);
            if ($value > 0) {
                $return['data']['lift_up_date_end'] = date('Y-m-d', $value);
                $return['data']['is_lift_up'] = (int) ($value > time());
            } else {
                $return['data']['lift_up_date_end'] = '0000-00-00 00:00:00';
                $return['data']['is_lift_up'] = 0;
            }
        }

        if (isset($data['lift_up_country_date_end'])) {
            $value = strtotime($data['lift_up_country_date_end']);
            if ($value > 0) {
                $return['data']['lift_up_country_date_end'] = date('Y-m-d', $value);
                if ($value > time()) {
                    $return['data']['is_lift_up_country'] = 1;
                } else {
                    $return['data']['is_lift_up_country'] = 0;
                }
            } else {
                $return['data']['lift_up_country_date_end'] = '0000-00-00 00:00:00';
                $return['data']['is_lift_up_country'] = 0;
            }
        }

        if (isset($data['lift_up_region_date_end'])) {
            $value = strtotime($data['lift_up_region_date_end']);
            if ($value > 0) {
                $return['data']['lift_up_region_date_end'] = date('Y-m-d', $value);
                if ($value > time()) {
                    $return['data']['is_lift_up_region'] = 1;
                } else {
                    $return['data']['is_lift_up_region'] = 0;
                }
            } else {
                $return['data']['lift_up_region_date_end'] = '0000-00-00 00:00:00';
                $return['data']['is_lift_up_region'] = 0;
            }
        }

        if (isset($data['lift_up_city_date_end'])) {
            $value = strtotime($data['lift_up_city_date_end']);
            if ($value > 0) {
                $return['data']['lift_up_city_date_end'] = date('Y-m-d', $value);
                if ($value > time()) {
                    $return['data']['is_lift_up_city'] = 1;
                } else {
                    $return['data']['is_lift_up_city'] = 0;
                }
            } else {
                $return['data']['lift_up_city_date_end'] = '0000-00-00 00:00:00';
                $return['data']['is_lift_up_city'] = 0;
            }
        }

        if (isset($data['highlight_date_end'])) {
            $value = strtotime($data['highlight_date_end']);
            if ($value > 0) {
                $return['data']['highlight_date_end'] = date('Y-m-d', $value);
                if ($value > time()) {
                    $return['data']['is_highlight'] = 1;
                } else {
                    $return['data']['is_highlight'] = 0;
                }
            } else {
                $return['data']['highlight_date_end'] = '0000-00-00 00:00:00';
                $return['data']['is_highlight'] = 0;
            }
        }

        if (isset($data['slide_show_date_end'])) {
            $value = strtotime($data['slide_show_date_end']);
            if ($value > 0) {
                $return['data']['slide_show_date_end'] = date('Y-m-d', $value);
                if ($value > time()) {
                    $return['data']['is_slide_show'] = 1;
                } else {
                    $return['data']['is_slide_show'] = 0;
                }
            } else {
                $return['data']['slide_show_date_end'] = '0000-00-00 00:00:00';
                $return['data']['is_slide_show'] = 0;
            }
        }

        if (isset($data['views'])) {
            $return['data']['views'] = intval($data['views']);
        }

        if (isset($data['set_to_subscribe'])) {
            $return['data']['set_to_subscribe'] = $data['set_to_subscribe'] ? 1 : 0;
        }

        // field editor validate
        if (isset($listing['id_type']) && isset($listing['id_category'])) {
            $this->ci->load->model('listings/models/Listings_field_editor_model');
            $property_type_gid = $this->get_field_editor_type($listing);
            $this->ci->load->model('Field_editor_model');
            $this->ci->Field_editor_model->initialize($property_type_gid);
            $params = array('where' => array('editor_type_gid' => $property_type_gid));
            $validate_data = $this->ci->Field_editor_model->validate_fields_for_save($params, $data);
            $fields = $this->ci->Field_editor_model->get_fields_list($params);
            foreach ($fields as $field_gid => $field_data) {
                if ($field_data['field_type'] != 'text' && $field_data['field_type'] != 'textarea')
                    continue;
                if (isset($data[$field_data['field_name']]) && !empty($data[$field_data['field_name']])) {
                    $data[$field_data['field_name']] = trim(strip_tags($data[$field_data['field_name']]));

                    if ($is_moderation_installed) {
                        $this->ci->load->model('moderation/models/Moderation_badwords_model');
                        $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type, $data[$field_data['field_name']]);
                        if ($bw_count) {
                            $message = str_replace('[field]', $field_data['name'], l('error_badwords_field', 'listings'));
                            $validate_data['errors'][] = $message;
                        }
                    }
                }
            }

            if (!empty($validate_data['errors'])) {
                $return['errors'] = array_merge($return['errors'], $validate_data['errors']);
            } else {
                foreach ($validate_data['data'] as $field => $value) {
                    $return['data'][$field] = $value;
                }
            }
        }

        if (isset($data['headline'])) {
            $return['data']['headline'] = trim(strip_tags($data['headline']));
            if (!empty($return['data']['headline'])) {
                if ($is_moderation_installed) {
                    $this->ci->load->model('moderation/models/Moderation_badwords_model');
                    $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_type, $return['data']['headline']);
                    if ($bw_count) {
                        $message = str_replace('[field]', l('field_headline', 'listings'), l('error_badwords_field', 'listings'));
                        $return['errors'][] = $message;
                    }
                }
            }
        }

        if (isset($data['is_map_panorama'])) {
            $return['data']['is_map_panorama'] = $data['is_map_panorama'] ? 1 : 0;
        }

        if (isset($data['id_seo_settings'])) {
            $return['data']['id_seo_settings'] = intval($data['id_seo_settings']);
        }

        return $return;
    }

    /**
     * Validate listing data for saving to data source
     *
     * @param integer $listing_id listing identifier
     * @param array $data listing data
     * @return array
     */
    public function validateForActivation($listing)
    {
        $return = array('errors' => array());

        if ($this->ci->pg_module->is_module_active("users")) {
            if (empty($listing['id_user'])) {
                $return['errors'][] = l('error_empty_user', 'listings');
            }
        }

        if (empty($listing['id_type'])) {
            $return['errors'][] = l('error_empty_operation_type', 'listings');
        }

        if (empty($listing['id_category'])) {
            $return['errors'][] = l('error_empty_category', 'listings');
        }

        if (empty($listing['property_type'])) {
            $return['errors'][] = l('error_empty_property_type', 'listings');
        }

        $is_payments_installed = $this->ci->pg_module->is_module_active('currencies');
        if ($is_payments_installed) {
            if (empty($listing['gid_currency'])) {
                if (empty($listing['price_negotiated'])) {
                    $return['errors'][] = l('error_empty_currency', 'listings');
                }
            }
        }

        $need_min_price = in_array($this->getOperationTypeById($listing['id_type']), array(self::OPERATION_SALE, self::OPERATION_RENT));
        if ($need_min_price) {
            if (empty($listing['price'])) {
                if (empty($listing['price_negotiated'])) {
                    $return['errors'][] = l('error_empty_price', 'listings');
                }
            }
        } elseif (empty($listing['price']) && empty($listing['price_max'])) {
            if (empty($listing['price_negotiated'])) {
                $return['errors'][] = l('error_empty_price', 'listings');
            }
        }

        if (in_array($this->getOperationTypeById($listing['id_type']), array(self::OPERATION_RENT, self::OPERATION_LEASE))) {
            if (empty($listing['price_period'])) {
                $return['errors'][] = l('error_empty_price_period', 'listings');
            } else {
                $value = ld_option('price_period', 'listings', $listing['price_period']);
                if (empty($value)) {
                    $return['errors'][] = l('error_invalid_price_period', 'listings');
                }
            }

            if (empty($listing['price_type'])) {
                $return['errors'][] = l('error_empty_price_type', 'listings');
            } else {
                $value = ld_option('price_type', 'listings', $listing['price_type']);
                if (empty($value)) {
                    $return['errors'][] = l('error_invalid_price_type', 'listings');
                }
            }
        }

        if (empty($listing['square_unit'])) {
            $return['errors'][] = l('error_empty_square_unit', 'listings');
        }

        if (empty($listing['square'])) {
            $return['errors'][] = l('error_empty_square', 'listings');
        }

        if (empty($listing['id_city'])) {
            $return['errors'][] = l('error_empty_location', 'listings');
        }

        return $return;
    }

    /**
     * Validate listings settings for saving to data source
     *
     * @param array $data settings data
     * @return array
     */
    public function validate_listings_settings($data)
    {
        $return = array('errors' => array(), 'data' => array());
        return $return;
    }

    /**
     * Format data of listing object
     *
     * @param array $data listing data
     * @return array
     */
    public function format_listing($data)
    {
        return current($this->format_listings(array($data)));
    }

    /**
     * Format data of listings objects
     *
     * @param array $listings listings data
     * @return array
     */
    public function format_listings(array $listings)
    {
        if (empty($this->format_settings['use_format']) || empty($listings)) {
            return $listings;
        }

        $is_file_uploads = $this->ci->pg_module->is_module_active('file_uploads');
        $is_video_uploads = $this->ci->pg_module->is_module_active('video_uploads');

        $users_ids = array();
        $locations_ids = array();
        $listings_ids = array();
        $property_types_ids = array();

        $this->ci->load->model('Properties_model');
        $property_types = $this->ci->Properties_model->get_categories(
            'property_types', $this->ci->pg_language->current_lang_id
        );
        $property_items = $this->ci->Properties_model->get_all_properties(
            'property_types', $this->ci->pg_language->current_lang_id
        );
        $square_units = $this->ci->pg_language->ds->get_reference(
            'listings', $this->square_units_gid, $this->ci->pg_language->current_lang_id
        );

        $dayhours = $this->ci->pg_language->ds->get_reference('start', 'dayhour-names')['option'];

        $this->ci->load->helper('listings');
        $this->ci->load->model('Uploads_model');
        $this->ci->load->library('mobile_detect');
        $isMobile = $this->ci->mobile_detect->isMobile();

        if ($this->format_settings['get_saved_listings']) {
            $this->ci->load->model('listings/models/Saved_listings_model');
            $this->ci->Saved_listings_model->set_format_settings('get_listing', false);
            $saved_listings = $this->ci->Saved_listings_model->get_saved_listings_ids();
            $this->ci->Saved_listings_model->set_format_settings('get_listing', true);
        }

        foreach ($listings as $key => $listing) {
            if (!empty($listing['id'])) {
                $listings_ids[] = $listing['id'];
            } else {
                $listing['id'] = 0;
            }

            // prefix
            if (!empty($listing['id']) && !empty($listing['id_user'])) {
                $listing['prefix'] = $listing['id_user'] . '/' . $listing['id'] . '/';
            }
            if (!empty($listing['id_type'])) {
                $listing['operation_type'] = $this->getOperationTypeById($listing['id_type']);
                $listing['operation_type_str'] = l('operation_type_' . $listing['operation_type'], 'listings');
            } else {
                $listing['operation_type'] = '';
                $listing['operation_type_str'] = '';
            }

            if (isset($listing['operation_type']) && $listing['operation_type'] == self::OPERATION_SALE && $listing[self::OPERATION_SOLD]) {
                $listing['sold_output'] = $listing['status_output'] = l('text_sold', 'listings');
            } elseif (!empty($listing['status'])) {
                $listing['sold_output'] = '';
                $listing['status_output'] = l('active_listing', 'listings');
            } else {
                $listing['sold_output'] = '';
                $listing['status_output'] = l('inactive_listing', 'listings');
            }

            if (isset($listing['id_category']) && !empty($property_types[$listing['id_category']])) {
                $listing['category_str'] = $listing['category'] = $property_types[$listing['id_category']];
                $listing['property_type_str'] = $listing['property'] = $property_items[$listing['id_category']][$listing['property_type']];
            } else {
                $listing['category_str'] = '';
                $listing['property_type_str'] = '';
            }

            $listing['field_editor_type'] = $this->get_field_editor_type($listing);

            // get_user
            if ($this->format_settings['get_user'] && isset($listing['id_user'])) {
                $users_ids[] = $listing['id_user'];
            }

            // get location
            if ($this->format_settings['get_location'] && isset($listing['id_country'])) {
                $locations_ids[$listing['id']] = array(
                    'country' => $listing['id_country'],
                    'region' => $listing['id_region'],
                    'city' => $listing['id_city'],
                    'district' => $listing['id_district'],
                    'address' => $listing['address'],
                    'zip' => $listing['zip']
                );
            }

            // get description
            if ($this->format_settings['get_description']) {
                $property_type_gid = $listing['field_editor_type'];
                if (!isset($property_types_ids[$property_type_gid])) {
                    $property_types_ids[$property_type_gid] = array();
                }
                $property_types_ids[$property_type_gid][] = $listing['id'];
            }

            // get_file
            if ($is_file_uploads) {
                if (!empty($listing['listing_file'])) {
                    if ($this->format_settings['get_file']) {
                        $this->ci->load->model('File_uploads_model');
                        $this->ci->load->model('listings/models/Listings_uploads_model');
                        $listing['listing_file_content'] = $this->ci->File_uploads_model->format_upload(
                            $this->ci->Listings_uploads_model->file_config_id, $listing['prefix'], $listing['listing_file']);
                    } else {
                        $listing['listing_file_content'] = '';
                    }
                    $listing['is_file'] = true;
                } else {
                    $listing['is_file'] = false;
                }
            } else {
                $listing['is_file'] = false;
            }

            // get_video
            if ($is_video_uploads) {
                if (!empty($listing['listing_video_data'])) {
                    $listing['listing_video_data'] = $listing['listing_video_data'] ? unserialize($listing['listing_video_data']) : array();
                    $this->ci->load->model('Video_uploads_model');
                    $this->ci->load->model('listings/models/Listings_uploads_model');
                    $video_url = $this->ci->Video_uploads_model->get_media_url(
                        $this->ci->Listings_uploads_model->video_config_id, $listing['prefix']
                    );
                }
                if (!empty($listing['listing_video']) && $listing['listing_video_data']['data']['upload_type'] == 'embed') {
                    if ($this->format_settings['get_video']) {
                        $listing['listing_video_content'] = $this->ci->Video_uploads_model->format_upload(
                            $this->ci->Listings_uploads_model->video_config_id, $listing['prefix'], $listing['listing_video_data']['data'], $listing['listing_video_image'], $listing['listing_video_data']['data']['upload_type']
                        );
                    } else {
                        $listing['listing_video_content'] = '';
                    }
                    $listing['is_video'] = true;
                } elseif (!empty($listing['listing_video']) && $listing['listing_video_data']['status'] == 'end') {
                    if ($this->format_settings['get_video'] && (!empty($listing['listing_video_data']['data']['isHTML5']) || !$isMobile)) {
                        if (!isset($listing['listing_video_data']['data']['isHTML5'])) {
                            $listing['listing_video_data']['data']['isHTML5'] = 0;
                        }
                        $listing['listing_video_content'] = $this->ci->Video_uploads_model->format_upload(
                            $this->ci->Listings_uploads_model->video_config_id, $listing['prefix'], $listing['listing_video'], $listing['listing_video_image'], $listing['listing_video_data']['data']['upload_type'], $listing['listing_video_data']['data']['isHTML5']
                        );
                    } else {
                        $listing['listing_video_content'] = '';
                    }
                    $listing['is_video'] = true;
                } else {
                    $listing['is_video'] = false;
                }
            } else {
                $listing['is_video'] = false;
            }

            // saved listings
            if ($this->format_settings['get_saved_listings']) {
                if (in_array($listing['id'], (array) $saved_listings)) {
                    $listing['is_saved'] = true;
                }
            }

            if (isset($listing['price'])) {
                $listing['price'] *= 1;
                $listing['price_max'] *= 1;
                $listing['price_week'] *= 1;
                $listing['price_month'] *= 1;
                $listing['price_reduced'] *= 1;
            }

            if (isset($listing['price_period'])) {
                $listing['price_period_str'] = ld_option('price_period', 'listings', $listing['price_period']);
            } else {
                $listing['price_period_str'] = '';
            }

            if (isset($listing['price_type'])) {
                $listing['price_type_str'] = ld_option('price_type', 'listings', $listing['price_type']);
            } else {
                $listing['price_type_str'] = '';
            }

            if ($this->format_settings['get_price_str']) {
                $listing['price_str'] = str_replace('&nbsp;', ' - ',
                    strip_tags(listing_price_block(array('data' => $listing, 'template' => 'small'))));
            } else {
                $listing['price_str'] = $this->formatPrice($listing);
            }

            $listing['square_output'] = '';

            if (isset($listing['operation_type'])) {
                switch ($listing['operation_type']) {
                    case self::OPERATION_SALE:
                        $listing['square_output'] .= $listing['square'];
                        break;
                    case self::OPERATION_BUY:
                        if ($listing['square']) {
                            $listing['square_output'] .= l('text_price_from', 'listings') . ' ' . $listing['square'] . ' ';
                        }
                        if ($listing['square_max']) {
                            $listing['square_output'] .= l('text_price_to', 'listings') . ' ' . $listing['square_max'] . ' ';
                        }
                        break;
                    case self::OPERATION_RENT:
                        $listing['square_output'] .= $listing['square'];
                        break;
                    case self::OPERATION_LEASE:
                        if ($listing['square']) {
                            $listing['square_output'] .= l('text_price_from', 'listings') . ' ' . $listing['square'] . ' ';
                        }
                        if ($listing['square_max']) {
                            $listing['square_output'] .= l('text_price_to', 'listings') . ' ' . $listing['square_max'] . ' ';
                        }
                        break;
                }
            }

            if (isset($listing['square_unit'])) {
                if (isset($square_units['option'][$listing['square_unit']])) {
                    $listing['square_unit_str'] = $square_units['option'][$listing['square_unit']];
                } else {
                    $listing['square_unit_str'] = $listing['square_unit'];
                }
                $listing['square_output'] .= ' ' . $listing['square_unit_str'];
            }

            $listing['square_output'] = trim($listing['square_output']);

            if (!empty($listing['date_open_begin'])) {
                $listing['date_open_begin_text'] = $dayhours[$listing['date_open_begin']];
            } else {
                $listing['date_open_begin_text'] = '';
            }

            if (!empty($listing['date_open_end'])) {
                $listing['date_open_end_text'] = $dayhours[$listing['date_open_end']];
            } else {
                $listing['date_open_end_text'] = '';
            }

            if (!empty($listing['id_wish_lists'])) {
                if (is_array($listing['id_wish_lists'])) {
                    $listing['id_wish_lists'] = serialize($listing['id_wish_lists']);
                }
                $listing['id_wish_lists'] = (array) unserialize($listing['id_wish_lists']);
            } else {
                $listing['id_wish_lists'] = array();
            }

            if (!empty($listing['is_featured'])) {
                $listing["service_status"] = l('status_featured', 'listings');
            }

            if (!empty($listing['is_lift_up'])) {
                $listing["service_status"] = l('status_lift_up', 'listings');
            }

            if (!empty($listing['is_lift_up_country'])) {
                $listing["service_status"] = l('status_lift_up', 'listings');
            }

            if (!empty($listing['is_lift_up_region'])) {
                $listing["service_status"] = l('status_lift_up', 'listings');
            }

            if (!empty($listing['is_lift_up_city'])) {
                $listing["service_status"] = l('status_lift_up', 'listings');
            }
            // user should see that service is already purchased
            if (!empty($listing['is_slide_show'])) { //&& !empty($listing['photo_count'])) {
                $listing['is_slide_show'] = true;
            } else {
                $listing['is_slide_show'] = false;
            }

            // thumbs
            if (!empty($listing['logo_image'])) {
                $listing['media']['photo'] = $this->ci->Uploads_model->format_upload($this->upload_config_id, $listing['id'], $listing['logo_image']);
            } else {
                $listing['media']['photo'] = $this->ci->Uploads_model->format_default_upload($this->upload_config_id);
            }

            // slider
            if (!empty($listing['slider_image'])) {
                $listing['media']['slider'] = $this->ci->Uploads_model->format_upload($this->slider_config_id, $listing['id'], $listing['slider_image']);
            } else {
                $listing['media']['slider'] = $this->ci->Uploads_model->format_default_upload($this->slider_config_id);
            }

            $listing['photo_title'] = str_replace('[id_ref]', $listing['id'], l('link_listing_view', 'listings'));
            if (!empty($listing['operation_type_str'])) {
                $listing['photo_alt'] = str_replace(
                    array('[id_ref]', '[operation_type]', '[property_type]'), array($listing['id'], $listing['operation_type_str'], $listing['property_type_str']), l('link_listing_view', 'listings')
                );
                $listing['panorama_alt'] = str_replace(
                    array('[id_ref]', '[operation_type]', '[property_type]'), array($listing['id'], $listing['operation_type_str'], $listing['property_type_str']), l('text_listing_vtour', 'listings')
                );
                $listing['photo_alt_number'] = str_replace(
                    array('[id_ref]', '[operation_type]', '[property_type]'), array($listing['id'], $listing['operation_type_str'], $listing['property_type_str']), l('text_listing_photo_number', 'listings')
                );
            }

            $listing['photo_title_number'] = str_replace('[id_ref]', $listing['id'], l('link_listing_photo_number', 'listings'));

            if (isset($listing['review_sorter'])) {
                $listing['review_sorter'] = round($listing['review_sorter'], 1);
            }
            if (isset($listing['review_value'])) {
                $listing['review_value'] = round($listing['review_value'], 1);
            }
            // nav settings
            $listing['section'] = 'overview';
            $listing['pdf'] = 'no';

            $listings[$key] = $listing;
        }

        // get user
        $this->ci->load->model('Start_model');
        $is_users_installed = $this->ci->pg_module->is_module_active("users");
        if ($is_users_installed) {
            if ($this->format_settings['get_user'] && !empty($users_ids)) {
                $this->ci->load->model('Users_model');
                $users_data = $this->ci->Users_model->get_users_list_by_key(null, null, null, array(), array_unique($users_ids));
                foreach ($listings as $key => $listing) {
                    if ($listing['id_user'] != 0) {
                        $listing['user'] = (isset($users_data[$listing['id_user']])) ?
                            $users_data[$listing['id_user']] : $this->ci->Users_model->format_default_user($listing['id_user']);
                    } else {
                        $listing['user'] = $this->ci->Start_model->getUserDefaultOnSite();
                    }
                    $listings[$key] = $listing;
                }
            }
        } else {
            $users_data[0] = $this->ci->Start_model->getUserDefaultOnSite();
            foreach ($listings as $key => $listing) {
                $listing['user'] = $users_data[0];
                $listings[$key] = $listing;
            }
        }

        // get location
        if (!empty($locations_ids)) {
            $this->ci->load->helper('countries');
            $listing_locations = address_output_format($locations_ids);
            $listing_locations_data = get_location_data($locations_ids, 'district');
            foreach ($listings as $key => $listing) {
                $listing['country'] = (isset($listing_locations_data['country'][$listing['id_country']])) ? $listing_locations_data['country'][$listing['id_country']]['name'] : '';
                $listing['region'] = (isset($listing_locations_data['region'][$listing['id_region']])) ? $listing_locations_data['region'][$listing['id_region']]['name'] : '';
                $listing['region_code'] = (isset($listing_locations_data['region'][$listing['id_region']])) ? $listing_locations_data['region'][$listing['id_region']]['code'] : '';
                $listing['city'] = (isset($listing_locations_data['city'][$listing['id_city']])) ? $listing_locations_data['city'][$listing['id_city']]['name'] : '';
                $listing['district'] = (isset($listing_locations_data['district'][$listing['id_district']])) ? $listing_locations_data['district'][$listing['id_district']]['name'] : '';
                $listing['output_name'] = $listing['location'] = $listing_locations[$listing['id']];

                $listing['photo_alt'] = str_replace('[location]', $listing['location'], $listing['photo_alt']);
                $listing['photo_alt_number'] = str_replace('[location]', $listing['location'], $listing['photo_alt_number']);
                $listing['panorama_alt'] = str_replace('[location]', $listing['location'], $listing['panorama_alt']);

                $listings[$key] = $listing;
            }
        } elseif (isset($listing['photo_alt'])) {
            foreach ($listings as $key => $listing) {
                $listing['photo_alt'] = str_replace('[location]', '', $listing['photo_alt']);
                $listing['photo_alt_number'] = str_replace('[location]', '', $listing['photo_alt_number']);
                $listing['panorama_alt'] = str_replace('[location]', '', $listing['panorama_alt']);
            }
        }

        // get description
        if ($this->format_settings['get_description'] && !empty($property_types_ids)) {
            $this->ci->load->model('listings/models/Listings_field_editor_model');
            $field_editor_data = array();
            foreach ($property_types_ids as $property_type_gid => $property_type_data) {
                $fields_for_select = $this->ci->Listings_field_editor_model->get_fields_for_select($property_type_gid);

                if (empty($fields_for_select)) {
                    continue;
                }
                $field_editor_data = $this->ci->Listings_field_editor_model->get_field_editor_data($property_type_gid, $fields_for_select, $property_type_data);

                foreach ($listings as $key => $listing) {
                    if (!isset($field_editor_data[$listing['id']])) {
                        continue;
                    }
                    foreach ($field_editor_data[$listing['id']] as $name => $value) {
                        if (isset($listing[$name])) {
                            continue;
                        }
                        $listing[$name] = $value;
                        $listings[$key] = $listing;
                    }
                }
            }

            $this->ci->load->model('Field_editor_model');
            $this->ci->load->model('listings/models/Listings_field_editor_model');
            $params = array('where_in' => array('editor_type_gid' => array_keys($property_types_ids)));
            $fields_data = $this->ci->Field_editor_model->format_list_fields_for_view($params, $listings);
            foreach ($listings as $key => $listing) {
                if (!isset($fields_data[$key])) {
                    continue;
                }
                $listings[$key]['field_editor'] = array();
                $fe_sections = $this->ci->Listings_field_editor_model->field_editor_sections[$listing['field_editor_type']];
                foreach ($fe_sections as $field_editor_section) {
                    foreach ($fields_data[$key] as $field_gid => $field_data) {
                        if ($field_data['section_gid'] != $field_editor_section) {
                            continue;
                        }
                        $listings[$key]['field_editor'][$field_gid] = $field_data;
                        switch ($field_data['field_type']) {
                            case 'select':
                                $listings[$key]['field_editor_' . $field_gid . '_output'] = $field_data['value'];
                                break;
                            case 'checkbox':
                                $listings[$key]['field_editor_' . $field_gid] = $field_data['value'];
                                $listings[$key]['field_editor_' . $field_gid . '_text'] = l($field_data['value'] ? 'yes_str' : 'no_str', 'start');
                                break;
                            case 'multiselect':
                                foreach ($field_data['value'] as $value => $name) {
                                    $listings[$key]['field_editor_' . $field_gid . '_option_' . $value] = 1;
                                    $listings[$key]['field_editor_' . $field_gid . '_option_' . $value . '_text'] = l('yes_str', 'start');
                                }
                                break;
                        }
                    }
                }
            }
        }

        // get photos
        if (($this->format_settings['get_photos'] || $this->format_settings['get_photos_all'])) {
            $this->ci->load->model('listings/models/Listings_uploads_model');
            $gallery_type_config = $this->ci->Listings_uploads_model->get_gallery_type();
            $default_gallery_upload = $this->ci->Uploads_model->format_default_upload($gallery_type_config['gid_upload_config']);

            $photos_by_object = array();

            $param = array();
            $param['where']['type_id'] = $gallery_type_config['id'];
            $param['where_in']['object_id'] = $listings_ids;
            if (!$this->format_settings['get_photos_all']) {
                $param['where']['status'] = 1;
            }
            $photos = $this->ci->Upload_gallery_model->get_files_by_param($param);
            foreach ($photos as $photo) {
                if (!isset($photos_by_object[$photo['object_id']])) {
                    $photos_by_object[$photo['object_id']] = array();
                }

                $photo['photo_alt'] = str_replace('[id_ref]', $photo['object_id'], l('text_listing_photo', 'listings'));
                $photo['photo_alt_number'] = str_replace('[id_ref]', $photo['object_id'], l('text_listing_photo_number', 'listings'));
                $photos_by_object[$photo['object_id']][] = $photo;
            }
            foreach ($listings as $key => $listing) {
                $listing['photos'] = isset($photos_by_object[$listing['id']]) ? $photos_by_object[$listing['id']] : array();
                $listing['photo_count'] = count($listing['photos']);
                $listing['photo_default'] = array('media' => $default_gallery_upload);
                $listings[$key] = $listing;
            }
        }

        // get virtual tours
        if ($this->format_settings['get_virtual_tours'] || $this->format_settings['get_virtual_tours_all']) {
            $this->ci->load->model('listings/models/Listings_uploads_model');
            $vtour_type_config = $this->ci->Listings_uploads_model->get_vtour_type();
            $vtours_by_object = array();
            $param = array();
            $param['where']['type_id'] = $vtour_type_config['id'];
            $param['where_in']['object_id'] = $listings_ids;
            if (!$this->format_settings['get_virtual_tours_all']) {
                $param['where']['status'] = 1;
            }
            $vtours = $this->ci->Upload_gallery_model->get_files_by_param($param);
            foreach ($vtours as $vtour) {
                if (!isset($vtours_by_object[$vtour['object_id']])) {
                    $vtours_by_object[$vtour['object_id']] = array();
                }
                $vtours_by_object[$vtour['object_id']][] = $vtour;
            }
            foreach ($listings as $key => $listing) {
                $listing['virtual_tour'] = isset($vtours_by_object[$listing['id']]) ? $vtours_by_object[$listing['id']] : array();
                $listing['virtual_tour_count'] = count($listing['virtual_tour']);
                $listings[$key] = $listing;
            }
        }

        // get moderation
        if ($this->format_settings['get_moderation']) {
            $is_moderation_installed = $this->ci->pg_module->is_module_active('moderation');
            if ($is_moderation_installed) {
                $this->ci->load->model('listings/models/Listings_moderation_model');
                $alert_data = $this->ci->Listings_moderation_model->get_listings_status($listings_ids);
                foreach ($listings as $key => $listing) {
                    $listing['moderation_status'] = $alert_data[$listing['id']];
                    $listings[$key] = $listing;
                }
            }
        }

        // qr code
        if ($this->format_settings['get_qr_code']) {
            foreach ($listings as $key => $listing) {
                $listing['qr_code'] = $this->get_qr_code($listing);
                $listings[$key] = $listing;
            }
        }

        // booking
        if ($this->ci->pg_module->is_module_active('listings_booking')) {
            $this->ci->load->model('listings_booking/models/Listings_booking_model');
            $cur_date = date("Y-m-d");
            $filter_param = array('listings' => $listings_ids, 'date_end >=' => $cur_date);
            $get_booking = false;
            if ($this->format_settings['get_booking']) {
                $get_booking = true;
            }
            if ($this->format_settings['get_booking_special_prices']) {
                $get_booking = true;
                $special_prices = array('status' => 'open');
                $filter_param = array_merge($filter_param, $special_prices);
            }
            if ($this->format_settings['get_booking_periods']) {
                $get_booking = true;
                $special_prices = array('status' => 'approve');
                $filter_param = array_merge($filter_param, $special_prices);
            }

            if ($get_booking) {
                $periods_data = $this->ci->Listings_booking_model->get_periods_list_by_key($filter_param, null, null, array('date_start' => 'DESC'));
                foreach ($listings as $key => $listing) {
                    $listing['booking']['periods'] = isset($periods_data[$listing['id']]) ? $periods_data[$listing['id']] : array();
                    $listings[$key] = $listing;
                }
            }
        }

        // all views
        if ($this->format_settings['get_views']) {
            $this->ci->load->model('listings/models/Listings_visitor_model');
            $listings_views = $this->ci->Listings_visitor_model->get_views_count($listings_ids);
            foreach ($listings as $key => $listing) {
                $listing['views'] += isset($listings_views[$listing['id']]) ? $listings_views[$listing['id']] : 0;
                $listings[$key] = $listing;
            }
        }

        foreach ($listings as &$listing) {
            $this->groupFields($listing);
        }
        return $listings;
    }

    private function groupFields(&$listing)
    {
        $field_groups = [
            'numerics' => [
                'field_editor' => [
                    'bd_rooms_1',
                    'bth_rooms_1',
                    'bd_rooms_4',
                    'bth_rooms_4',
                ],
            ],
        ];
        $sections = [];
        foreach ($field_groups as $section => $sub_groups) {
            foreach ($sub_groups['field_editor'] as $fe_field) {
                if (!isset($listing['field_editor'][$fe_field])) {
                    continue;
                }
                $name = preg_replace('/_[\d]+$/', '', $fe_field);
                $sections[$section][$fe_field] = [
                    'fe_name' => $name,
                    'name' => $name,
                    'value' => &$listing['field_editor'][$fe_field]['value_str'],
                    'value_str' => &$listing['field_editor'][$fe_field]['value_str'],
                ];
            }
        }
        $sections['numerics']['square'] = [
            'name' => 'square',
            'value' => &$listing['square'],
            'value_str' => &$listing['square_output'],
        ];
        $listing['field_section'] = $sections;
        return $listing;
    }

    /**
     * Format data of listing object by default
     *
     * @param integer $listing_id listing identifier
     * @return array
     */
    public function format_default_listing($listing_id)
    {
        $this->ci->load->model('Uploads_model');
        $data['postfix'] = $listing_id;
        $data['output_name'] = 'Listing is deleted';
        $data['media']['photo'] = $this->ci->Uploads_model->format_default_upload($this->upload_config_id);
        return $data;
    }

    /**
     * Return display sections of listing content
     *
     * @param array $data listing data
     * @return array
     */
    public function get_display_sections($data)
    {
        $display_sections = $this->display_sections;

        $is_install_listings_booking = $this->ci->pg_module->is_module_active('listings_booking');
        if ($data['operation_type'] != self::OPERATION_RENT || !$data['use_calendar'] || empty($is_install_listings_booking)) {
            $display_sections['bottom']['calendar'] = 0;
        }
        if (((float) $data['lat'] != 0 || (float) $data['lon'] != 0) && $this->ci->pg_module->is_module_active('geomap')) {
            if (!$data['is_map_panorama']) {
                $display_sections['top']['panorama'] = 0;
            } else {
                $this->ci->load->helpers('geomap');
                if (!geomap_panorama_available('listing_view'))
                    $display_sections['top']['panorama'] = 0;
            }
        }else {
            $display_sections['top']['map'] = 0;
            $display_sections['top']['panorama'] = 0;
        }

        if (empty($data['is_file'])) {
            $display_sections['bottom']['file'] = 0;
        }

        if (empty($data['is_video'])) {
            $display_sections['top']['video'] = 0;
        }

        if (empty($data['review_count'])) {
            $display_sections['bottom']['reviews'] = 0;
        }

        if (empty($data['is_vtour'])) {
            $display_sections['top']['virtual_tour'] = 0;
        }

        return $display_sections;
    }

    /**
     * Render pdf version of listing content
     *
     * @param string $content listing content
     * @param array $data listing data
     * @return void
     */
    public function get_pdf_file($content, $data)
    {
        $content = preg_replace('/<script(.*?)<\/script>/is', '', $content);

        $content = str_replace('<meta charset="utf-8">', '<meta http-equiv="content-type" content="text/html; charset=utf-8" />', $content);

        $current_lang_id = $this->ci->pg_language->current_lang_id;
        $current_lang = $this->ci->pg_language->get_lang_by_id($current_lang_id);

        if ($current_lang['code'] == 'ma' || $current_lang['code'] == 'zh') {
            $content = str_replace('</head>', '<style>@font-face {font-family:\'Noto Sans\';font-style: normal;font-weight: 400;src: url(' . site_url() . 'application/libraries/dompdf/lib/fonts/fireflysung.ttf) format(\'truetype\');}*{font-family: Noto Sans !important;}</style></head>', $content);
        }

        $content = str_replace('&nbsp;', ' ', $content);

        $this->ci->load->library('dompdf');

        $this->ci->dompdf->set_paper('a4', 'portrait');
        $this->ci->dompdf->load_html($content);
        $this->ci->dompdf->render();
        $file_name = $data['gid'] . '.pdf';
        $this->ci->dompdf->stream($file_name);
    }

    /**
     * Return module settings for seo urls
     *
     * @param string $method controller method
     * @param integer $lang_id language identifier
     * @return array
     */
    public function get_seo_settings($method = '', $lang_id = '')
    {
        if (!empty($method)) {
            return $this->_get_seo_settings($method, $lang_id);
        } else {
            $actions = array('index', 'category', 'country', 'region', 'location', 'user', 'search', 'open_house', 'privates', 'agents', 'discount', 'view', 'wish_lists', 'wish_list');
            $return = array();
            foreach ($actions as $action) {
                $return[$action] = $this->_get_seo_settings($action, $lang_id);
            }
            return $return;
        }
    }

    /**
     * Return module settings for seo urls (internal)
     *
     * @param string $method controller method
     * @param integer $lang_id language identifier
     * @return array
     */
    protected function _get_seo_settings($method, $lang_id = '')
    {
        switch ($method) {
            case 'index':
                return array(
                    'templates' => array('operation_type', 'operation_type_str', 'category', 'property', 'country', 'region', 'region_code', 'city'),
                    'url_vars' => array(
                        'operation_type' => array('operation_type' => 'literal', 'operation_type_str' => 'literal'),
                    ),
                    'url_postfix' => array(
                        'order' => array('order' => 'literal'),
                        'order_direction' => array('order_direction' => 'literal'),
                        'page' => array('page' => 'literal'),
                    ),
                );
                break;
            case 'category':
                return array(
                    'templates' => array('operation_type', 'operation_type_str', 'category', 'property'),
                    'url_vars' => array(
                        'id_category' => array('category' => 'literal'),
                        'property_type' => array('property' => 'literal'),
                        'operation_type' => array('operation_type' => 'literal', 'operation_type_str' => 'literal'),
                    ),
                    'url_postfix' => array(
                        'order' => array('order' => 'literal'),
                        'order_direction' => array('order_direction' => 'literal'),
                        'page' => array('page' => 'literal'),
                    ),
                );
                break;
            case 'country':
                return array(
                    'templates' => array('operation_type', 'operation_type_str', 'country'),
                    'url_vars' => array(
                        'id_country' => array('country' => 'literal'),
                        'operation_type' => array('operation_type' => 'literal', 'operation_type_str' => 'literal'),
                    ),
                    'url_postfix' => array(
                        'order' => array('order' => 'literal'),
                        'order_direction' => array('order_direction' => 'literal'),
                        'page' => array('page' => 'literal'),
                    ),
                );
                break;
            case 'region':
                return array(
                    'templates' => array('operation_type', 'operation_type_str', 'country', 'region', 'region_code'),
                    'url_vars' => array(
                        'id_country' => array('country' => 'literal'),
                        'id_region' => array('region' => 'literal'),
                        'operation_type' => array('operation_type' => 'literal', 'operation_type_str' => 'literal'),
                    ),
                    'url_postfix' => array(
                        'order' => array('order' => 'literal'),
                        'order_direction' => array('order_direction' => 'literal'),
                        'page' => array('page' => 'literal'),
                    ),
                );
                break;
            case 'location':
                return array(
                    'templates' => array('operation_type', 'operation_type_str', 'country', 'region', 'region_code', 'city'),
                    'url_vars' => array(
                        'id_country' => array('country' => 'literal'),
                        'id_region' => array('region' => 'literal'),
                        'id_city' => array('city' => 'literal'),
                        'operation_type' => array('operation_type' => 'literal', 'operation_type_str' => 'literal'),
                    ),
                    'url_postfix' => array(
                        'order' => array('order' => 'literal'),
                        'order_direction' => array('order_direction' => 'literal'),
                        'page' => array('page' => 'literal'),
                    ),
                );
                break;
            case 'user':
                return array(
                    'templates' => array('name', 'user_type', 'user_type_str', 'operation_type', 'operation_type_str'),
                    'url_vars' => array(
                        'id_user' => array('id_user' => 'numeric'),
                        'operation_type' => array('operation_type' => 'literal', 'operation_type_str' => 'literal'),
                    ),
                    'optional' => array(
                        'user' => array('user' => 'literal'),
                    ),
                    'url_postfix' => array(
                        'order' => array('order' => 'literal'),
                        'order_direction' => array('order_direction' => 'literal'),
                        'page' => array('page' => 'literal'),
                    ),
                );
                break;
            case 'search':
                return array(
                    'noindex' => 1,
                    'templates' => array('operation_type', 'operation_type_str', 'category', 'property', 'country', 'region', 'region_code', 'city'),
                    'url_vars' => array(
                        'keyword' => array('keyword' => 'literal'),
                        'operation_type' => array('operation_type' => 'literal', 'operation_type_str' => 'literal'),
                    ),
                    'url_postfix' => array(
                        'order' => array('order' => 'literal'),
                        'order_direction' => array('order_direction' => 'literal'),
                        'page' => array('page' => 'literal'),
                    ),
                );
                break;
            case 'open_house':
                return array(
                    'templates' => array('operation_type', 'operation_type_str'),
                    'header' => l('seo_tags_open_house_header', 'listings', $lang_id, 'seo'),
                    'url_vars' => array(
                        'operation_type' => array('operation_type' => 'literal', 'operation_type_str' => 'literal'),
                    ),
                    'url_postfix' => array(
                        'order' => array('order' => 'literal'),
                        'order_direction' => array('order_direction' => 'literal'),
                        'page' => array('page' => 'literal'),
                    ),
                );
                break;
            case 'privates':
                return array(
                    'templates' => array('operation_type', 'operation_type_str'),
                    'url_vars' => array(
                        'operation_type' => array('operation_type' => 'literal', 'operation_type_str' => 'literal'),
                    ),
                    'url_postfix' => array(
                        'order' => array('order' => 'literal'),
                        'order_direction' => array('order_direction' => 'literal'),
                        'page' => array('page' => 'literal'),
                    ),
                );
                break;
            case 'agents':
                return array(
                    'templates' => array('operation_type', 'operation_type_str'),
                    'url_vars' => array(
                        'operation_type' => array('operation_type' => 'literal', 'operation_type_str' => 'literal'),
                    ),
                    'url_postfix' => array(
                        'order' => array('order' => 'literal'),
                        'order_direction' => array('order_direction' => 'literal'),
                        'page' => array('page' => 'literal'),
                    ),
                );
                break;
            case 'discount':
                return array(
                    'templates' => array('operation_type', 'operation_type_str'),
                    'url_vars' => array(
                        'operation_type' => array('operation_type' => 'literal', 'operation_type_str' => 'literal'),
                    ),
                    'url_postfix' => array(
                        'order' => array('order' => 'literal'),
                        'order_direction' => array('order_direction' => 'literal'),
                        'page' => array('page' => 'literal'),
                    ),
                );
                break;
            case 'view':
                return array(
                    'templates' => array('id', 'operation_type', 'operation_type_str', 'property', 'location', 'country', 'region', 'city', 'address', 'section', 'pdf', 'headline', 'price', 'status', 'sold'),
                    'url_vars' => array(
                        'id' => array('id' => 'numeric', 'gid' => 'literal'),
                        'section' => array('section' => 'literal'),
                    ),
                    'optional' => array(
                        'operation_type' => array('operation_type' => 'literal', 'operation_type_str' => 'literal'),
                        'property' => array('property' => 'literal'),
                        'country' => array('country' => 'literal'),
                        'region' => array('region' => 'literal'),
                        'city' => array('city' => 'literal'),
                        'address' => array('address' => 'literal'),
                    ),
                );
                break;
            case 'wish_lists':
                return array(
                    'templates' => array(),
                    'url_vars' => array(),
                    'url_postfix' => array(
                        'order' => array('order' => 'literal'),
                        'order_direction' => array('order_direction' => 'literal'),
                        'page' => array('page' => 'literal'),
                    ),
                );
                break;
            case 'wish_list':
                return array(
                    'templates' => array('id', 'name'),
                    'url_vars' => array(
                        'id' => array('id' => 'numeric'),
                    ),
                    'url_postfix' => array(
                        'page' => array('page' => 'literal'),
                    ),
                );
                break;
        }
    }

    /**
     * Transform seo values from query string to method parameters
     *
     * @param string $var_name_from variable name from url
     * @param string $var_name_to variable name to method
     * @param mixed $value parameter value
     * @return mixed
     */
    public function request_seo_rewrite($var_name_from, $var_name_to, $value)
    {
        $listing_data = array();

        $default_lang = $this->ci->pg_language->current_lang_id;

        if ($var_name_from == 'pdf' && $var_name_to == 'pdf') {
            return ($value == 'no') ? false : true;
        }

        if ($var_name_from == 'operation_type_str' && $var_name_to == 'operation_type') {
            $operation_types = $this->get_operation_types(true);

            foreach ($operation_types as $operation_type) {
                $option_name = l('operation_search_' . $operation_type, 'listings');
                if ($option_name == $value || str_replace('/', ' ', $option_name) == $value || $option_name == str_replace('_', ' ', $value)) {
                    return $operation_type;
                }
            }

            foreach ($this->ci->pg_language->languages as $lid => $lang_data) {
                if ($lid == $default_lang) {
                    continue;
                }
                $option_name = l('operation_search_' . $operation_type, 'listings', $lid);
                if ($option_name == $value || str_replace('/', ' ', $option_name) == $value || $option_name == str_replace('_', ' ', $value)) {
                    return $operation_type;
                }
            }

            return $this->get_operation_type_gid_default(true);
        }

        if ($var_name_from == 'category' && $var_name_to == 'id_category') {
            $this->ci->load->model('Properties_model');
            $property_types = $this->ci->Properties_model->get_categories('property_types', $default_lang);
            foreach ($property_types as $option_value => $option_name) {
                if ($option_name == $value || str_replace('/', ' ', $option_name) == $value || $option_name == str_replace('_', ' ', $value)) {
                    return $option_value;
                }
            }
            foreach ($this->ci->pg_language->languages as $lid => $lang_data) {
                if ($lid == $default_lang)
                    continue;
                $property_types = $this->ci->Properties_model->get_categories('property_types', $lid);
                foreach ($property_types as $option_value => $option_name) {
                    if ($option_name == $value || str_replace('/', ' ', $option_name) == $value || $option_name == str_replace('_', ' ', $value)) {
                        return $option_value;
                    }
                }
            }
            return 0;
        }

        if ($var_name_from == 'property' && $var_name_to == 'property_type') {
            $category_ids = $this->get_category_ids();
            foreach ($category_ids as $category_id) {
                $this->ci->load->model('Properties_model');
                $property_items = $this->ci->Properties_model->get_properties($category_id, $default_lang);
                if (!empty($property_items)) {
                    foreach ($property_items as $option_value => $option_name) {
                        if ($option_name == $value || str_replace('/', ' ', $option_name) == $value || $option_name == str_replace('_', ' ', $value)) {
                            return $option_value;
                        }
                        $option_name .= '-' . $category_id;
                        if ($option_name == $value || str_replace('/', ' ', $option_name) == $value || $option_name == str_replace('_', ' ', $value)) {
                            return $option_value;
                        }
                    }

                    foreach ($this->ci->pg_language->languages as $lid => $lang_data) {
                        if ($lid == $default_lang)
                            continue;
                        $property_items = $this->ci->Properties_model->get_properties($category_id, $lid);
                        if (empty($property_items))
                            continue;
                        foreach ($property_items as $option_value => $option_name) {
                            if ($option_name == $value || str_replace('/', ' ', $option_name) == $value || $option_name == str_replace('_', ' ', $value)) {
                                return $option_value;
                            }
                            $option_name .= '-' . $category_id;
                            if ($option_name == $value || str_replace('/', ' ', $option_name) == $value || $option_name == str_replace('_', ' ', $value)) {
                                return $option_value;
                            }
                        }
                    }
                }
            }
            return 0;
        }

        if ($var_name_from == 'country' && $var_name_to == 'id_country') {
            $this->ci->load->model('Countries_model');
            $country = $this->ci->Countries_model->get_country_by_name($value);
            if (!empty($country)) {
                return $country['code'];
            }
            return '';
        }

        if ($var_name_from == 'region' && $var_name_to == 'id_region') {
            $this->ci->load->model('Countries_model');
            $region = $this->ci->Countries_model->get_region_by_name($value);
            if (!empty($region)) {
                return $region['id'];
            }
            return 0;
        }

        if ($var_name_from == 'city' && $var_name_to == 'id_city') {
            $this->ci->load->model('Countries_model');
            $city = $this->ci->Countries_model->get_city_by_name($value);
            if (!empty($city)) {
                return $city['id'];
            }
            return 0;
        }

        if ($var_name_from == 'user' && $var_name_to == 'id_user') {
            $is_users_installed = $this->ci->pg_module->is_module_active("users");
            if ($is_users_installed) {
                $this->ci->load->model('Users_model');
                $users = $this->ci->Users_model->get_users_list_by_filters(array('name' => $value), 1, 1);
            } else {
                $this->ci->load->model('Start_model');
                $users = $this->ci->Start_model->getUserDefaultOnSite();
            }
            if (!empty($users)) {
                return $users[0]['id'];
            }
            return 0;
        }

        return $value;
    }

    /**
     * Return urls for xml site map as array
     *
     * @return array
     */
    public function get_sitemap_xml_urls()
    {
        $this->ci->load->helper('seo');
        $this->ci->load->library('Translit');

        $lang_canonical = true;
        if ($this->ci->pg_module->is_module_active('seo')) {
            $lang_canonical = $this->ci->pg_module->get_module_config('seo', 'lang_canonical');
        }

        if ($lang_canonical) {
            $default_lang_id = $this->ci->pg_language->get_default_lang_id();
            $default_lang_code = $this->ci->pg_language->get_lang_code_by_id($default_lang_id);
            $langs[$default_lang_id] = $default_lang_code;
        } else {
            foreach ($this->ci->pg_language->languages as $lang_id => $lang_data) {
                $langs[$lang_id] = $lang_data['code'];
            }
        }

        $operation_types = $this->get_operation_types(true);

        $return = array();

        $user_settings = $this->pg_seo->get_settings('user', 'listings', 'index');
        if (!$user_settings['noindex']) {
            foreach ($langs as $lang_id => $lang_code) {
                foreach ($operation_types as $operation_type) {
                    $return[] = array(
                        'url' => rewrite_link('listings', 'index', array('operation_type' => $operation_type), false, $lang_code, $lang_canonical),
                        'priority' => 0.5,
                    );
                }
            }
        }

        $user_settings = $this->pg_seo->get_settings('user', 'listings', 'category');
        if (!$user_settings['noindex']) {
            $this->ci->load->model('Properties_model');
            foreach ($langs as $lang_id => $lang_code) {
                $property_types = $this->ci->Properties_model->get_categories('property_types', $lang_id);
                $property_items = $this->ci->Properties_model->get_all_properties('property_types', $lang_id);
                foreach ($operation_types as $operation_type) {
                    foreach ($property_types as $property_type_gid => $property_type_name) {
                        $return[] = array(
                            'url' => rewrite_link('listings', 'category', array('operation_type' => $operation_type, 'id_category' => $property_type_gid, 'category' => $property_type_name), false, $lang_code, $lang_canonical),
                            'priority' => 0.5,
                        );
                        foreach ($property_items[$property_type_gid] as $subtype_gid => $subtype_name) {
                            $return[] = array(
                                'url' => rewrite_link('listings', 'category', array('operation_type' => $operation_type, 'id_category' => $property_type_gid, 'category' => $property_type_name, 'property_type' => $subtype_gid, 'property' => $subtype_name), false, $lang_code, $lang_canonical),
                                'priority' => 0.5,
                            );
                        }
                    }
                }
            }
        }

        $user_settings = $this->pg_seo->get_settings('user', 'listings', 'location');
        if (!$user_settings['noindex']) {
            $this->ci->load->model('Countries_model');
            foreach ($langs as $lang_id => $lang_code) {
                $locations = $this->ci->Countries_model->get_countries(null, array(), array(), $lang_id);
                if (count($locations) == 1) {
                    $locations = $this->ci->Countries_model->get_regions($country['code'], null, array(), array(), $lang_id);
                }
                foreach ($operation_types as $operation_type) {
                    foreach ($locations as $location) {
                        if (isset($location['country_code'])) {
                            $return[] = array(
                                'url' => rewrite_link('listings', 'location', array('operation_type' => $operation_type, 'id_country' => $country_id, 'country' => $country_name, 'id_region' => $location['id'], 'region' => $location['name']), false, $lang_code, $lang_canonical),
                                'priority' => 0.5,
                            );
                        } else {
                            $country_id = $location['code'];
                            $country_name = $location['name'];
                            $return[] = array(
                                'url' => rewrite_link('listings', 'location', array('operation_type' => $operation_type, 'id_country' => $country_id, 'country' => $country_name), false, $lang_code, $lang_canonical),
                                'priority' => 0.5,
                            );
                        }
                    }
                }
            }
        }

        $user_settings = $this->pg_seo->get_settings('user', 'listings', 'open_house');
        if (!$user_settings['noindex']) {
            foreach ($langs as $lang_id => $lang_code) {
                foreach ($operation_types as $operation_type) {
                    $return[] = array(
                        'url' => rewrite_link('listings', 'open_house', array('operation_type' => $operation_type), false, $lang_code, $lang_canonical),
                        'priority' => 0.5,
                    );
                }
            }
        }

        $user_settings = $this->pg_seo->get_settings('user', 'listings', 'privates');
        if (!$user_settings['noindex']) {
            foreach ($langs as $lang_id => $lang_code) {
                foreach ($operation_types as $operation_type) {
                    $return[] = array(
                        'url' => rewrite_link('listings', 'privates', array('operation_type' => $operation_type), false, $lang_code, $lang_canonical),
                        'priority' => 0.5,
                    );
                }
            }
        }

        $user_settings = $this->pg_seo->get_settings('user', 'listings', 'agents');
        if (!$user_settings['noindex']) {
            foreach ($langs as $lang_id => $lang_code) {
                foreach ($operation_types as $operation_type) {
                    $return[] = array(
                        'url' => rewrite_link('listings', 'agents', array('operation_type' => $operation_type), false, $lang_code, $lang_canonical),
                        'priority' => 0.5,
                    );
                }
            }
        }

        $user_settings = $this->pg_seo->get_settings('user', 'listings', 'discount');
        if (!$user_settings['noindex']) {
            foreach ($langs as $lang_id => $lang_code) {
                foreach ($operation_types as $operation_type) {
                    $return[] = array(
                        'url' => rewrite_link('listings', 'discount', array('operation_type' => $operation_type), false, $lang_code, $lang_canonical),
                        'priority' => 0.5,
                    );
                }
            }
        }

        /*$user_settings = $this->pg_seo->get_settings('user', 'listings', 'view');
        if (!$user_settings['noindex']) {
            foreach ($langs as $lang_id => $lang_code) {
                foreach ($operation_types as $operation_type) {
                    $listings = $this->get_listings_list(array('active' => 1, 'type' => $operation_type));
                    foreach ($listings as $seo_listing) {
                        $return[] = array(
                            'url' => rewrite_link('listings', 'view', $seo_listing, false, $lang_code, $lang_canonical),
                            'priority' => 0.8,
                        );
                    }
                }
            }
        }*/

        $user_settings = $this->pg_seo->get_settings('user', 'listings', 'wish_lists');
        if (!$user_settings['noindex']) {
            foreach ($langs as $lang_id => $lang_code) {
                $return[] = array(
                    'url' => rewrite_link('listings', 'wish_lists', array(), false, $lang_code, $lang_canonical),
                    'priority' => 0.5,
                );
            }
        }

        $user_settings = $this->pg_seo->get_settings('user', 'listings', 'wish_list');
        if (!$user_settings['noindex']) {
            $this->ci->load->model('listings/models/Wish_list_model');
            foreach ($langs as $lang_id => $lang_code) {
                $wish_lists = $this->ci->Wish_list_model->get_wish_lists_list(array('active' => 1));
                foreach ($wish_lists as $wish_list) {
                    $return[] = array(
                        'url' => rewrite_link('listings', 'wish_list', array('id' => $wish_list['id']), false, $lang_code, $lang_canonical),
                        'priority' => 0.6,
                    );
                }
            }
        }

        return $return;
    }

    /**
     * Return urls data for site map module
     *
     * @return array
     */
    public function get_sitemap_urls()
    {
        $auth = $this->ci->session->userdata('auth_type') === 'user';
        $block = array();

        $operation_types = $this->get_operation_types(true);
        if (empty($operation_types)) {
            return $block;
        }

        $items = array();
        foreach ($operation_types as $type) {
            if ($type === self::OPERATION_ALL) {
                continue;
            }
            $link = $this->ci->pg_seo->create_url('listings', 'index', array(
                'operation_type' => $type,
                'operation_type_str' => l("operation_search_$type", 'listings'),
                'id_category' => 0,
                'property_type' => 0,
                'order' => 'default',
                'order_direction' => 'DESC',
                'page' => 1
            ));
            $items[] = array(
                'name' => l('operation_search_' . $type, 'listings'),
                'link' => $link,
                'clickable' => true,
            );
        }

        if (PACKAGE_NAME !== 'agent') {
            $items[] = array(
                'name' => l('header_my_saved_listings', 'listings'),
                'link' => site_url() . 'listings/saved',
                'clickable' => (bool) $auth,
            );

            $items[] = array(
                'name' => l('header_saved_searches', 'listings'),
                'link' => site_url() . 'listings/preferences',
                'clickable' => (bool) $auth,
            );
        }

        reset($operation_types);
        $block[] = array(
            'name' => l('header_search_listings', 'listings'),
            'link' => $this->ci->pg_seo->create_url('listings', 'index', array('operation_type' => current($operation_types), 'id_category' => 0, 'property_type' => 0, 'order' => 'default', 'order_direction' => 'DESC', 'page' => 1)),
            'clickable' => true,
            'items' => $items,
        );

        return $block;
    }

    /**
     * Install fields for reviews data
     *
     * @param array $fields reviews fields
     * @return void
     */
    public function install_reviews_fields($fields = array())
    {
        if (empty($fields)) {
            return;
        }
        $this->ci->load->dbforge();
        $table_fields = $this->ci->db->list_fields(LISTINGS_TABLE);
        foreach ((array) $fields as $field_name => $field_data) {
            if (!in_array($field_name, $table_fields)) {
                $this->ci->dbforge->add_column(LISTINGS_TABLE, array($field_name => $field_data));
            }
        }
    }

    /**
     * Uninstall fields for reviews data
     *
     * @param array $fields reviews fields
     * @return void
     */
    public function deinstall_reviews_fields($fields = array())
    {
        if (empty($fields))
            return;
        $this->ci->load->dbforge();
        $table_fields = $this->ci->db->list_fields(LISTINGS_TABLE);
        foreach ($fields as $field_name) {
            if (in_array($field_name, $table_fields)) {
                $this->ci->dbforge->drop_column(LISTINGS_TABLE, $field_name);
            }
        }
    }

    /**
     * Generate qr code for listing url
     *
     * @param array $listing listing data
     * @return string
     */
    public function get_qr_code($listing)
    {
        $this->ci->load->helper('seo');
        $this->ci->load->library('Translit');

        $current_lang = $this->ci->pg_language->get_lang_by_id($this->ci->pg_language->current_lang_id);

        $link = rewrite_link('listings', 'view', $listing);
        $theme = $this->ci->pg_theme->get_active_settings();

        $site_color_hex = $theme['user']['scheme_data']['color_settings']['main_bg'];
        $site_color_bg_hex = $theme['user']['scheme_data']['color_settings']['html_bg'];

        $site_color_val = hexdec($site_color_hex);
        $site_color_bg_val = hexdec($site_color_bg_hex);

        $site_color_rgb = (0xFF & ($site_color_val >> 0x10)) . ',' . (0xFF & ($site_color_val >> 0x8)) . ',' . (0xFF & $site_color_val);
        $site_color_bg_rgb = (0xFF & ($site_color_bg_val >> 0x10)) . ',' . (0xFF & ($site_color_bg_val >> 0x8)) . ',' . (0xFF & $site_color_bg_val);

        $filename = $listing['id'] . '_' . $site_color_hex . '.jpg';

        $this->ci->load->library('QRcode');

        $filepath = FRONTEND_PATH . 'qr/' . $site_color_hex . '/' . $filename;

        if (!file_exists($filepath)) {
            if (!file_exists(FRONTEND_PATH . 'qr/' . $site_color_hex . '/')) {
                @mkdir(FRONTEND_PATH . 'qr/' . $site_color_hex . '/');
            }
            \QRcode::jpg($link, $filepath, 'H', 2, 2, 85, $site_color_rgb, $site_color_bg_rgb);
        } elseif (@filemtime($filepath) < time() - $this->qr_update_time * 60 * 60) {
            \QRcode::jpg($link, $filepath, 'H', 2, 2, 85, $site_color_rgb, $site_color_bg_rgb);
        }

        $url = FRONTEND_URL . 'qr/' . $site_color_hex . '/' . $filename;
        return $url;
    }

    /**
     * Processing events from spam module
     *
     * @param string $action action name
     * @param mix $data listing identifiers
     *
     * @return string
     */
    public function spam_callback($action, $data)
    {
        switch ($action) {
            case 'ban':
                $this->save_listing((int) $data, array('banned' => 1));
                return 'banned';
                break;
            case 'unban':
                $this->save_listing((int) $data, array('banned' => 0));
                return 'unbanned';
                break;
            case 'delete':
                $this->delete_listing((int) $data);
                return 'removed';
                break;
            case 'get_content':
                if (empty($data)) {
                    return array();
                }
                $listings = $this->get_listings_list(array('ids' => (array) $data));

                $this->ci->load->helper('listings');

                $return = array();
                foreach ($listings as $listing) {
                    $return[$listing['id']] = $listing['output_name'] . '<br>' .
                        $listing['property_type_str'] . ' ' .
                        $listing['operation_type_str'] . '<br>' .
                        listing_price_block(array('data' => $listing, 'template' => 'small'));
                }
                return $return;
                break;
            case 'get_link':
                if (empty($data)) {
                    return array();
                }
                $listings = $this->get_listings_list(array('ids' => (array) $data));
                $return = array();
                foreach ($listings as $listing) {
                    $return[$listing['id']] = site_url() . 'admin/listings/edit/' . $listing['id'];
                }
                return $return;
                break;
            case 'get_object':
                if (empty($data)) {
                    return array();
                }
                $listings = $this->get_listings_list(array('ids' => (array) $data));
                $return = array();
                foreach ($listings as $listing) {
                    $return[$listing['id']] = $listing;
                }
                return $return;
                break;
        }
    }

    /**
     * Propcessing events from reviews module
     *
     * @param string $action action name
     * @param array $data review data
     * @return string
     */
    public function callback_reviews($action, $data)
    {
        switch ($action) {
            case 'update':
                $listing_data['review_type'] = $data['type_gid'];
                $listing_data['review_sorter'] = $data['review_sorter'];
                $listing_data['review_value'] = $data['review_value'];
                $listing_data['review_count'] = $data['review_count'];
                $listing_data['review_data'] = serialize($data['review_data']);
                $this->save_listing($data['id_object'], $listing_data);
                break;
            case 'get_object':
                if (empty($data)) {
                    return array();
                }
                $listings = $this->get_listings_list(array('ids' => (array) $data));
                $return = array();
                foreach ($listings as $listing) {
                    $return[$listing['id']] = $listing;
                }
                return $return;
                break;
        }
    }

    /**
     * Return available banners places
     *
     * Callback method for banners module
     *
     * @return array
     */
    public function _banner_available_pages()
    {
        $return[] = array('link' => 'listings/index', 'name' => l('header_listings', 'listings'));
        $return[] = array('link' => 'listings/category', 'name' => l('header_category_listings', 'listings'));
        $return[] = array('link' => 'listings/location', 'name' => l('header_location_listings', 'listings'));
        $return[] = array('link' => 'listings/search', 'name' => l('header_search_listings', 'listings'));
        $return[] = array('link' => 'listings/open_house', 'name' => l('header_open_house_listings', 'listings'));

        if ($this->ci->pg_module->is_module_active("users_private")) {
            $return[] = array('link' => 'listings/privates', 'name' => l('header_privates_listings', 'listings'));
        }

        $return[] = array('link' => 'listings/agents', 'name' => l('header_agents_listings', 'listings'));
        $return[] = array('link' => 'listings/discount', 'name' => l('header_discount_listings', 'listings'));
        $return[] = array('link' => 'listings/user', 'name' => l('header_user_listings', 'listings'));
        $return[] = array('link' => 'listings/view', 'name' => l('header_view_listing', 'listings'));
        return $return;
    }

    /**
     * Return listings rss settings as array
     *
     * @return array
     */
    public function get_rss_settings()
    {
        $data = array(
            'rss_feed_channel_title' => $this->ci->pg_module->get_module_config('listings', 'rss_feed_channel_title'),
            'rss_feed_channel_description' => $this->ci->pg_module->get_module_config('listings', 'rss_feed_channel_description'),
            'rss_feed_image_url' => $this->ci->pg_module->get_module_config('listings', 'rss_feed_image_url'),
            'rss_feed_image_title' => $this->ci->pg_module->get_module_config('listings', 'rss_feed_image_title'),
            'rss_listings_max_count' => intval($this->ci->pg_module->get_module_config('listings', 'rss_listings_max_count')),
        );
        if ($data['rss_feed_image_url']) {
            $this->ci->load->model('Uploads_model');
            $data['rss_feed_image_media'] = $this->ci->Uploads_model->format_upload($this->rss_config_id, '', $data['rss_feed_image_url']);
        }
        return $data;
    }

    /**
     * Display listing with status "Sold"
     *
     * @param integer $listing_id listing identifier
     * @param integer $status sold status
     * @return void
     */
    public function make_sold($listing_id, $status)
    {
        $data[self::OPERATION_SOLD] = $status ? 1 : 0;
        $this->save_listing($listing_id, $data);
    }

    /**
     * Validate data from share form
     *
     * @param integer $listing_id listing identifier
     * @param array $data data to validation
     * @return array
     */
    public function validate_share($listing_id, $data)
    {
        $return = array('errors' => array(), 'data' => array());

        $is_moderation_installed = $this->ci->pg_module->is_module_active("moderation");

        // email
        if (isset($data['email'])) {
            $return['data']['email'] = trim(strip_tags($data['email']));
            if (empty($return['data']['email'])) {
                $return['errors'][] = l('error_empty_email', 'listings');
            } else {
                $this->ci->config->load('reg_exps', TRUE);
                $email_expr = $this->ci->config->item('email', 'reg_exps');
                if (!preg_match($email_expr, $return['data']['email'])) {
                    $return['errors'][] = l('error_email_incorrect', 'listings');
                }
            }
        } elseif (!$listing_id) {
            $return['errors'][] = l('error_empty_email', 'listings');
        }

        // user
        if (isset($data['user'])) {
            if (!empty($data['user'])) {
                $data['user'] = trim(strip_tags($data['user']));
            }
            if (!empty($data['user'])) {
                $return['data']['user'] = $data['user'];
                if ($is_moderation_installed) {
                    $this->ci->load->model('moderation/models/Moderation_badwords_model');
                    $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_share_type, $return['data']['user']);
                    if ($bw_count) {
                        $return['errors'][] = l('error_badwords_user', 'listings');
                    }
                }
            } else {
                $return['errors'][] = l('error_empty_user', 'listings');
            }
        } elseif (!$listing_id) {
            $return['errors'][] = l('error_empty_user', 'listings');
        }

        // message
        if (isset($data['message'])) {
            if (!empty($data['message'])) {
                $data['message'] = trim(strip_tags($data['message']));
            }
            if (!empty($data['message'])) {
                $return['data']['message'] = $data['message'];
                if ($is_moderation_installed) {
                    $this->ci->load->model('moderation/models/Moderation_badwords_model');
                    $bw_count = $this->ci->Moderation_badwords_model->check_badwords($this->moderation_share_type, $return['data']['message']);
                    if ($bw_count) {
                        $return['errors'][] = l('error_badwords_message', 'listings');
                    }
                }
            } else {
                $return['errors'][] = l('error_empty_message', 'listings');
            }
        } elseif (!$listing_id) {
            $return['errors'][] = l('error_empty_message', 'listings');
        }

        return $return;
    }

    public function applyFormatSettings($settings = array())
    {
        $this->old_settings = $this->format_settings;
        $this->set_format_settings($settings);
    }

    public function revertFormatSettings()
    {
        $this->format_settings = $this->old_settings;
    }

    /**
     * Return criteria for used in fulltext search
     *
     * @param ineteger $listing_id listing identifier
     * @param array $fields fulltext fields
     * @return array
     */
    public function get_fulltext_data($listing_id, $fields)
    {
        $all_langs = $this->ci->pg_language->return_langs();
        $default_lang = $this->ci->pg_language->get_default_lang_id();

        $return = array(
            'main_fields' => array(),
            'fe_fields' => array(),
            'default_lang_id' => $default_lang,
            'object_lang_id' => $default_lang,
        );

        $this->applyFormatSettings(array('get_description' => true, 'get_location' => false, 'get_price_str' => false, 'use_format' => true));
        $data = $this->get_listing_by_id($listing_id);
        $this->revertFormatSettings();

        $use_multilang_search = $this->ci->pg_module->get_module_config('field_editor', 'use_multilang_search');
        if ($use_multilang_search) {
            $langs = $all_langs;
        } else {
            if (in_array($data['user']['lang_id'], $this->ci->pg_language->languages)) {
                $return['object_lang_id'] = $data['user']['lang_id'];
            }

            $current_lang = $return['object_lang_id'];
            $langs[$current_lang] = $all_langs[$current_lang];
            if ($current_lang != $default_lang) {
                $langs[$default_lang] = $all_langs[$default_lang];
            }
        }

        $return['main_fields'] = array('user' => $data['user']['output_name']);
        if (!empty($data['headline'])) {
            $return['main_fields']['headline'] = $data['headline'];
        }

        $this->ci->load->model('Properties_model');
        $this->ci->load->model('Countries_model');

        $this->load->helper('start');
        switch ($data['operation_type']) {
            case self::OPERATION_SALE:
                if (!$data['price_negotiated']) {
                    if ($data['price_reduced'] > 0) {
                        $return['main_fields']['price'] = strip_tags(currency_format_output(array('value' => $data['price_reduced'], 'cur_gid' => $data['gid_currency'])));
                    } else {
                        $return['main_fields']['price'] = strip_tags(currency_format_output(array('value' => $data['price'], 'cur_gid' => $data['gid_currency'])));
                    }
                }
                break;
            case self::OPERATION_BUY:
                if (!$data['price_negotiated']) {
                    $price = array();
                    if ($data['price'] > 0) {
                        $price[] = l('text_price_from', 'listings') . ' ' . strip_tags(currency_format_output(array('value' => $data['price'], 'cur_gid' => $data['gid_currency'])));
                    }
                    if ($data['price_max'] > 0) {
                        $price[] = l('text_price_to', 'listings') . ' ' . strip_tags(currency_format_output(array('value' => $data['price_max'], 'cur_gid' => $data['gid_currency'])));
                    }
                    $return['main_fields']['price'] = implode(' ', $price);
                }
                break;
            case self::OPERATION_RENT:
                if (!$data['price_negotiated']) {
                    if ($data['price_reduced'] > 0) {
                        $return['main_fields']['price'] = strip_tags(currency_format_output(array('value' => $data['price_reduced'], 'cur_gid' => $data['gid_currency'])));
                    } else {
                        $return['main_fields']['price'] = strip_tags(currency_format_output(array('value' => $data['price'], 'cur_gid' => $data['gid_currency'])));
                    }
                    $return['main_fields']['price'] .= ' ' . ld_option('price_period', 'listings', $data['price_period']);
                    if (!empty($data['price_type'])) {
                        $return['main_fields']['price'] .= '/' . ld_option('price_period', 'listings', $data['price_type']);
                    }
                }
                break;
            case self::OPERATION_LEASE:
                if (!$data['price_negotiated']) {
                    $price = array();
                    if ($data['price'] > 0) {
                        $price[] = l('text_price_from', 'listings') . ' ' . strip_tags(currency_format_output(array('value' => $data['price'], 'cur_gid' => $data['gid_currency'])));
                    }
                    if ($data['price_max'] > 0) {
                        $price[] = l('text_price_to', 'listings') . ' ' . strip_tags(currency_format_output(array('value' => $data['price_max'], 'cur_gid' => $data['gid_currency'])));
                    }
                    $return['main_fields']['price'] = implode(' ', $price);
                    $return['main_fields']['price'] .= ' ' . ld_option('price_period', 'listings', $data['price_period']);
                    if ($return['main_fields']['price_type']) {
                        $return['main_fields']['price'] .= '/' . ld_option('price_period', 'listings', $data['price_type']);
                    }
                }
                break;
        }

        foreach ($langs as $lang) {
            $square_units = $this->ci->pg_language->ds->get_reference('listings', $this->square_units_gid, $lang['id']);

            $operation_types = $this->ci->pg_language->ds->get_reference('listings', 'operation_type', $lang['id']);
            $operation_types = $operation_types['option'];
            $return['main_fields']['operation_type_' . $lang['code']] = $operation_types[$data['operation_type']];

            $property_types = $this->ci->Properties_model->get_categories('property_types', $lang['id']);
            $property_items = $this->ci->Properties_model->get_all_properties('property_types', $lang['id']);
            $return['main_fields']['category_' . $lang['code']] = $property_types[$data['id_category']];
            $return['main_fields']['property_' . $lang['code']] = $property_items[$data['id_category']][$data['property_type']];

            if ($data[self::OPERATION_SOLD]) {
                $return['main_fields']['sold_' . $lang['code']] = l('field_sold', 'listings', $lang['id']);
            }

            $country = $this->ci->Countries_model->get_country($data['id_country'], $lang['id']);
            if (isset($country['name']) && !empty($country['name'])) {
                $return['main_fields']['location_' . $lang['code']] = $country['name'];

                $region = $this->ci->Countries_model->get_region($data['id_region'], $lang['id']);
                if (isset($region['name']) && !empty($region['name'])) {
                    $return['main_fields']['location_' . $lang['code']] .= ' ' . $region['name'];

                    $city = $this->ci->Countries_model->get_city($data['id_city'], $lang['id']);
                    if (isset($city['name']) && !empty($city['name'])) {
                        $return['main_fields']['location_' . $lang['code']] .= ' ' . $city['name'];

                        if ($data['id_district']) {
                            $district = $this->ci->Countries_model->get_district($data['id_district'], $lang['id']);

                            if (isset($district['name']) && !empty($district['name'])) {
                                $return['main_fields']['location_' . $lang['code']] .= ' ' . $district['name'];
                            }
                        }
                    }
                }
            }

            if (!empty($data['address'])) {
                $return['main_fields']['location_' . $lang['code']] .= ' ' . $data['address'];
            }

            if (isset($data['zip'])) {
                $return['main_fields']['zip_' . $lang['code']] = l('field_zip', 'listings', $lang['id']) . ' ' . $data['zip'];
            }

            if ($data['price_negotiated']) {
                $return['main_fields']['price_' . $lang['code']] = l('field_price_negotiated_' . $data['operation_type'], 'listings', $lang['id']);
            } else {
                $return['main_fields']['price_' . $lang['code']] = l('field_price', 'listings', $lang['id']) . ' ' . $return['main_fields']['price'];
            }

            $return['main_fields']['square_' . $lang['code']] = l('field_square', 'listings', $lang['id']) . ' ' . $data['square'];
            if ($data['square_unit']) {
                $return['main_fields']['square_' . $lang['code']] .= ' ';
                if (isset($square_units['option'][$data['square_unit']])) {
                    $return['main_fields']['square_' . $lang['code']] .= $square_units['option'][$data['square_unit']];
                } else {
                    $return['main_fields']['square_' . $lang['code']] .= $data['square_unit'];
                }
            }
        }

        if (isset($return['main_fields']['price'])) {
            unset($return['main_fields']['price']);
        }

        foreach ($fields as $field) {
            if (empty($data[$field])) {
                continue;
            }
            $return['fe_fields'][$field] = $data[$field];
        }

        return $return;
    }

    /**
     * Return seo data to rewrite url of search page
     *
     * @return array
     */
    public function get_search_link_data()
    {
        //load properties
        $this->ci->load->model('Properties_model');
        $property_types = $this->ci->Properties_model->get_categories('property_types', $this->ci->pg_language->current_lang_id);
        $property_items = $this->ci->Properties_model->get_all_properties('property_types', $this->ci->pg_language->current_lang_id);

        $current_settings = isset($_SESSION['listings_search']) ? $_SESSION['listings_search'] : array();

        $operation_types = $this->get_operation_types();
        if (in_array(self::OPERATION_SALE, $operation_types))
            $operation_types[] = self::OPERATION_SOLD;
        if (isset($current_settings['data']['type']) &&
            in_array($current_settings['data']['type'], $operation_types)) {
            $operation_type = $current_settings['data']['type'];
        } else {
            $operation_type = $this->get_operation_type_gid_default();
        }

        $category_id = $property_type = 0;
        if (isset($current_settings['data']['id_category'])) {
            $category_id = intval($current_settings['data']['id_category']);
            if ($category_id && isset($current_settings['data']['property_type'])) {
                $property_type = intval($current_settings['data']['property_type']);
            }
        }

        if (isset($current_settings['data']['id_country']) || isset($current_settings['data']['id_region']) || isset($current_settings['data']['id_city']) || isset($current_settings['data']['id_district'])) {
            $this->ci->load->helper('countries');
            $locations_data = get_location_data(
                array(
                    array(
                        'country' => isset($current_settings['data']['id_country']) ? $current_settings['data']['id_country'] : '',
                        'region' => isset($current_settings['data']['id_region']) ? $current_settings['data']['id_region'] : '',
                        'city' => isset($current_settings['data']['id_city']) ? $current_settings['data']['id_city'] : '',
                        'district' => isset($current_settings['data']['id_district']) ? $current_settings['data']['id_district'] : '',
                    )
                )
            );
            $country = (isset($current_settings['data']['id_country']) && isset($locations_data['country'][$current_settings['data']['id_country']]['name'])) ? $locations_data['country'][$current_settings['data']['id_country']]['name'] : '';
            $region = (isset($current_settings['data']['id_region']) && isset($locations_data['region'][$current_settings['data']['id_region']]['name'])) ? $locations_data['region'][$current_settings['data']['id_region']]['name'] : '';
            $region_code = (isset($current_settings['data']['id_region']) && isset($locations_data['region'][$current_settings['data']['id_region']]['code'])) ? $locations_data['region'][$current_settings['data']['id_region']]['code'] : '';
            $city = (isset($current_settings['data']['id_city']) && isset($locations_data['city'][$current_settings['data']['id_city']]['name'])) ? $locations_data['city'][$current_settings['data']['id_city']]['name'] : '';
            $district = (isset($current_settings['data']['id_district']) && isset($locations_data['district'][$current_settings['data']['id_district']]['name'])) ? $locations_data['district'][$current_settings['data']['id_district']]['name'] : '';
        } else {
            $country = '';
            $region = '';
            $region_code = '';
            $city = '';
            $district = '';
        }

        $keyword = '';
        if (isset($current_settings['data']['keyword'])) {
            $keyword = $current_settings['data']['keyword'];
        }

        $order = 'default';
        $order_direction = 'DESC';
        $page = 1;

        if (!empty($current_settings)) {
            $order = get($current_settings['order'], $order);
            $order_direction = get($current_settings['order_direction'], $order_direction);
            $page = get($current_settings['page'], $page);
        }

        $action_data = array(
            'operation_type' => $operation_type,
            'operation_type_str' => l('operation_search_' . $operation_type, 'listings'),
            'id_category' => $category_id,
            'category' => $category_id ? $property_types[$category_id] : 'all',
            'property_type' => $property_type,
            'property' => $property_type ? $property_items[$category_id][$property_type] : 'all',
            'country' => $country,
            'region' => $region,
            'region_code' => $region_code,
            'city' => $city,
            'district' => $district,
            'keyword' => $keyword,
            'order' => $order,
            'order_direction' => $order_direction,
            'page' => $page,
        );

        return $action_data;
    }

    /**
     * Return data of current search criteria
     *
     * @return array
     */
    public function get_search_data()
    {
        $filters = isset($_SESSION['listings_search']) ? $_SESSION['listings_search']['data'] : array();

        $operation_types = $this->get_operation_types(true);

        $current_search_data = array();
        if (isset($filters['type']) && in_array($filters['type'], $operation_types)) {
            $operation_type = $filters['type'];
            unset($filters['type']);
        } else {
            $operation_type = current($operation_types);
        }

        if ($operation_type == self::OPERATION_SOLD) {
            $current_search_data['type'] = array(
                'name' => l('field_listing_type', 'listings'),
                'value' => self::OPERATION_SOLD,
                'label' => l('text_sold', 'listings'),
            );
        } else {
            $current_search_data['type'] = array(
                'name' => l('field_listing_type', 'listings'),
                'value' => $operation_type,
                'label' => l('operation_search_' . $operation_type, 'listings'),
            );
        }

        if (isset($filters['id_country']) && !empty($filters['id_country'])) {
            $location_data = array();
            $location_data['country'] = $filters['id_country'];
            unset($filters['id_country']);

            $this->ci->load->helper('countries');

            if (isset($filters['id_region']) && !empty($filters['id_region'])) {
                $location_data['region'] = $filters['id_region'];
                unset($filters['id_region']);

                if (isset($filters['id_city']) && !empty($filters['id_city'])) {
                    $location_data['city'] = $filters['id_city'];
                    unset($filters['id_city']);

                    if (isset($filters['id_district']) && !empty($filters['id_district'])) {
                        $location_data['district'] = $filters['id_district'];
                        unset($filters['id_district']);

                        $location = districts_output_format(array($location_data));
                        if (!empty($location)) {
                            $current_search_data['location'] = array(
                                'name' => l('field_location', 'listings'),
                                'value' => array($location_data['country'], (int) $location_data['region'], (int) $location_data['city'], (int) $location_data['district']),
                                'label' => $location[0],
                            );
                        }
                    } else {
                        $location = cities_output_format(array($location_data));
                        if (!empty($location)) {
                            $current_search_data['location'] = array(
                                'name' => l('field_location', 'listings'),
                                'value' => array($location_data['country'], (int) $location_data['region'], (int) $location_data['city']),
                                'label' => $location[0],
                            );
                        }
                    }
                } else {
                    $location = regions_output_format(array($location_data));
                    if (!empty($location)) {
                        $current_search_data['location'] = array(
                            'name' => l('field_location', 'listings'),
                            'value' => array($location_data['country'], (int) $location_data['region']),
                            'label' => $location[0],
                        );
                    }
                }
            } else {
                $location = countries_output_format(array($location_data));
                if (!empty($location)) {
                    $current_search_data['location'] = array(
                        'name' => l('field_location', 'listings'),
                        'value' => array($location_data['country']),
                        'label' => $location[0],
                    );
                }
            }
        }

        if (isset($filters['id_category'])) {
            $this->ci->load->helper('properties');
            if (isset($filters['property_type'])) {
                $value = property_value($filters);
                $current_search_data['property_type'] = array(
                    'name' => l('field_category', 'listings'),
                    'value' => $filters['id_category'] . '_' . $filters['property_type'],
                    'label' => $value ? $value : $filters['property_type'],
                );
                unset($filters['property_type']);
            } else {
                $value = category_value($filters);
                $current_search_data['id_category'] = array(
                    'name' => l('field_category', 'listings'),
                    'value' => $filters['id_category'],
                    'label' => $value ? $value : $filters['id_category'],
                );
            }
            unset($filters['id_category']);
        }

        $this->ci->load->helper('start');

        // price
        if (isset($filters['price_max'])) {
            if (!isset($filters['price_min'])) {
                $filters['price_min'] = 0;
            }
            $filters['price'] = ' ' . l('text_price_to', 'listings') . ' ' . strip_tags(currency_format_output(array('value' => $filters['price_max'])));
            unset($filters['price_max']);
        }

        if (isset($filters['price_min'])) {
            $filters['price'] = l('text_price_from', 'listings') . ' ' . strip_tags(currency_format_output(array('value' => $filters['price_min']))) . (isset($filters['price']) ? $filters['price'] : '');
            unset($filters['price_min']);
        }

        // square
        if (isset($filters['square_max'])) {
            if (!isset($filters['square_min'])) {
                $filters['square_min'] = 0;
            }
            $filters['square'] = ' ' . l('text_price_to', 'listings') . ' ' . $filters['square_max'];
            unset($filters['square_max']);
        }

        if (isset($filters['square_min'])) {
            $filters['square'] = l('text_price_from', 'listings') . ' ' . $filters['square_min'] . (isset($filters['square']) ? $filters['square'] : '');
            unset($filters['square_min']);
        }

        $this->config->load('date_formats', TRUE);
        $date_format = $this->ci->pg_date->get_format('date_literal', 'date');

        if ($this->ci->pg_module->is_module_active('listings_booking')) {
            if (isset($filters['booking_date_start'])) {
                $filters['booking_date_start'] = date($date_format, strtotime($filters['booking_date_start']));
            }

            if (isset($filters['booking_date_end'])) {
                $filters['booking_date_end'] = date($date_format, strtotime($filters['booking_date_end']));
            }
        }

        if (isset($filters['by_agents']) && $filters['by_agents']) {
            $filters['by_agents'] = l('yes_str', 'start');
        }

        if (isset($filters['by_private']) && $filters['by_private']) {
            $filters['by_private'] = l('yes_str', 'start');
        }

        if (isset($filters['by_open_house']) && $filters['by_open_house']) {
            $filters['by_open_house'] = l('yes_str', 'start');
        }

        if (isset($filters['with_photo']) && $filters['with_photo']) {
            $filters['with_photo'] = l('yes_str', 'start');
        }

        foreach ((array) $filters as $field_name => $field_value) {
            if ($field_name == 'field_editor_data')
                continue;
            $label = $value = is_array($field_value) ? implode(',', $field_value) : $field_value;
            $current_search_data[$field_name] = array(
                'name' => l('field_' . $field_name, 'listings'),
                'value' => $label,
                'label' => $value,
            );
        }

        if (isset($filters['field_editor_data'])) {
            $property_type_gid = $this->get_field_editor_type($filters);

            $this->ci->load->model('Field_editor_model');
            $this->ci->Field_editor_model->initialize($property_type_gid);

            $fields_gids = array();
            foreach ($filters['field_editor_data'] as $form_name => $form_data) {
                $fields_gids = array_merge($fields_gids, array_keys($form_data));
            }

            $params = array('where_in' => array('gid' => $fields_gids));
            $fields_data = $this->ci->Field_editor_model->get_fields_list($params);

            $field_editor_data = array('gids' => array(), 'data' => array());
            $field_editor_range = array('gids' => array(), 'data' => array());
            foreach ($filters['field_editor_data'] as $form_name => $form_data) {
                foreach ($form_data as $field_gid => $field_data) {
                    if (isset($field_data['range']) && is_array($field_data['range'])) {
                        if (isset($field_data['range']['min'])) {
                            $field_editor_data['gids'][$field_gid] = $field_data['range']['min'];
                            $field_editor_data['data'][$fields_data[$field_gid]['field_name']] = $field_data['range']['min'];
                        } else {
                            $field_editor_data['gids'][$field_gid] = 0;
                            $field_editor_data['data'][$fields_data[$field_gid]['field_name']] = 0;
                        }
                        if (isset($field_data['range']['max'])) {
                            $field_editor_range['gids'][$field_gid] = $field_data['range']['max'];
                            $field_editor_range['data'][$fields_data[$field_gid]['field_name']] = $field_data['range']['max'];
                        } else {
                            $field_editor_range['gids'][$field_gid] = 0;
                            $field_editor_range['data'][$fields_data[$field_gid]['field_name']] = 0;
                        }
                    } else {
                        $field_editor_data['gids'][$field_gid] = $field_data;
                        $field_editor_data['data'][$fields_data[$field_gid]['field_name']] = $field_data;
                    }
                }
            }

            $params = array('where_in' => array('gid' => array_keys($field_editor_data['gids'])));
            $results = $this->ci->Field_editor_model->format_list_fields_for_view($params, array($field_editor_data['data']));
            foreach ($results as $result) {
                foreach ($result as $field_gid => $field_data) {
                    if (!empty($field_data['value_str'])) {
                        $value = $field_data['value_str'];
                    } elseif (!empty($field_data['value_dec'])) {
                        $value = $field_data['value_dec'];
                    } elseif (!empty($field_data['value_original'])) {
                        $value = $field_data['value_original'];
                    } else {
                        $value = $field_data['value'];
                    }
                    $current_search_data['field_editor/' . $field_gid] = array(
                        'name' => $field_data['name'],
                        'value' => isset($filters['field_editor_data'][$field_gid]) ? $filters['field_editor_data'][$field_gid] : '',
                        'label' => $value != '' || $field_editor_data['gids'][$field_gid] !== 0 ? $value : 0,
                    );
                }
            }

            if (!empty($field_editor_range['gids'])) {
                $params = array('where_in' => array('gid' => array_keys($field_editor_range['gids'])));
                $results = $this->ci->Field_editor_model->format_list_fields_for_view($params, array($field_editor_range['data']));
                foreach ($results as $result) {
                    foreach ($result as $field_gid => $field_data) {
                        if (!empty($field_data['value_str'])) {
                            $value = $field_data['value_str'];
                        } elseif (!empty($field_data['value_dec'])) {
                            $value = $field_data['value_dec'];
                        } elseif (!empty($field_data['value_original'])) {
                            $value = $field_data['value_original'];
                        } else {
                            $value = $field_data['value'];
                        }
                        if ($value) {
                            if (isset($current_search_data['field_editor/' . $field_gid]) && $current_search_data['field_editor/' . $field_gid]['label']) {
                                $current_search_data['field_editor/' . $field_gid]['label'] .= '-' . $value;
                            } else {
                                $current_search_data['field_editor/' . $field_gid]['label'] = '&le;' . $value;
                            }
                        } elseif (isset($current_search_data['field_editor/' . $field_gid])) {
                            $current_search_data['field_editor/' . $field_gid]['label'] = '&ge;' . $current_search_data['field_editor/' . $field_gid]['label'];
                        }
                    }
                }
            }
        }

        return $current_search_data;
    }

    /**
     * Return available user types with names
     *
     * @param string $template_gid template guid
     * @return array
     */
    public function service_user_type_method($template_gid)
    {
        $is_users_installed = $this->ci->pg_module->is_module_active("users");
        if ($is_users_installed) {
            $this->ci->load->model('Users_model');
            $user_types = $this->ci->Users_model->getUserTypesNames();
        } else {
            $this->ci->load->model('Start_model');
            $user = $this->ci->Start_model->getUserDefaultOnSite();
            $user_types = $user['user_type'];
        }
        return $user_types;
    }

    /**
     * Return field editor type from listing data
     *
     * @param array $data input data
     * @param boolean $all use disabled
     * @return string
     */
    public function get_field_editor_type($data, $all = true)
    {
        if (isset($data['field_editor_type'])) {
            return $data['field_editor_type'];
        }

        if ($all) {
            $operation_types = $this->operations_arr;
        } else {
            $operation_types = $this->get_operation_types();
        }

        if (empty($operation_types)) {
            return '';
        }

        if (isset($data['operation_type']) && in_array($data['operation_type'], $operation_types)) {
            $operation_type = $data['operation_type'];
        } elseif (isset($data['type']) && in_array($data['type'], $operation_types)) {
            $operation_type = $data['type'];
        } elseif (isset($data['id_type']) && isset($operation_types[$data['id_type']])) {
            $operation_type = $operation_types[$data['id_type']];
        } else {
            $operation_type = current($operation_types);
        }

        if (isset($data['id_category']) && isset($this->field_editor_types[$operation_type][$data['id_category']])) {
            $field_editor_type = $this->field_editor_types[$operation_type][$data['id_category']];
        } else {
            $field_editor_type = current($this->field_editor_types[$operation_type]);
        }

        return $field_editor_type;
    }

    /**
     * Return field editor type by default
     *
     * @return string
     */
    public function get_field_editor_type_default()
    {
        $operation_type = $this->get_operation_type_gid_default();
        if ($operation_type == self::OPERATION_ALL) {
            return '';
        }
        return current($this->field_editor_types[$operation_type]);
    }

    /**
     * Update service date end
     *
     * @param string $date_field date field
     * @param array $service_field service field
     * @return void
     */
    public function update_service_date_end($date_field, $service_field)
    {
        $this->ci->db->where($date_field . ' <', date('Y-m-d H:i:s'));
        $this->ci->db->where('UNIX_TIMESTAMP(' . $date_field . ') >', '86400');
        $this->ci->db->update(LISTINGS_TABLE, array($service_field => 0, $date_field => '0000-00-00 00:00:00'));
    }

    /**
     * Recalculate min/max price range
     *
     * @param integer $listing_id listing identifier
     * @param string $operation_type operation type
     * @return array
     */
    public function getPriceRange($listing_id, $operation_type)
    {
        $this->ci->db->select('MIN(CASE WHEN price_reduced>0 THEN price_reduced ELSE price END) AS min, MAX(CASE WHEN price_max>0 THEN price_max WHEN price_reduced>0 THEN price_reduced ELSE price END) AS max')
            ->from(LISTINGS_TABLE)
            ->where('status', self::STATUS_ACTIVE)
            ->where('price_negotiated', 0)
            ->where('id !=', $listing_id);
        if ($operation_type == self::OPERATION_SOLD) {
            $operation_type = self::OPERATION_SALE;
            $this->ci->db->where('sold', 1);
        } else {
            $this->ci->db->where('sold', 0);
        }
        $this->ci->db->where('id_type', $this->get_operation_type_by_gid($operation_type));
        $results = $this->ci->db->get()->result_array();
        return [
            'min' => get($results[0]['min']),
            'max' => get($results[0]['max']),
        ];
    }

    /**
     * Recalculate min/max square range
     *
     * @param integer $listing_id listing identifier
     * @param string $operation_type operation type
     * @return array
     */
    public function getSquareRange($listing_id, $operation_type)
    {
        $this->ci->db->select('MIN(square) AS min, MAX(CASE WHEN square_max>0 THEN square_max ELSE square END) AS max')
            ->from(LISTINGS_TABLE)
            ->where('status', self::STATUS_ACTIVE)
            ->where('id !=', $listing_id);
        if ($operation_type == self::OPERATION_SOLD) {
            $operation_type = self::OPERATION_SALE;
            $this->ci->db->where('sold', 1);
        } else {
            $this->ci->db->where('sold', 0);
        }
        $this->ci->db->where('id_type', $this->get_operation_type_by_gid($operation_type));
        $results = $this->ci->db->get()->result_array();
        return [
            'min' => get($results[0]['min']),
            'max' => get($results[0]['max']),
        ];
    }

    /**
     * Resets ranges config
     */
    public function resetRanges()
    {
        foreach ($this->operations_arr as $operation_type) {
            $price_range = $this->getPriceRange(-1, $operation_type);
            $this->ci->pg_module->set_module_config('listings', 'price_min_' . $operation_type, (float) $price_range['min']);
            $this->ci->pg_module->set_module_config('listings', 'price_max_' . $operation_type, (float) $price_range['max']);

            $square_range = $this->getSquareRange(-1, $operation_type);
            $this->ci->pg_module->set_module_config('listings', 'square_min_' . $operation_type, (float) $square_range['min']);
            $this->ci->pg_module->set_module_config('listings', 'square_max_' . $operation_type, (float) $square_range['max']);
        }
    }

    public function getSquareUnits($lang_id = null)
    {
        if (is_null($lang_id)) {
            $lang_id = $this->ci->pg_language->current_lang_id;
        }
        return $this->ci->pg_language->ds->get_reference('listings', $this->square_units_gid, $lang_id);
    }

    public function formatPrice($data, $template='')
    {
        $this->ci->load->helper('start');

        if ($data['price_negotiated']) {
            return l('text_negotiated_price_' . $data['operation_type'], 'listings');
        }

        if ($data['price'] > 0) {
            $price_str = currency_format_output([
                'value' => $data['price'],
                'cur_gid' => $data['gid_currency']
            ]);
        } else {
            $price_str = '';
        }

        if ($data['price_max'] > 0) {
            $price_max_str = currency_format_output([
                'value' => $data['price_max'],
                'cur_gid' => $data['gid_currency']
            ]);
        } else {
            $price_max_str = '';
        }

        if ($data['price_reduced'] > 0) {
            $price_reduce_str = currency_format_output([
                'value' => $data['price_reduced'],
                'cur_gid' => $data['gid_currency']
            ]);
        } else {
            $price_reduce_str = '';
        }

        $value = '';

        switch ($data['operation_type']) {
            case self::OPERATION_SALE:
                if ($data['price_reduced'] > 0) {
                    if ($template == 'small') {
                        $value .= $price_reduce_str;
                    } else {
                        $value .= '<span>' . $price_reduce_str . '&nbsp;</span>'
                               .  '<del>' . $price_str . '</del>';
                    }
                } else {
                    $value .= '<span>' . $price_str . '</span>';
                }
                break;

            case self::OPERATION_BUY:
                if ($data['price'] > 0) {
                    $value .= ' ' . l('text_price_from', 'listings') . ' ' . $price_str;
                }
                if ($data['price_max'] > 0) {
                    $value .= ' ' . l('text_price_to', 'listings') . ' ' . $price_max_str;
                }
                break;

            case self::OPERATION_RENT:
                $price_value = $data['price'];

                if ($data['price_week'] && $data['price_week'] < $price_value) {
                    $price_value = $data['price_week'];
                }

                if ($data['price_month'] && $data['price_month'] < $price_value) {
                    $price_value = $data['price_month'];
                }

                if ($data['price_week'] || $data['price_month']) {
                    $value .= ' ' . l('text_price_from', 'listings') . '&nbsp;';
                }

                if ($data['price_reduced'] > 0) {
                    if ($template == 'small') {
                        if ($data['price_reduced'] < $price_value) {
                            $price_value = $data['price_reduced'];
                        }

                        $price_value_str = currency_format_output([
                            'value' => $price_value,
                            'cur_gid' => $data['gid_currency']
                        ]);

                        $value = '<span>' . $price_value_str . '</span>';
                    } else {
                        $price_value_str = currency_format_output([
                            'value' => $price_value,
                            'cur_gid' => $data['gid_currency']
                        ]);

                        if ($data['price_reduced'] < $price_value) {
                            $value .= '<span>' . $price_reduce_str . '&nbsp;</span>'
                                   .  '<del>' . $price_value_str . '</del>';
                        } else {
                            $value .= '<span>' . $price_value_str . '</span>';
                        }
                    }
                } else {
                    $price_value_str = currency_format_output([
                        'value' => $price_value,
                        'cur_gid' => $data['gid_currency']
                    ]);

                    $value .= '<span>' . $price_value_str . '</span>';
                }

                $value .= '&nbsp;' . $data['price_period_str'];
                break;

            case self::OPERATION_LEASE:
                if ($data['price'] > 0) {
                    $value .= ' ' . l('text_price_from', 'listings') . ' ' . $price_str;
                }
                if ($data['price_max'] > 0) {
                    $value .= ' ' . l('text_price_to', 'listings') . ' ' . $price_max_str;
                }

                $value .= '&nbsp;' . $data['price_period_str'];
                break;
        }

        return trim($value);
    }
}
